// ====================================================================
// This file is part of FlexibleSUSY.
//
// FlexibleSUSY is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// FlexibleSUSY is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with FlexibleSUSY.  If not, see
// <http://www.gnu.org/licenses/>.
// ====================================================================

// File generated at Thu 17 Aug 2017 11:21:23

/**
 * @file MSSMNoFVHimalaya_a_muon.cpp
 *
 * This file was generated at Thu 17 Aug 2017 11:21:23 with FlexibleSUSY
 * 1.7.4-Himalaya and SARAH 4.12.0 .
 */

#include "MSSMNoFVHimalaya_a_muon.hpp"
#include "MSSMNoFVHimalaya_mass_eigenstates.hpp"

#include "error.hpp"
#include "logger.hpp"
#include "numerics2.hpp"
#include "wrappers.hpp"

#include <array>
#include <algorithm>
#include <vector>
#include <boost/mpl/vector_c.hpp>
#include <boost/mpl/at.hpp>

#define INPUTPARAMETER(p) context.model.get_input().p
#define MODELPARAMETER(p) context.model.get_##p()
#define DERIVEDPARAMETER(p) context.model.p()

namespace flexiblesusy {
namespace {
/**
 * @class IndexBounds<N>
 * @brief A class representing multiple (N) index ranges.
 *
 * N is a non-negative integer.
 * The ranges are specified the c++ way; The range begins are inclusive
 * and the range ends are exclusive. Misspecified ranges result in
 * undefined behaviour!
 * The intended use is to iterate over an IndexBounds<N> using the normal
 * begin()/end() syntax, which will iterate over every possible combination
 * of the indices within their respective ranges.
 * The ranges are const! Once they are initialized they cannot be changed.
 * Initialization is done like = {{ beg1, beg2, ... }, { end1, end2, ... }}
 */
template<unsigned N> struct IndexBounds;

/**
 * @class IndexIterator<N>
 * @brief The iterator class used by IndexBounds<N>.
 *
 * It only fulfils the input iterator requirements!
 * The value_type is a const std::array<unsigned, N>
 * containing the current indices.
 * Incrementing the iterator results in a new index combination.
 */
template<unsigned N> struct IndexIterator
: public std::iterator<
std::input_iterator_tag,
const std::array<unsigned, N>>
{
   friend struct IndexBounds<N>;
private:
   const IndexBounds<N> *bounds;
   std::array<unsigned, N> indices;

   IndexIterator(const IndexBounds<N> &b, const unsigned *i)
      : bounds(&b)
   {
      std::memcpy(indices.data(), i, N * sizeof(unsigned));
   }
public:
   const std::array<unsigned, N> &operator*() const {
      return indices;
   }

   const std::array<unsigned, N> *operator->() const {
      return &indices;
   }

   IndexIterator &operator++() {
      for (unsigned i = 0; i != N; i++) {
         indices[i]++;
         if (indices[i] == bounds->indexEnd[i]) {
            indices[i] = bounds->indexBegin[i];
            continue;
         }

         return *this;
      }

      std::memcpy(indices.data(), bounds->indexEnd, N * sizeof(unsigned));
      return *this;
   }

   IndexIterator operator++(int) {
      IndexIterator it(*this);
      this->operator++();
      return it;
   }

   template<unsigned M>
   friend bool operator==(const IndexIterator<M> &it1, const IndexIterator<M> &it2);
};

template<unsigned N>
bool operator==(const IndexIterator<N> &it1, const IndexIterator<N> &it2)
{
   return it1.indices == it2.indices;
}

template<unsigned N>
bool operator!=(const IndexIterator<N> &it1, const IndexIterator<N> &it2)
{
   return !(it1 == it2);
}

template<unsigned N> struct IndexBounds {
   typedef const std::array<unsigned, N> indices_type;

   unsigned indexBegin[N];
   unsigned indexEnd[N];

   typedef IndexIterator<N> const_iterator;

   const_iterator begin() const {
      return const_iterator(*this, indexBegin);
   }

   const_iterator end() const {
      return const_iterator(*this, indexEnd);
   }
};

template<> struct IndexBounds<0> {
   typedef const std::array<unsigned, 0> indices_type;
   typedef indices_type * const_iterator;

   static const indices_type dummyIndex;

   const_iterator begin() const
   { return &dummyIndex; }
   const_iterator end() const
   { return (begin()+1); }
};
IndexBounds<0>::indices_type IndexBounds<0>::dummyIndex{};

// Particles (SARAH-style)
struct Particle {};

struct VG: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool VG::is_sm_particle[VG::numberOfGenerations] = { true };

struct gG: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool gG::is_sm_particle[gG::numberOfGenerations] = { true };

struct Glu: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Glu::is_sm_particle[Glu::numberOfGenerations] = { false };

struct VP: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool VP::is_sm_particle[VP::numberOfGenerations] = { true };

struct VZ: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool VZ::is_sm_particle[VZ::numberOfGenerations] = { true };

struct gP: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool gP::is_sm_particle[gP::numberOfGenerations] = { true };

struct gZ: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool gZ::is_sm_particle[gZ::numberOfGenerations] = { true };

struct gWm: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool gWm::is_sm_particle[gWm::numberOfGenerations] = { true };

struct gWmC: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool gWmC::is_sm_particle[gWmC::numberOfGenerations] = { true };

struct Fd: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Fd::is_sm_particle[Fd::numberOfGenerations] = { true };

struct Fs: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Fs::is_sm_particle[Fs::numberOfGenerations] = { true };

struct Fb: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Fb::is_sm_particle[Fb::numberOfGenerations] = { true };

struct Fu: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Fu::is_sm_particle[Fu::numberOfGenerations] = { true };

struct Fc: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Fc::is_sm_particle[Fc::numberOfGenerations] = { true };

struct Ft: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Ft::is_sm_particle[Ft::numberOfGenerations] = { true };

struct Fve: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Fve::is_sm_particle[Fve::numberOfGenerations] = { true };

struct Fvm: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Fvm::is_sm_particle[Fvm::numberOfGenerations] = { true };

struct Fvt: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Fvt::is_sm_particle[Fvt::numberOfGenerations] = { true };

struct Fe: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Fe::is_sm_particle[Fe::numberOfGenerations] = { true };

struct Fm: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Fm::is_sm_particle[Fm::numberOfGenerations] = { true };

struct Ftau: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Ftau::is_sm_particle[Ftau::numberOfGenerations] = { true };

struct SveL: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool SveL::is_sm_particle[SveL::numberOfGenerations] = { false };

struct SvmL: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool SvmL::is_sm_particle[SvmL::numberOfGenerations] = { false };

struct SvtL: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool SvtL::is_sm_particle[SvtL::numberOfGenerations] = { false };

struct Sd: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Sd::is_sm_particle[Sd::numberOfGenerations] = { false, false };

struct Su: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Su::is_sm_particle[Su::numberOfGenerations] = { false, false };

struct Se: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Se::is_sm_particle[Se::numberOfGenerations] = { false, false };

struct Sm: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Sm::is_sm_particle[Sm::numberOfGenerations] = { false, false };

struct Stau: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Stau::is_sm_particle[Stau::numberOfGenerations] = { false, false };

struct Ss: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Ss::is_sm_particle[Ss::numberOfGenerations] = { false, false };

struct Sc: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Sc::is_sm_particle[Sc::numberOfGenerations] = { false, false };

struct Sb: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Sb::is_sm_particle[Sb::numberOfGenerations] = { false, false };

struct St: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool St::is_sm_particle[St::numberOfGenerations] = { false, false };

struct hh: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool hh::is_sm_particle[hh::numberOfGenerations] = { false, false };

struct Ah: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Ah::is_sm_particle[Ah::numberOfGenerations] = { false, false };

struct Hpm: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Hpm::is_sm_particle[Hpm::numberOfGenerations] = { false, false };

struct Chi: public Particle {
   static const unsigned numberOfGenerations = 4;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Chi::is_sm_particle[Chi::numberOfGenerations] = { false, false, false, false };

struct Cha: public Particle {
   static const unsigned numberOfGenerations = 2;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool Cha::is_sm_particle[Cha::numberOfGenerations] = { false, false };

struct VWm: public Particle {
   static const unsigned numberOfGenerations = 1;
   static const bool is_sm_particle[numberOfGenerations];
};
const bool VWm::is_sm_particle[VWm::numberOfGenerations] = { true };


// Special particle families
typedef VP Photon;
typedef Fm MuonFamily;

// AntiParticles
template<class P> struct anti : public Particle
{
   static const unsigned numberOfGenerations = P::numberOfGenerations;
   typedef anti<P> type;
};
template<class P> struct anti<anti<P>> { typedef P type; };

// Particles that are their own antiparticles
template<> struct anti<VG> { typedef VG type; };
template<> struct anti<Glu> { typedef Glu type; };
template<> struct anti<VP> { typedef VP type; };
template<> struct anti<VZ> { typedef VZ type; };
template<> struct anti<hh> { typedef hh type; };
template<> struct anti<Ah> { typedef Ah type; };
template<> struct anti<Chi> { typedef Chi type; };

template<class P> struct remove_anti { typedef P type; };
template<class P> struct remove_anti<anti<P>> { typedef P type; };

/**
 * @class EvaluationContext
 * @brief Represents an evaluation context.
 *
 * Actually it simply contains a reference to a model object.
 * All computational functions are forwarded to that object,
 * e.g. mass calculation functions.
 */
struct EvaluationContext {
   MSSMNoFVHimalaya_mass_eigenstates& model; ///< The model object.

   /**
    * @fn mass<P>
    * @brief Returns the mass of a particle.
    */
   template<class P> double mass() const {
      return mass<typename anti<P>::type>();
   }

   template<class P> double mass(unsigned index) const {
      return mass<typename anti<P>::type>(index);
   }
};

/**
 * @class DiagramEvaluator
 * @brief A template that can calculate disagrams.
 *
 * This template is completely general. The intended use is
 * to have specialisations of the kind:
 *   DiagramEvaluator<DiagramType, Particle1, Particle2, ...>
 * along with a static member function:
 *   static double value( EvaluationContext &context );
 * that calculates the contribution to a_muon
 * for a specific diagram class with specified particles.
 */
template<class ...Args> struct DiagramEvaluator;

double OneLoopFunctionF1C(double);
double OneLoopFunctionF2C(double);
double OneLoopFunctionF1N(double);
double OneLoopFunctionF2N(double);

double get_QED_2L(EvaluationContext& context);



// The different diagram types that contribute to the muon magnetic moment
template<unsigned> class OneLoopDiagram;

// Indexed diagram types
template<> class OneLoopDiagram<3> {};
template<> class OneLoopDiagram<4> {};

template<class PhotonEmitter, class ExchangeParticle>
struct DiagramEvaluator<OneLoopDiagram<3>, PhotonEmitter, ExchangeParticle>
{ static double value(EvaluationContext& context); };

template<class PhotonEmitter, class ExchangeParticle>
struct DiagramEvaluator<OneLoopDiagram<4>, PhotonEmitter, ExchangeParticle>
{ static double value(EvaluationContext& context); };

/**
 * @class SingleComponentedVertex
 * @brief A vertex whose value can be represented by a single complex number
 */
class SingleComponentedVertex {
private:
   std::complex<double> val; ///< The value
public:
   explicit SingleComponentedVertex(std::complex<double> v)
      : val(v) {}

   /**
    * @fn value
    * @brief Returns the value of the vertex.
    */
   std::complex<double> value() const {
      return val;
   }

   /**
    * @fn isZero
    * @brief Tests whether the value is numerically significant
    */
   bool isZero() const {
      return (is_zero(val.real()) && is_zero(val.imag()));
   }
};

/**
 * @class LeftAndRightComponentedVertex
 * @brief A vertex whose value can be represented by two complex numbers
 */
class LeftAndRightComponentedVertex {
private:
   std::pair<std::complex<double>, std::complex<double>> value;  ///< The values
public:
   LeftAndRightComponentedVertex(const std::complex<double> &left,
                                 const std::complex<double> &right)
      : value(left, right) {}

   /**
    * @fn left
    * @brief Returns the left component of the vertex.
    */
   std::complex<double> left() const {
      return value.first;
   }

   /**
    * @fn right
    * @brief Returns the left component of the vertex.
    */
   std::complex<double> right() const {
      return value.second;
   }

   /**
    * @fn isZero
    * @brief Tests whether the values are numerically significant
    */
   bool isZero() const {
      return (is_zero(value.first.real()) && is_zero(value.first.imag()) &&
              is_zero(value.second.real()) && is_zero(value.second.imag()));
   }
};

/**
 * @class VertexFunctionData<P...>
 * @brief Vertex data for a vertex with the particles specified by P...
 *
 * All elements in P... have to be publicly derived from Particle.
 * The data includes the type of the vertex, whether P... is in
 * canonical order or permuted as well as index bounds referring to
 * the specific particles (generation and colour indices, etc).
 * The intention is to have a program generate VertexFunctionData<>
 * specialisations that are then used by the VertexFunction<> template.
 */
template<class ...P> struct VertexFunctionData;

/**
 * @class VertexFunctionHelper<bool, P...>
 * @brief A helper template that is used by VertexFunction<P...>
 *
 * You should never have to use this class directly. It is just a
 * helper class.
 */
template<bool, class ...P> class VertexFunctionHelper;

template<class ...P> class VertexFunctionHelper<false, P...>
   : public VertexFunctionData<P...> {
private:
   typedef VertexFunctionData<P...> Base;
   typedef typename Base::particleIndexStart particleIndexStart;
   using Base::indexB;
public:
   typedef typename Base::index_bounds index_bounds;
   typedef typename index_bounds::indices_type indices_type;
   typedef typename Base::vertex_type vertex_type;
public:
   static const index_bounds &indexBounds() {
      return indexB;
   }

   template<unsigned particleIndex>
   static std::vector<unsigned> particleIndices(const indices_type &indices)
   {
      auto begin = indices.begin() + boost::mpl::at_c<particleIndexStart, particleIndex>::type::value;
      auto end = indices.begin() + boost::mpl::at_c<particleIndexStart, particleIndex+1>::type::value;
      return std::vector<unsigned>(begin, end);
   }
};

template<class ...P> class VertexFunctionHelper<true, P...>
   : public VertexFunctionData<P...>::orig_type {
private:
   typedef typename VertexFunctionData<P...>::orig_type orig_type;
   typedef typename VertexFunctionData<P...>::particlePermutation particlePermutation;
public:
   typedef typename orig_type::index_bounds index_bounds;
   typedef typename index_bounds::indices_type indices_type;
   typedef typename orig_type::vertex_type vertex_type;
public:
   template<unsigned particleIndex>
   static std::vector<unsigned> particleIndices(const indices_type &indices)
   {
      return orig_type::template particleIndices<
                boost::mpl::at_c<particlePermutation, particleIndex>::type::value
                > (indices);
   }
};

/**
 * @class VertexFunction<P...>
 * @brief A template that represents a vertex with open particle indices.
 *
 * All elements in P... have to be publicly derived from Particle.
 * To obtain a conrete value, use the static member function vertex()
 * along with the desired particle indices.
 * The intention here is to have a program generate the vertex calculation
 * code for a canonically ordered P... and have c++ take care of
 * possible particle permutations.
 */
template<class ...P> class VertexFunction
   : public VertexFunctionHelper<VertexFunctionData<P...>::is_permutation, P...> {
private:
   typedef VertexFunctionHelper<VertexFunctionData<P...>::is_permutation, P...> Base;
public:
   typedef typename Base::indices_type indices_type;
   typedef typename Base::index_bounds index_bounds;
   typedef typename Base::vertex_type vertex_type;

   /**
    * @fn indexBounds
    * @brief Returns the IndexBounds<> object of the VertexFunction.
    */
   using Base::indexBounds;

   /**
    * @fn particleIndices
    * @brief Returns the subset of indices belonging to
    *        a particle at a given index.
    * @param indices The complete set of indices for all particles
    *
    * The particle indexing is in the same order as the template arguments
    * and starts at 0.
    */
   using Base::particleIndices;

   /**
    * @fn vertex
    * @brief Calculates the vertex for a given set of particle indices.
    * @param indices The particle indices
    * @param context The evaluation context
    *
    * Q is just used for SFINAE (c++)
    * This definition only kicks in for permuted P...
    * For canonical P... the definitions have to be supplied separately.
    */
   template<class Q = Base>
   static vertex_type vertex(const indices_type &indices, EvaluationContext &context)
   {
      return Q::vertex(indices, context);
   }
};

template<> struct VertexFunctionData<Cha, VP, anti<Cha>::type>
{
   static const bool is_permutation = false;
   typedef IndexBounds<2> index_bounds;
   typedef LeftAndRightComponentedVertex vertex_type;
   typedef boost::mpl::vector_c<unsigned, 0, 1, 1, 2> particleIndexStart;
   static const index_bounds indexB;
};
const VertexFunctionData<Cha, VP, anti<Cha>::type>::index_bounds VertexFunctionData<Cha, VP, anti<Cha>::type>::indexB = { { 0, 0 }, { 2, 2 } };

template<> struct VertexFunctionData<VP, Cha, anti<Cha>::type>
{
   static const bool is_permutation = true;
   typedef VertexFunction<Cha, VP, anti<Cha>::type> orig_type;
   typedef boost::mpl::vector_c<unsigned, 1, 0, 2> particlePermutation;
};

template<> struct VertexFunctionData<Fm, anti<Cha>::type, anti<SvmL>::type>
{
   static const bool is_permutation = false;
   typedef IndexBounds<1> index_bounds;
   typedef LeftAndRightComponentedVertex vertex_type;
   typedef boost::mpl::vector_c<unsigned, 0, 0, 1, 1> particleIndexStart;
   static const index_bounds indexB;
};
const VertexFunctionData<Fm, anti<Cha>::type, anti<SvmL>::type>::index_bounds VertexFunctionData<Fm, anti<Cha>::type, anti<SvmL>::type>::indexB = { { 0 }, { 2 } };

template<> struct VertexFunctionData<Fm, VP, anti<Fm>::type>
{
   static const bool is_permutation = false;
   typedef IndexBounds<0> index_bounds;
   typedef LeftAndRightComponentedVertex vertex_type;
   typedef boost::mpl::vector_c<unsigned, 0, 0, 0, 0> particleIndexStart;
   static const index_bounds indexB;
};
const VertexFunctionData<Fm, VP, anti<Fm>::type>::index_bounds VertexFunctionData<Fm, VP, anti<Fm>::type>::indexB{};

template<> struct VertexFunctionData<VP, Fm, anti<Fm>::type>
{
   static const bool is_permutation = true;
   typedef VertexFunction<Fm, VP, anti<Fm>::type> orig_type;
   typedef boost::mpl::vector_c<unsigned, 1, 0, 2> particlePermutation;
};

template<> struct VertexFunctionData<Ah, Fm, anti<Fm>::type>
{
   static const bool is_permutation = false;
   typedef IndexBounds<1> index_bounds;
   typedef LeftAndRightComponentedVertex vertex_type;
   typedef boost::mpl::vector_c<unsigned, 0, 1, 1, 1> particleIndexStart;
   static const index_bounds indexB;
};
const VertexFunctionData<Ah, Fm, anti<Fm>::type>::index_bounds VertexFunctionData<Ah, Fm, anti<Fm>::type>::indexB = { { 0 }, { 2 } };

template<> struct VertexFunctionData<Fm, anti<Fm>::type, Ah>
{
   static const bool is_permutation = true;
   typedef VertexFunction<Ah, Fm, anti<Fm>::type> orig_type;
   typedef boost::mpl::vector_c<unsigned, 1, 2, 0> particlePermutation;
};

template<> struct VertexFunctionData<Fm, hh, anti<Fm>::type>
{
   static const bool is_permutation = false;
   typedef IndexBounds<1> index_bounds;
   typedef LeftAndRightComponentedVertex vertex_type;
   typedef boost::mpl::vector_c<unsigned, 0, 0, 1, 1> particleIndexStart;
   static const index_bounds indexB;
};
const VertexFunctionData<Fm, hh, anti<Fm>::type>::index_bounds VertexFunctionData<Fm, hh, anti<Fm>::type>::indexB = { { 0 }, { 2 } };

template<> struct VertexFunctionData<Fm, anti<Fm>::type, hh>
{
   static const bool is_permutation = true;
   typedef VertexFunction<Fm, hh, anti<Fm>::type> orig_type;
   typedef boost::mpl::vector_c<unsigned, 0, 2, 1> particlePermutation;
};

template<> struct VertexFunctionData<Hpm, VP, anti<Hpm>::type>
{
   static const bool is_permutation = false;
   typedef IndexBounds<2> index_bounds;
   typedef SingleComponentedVertex vertex_type;
   typedef boost::mpl::vector_c<unsigned, 0, 1, 1, 2> particleIndexStart;
   static const index_bounds indexB;
};
const VertexFunctionData<Hpm, VP, anti<Hpm>::type>::index_bounds VertexFunctionData<Hpm, VP, anti<Hpm>::type>::indexB = { { 0, 0 }, { 2, 2 } };

template<> struct VertexFunctionData<VP, Hpm, anti<Hpm>::type>
{
   static const bool is_permutation = true;
   typedef VertexFunction<Hpm, VP, anti<Hpm>::type> orig_type;
   typedef boost::mpl::vector_c<unsigned, 1, 0, 2> particlePermutation;
};

template<> struct VertexFunctionData<Fm, anti<Fvm>::type, anti<Hpm>::type>
{
   static const bool is_permutation = false;
   typedef IndexBounds<1> index_bounds;
   typedef LeftAndRightComponentedVertex vertex_type;
   typedef boost::mpl::vector_c<unsigned, 0, 0, 0, 1> particleIndexStart;
   static const index_bounds indexB;
};
const VertexFunctionData<Fm, anti<Fvm>::type, anti<Hpm>::type>::index_bounds VertexFunctionData<Fm, anti<Fvm>::type, anti<Hpm>::type>::indexB = { { 0 }, { 2 } };

template<> struct VertexFunctionData<Fm, anti<Hpm>::type, anti<Fvm>::type>
{
   static const bool is_permutation = true;
   typedef VertexFunction<Fm, anti<Fvm>::type, anti<Hpm>::type> orig_type;
   typedef boost::mpl::vector_c<unsigned, 0, 2, 1> particlePermutation;
};

template<> struct VertexFunctionData<Sm, VP, anti<Sm>::type>
{
   static const bool is_permutation = false;
   typedef IndexBounds<2> index_bounds;
   typedef SingleComponentedVertex vertex_type;
   typedef boost::mpl::vector_c<unsigned, 0, 1, 1, 2> particleIndexStart;
   static const index_bounds indexB;
};
const VertexFunctionData<Sm, VP, anti<Sm>::type>::index_bounds VertexFunctionData<Sm, VP, anti<Sm>::type>::indexB = { { 0, 0 }, { 2, 2 } };

template<> struct VertexFunctionData<VP, Sm, anti<Sm>::type>
{
   static const bool is_permutation = true;
   typedef VertexFunction<Sm, VP, anti<Sm>::type> orig_type;
   typedef boost::mpl::vector_c<unsigned, 1, 0, 2> particlePermutation;
};

template<> struct VertexFunctionData<Chi, Fm, anti<Sm>::type>
{
   static const bool is_permutation = false;
   typedef IndexBounds<2> index_bounds;
   typedef LeftAndRightComponentedVertex vertex_type;
   typedef boost::mpl::vector_c<unsigned, 0, 1, 1, 2> particleIndexStart;
   static const index_bounds indexB;
};
const VertexFunctionData<Chi, Fm, anti<Sm>::type>::index_bounds VertexFunctionData<Chi, Fm, anti<Sm>::type>::indexB = { { 0, 0 }, { 4, 2 } };

template<> struct VertexFunctionData<Fm, anti<Sm>::type, Chi>
{
   static const bool is_permutation = true;
   typedef VertexFunction<Chi, Fm, anti<Sm>::type> orig_type;
   typedef boost::mpl::vector_c<unsigned, 1, 2, 0> particlePermutation;
};

double get_MSUSY(const MSSMNoFVHimalaya_mass_eigenstates& model)
{
   return Min(model.get_MSd().tail<2>().minCoeff(), model.get_MSu().tail<2>()
      .minCoeff(), model.get_MSe().tail<2>().minCoeff(), model.get_MSm().tail<2>()
      .minCoeff(), model.get_MStau().tail<2>().minCoeff(), model.get_MSs().tail<2>
      ().minCoeff(), model.get_MSc().tail<2>().minCoeff(), model.get_MSb().tail<2>
      ().minCoeff(), model.get_MSt().tail<2>().minCoeff(), model.get_MHpm().tail<1
      >().minCoeff(), model.get_MCha().tail<2>().minCoeff());

}

void run_to_MSUSY(MSSMNoFVHimalaya_mass_eigenstates& model)
{
   const double precision_goal = model.get_precision();
   const int Nmax = static_cast<int>(-log10(precision_goal) * 10);
   int it = 0;
   double precision = 0.;
   double MSUSY_old = 0., MSUSY_new = 0.;

   VERBOSE_MSG("MSSMNoFVHimalaya_a_muon: iterate to run to MSUSY ...");

   do {
      MSUSY_new = get_MSUSY(model);

      if (MSUSY_new <= 0.)
         return;

      VERBOSE_MSG("MSSMNoFVHimalaya_a_muon:    running to new MSUSY = " << MSUSY_new);

      model.run_to(MSUSY_new);
      model.calculate_DRbar_masses();

      precision = MaxRelDiff(MSUSY_old, MSUSY_new);
      MSUSY_old = MSUSY_new;
   } while (precision > precision_goal && ++it < Nmax);

   VERBOSE_MSG("MSSMNoFVHimalaya_a_muon: iteration finished. MSUSY = " << MSUSY_new);

   if (precision > precision_goal) {
      WARNING("MSSMNoFVHimalaya_a_muon: run_to_MSUSY(): "
              "Iteration did not converge after " << Nmax
              << " iterations (precision goal: " << precision_goal << ")");
   }
}

double calculate_a_muon_impl(MSSMNoFVHimalaya_mass_eigenstates& model)
{
   VERBOSE_MSG("MSSMNoFVHimalaya_a_muon: calculating a_mu at Q = " << model.get_scale());

   /********** GMuonMinus2.m generated calculation code **********/

   EvaluationContext context{ model };
   double val = 0.0;

   val += DiagramEvaluator<OneLoopDiagram<3>, Cha, SvmL>::value(context);
   val += DiagramEvaluator<OneLoopDiagram<3>, Fm, Ah>::value(context);
   val += DiagramEvaluator<OneLoopDiagram<3>, Fm, hh>::value(context);
   val += DiagramEvaluator<OneLoopDiagram<4>, Hpm, Fvm>::value(context);
   val += DiagramEvaluator<OneLoopDiagram<4>, Sm, Chi>::value(context);

   // add 2-loop QED logarithms
   val *= 1. + get_QED_2L(context);

   return val;
}

/// generates array with N scales from mean/factor to mean*factor
template <int N>
std::array<double,N> generate_scales(double mean, double factor)
{
   static_assert(N > 1, "N must be greater than 1!");

   const double start = mean / factor, stop = mean * factor;
   std::array<double,N> scales;

   scales[0] = start;

   for (int i = 1; i < (N-1); i++)
      scales[i] = std::exp(std::log(start) + (std::log(stop) - std::log(start))*i / N);

   scales[N-1] = stop;

   return scales;
}

/// returns minimum and maximum a_mu when scale is varied by a factor 2
std::pair<double,double> vary_scale(const MSSMNoFVHimalaya_mass_eigenstates& model)
{
   auto scales = generate_scales<7>(model.get_scale(), 2.);

   std::transform(scales.begin(), scales.end(), scales.begin(),
                  [&model] (double scale) {
                     double amu = 0.;
                     try {
                        auto m = model;
                        m.run_to(scale);
                        m.get_physical().clear();
                        m.calculate_DRbar_masses();
                        amu = calculate_a_muon_impl(m);
                     } catch(const Error& e) {
                        ERROR("MSSMNoFVHimalaya_a_muon: scale variation: " << e.what());
                     }
                     return amu;
                  });

   const auto minmax = std::minmax_element(scales.cbegin(), scales.cend());

   return std::make_pair(*(minmax.first), *(minmax.second));
}

} // anonymous namespace

double MSSMNoFVHimalaya_a_muon::calculate_a_muon(const MSSMNoFVHimalaya_mass_eigenstates& model_)
{
   MSSMNoFVHimalaya_mass_eigenstates model(model_);

   VERBOSE_MSG("MSSMNoFVHimalaya_a_muon: starting calculation of a_mu ...");

   try {
      run_to_MSUSY(model);
      model.get_physical().clear();
   } catch (const Error& e) {
      ERROR("MSSMNoFVHimalaya_a_muon:" << e.what());
      return std::numeric_limits<double>::quiet_NaN();
   }

   return calculate_a_muon_impl(model);
}

double MSSMNoFVHimalaya_a_muon::calculate_a_muon_uncertainty(const MSSMNoFVHimalaya_mass_eigenstates& model_)
{
   MSSMNoFVHimalaya_mass_eigenstates model(model_);

   VERBOSE_MSG("MSSMNoFVHimalaya_a_muon: starting calculation of a_mu uncertainty ...");

   try {
      run_to_MSUSY(model);
      model.get_physical().clear();
   } catch (const Error& e) {
      ERROR("MSSMNoFVHimalaya_a_muon uncertainty: " << e.what());
      return std::numeric_limits<double>::quiet_NaN();
   }

   const auto delta_amu_scale_minmax = vary_scale(model);
   const auto delta_amu_scale = std::abs(delta_amu_scale_minmax.second - delta_amu_scale_minmax.first);

   return delta_amu_scale;
}

namespace {
/**
* @defgroup LoopFunctions LoopFunctions
* @brief The loop functions necessary for \f$a_\mu\f$ one-loop calculations.
*
* These are OneLoopFunctionF1C(), OneLoopFunctionF2C(),
* OneLoopFunctionF1N() and OneLoopFunctionF2N()
* as specified in arXiv:1311.1775
*/

double OneLoopFunctionF1C(double x)
{
   if (is_zero(x))
      return 4.0;

   // error around x=1 is <= 10^-12 on an intel i7
   const double y = x - 1.0;

   if (std::abs(y) < 0.21) {
      return (1.0000000000000000000 -
              0.60000000000000000000  * y +
              0.40000000000000000000  * y * y -
              0.28571428571428571429  * y * y * y +
              0.21428571428571428571  * y * y * y * y -
              0.16666666666666666667  * y * y * y * y * y +
              0.13333333333333333333  * y * y * y * y * y * y -
              0.10909090909090909091  * y * y * y * y * y * y * y +
              0.090909090909090909091 * y * y * y * y * y * y * y * y -
              0.076923076923076923077 * y * y * y * y * y * y * y * y * y +
              0.065934065934065934066 * y * y * y * y * y * y * y * y * y * y -
              0.057142857142857142857 * y * y * y * y * y * y * y * y * y * y * y +
              0.050000000000000000000 * y * y * y * y * y * y * y * y * y * y * y * y -
              0.044117647058823529412 * y * y * y * y * y * y * y * y * y * y * y * y * y +
              0.039215686274509803922 * y * y * y * y * y * y * y * y * y * y * y * y * y * y -
              0.035087719298245614035 * y * y * y * y * y * y * y * y * y * y * y * y * y * y * y);
   }

   return 2.0 / (y * y * y * y) * (2.0 + 3.0 * x - 6.0 * x * x + x * x * x + 6.0 * x * std::log(x));
}

double OneLoopFunctionF2C(double x)
{
   // error around x=1 is <= 10^-13 on an intel i7
   const double y = x - 1.0;

   if (std::abs(y) < 0.155)
      return (1.0 - 0.75 * y + 0.6 * y * y -
              0.50000000000000000000 * y * y * y +
              0.4285714285714285714  * y * y * y * y -
              0.37500000000000000000 * y * y * y * y * y +
              0.33333333333333333333 * y * y * y * y * y * y -
              0.3000000000000000000  * y * y * y * y * y * y * y +
              0.2727272727272727273  * y * y * y * y * y * y * y * y -
              0.2500000000000000000  * y * y * y * y * y * y * y * y * y +
              0.23076923076923076923 * y * y * y * y * y * y * y * y * y * y -
              0.21428571428571428571 * y * y * y * y * y * y * y * y * y * y * y +
              0.2000000000000000000  * y * y * y * y * y * y * y * y * y * y * y * y -
              0.1875000000000000000  * y * y * y * y * y * y * y * y * y * y * y * y * y +
              0.1764705882352941176  * y * y * y * y * y * y * y * y * y * y * y * y * y * y -
              0.16666666666666666667 * y * y * y * y * y * y * y * y * y * y * y * y * y * y * y);

   return -3.0 / (2.0 * y * y * y) * (-3.0 + 4.0 * x - x * x - 2.0 * std::log(x));
}

double OneLoopFunctionF1N(double x)
{
   if (is_zero(x))
      return 2.0;

   // error around x=1 is <= 10^-12 on an intel i7
   const double y = x - 1.0;

   if (std::abs(y) < 0.23)
      return (1.0000000000000000000 -
              0.4000000000000000000  * y +
              0.2000000000000000000  * y * y -
              0.11428571428571428571 * y * y * y +
              0.07142857142857142857 * y * y * y * y -
              0.04761904761904761905 * y * y * y * y * y +
              0.03333333333333333333 * y * y * y * y * y * y -
              0.02424242424242424242 * y * y * y * y * y * y * y +
              0.0181818181818181818  * y * y * y * y * y * y * y * y -
              0.01398601398601398601 * y * y * y * y * y * y * y * y * y +
              0.01098901098901098901 * y * y * y * y * y * y * y * y * y * y -
              0.0087912087912087912  * y * y * y * y * y * y * y * y * y * y * y +
              0.00714285714285714286 * y * y * y * y * y * y * y * y * y * y * y * y -
              0.0058823529411764706  * y * y * y * y * y * y * y * y * y * y * y * y * y +
              0.0049019607843137255  * y * y * y * y * y * y * y * y * y * y * y * y * y * y -
              0.0041279669762641899  * y * y * y * y * y * y * y * y * y * y * y * y * y * y * y);

   return 2.0 / (y * y * y * y) * (1.0 - 6.0 * x + 3.0 * x * x + 2.0 * x * x * x - 6.0 * x * x * std::log(x));
}

double OneLoopFunctionF2N(double x)
{
   if (is_zero(x))
      return 3.0;

   // error around x=1 is <= 10^-13 on an intel i7
   const double y = x - 1.0;

   if (std::abs(y) < 0.185)
      return (1.0000000000000000000 -
              0.50000000000000000000 * y +
              0.30000000000000000000 * y * y -
              0.2000000000000000000  * y * y * y +
              0.14285714285714285714 * y * y * y * y -
              0.10714285714285714286 * y * y * y * y * y +
              0.08333333333333333333 * y * y * y * y * y * y -
              0.06666666666666666667 * y * y * y * y * y * y * y +
              0.05454545454545454545 * y * y * y * y * y * y * y * y -
              0.0454545454545454545  * y * y * y * y * y * y * y * y * y +
              0.0384615384615384615  * y * y * y * y * y * y * y * y * y * y -
              0.03296703296703296703 * y * y * y * y * y * y * y * y * y * y * y +
              0.0285714285714285714  * y * y * y * y * y * y * y * y * y * y * y * y -
              0.02500000000000000000 * y * y * y * y * y * y * y * y * y * y * y * y * y +
              0.0220588235294117647  * y * y * y * y * y * y * y * y * y * y * y * y * y * y -
              0.0196078431372549020  * y * y * y * y * y * y * y * y * y * y * y * y * y * y * y);

   return -3.0 / (y * y * y) * (1.0 - x * x + 2.0 * x * std::log(x));
}

template<> double EvaluationContext::mass<VG>() const
{ return model.get_MVG(); }

template<> double EvaluationContext::mass<Glu>() const
{ return model.get_MGlu(); }

template<> double EvaluationContext::mass<VP>() const
{ return model.get_MVP(); }

template<> double EvaluationContext::mass<VZ>() const
{ return model.get_MVZ(); }

template<> double EvaluationContext::mass<Fd>() const
{ return model.get_MFd(); }

template<> double EvaluationContext::mass<Fs>() const
{ return model.get_MFs(); }

template<> double EvaluationContext::mass<Fb>() const
{ return model.get_MFb(); }

template<> double EvaluationContext::mass<Fu>() const
{ return model.get_MFu(); }

template<> double EvaluationContext::mass<Fc>() const
{ return model.get_MFc(); }

template<> double EvaluationContext::mass<Ft>() const
{ return model.get_MFt(); }

template<> double EvaluationContext::mass<Fve>() const
{ return model.get_MFve(); }

template<> double EvaluationContext::mass<Fvm>() const
{ return model.get_MFvm(); }

template<> double EvaluationContext::mass<Fvt>() const
{ return model.get_MFvt(); }

template<> double EvaluationContext::mass<Fe>() const
{ return model.get_MFe(); }

template<> double EvaluationContext::mass<Fm>() const
{ return model.get_MFm(); }

template<> double EvaluationContext::mass<Ftau>() const
{ return model.get_MFtau(); }

template<> double EvaluationContext::mass<SveL>() const
{ return model.get_MSveL(); }

template<> double EvaluationContext::mass<SvmL>() const
{ return model.get_MSvmL(); }

template<> double EvaluationContext::mass<SvtL>() const
{ return model.get_MSvtL(); }

template<> double EvaluationContext::mass<Sd>(unsigned index) const
{ return model.get_MSd(index); }

template<> double EvaluationContext::mass<Su>(unsigned index) const
{ return model.get_MSu(index); }

template<> double EvaluationContext::mass<Se>(unsigned index) const
{ return model.get_MSe(index); }

template<> double EvaluationContext::mass<Sm>(unsigned index) const
{ return model.get_MSm(index); }

template<> double EvaluationContext::mass<Stau>(unsigned index) const
{ return model.get_MStau(index); }

template<> double EvaluationContext::mass<Ss>(unsigned index) const
{ return model.get_MSs(index); }

template<> double EvaluationContext::mass<Sc>(unsigned index) const
{ return model.get_MSc(index); }

template<> double EvaluationContext::mass<Sb>(unsigned index) const
{ return model.get_MSb(index); }

template<> double EvaluationContext::mass<St>(unsigned index) const
{ return model.get_MSt(index); }

template<> double EvaluationContext::mass<hh>(unsigned index) const
{ return model.get_Mhh(index); }

template<> double EvaluationContext::mass<Ah>(unsigned index) const
{ return model.get_MAh(index); }

template<> double EvaluationContext::mass<Hpm>(unsigned index) const
{ return model.get_MHpm(index); }

template<> double EvaluationContext::mass<Chi>(unsigned index) const
{ return model.get_MChi(index); }

template<> double EvaluationContext::mass<Cha>(unsigned index) const
{ return model.get_MCha(index); }

template<> double EvaluationContext::mass<VWm>() const
{ return model.get_MVWm(); }

constexpr unsigned muonIndex()
{ return 0; }

double muonPhysicalMass(EvaluationContext& context)
{
   double m_muon_pole = context.model.get_physical().MFm;

   if (m_muon_pole == 0.0) {
      context.model.solve_ewsb();
      context.model.calculate_MFm_pole();
      m_muon_pole = context.model.get_physical().MFm;
   }

   return m_muon_pole;
}

double muonCharge(EvaluationContext& context)
{
   typedef VertexFunction<VP, Fm, anti<Fm>::type> VF;
   const std::array<unsigned, 0> indices{};
   return VF::vertex(indices, context).left().real();
}

template<> template<> VertexFunction<Cha, VP, anti<Cha>::type>::vertex_type
VertexFunction<Cha, VP, anti<Cha>::type>::vertex(const indices_type &indices, EvaluationContext &context)
{
   const unsigned gt1 = indices[0];
   const unsigned gt3 = indices[1];
   const auto g2 = MODELPARAMETER(g2);
   const auto g1 = MODELPARAMETER(g1);
   const auto UM = MODELPARAMETER(UM);
   const auto UP = MODELPARAMETER(UP);
   const auto ThetaW = DERIVEDPARAMETER(ThetaW);

   const std::complex<double> left = -(g2*Conj(UP(gt3,0))*Sin(ThetaW)*UP(gt1,0)) - 0.1*Conj(UP(gt3,1))*(3.872983346207417*g1*Cos(ThetaW) + 5*g2*Sin(ThetaW))*UP(gt1,1);

   const std::complex<double> right = -(g2*Conj(UM(gt1,0))*Sin(ThetaW)*UM(gt3,0)) - 0.1*Conj(UM(gt1,1))*(3.872983346207417*g1*Cos(ThetaW) + 5*g2*Sin(ThetaW))*UM(gt3,1);

   return vertex_type(left, right);
}

template<> template<> VertexFunction<Fm, anti<Cha>::type, anti<SvmL>::type>::vertex_type
VertexFunction<Fm, anti<Cha>::type, anti<SvmL>::type>::vertex(const indices_type &indices, EvaluationContext &context)
{
   const unsigned gt2 = indices[0];
   const auto g2 = MODELPARAMETER(g2);
   const auto Ye = MODELPARAMETER(Ye);
   const auto UP = MODELPARAMETER(UP);
   const auto UM = MODELPARAMETER(UM);

   const std::complex<double> left = -(g2*Conj(UP(gt2,0)));

   const std::complex<double> right = Conj(Ye(1,1))*UM(gt2,1);

   return vertex_type(left, right);
}

template<> template<> VertexFunction<Fm, VP, anti<Fm>::type>::vertex_type
VertexFunction<Fm, VP, anti<Fm>::type>::vertex(const indices_type &indices, EvaluationContext &context)
{
   const auto g1 = MODELPARAMETER(g1);
   const auto g2 = MODELPARAMETER(g2);
   const auto ThetaW = DERIVEDPARAMETER(ThetaW);

   const std::complex<double> left = -0.7745966692414834*g1*Cos(ThetaW);

   const std::complex<double> right = 0.1*(-3.872983346207417*g1*Cos(ThetaW) - 5*g2*Sin(ThetaW));

   return vertex_type(left, right);
}

template<> template<> VertexFunction<Ah, Fm, anti<Fm>::type>::vertex_type
VertexFunction<Ah, Fm, anti<Fm>::type>::vertex(const indices_type &indices, EvaluationContext &context)
{
   const unsigned gt1 = indices[0];
   const auto Ye = MODELPARAMETER(Ye);
   const auto ZA = MODELPARAMETER(ZA);

   const std::complex<double> left = std::complex<double>(0.,-0.7071067811865475)*Ye(1,1)*ZA(gt1,0);

   const std::complex<double> right = std::complex<double>(0.,0.7071067811865475)*Conj(Ye(1,1))*ZA(gt1,0);

   return vertex_type(left, right);
}

template<> template<> VertexFunction<Fm, hh, anti<Fm>::type>::vertex_type
VertexFunction<Fm, hh, anti<Fm>::type>::vertex(const indices_type &indices, EvaluationContext &context)
{
   const unsigned gt2 = indices[0];
   const auto Ye = MODELPARAMETER(Ye);
   const auto ZH = MODELPARAMETER(ZH);

   const std::complex<double> left = -0.7071067811865475*Ye(1,1)*ZH(gt2,0);

   const std::complex<double> right = -0.7071067811865475*Conj(Ye(1,1))*ZH(gt2,0);

   return vertex_type(left, right);
}

template<> template<> VertexFunction<Hpm, VP, anti<Hpm>::type>::vertex_type
VertexFunction<Hpm, VP, anti<Hpm>::type>::vertex(const indices_type &indices, EvaluationContext &context)
{
   const unsigned gt1 = indices[0];
   const unsigned gt3 = indices[1];
   const auto g1 = MODELPARAMETER(g1);
   const auto g2 = MODELPARAMETER(g2);
   const auto ThetaW = DERIVEDPARAMETER(ThetaW);

   const std::complex<double> result = -0.1*KroneckerDelta(gt1,gt3)*(3.872983346207417*g1*Cos(ThetaW) + 5*g2*Sin(ThetaW));

   return vertex_type(result);
}

template<> template<> VertexFunction<Fm, anti<Fvm>::type, anti<Hpm>::type>::vertex_type
VertexFunction<Fm, anti<Fvm>::type, anti<Hpm>::type>::vertex(const indices_type &indices, EvaluationContext &context)
{
   const unsigned gt3 = indices[0];
   const auto Ye = MODELPARAMETER(Ye);
   const auto ZP = MODELPARAMETER(ZP);

   const std::complex<double> left = 0;

   const std::complex<double> right = Conj(Ye(1,1))*ZP(gt3,0);

   return vertex_type(left, right);
}

template<> template<> VertexFunction<Sm, VP, anti<Sm>::type>::vertex_type
VertexFunction<Sm, VP, anti<Sm>::type>::vertex(const indices_type &indices, EvaluationContext &context)
{
   const unsigned gt1 = indices[0];
   const unsigned gt3 = indices[1];
   const auto g1 = MODELPARAMETER(g1);
   const auto g2 = MODELPARAMETER(g2);
   const auto ZM = MODELPARAMETER(ZM);
   const auto ThetaW = DERIVEDPARAMETER(ThetaW);

   const std::complex<double> result = 0.1*(-(Conj(ZM(gt1,0))*(3.872983346207417*g1*Cos(ThetaW) + 5*g2*Sin(ThetaW))*ZM(gt3,0)) - 7.745966692414834*g1*Conj(ZM(gt1,1))*Cos(ThetaW)*ZM(gt3,1));

   return vertex_type(result);
}

template<> template<> VertexFunction<Chi, Fm, anti<Sm>::type>::vertex_type
VertexFunction<Chi, Fm, anti<Sm>::type>::vertex(const indices_type &indices, EvaluationContext &context)
{
   const unsigned gt1 = indices[0];
   const unsigned gt3 = indices[1];
   const auto g1 = MODELPARAMETER(g1);
   const auto g2 = MODELPARAMETER(g2);
   const auto Ye = MODELPARAMETER(Ye);
   const auto ZN = MODELPARAMETER(ZN);
   const auto ZM = MODELPARAMETER(ZM);

   const std::complex<double> left = 0.5477225575051661*g1*Conj(ZN(gt1,0))*ZM(gt3,0) + 0.7071067811865475*g2*Conj(ZN(gt1,1))*ZM(gt3,0) - Conj(ZN(gt1,2))*Ye(1,1)*ZM(gt3,1);

   const std::complex<double> right = -1.0954451150103321*g1*ZM(gt3,1)*ZN(gt1,0) - Conj(Ye(1,1))*ZM(gt3,0)*ZN(gt1,2);

   return vertex_type(left, right);
}

double get_QED_2L(EvaluationContext& context)
{
   const double MSUSY = Abs(get_MSUSY(context.model));
   const double m_muon = muonPhysicalMass(context);
   const double alpha_em = Sqr(muonCharge(context))/(4*Pi);
   const double qed_2L = alpha_em/(4*Pi) * 16 * FiniteLog(m_muon/MSUSY);

   return qed_2L;

}

template<class PhotonEmitter, class ExchangeParticle>
double DiagramEvaluator<OneLoopDiagram<3>, PhotonEmitter, ExchangeParticle>::value(EvaluationContext &context)
{
   double res = 0.0;

   typedef VertexFunction<Photon, PhotonEmitter, typename anti<PhotonEmitter>::type> photonVertexFunction;
   typedef VertexFunction<MuonFamily, typename anti<PhotonEmitter>::type, typename anti<ExchangeParticle>::type> muonVertexFunction;

   for (auto photonVertexIndexIt = photonVertexFunction::indexBounds().begin();
        photonVertexIndexIt != photonVertexFunction::indexBounds().end();
        ++photonVertexIndexIt) {
      const auto &photonVertexIndices = *photonVertexIndexIt;
      const auto &photonVertex = photonVertexFunction::vertex(photonVertexIndices, context);
      if (photonVertex.isZero())
         continue;

      // FIXME: We assume that the photonVertex is symmetric between its left and right parts!
      const double photonEmitterChargeCount = photonVertex.left().real() / muonCharge(context);

      const auto &photonEmitterIndices = photonVertexFunction::template particleIndices<2>(photonVertexIndices);

      for (auto muonVertexIndexIt = muonVertexFunction::indexBounds().begin();
           muonVertexIndexIt != muonVertexFunction::indexBounds().end();
           ++muonVertexIndexIt) {
         const auto &muonVertexIndices = *muonVertexIndexIt;
         const auto &muonFamilyIndices = muonVertexFunction::template particleIndices<0>(muonVertexIndices);
         if (MuonFamily::numberOfGenerations != 1 && muonFamilyIndices[0] != muonIndex())
            continue;

         if (photonEmitterIndices != muonVertexFunction::template particleIndices<1>(muonVertexIndices))
            continue;

         const auto &muonVertex = muonVertexFunction::vertex(muonVertexIndices, context);
         if (muonVertex.isZero())
            continue;

         const auto &exchangeIndices = muonVertexFunction::template particleIndices<2>(muonVertexIndices);

         // skip diagrams with only Standard Model particles in the loop
         if (((PhotonEmitter::numberOfGenerations == 1 &&
               remove_anti<PhotonEmitter>::type::is_sm_particle[0]) ||
              (PhotonEmitter::numberOfGenerations > 1 &&
               remove_anti<PhotonEmitter>::type::is_sm_particle[photonEmitterIndices[0]])) &&
             ((ExchangeParticle::numberOfGenerations == 1 &&
               remove_anti<ExchangeParticle>::type::is_sm_particle[0]) ||
              (ExchangeParticle::numberOfGenerations > 1 &&
               remove_anti<ExchangeParticle>::type::is_sm_particle[exchangeIndices[0]])))
               continue;

         const std::complex<double> zL = muonVertex.left();
         const std::complex<double> zR = muonVertex.right();

         const double coeffA = std::norm(zL) + std::norm(zR);
         const double coeffB = (zL * std::conj(zR) + zR * std::conj(zL)).real();

         const double muonMass =
            MuonFamily::numberOfGenerations == 1 ?
            context.mass<MuonFamily>() :
            context.mass<MuonFamily>(muonIndex());

         const double photonEmitterMass =
            PhotonEmitter::numberOfGenerations == 1 ?
            context.mass<PhotonEmitter>() :
            context.mass<PhotonEmitter>(photonEmitterIndices[0]);

         const double exchangeMass =
            ExchangeParticle::numberOfGenerations == 1 ?
            context.mass<ExchangeParticle>() :
            context.mass<ExchangeParticle>(exchangeIndices[0]);

         const double massRatioSquared = Sqr(photonEmitterMass / exchangeMass);

         const double part1 = muonMass / 12.0 * coeffA * OneLoopFunctionF1C(massRatioSquared);

         const double part2 =
            is_zero(massRatioSquared) ? 0. :
            photonEmitterMass / 3.0 * coeffB * OneLoopFunctionF2C(massRatioSquared);

         const double preFactor = oneOver16PiSqr * photonEmitterChargeCount
            * muonPhysicalMass(context) / (exchangeMass * exchangeMass);

         res += preFactor * (part1 + part2);
      }
   }

   return res;
}

template<class PhotonEmitter, class ExchangeParticle>
double DiagramEvaluator<OneLoopDiagram<4>, PhotonEmitter, ExchangeParticle>::value(EvaluationContext &context)
{
   double res = 0.0;

   typedef VertexFunction<Photon, PhotonEmitter, typename anti<PhotonEmitter>::type> photonVertexFunction;
   typedef VertexFunction<MuonFamily, typename anti<PhotonEmitter>::type, typename anti<ExchangeParticle>::type> muonVertexFunction;

   for (auto photonVertexIndexIt = photonVertexFunction::indexBounds().begin();
        photonVertexIndexIt != photonVertexFunction::indexBounds().end();
        ++photonVertexIndexIt) {
      const auto &photonVertexIndices = *photonVertexIndexIt;
      const auto &photonVertex = photonVertexFunction::vertex(photonVertexIndices, context);
      if (photonVertex.isZero())
         continue;

      const double photonEmitterChargeCount = photonVertex.value().real() / muonCharge(context);

      const auto &photonEmitterIndices = photonVertexFunction::template particleIndices<2>(photonVertexIndices);

      for (auto muonVertexIndexIt = muonVertexFunction::indexBounds().begin();
           muonVertexIndexIt != muonVertexFunction::indexBounds().end();
           ++muonVertexIndexIt) {
         const auto &muonVertexIndices = *muonVertexIndexIt;
         const auto &muonFamilyIndices = muonVertexFunction::template particleIndices<0>(muonVertexIndices);
         if (MuonFamily::numberOfGenerations != 1 && muonFamilyIndices[0] != muonIndex())
            continue;

         if (photonEmitterIndices != muonVertexFunction::template particleIndices<1>(muonVertexIndices))
            continue;

         const auto &muonVertex = muonVertexFunction::vertex(muonVertexIndices, context);
         if (muonVertex.isZero())
            continue;

         const auto &exchangeIndices = muonVertexFunction::template particleIndices<2>(muonVertexIndices);

         // skip diagrams with only Standard Model particles in the loop
         if (((PhotonEmitter::numberOfGenerations == 1 &&
               remove_anti<PhotonEmitter>::type::is_sm_particle[0]) ||
              (PhotonEmitter::numberOfGenerations > 1 &&
               remove_anti<PhotonEmitter>::type::is_sm_particle[photonEmitterIndices[0]])) &&
             ((ExchangeParticle::numberOfGenerations == 1 &&
               remove_anti<ExchangeParticle>::type::is_sm_particle[0]) ||
              (ExchangeParticle::numberOfGenerations > 1 &&
               remove_anti<ExchangeParticle>::type::is_sm_particle[exchangeIndices[0]])))
               continue;

         const std::complex<double> zL = muonVertex.left();
         const std::complex<double> zR = muonVertex.right();

         const double coeffA = std::norm(zL) + std::norm(zR);
         const double coeffB = (zL * std::conj(zR) + zR * std::conj(zL)).real();

         const double muonMass =
            MuonFamily::numberOfGenerations == 1 ?
            context.mass<MuonFamily>() :
            context.mass<MuonFamily>(muonIndex());

         const double photonEmitterMass =
            PhotonEmitter::numberOfGenerations == 1 ?
            context.mass<PhotonEmitter>() :
            context.mass<PhotonEmitter>(photonEmitterIndices[0]);

         const double exchangeMass =
            ExchangeParticle::numberOfGenerations == 1 ?
            context.mass<ExchangeParticle>() :
            context.mass<ExchangeParticle>(exchangeIndices[0]);

         const double massRatioSquared = Sqr(exchangeMass / photonEmitterMass);

         const double part1 = 1.0 / 12.0 * coeffA * OneLoopFunctionF1N(massRatioSquared);
         const double part2 = exchangeMass / (6.0 * muonMass) * coeffB * OneLoopFunctionF2N(massRatioSquared);

         const double preFactor = - oneOver16PiSqr * photonEmitterChargeCount
            * muonPhysicalMass(context) * muonMass / (photonEmitterMass * photonEmitterMass);

         res += preFactor * (part1 + part2);
      }
   }

   return res;
}
} // anonymous namespace
} // namespace flexiblesusy
