// ====================================================================
// This file is part of FlexibleSUSY.
//
// FlexibleSUSY is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// FlexibleSUSY is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with FlexibleSUSY.  If not, see
// <http://www.gnu.org/licenses/>.
// ====================================================================

// File generated at Thu 17 Aug 2017 11:21:06

/**
 * @file NUHMSSMNoFVHimalaya_mass_eigenstates.hpp
 *
 * @brief contains class for model with routines needed to solve boundary
 *        value problem using the two_scale solver by solving EWSB
 *        and determine the pole masses and mixings
 *
 * This file was generated at Thu 17 Aug 2017 11:21:06 with FlexibleSUSY
 * 1.7.4-Himalaya (git commit: 20b32b788451d6463766eef390bf6d35c0983e74) and SARAH 4.12.0 .
 */

#ifndef NUHMSSMNoFVHimalaya_MASS_EIGENSTATES_H
#define NUHMSSMNoFVHimalaya_MASS_EIGENSTATES_H

#include "NUHMSSMNoFVHimalaya_soft_parameters.hpp"
#include "NUHMSSMNoFVHimalaya_physical.hpp"
#include "NUHMSSMNoFVHimalaya_info.hpp"
#include "loop_corrections.hpp"
#include "threshold_corrections.hpp"
#include "error.hpp"
#include "problems.hpp"
#include "config.h"

#include <iosfwd>
#include <string>

#include <Eigen/Core>

namespace flexiblesusy {

class EWSB_solver;
/**
 * @class NUHMSSMNoFVHimalaya_mass_eigenstates
 * @brief model class with routines for determing masses and mixinga and EWSB
 */
class NUHMSSMNoFVHimalaya_mass_eigenstates : public NUHMSSMNoFVHimalaya_soft_parameters {
public:
   explicit NUHMSSMNoFVHimalaya_mass_eigenstates(const NUHMSSMNoFVHimalaya_input_parameters& input_ = NUHMSSMNoFVHimalaya_input_parameters());
   NUHMSSMNoFVHimalaya_mass_eigenstates(const NUHMSSMNoFVHimalaya_mass_eigenstates&) = default;
   NUHMSSMNoFVHimalaya_mass_eigenstates(NUHMSSMNoFVHimalaya_mass_eigenstates&&) = default;
   virtual ~NUHMSSMNoFVHimalaya_mass_eigenstates() = default;
   NUHMSSMNoFVHimalaya_mass_eigenstates& operator=(const NUHMSSMNoFVHimalaya_mass_eigenstates&) = default;
   NUHMSSMNoFVHimalaya_mass_eigenstates& operator=(NUHMSSMNoFVHimalaya_mass_eigenstates&&) = default;

   /// number of EWSB equations
   static const int number_of_ewsb_equations = 2;

   void calculate_DRbar_masses();
   void calculate_pole_masses();
   void check_pole_masses_for_tachyons();
   virtual void clear() override;
   void clear_DRbar_parameters();
   Eigen::ArrayXd get_DRbar_masses() const;
   Eigen::ArrayXd get_DRbar_masses_and_mixings() const;
   void do_calculate_sm_pole_masses(bool);
   bool do_calculate_sm_pole_masses() const;
   void do_calculate_bsm_pole_masses(bool);
   bool do_calculate_bsm_pole_masses() const;
   void do_force_output(bool);
   bool do_force_output() const;
   void reorder_DRbar_masses();
   void reorder_pole_masses();
   void set_ewsb_iteration_precision(double);
   void set_ewsb_loop_order(int);
   void set_loop_corrections(const Loop_corrections&);
   const Loop_corrections& get_loop_corrections() const;
   void set_threshold_corrections(const Threshold_corrections&);
   const Threshold_corrections& get_threshold_corrections() const;
   void set_DRbar_masses(const Eigen::ArrayXd&);
   void set_DRbar_masses_and_mixings(const Eigen::ArrayXd&);
   void set_pole_mass_loop_order(int);
   int get_pole_mass_loop_order() const;
   void set_physical(const NUHMSSMNoFVHimalaya_physical&);
   double get_ewsb_iteration_precision() const;
   double get_ewsb_loop_order() const;
   const NUHMSSMNoFVHimalaya_physical& get_physical() const;
   NUHMSSMNoFVHimalaya_physical& get_physical();
   const Problems<NUHMSSMNoFVHimalaya_info::NUMBER_OF_PARTICLES, NUHMSSMNoFVHimalaya_info::NUMBER_OF_PARAMETERS>& get_problems() const;
   Problems<NUHMSSMNoFVHimalaya_info::NUMBER_OF_PARTICLES, NUHMSSMNoFVHimalaya_info::NUMBER_OF_PARAMETERS>& get_problems();
   int solve_ewsb_tree_level();
   int solve_ewsb_one_loop();
   int solve_ewsb();            ///< solve EWSB at ewsb_loop_order level

   void calculate_spectrum();
   void clear_problems();
   std::string name() const;
   void run_to(double scale, double eps = -1.0) override;
   void print(std::ostream& out = std::cout) const override;
   void set_precision(double);
   double get_precision() const;


   double get_MVG() const { return MVG; }
   double get_MGlu() const { return MGlu; }
   double get_MFd() const { return MFd; }
   double get_MFs() const { return MFs; }
   double get_MFb() const { return MFb; }
   double get_MFu() const { return MFu; }
   double get_MFc() const { return MFc; }
   double get_MFt() const { return MFt; }
   double get_MFve() const { return MFve; }
   double get_MFvm() const { return MFvm; }
   double get_MFvt() const { return MFvt; }
   double get_MFe() const { return MFe; }
   double get_MFm() const { return MFm; }
   double get_MFtau() const { return MFtau; }
   double get_MSveL() const { return MSveL; }
   double get_MSvmL() const { return MSvmL; }
   double get_MSvtL() const { return MSvtL; }
   const Eigen::Array<double,2,1>& get_MSd() const { return MSd; }
   double get_MSd(int i) const { return MSd(i); }
   const Eigen::Array<double,2,1>& get_MSu() const { return MSu; }
   double get_MSu(int i) const { return MSu(i); }
   const Eigen::Array<double,2,1>& get_MSe() const { return MSe; }
   double get_MSe(int i) const { return MSe(i); }
   const Eigen::Array<double,2,1>& get_MSm() const { return MSm; }
   double get_MSm(int i) const { return MSm(i); }
   const Eigen::Array<double,2,1>& get_MStau() const { return MStau; }
   double get_MStau(int i) const { return MStau(i); }
   const Eigen::Array<double,2,1>& get_MSs() const { return MSs; }
   double get_MSs(int i) const { return MSs(i); }
   const Eigen::Array<double,2,1>& get_MSc() const { return MSc; }
   double get_MSc(int i) const { return MSc(i); }
   const Eigen::Array<double,2,1>& get_MSb() const { return MSb; }
   double get_MSb(int i) const { return MSb(i); }
   const Eigen::Array<double,2,1>& get_MSt() const { return MSt; }
   double get_MSt(int i) const { return MSt(i); }
   const Eigen::Array<double,2,1>& get_Mhh() const { return Mhh; }
   double get_Mhh(int i) const { return Mhh(i); }
   const Eigen::Array<double,2,1>& get_MAh() const { return MAh; }
   double get_MAh(int i) const { return MAh(i); }
   const Eigen::Array<double,2,1>& get_MHpm() const { return MHpm; }
   double get_MHpm(int i) const { return MHpm(i); }
   const Eigen::Array<double,4,1>& get_MChi() const { return MChi; }
   double get_MChi(int i) const { return MChi(i); }
   const Eigen::Array<double,2,1>& get_MCha() const { return MCha; }
   double get_MCha(int i) const { return MCha(i); }
   double get_MVWm() const { return MVWm; }
   double get_MVP() const { return MVP; }
   double get_MVZ() const { return MVZ; }

   
   Eigen::Array<double,1,1> get_MChargedHiggs() const;

   Eigen::Array<double,1,1> get_MPseudoscalarHiggs() const;

   const Eigen::Matrix<double,2,2>& get_ZD() const { return ZD; }
   double get_ZD(int i, int k) const { return ZD(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZU() const { return ZU; }
   double get_ZU(int i, int k) const { return ZU(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZE() const { return ZE; }
   double get_ZE(int i, int k) const { return ZE(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZM() const { return ZM; }
   double get_ZM(int i, int k) const { return ZM(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZTau() const { return ZTau; }
   double get_ZTau(int i, int k) const { return ZTau(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZS() const { return ZS; }
   double get_ZS(int i, int k) const { return ZS(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZC() const { return ZC; }
   double get_ZC(int i, int k) const { return ZC(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZB() const { return ZB; }
   double get_ZB(int i, int k) const { return ZB(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZT() const { return ZT; }
   double get_ZT(int i, int k) const { return ZT(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZH() const { return ZH; }
   double get_ZH(int i, int k) const { return ZH(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZA() const { return ZA; }
   double get_ZA(int i, int k) const { return ZA(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZP() const { return ZP; }
   double get_ZP(int i, int k) const { return ZP(i,k); }
   const Eigen::Matrix<std::complex<double>,4,4>& get_ZN() const { return ZN; }
   std::complex<double> get_ZN(int i, int k) const { return ZN(i,k); }
   const Eigen::Matrix<std::complex<double>,2,2>& get_UM() const { return UM; }
   std::complex<double> get_UM(int i, int k) const { return UM(i,k); }
   const Eigen::Matrix<std::complex<double>,2,2>& get_UP() const { return UP; }
   std::complex<double> get_UP(int i, int k) const { return UP(i,k); }
   const Eigen::Matrix<double,2,2>& get_ZZ() const { return ZZ; }
   double get_ZZ(int i, int k) const { return ZZ(i,k); }

   void set_PhaseGlu(std::complex<double> PhaseGlu_) { PhaseGlu = PhaseGlu_; }
   std::complex<double> get_PhaseGlu() const { return PhaseGlu; }



   double get_mass_matrix_VG() const;
   void calculate_MVG();
   double get_mass_matrix_Glu() const;
   void calculate_MGlu();
   double get_mass_matrix_Fd() const;
   void calculate_MFd();
   double get_mass_matrix_Fs() const;
   void calculate_MFs();
   double get_mass_matrix_Fb() const;
   void calculate_MFb();
   double get_mass_matrix_Fu() const;
   void calculate_MFu();
   double get_mass_matrix_Fc() const;
   void calculate_MFc();
   double get_mass_matrix_Ft() const;
   void calculate_MFt();
   double get_mass_matrix_Fve() const;
   void calculate_MFve();
   double get_mass_matrix_Fvm() const;
   void calculate_MFvm();
   double get_mass_matrix_Fvt() const;
   void calculate_MFvt();
   double get_mass_matrix_Fe() const;
   void calculate_MFe();
   double get_mass_matrix_Fm() const;
   void calculate_MFm();
   double get_mass_matrix_Ftau() const;
   void calculate_MFtau();
   double get_mass_matrix_SveL() const;
   void calculate_MSveL();
   double get_mass_matrix_SvmL() const;
   void calculate_MSvmL();
   double get_mass_matrix_SvtL() const;
   void calculate_MSvtL();
   Eigen::Matrix<double,2,2> get_mass_matrix_Sd() const;
   void calculate_MSd();
   Eigen::Matrix<double,2,2> get_mass_matrix_Su() const;
   void calculate_MSu();
   Eigen::Matrix<double,2,2> get_mass_matrix_Se() const;
   void calculate_MSe();
   Eigen::Matrix<double,2,2> get_mass_matrix_Sm() const;
   void calculate_MSm();
   Eigen::Matrix<double,2,2> get_mass_matrix_Stau() const;
   void calculate_MStau();
   Eigen::Matrix<double,2,2> get_mass_matrix_Ss() const;
   void calculate_MSs();
   Eigen::Matrix<double,2,2> get_mass_matrix_Sc() const;
   void calculate_MSc();
   Eigen::Matrix<double,2,2> get_mass_matrix_Sb() const;
   void calculate_MSb();
   Eigen::Matrix<double,2,2> get_mass_matrix_St() const;
   void calculate_MSt();
   Eigen::Matrix<double,2,2> get_mass_matrix_hh() const;
   void calculate_Mhh();
   Eigen::Matrix<double,2,2> get_mass_matrix_Ah() const;
   void calculate_MAh();
   Eigen::Matrix<double,2,2> get_mass_matrix_Hpm() const;
   void calculate_MHpm();
   Eigen::Matrix<double,4,4> get_mass_matrix_Chi() const;
   void calculate_MChi();
   Eigen::Matrix<double,2,2> get_mass_matrix_Cha() const;
   void calculate_MCha();
   double get_mass_matrix_VWm() const;
   void calculate_MVWm();
   Eigen::Matrix<double,2,2> get_mass_matrix_VPVZ() const;
   void calculate_MVPVZ();

   double get_ewsb_eq_hh_1() const;
   double get_ewsb_eq_hh_2() const;

   std::complex<double> CpUSdconjUSdVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUSdconjUSdconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUSdconjUSdconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUSdconjUSdconjSvtLSvtL(int gO1, int gO2) const;
   double CpUSdconjUSdconjVWmVWm(int gO1, int gO2) const;
   std::complex<double> CpUSdconjUSdAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSdconjUSdhhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSdSdAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSdSdhh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSdSuHpm(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSdVGSd(int gO2, int gI2) const;
   std::complex<double> CpconjUSdVPSd(int gO2, int gI2) const;
   std::complex<double> CpconjUSdVZSd(int gO2, int gI2) const;
   std::complex<double> CpconjUSdVWmSu(int gO2, int gI2) const;
   std::complex<double> CpconjUSdFuChaPR(int gO2, int , int gI2) const;
   std::complex<double> CpconjUSdFuChaPL(int gO1, int , int gI2) const;
   std::complex<double> CpconjUSdFdChiPR(int gO2, int , int gI2) const;
   std::complex<double> CpconjUSdFdChiPL(int gO1, int , int gI2) const;
   std::complex<double> CpconjUSdGluFdPR(int gO2, int , int ) const;
   std::complex<double> CpconjUSdGluFdPL(int gO1, int , int ) const;
   std::complex<double> CpUSuconjUSuVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUSuconjUSuconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUSuconjUSuconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUSuconjUSuconjSvtLSvtL(int gO1, int gO2) const;
   double CpUSuconjUSuconjVWmVWm(int gO1, int gO2) const;
   std::complex<double> CpUSuconjUSuAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSuconjUSuhhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSuconjHpmSd(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSuSuAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSuSuhh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSubarChaFdPR(int gO2, int gI1, int ) const;
   std::complex<double> CpconjUSubarChaFdPL(int gO1, int gI1, int ) const;
   std::complex<double> CpconjUSuconjVWmSd(int gO2, int gI2) const;
   std::complex<double> CpconjUSuVGSu(int gO2, int gI2) const;
   std::complex<double> CpconjUSuVPSu(int gO2, int gI2) const;
   std::complex<double> CpconjUSuVZSu(int gO2, int gI2) const;
   std::complex<double> CpconjUSuFuChiPR(int gO2, int , int gI2) const;
   std::complex<double> CpconjUSuFuChiPL(int gO1, int , int gI2) const;
   std::complex<double> CpconjUSuGluFuPR(int gO2, int , int ) const;
   std::complex<double> CpconjUSuGluFuPL(int gO1, int , int ) const;
   std::complex<double> CpUSeconjUSeVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUSeconjUSeconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUSeconjUSeconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUSeconjUSeconjSvtLSvtL(int gO1, int gO2) const;
   double CpUSeconjUSeconjVWmVWm(int gO1, int gO2) const;
   double CpconjUSeVWmSveL(int gO2) const;
   std::complex<double> CpUSeconjUSeAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSeconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSeconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSeconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSeconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSeconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSeconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSeconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSeconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSeconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSeconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSeconjUSehhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSeSeAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSeSehh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSeSveLHpm(int gO2, int gI2) const;
   std::complex<double> CpconjUSeVPSe(int gO2, int gI2) const;
   std::complex<double> CpconjUSeVZSe(int gO2, int gI2) const;
   double CpconjUSeFveChaPR(int , int ) const;
   std::complex<double> CpconjUSeFveChaPL(int gO1, int gI2) const;
   std::complex<double> CpconjUSeFeChiPR(int gO2, int gI2) const;
   std::complex<double> CpconjUSeFeChiPL(int gO1, int gI2) const;
   std::complex<double> CpUSmconjUSmVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUSmconjUSmconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUSmconjUSmconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUSmconjUSmconjSvtLSvtL(int gO1, int gO2) const;
   double CpUSmconjUSmconjVWmVWm(int gO1, int gO2) const;
   double CpconjUSmVWmSvmL(int gO2) const;
   std::complex<double> CpUSmconjUSmAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSmconjUSmhhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSmSmAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSmSmhh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSmSvmLHpm(int gO2, int gI2) const;
   std::complex<double> CpconjUSmVPSm(int gO2, int gI2) const;
   std::complex<double> CpconjUSmVZSm(int gO2, int gI2) const;
   double CpconjUSmFvmChaPR(int , int ) const;
   std::complex<double> CpconjUSmFvmChaPL(int gO1, int gI2) const;
   std::complex<double> CpconjUSmFmChiPR(int gO2, int gI2) const;
   std::complex<double> CpconjUSmFmChiPL(int gO1, int gI2) const;
   std::complex<double> CpUStauconjUStauVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUStauconjUStauconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUStauconjUStauconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUStauconjUStauconjSvtLSvtL(int gO1, int gO2) const;
   double CpUStauconjUStauconjVWmVWm(int gO1, int gO2) const;
   double CpconjUStauVWmSvtL(int gO2) const;
   std::complex<double> CpUStauconjUStauAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStauconjUStauhhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUStauStauAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUStauStauhh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUStauSvtLHpm(int gO2, int gI2) const;
   std::complex<double> CpconjUStauVPStau(int gO2, int gI2) const;
   std::complex<double> CpconjUStauVZStau(int gO2, int gI2) const;
   double CpconjUStauFvtChaPR(int , int ) const;
   std::complex<double> CpconjUStauFvtChaPL(int gO1, int gI2) const;
   std::complex<double> CpconjUStauFtauChiPR(int gO2, int gI2) const;
   std::complex<double> CpconjUStauFtauChiPL(int gO1, int gI2) const;
   std::complex<double> CpUSsconjUSsVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUSsconjUSsconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUSsconjUSsconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUSsconjUSsconjSvtLSvtL(int gO1, int gO2) const;
   double CpUSsconjUSsconjVWmVWm(int gO1, int gO2) const;
   std::complex<double> CpUSsconjUSsAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSsconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSsconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSsconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSsconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSsconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSsconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSsconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSsconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSsconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSsconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSsconjUSshhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSsScHpm(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSsSsAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSsSshh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSsVWmSc(int gO2, int gI2) const;
   std::complex<double> CpconjUSsVGSs(int gO2, int gI2) const;
   std::complex<double> CpconjUSsVPSs(int gO2, int gI2) const;
   std::complex<double> CpconjUSsVZSs(int gO2, int gI2) const;
   std::complex<double> CpconjUSsFcChaPR(int gO2, int , int gI2) const;
   std::complex<double> CpconjUSsFcChaPL(int gO1, int , int gI2) const;
   std::complex<double> CpconjUSsFsChiPR(int gO2, int , int gI2) const;
   std::complex<double> CpconjUSsFsChiPL(int gO1, int , int gI2) const;
   std::complex<double> CpconjUSsGluFsPR(int gO2, int , int ) const;
   std::complex<double> CpconjUSsGluFsPL(int gO1, int , int ) const;
   std::complex<double> CpUScconjUScVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUScconjUScconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUScconjUScconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUScconjUScconjSvtLSvtL(int gO1, int gO2) const;
   double CpUScconjUScconjVWmVWm(int gO1, int gO2) const;
   std::complex<double> CpUScconjUScAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUScconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUScconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUScconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUScconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUScconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUScconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUScconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUScconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUScconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUScconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUScconjUSchhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUScconjHpmSs(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUScScAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUScSchh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUScbarChaFsPR(int gO2, int gI1, int ) const;
   std::complex<double> CpconjUScbarChaFsPL(int gO1, int gI1, int ) const;
   std::complex<double> CpconjUScVGSc(int gO2, int gI2) const;
   std::complex<double> CpconjUScVPSc(int gO2, int gI2) const;
   std::complex<double> CpconjUScVZSc(int gO2, int gI2) const;
   std::complex<double> CpconjUScconjVWmSs(int gO2, int gI2) const;
   std::complex<double> CpconjUScFcChiPR(int gO2, int , int gI2) const;
   std::complex<double> CpconjUScFcChiPL(int gO1, int , int gI2) const;
   std::complex<double> CpconjUScGluFcPR(int gO2, int , int ) const;
   std::complex<double> CpconjUScGluFcPL(int gO1, int , int ) const;
   std::complex<double> CpUSbconjUSbVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUSbconjUSbconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUSbconjUSbconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUSbconjUSbconjSvtLSvtL(int gO1, int gO2) const;
   double CpUSbconjUSbconjVWmVWm(int gO1, int gO2) const;
   std::complex<double> CpUSbconjUSbAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUSbconjUSbhhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSbSbAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSbSbhh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSbStHpm(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUSbVGSb(int gO2, int gI2) const;
   std::complex<double> CpconjUSbVPSb(int gO2, int gI2) const;
   std::complex<double> CpconjUSbVZSb(int gO2, int gI2) const;
   std::complex<double> CpconjUSbVWmSt(int gO2, int gI2) const;
   std::complex<double> CpconjUSbFtChaPR(int gO2, int , int gI2) const;
   std::complex<double> CpconjUSbFtChaPL(int gO1, int , int gI2) const;
   std::complex<double> CpconjUSbFbChiPR(int gO2, int , int gI2) const;
   std::complex<double> CpconjUSbFbChiPL(int gO1, int , int gI2) const;
   std::complex<double> CpconjUSbGluFbPR(int gO2, int , int ) const;
   std::complex<double> CpconjUSbGluFbPL(int gO1, int , int ) const;
   std::complex<double> CpUStconjUStVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUStconjUStconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUStconjUStconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUStconjUStconjSvtLSvtL(int gO1, int gO2) const;
   double CpUStconjUStconjVWmVWm(int gO1, int gO2) const;
   std::complex<double> CpUStconjUStAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUStconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUStconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUStconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUStconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUStconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUStconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUStconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUStconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUStconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUStconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUStconjUSthhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUStconjHpmSb(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUStStAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUStSthh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUStbarChaFbPR(int gO2, int gI1, int ) const;
   std::complex<double> CpconjUStbarChaFbPL(int gO1, int gI1, int ) const;
   std::complex<double> CpconjUStconjVWmSb(int gO2, int gI2) const;
   std::complex<double> CpconjUStVGSt(int gO2, int gI2) const;
   std::complex<double> CpconjUStVPSt(int gO2, int gI2) const;
   std::complex<double> CpconjUStVZSt(int gO2, int gI2) const;
   std::complex<double> CpconjUStFtChiPR(int gO2, int , int gI2) const;
   std::complex<double> CpconjUStFtChiPL(int gO1, int , int gI2) const;
   std::complex<double> CpconjUStGluFtPR(int gO2, int , int ) const;
   std::complex<double> CpconjUStGluFtPL(int gO1, int , int ) const;
   std::complex<double> CpUhhVZVZ(int gO2) const;
   std::complex<double> CpUhhconjSveLSveL(int gO2) const;
   std::complex<double> CpUhhconjSvmLSvmL(int gO2) const;
   std::complex<double> CpUhhconjSvtLSvtL(int gO2) const;
   std::complex<double> CpUhhconjVWmVWm(int gO2) const;
   std::complex<double> CpUhhbargWmgWm(int gO1) const;
   std::complex<double> CpUhhbargWmCgWmC(int gO1) const;
   std::complex<double> CpUhhbargZgZ(int gO1) const;
   std::complex<double> CpUhhUhhVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUhhUhhconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUhhUhhconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUhhUhhconjSvtLSvtL(int gO1, int gO2) const;
   std::complex<double> CpUhhUhhconjVWmVWm(int gO1, int gO2) const;
   std::complex<double> CpUhhUhhAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhUhhhhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhAhAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhconjHpmHpm(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhconjSbSb(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhconjScSc(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhconjSdSd(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhconjSeSe(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhconjSmSm(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhconjSsSs(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhconjStSt(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhconjStauStau(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhconjSuSu(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhhhhh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhbarChaChaPR(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhbarChaChaPL(int gO1, int gI1, int gI2) const;
   std::complex<double> CpUhhChiChiPR(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUhhChiChiPL(int gO1, int gI1, int gI2) const;
   std::complex<double> CpUhhVZAh(int gO2, int gI2) const;
   std::complex<double> CpUhhconjVWmHpm(int gO2, int gI2) const;
   double CpUhhbarFeFePR(int gO2) const;
   double CpUhhbarFeFePL(int gO1) const;
   double CpUhhbarFmFmPR(int gO2) const;
   double CpUhhbarFmFmPL(int gO1) const;
   double CpUhhbarFtauFtauPR(int gO2) const;
   double CpUhhbarFtauFtauPL(int gO1) const;
   double CpUhhbarFbFbPR(int gO2, int , int ) const;
   double CpUhhbarFbFbPL(int gO1, int , int ) const;
   double CpUhhbarFcFcPR(int gO2, int , int ) const;
   double CpUhhbarFcFcPL(int gO1, int , int ) const;
   double CpUhhbarFdFdPR(int gO2, int , int ) const;
   double CpUhhbarFdFdPL(int gO1, int , int ) const;
   double CpUhhbarFsFsPR(int gO2, int , int ) const;
   double CpUhhbarFsFsPL(int gO1, int , int ) const;
   double CpUhhbarFtFtPR(int gO2, int , int ) const;
   double CpUhhbarFtFtPL(int gO1, int , int ) const;
   double CpUhhbarFuFuPR(int gO2, int , int ) const;
   double CpUhhbarFuFuPL(int gO1, int , int ) const;
   std::complex<double> CpUAhbargWmgWm(int gO1) const;
   std::complex<double> CpUAhbargWmCgWmC(int gO1) const;
   std::complex<double> CpUAhUAhVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUAhUAhconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUAhUAhconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUAhUAhconjSvtLSvtL(int gO1, int gO2) const;
   std::complex<double> CpUAhUAhconjVWmVWm(int gO1, int gO2) const;
   std::complex<double> CpUAhUAhAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhUAhhhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhconjHpmHpm(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhconjSbSb(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhconjScSc(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhconjSdSd(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhconjSeSe(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhconjSmSm(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhconjSsSs(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhconjStSt(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhconjStauStau(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhconjSuSu(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhhhAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhbarChaChaPR(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhbarChaChaPL(int gO1, int gI1, int gI2) const;
   std::complex<double> CpUAhChiChiPR(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUAhChiChiPL(int gO1, int gI1, int gI2) const;
   std::complex<double> CpUAhVZhh(int gO2, int gI2) const;
   std::complex<double> CpUAhconjVWmHpm(int gO2, int gI2) const;
   std::complex<double> CpUAhbarFeFePR(int gO2) const;
   std::complex<double> CpUAhbarFeFePL(int gO1) const;
   std::complex<double> CpUAhbarFmFmPR(int gO2) const;
   std::complex<double> CpUAhbarFmFmPL(int gO1) const;
   std::complex<double> CpUAhbarFtauFtauPR(int gO2) const;
   std::complex<double> CpUAhbarFtauFtauPL(int gO1) const;
   std::complex<double> CpUAhbarFbFbPR(int gO2, int , int ) const;
   std::complex<double> CpUAhbarFbFbPL(int gO1, int , int ) const;
   std::complex<double> CpUAhbarFcFcPR(int gO2, int , int ) const;
   std::complex<double> CpUAhbarFcFcPL(int gO1, int , int ) const;
   std::complex<double> CpUAhbarFdFdPR(int gO2, int , int ) const;
   std::complex<double> CpUAhbarFdFdPL(int gO1, int , int ) const;
   std::complex<double> CpUAhbarFsFsPR(int gO2, int , int ) const;
   std::complex<double> CpUAhbarFsFsPL(int gO1, int , int ) const;
   std::complex<double> CpUAhbarFtFtPR(int gO2, int , int ) const;
   std::complex<double> CpUAhbarFtFtPL(int gO1, int , int ) const;
   std::complex<double> CpUAhbarFuFuPR(int gO2, int , int ) const;
   std::complex<double> CpUAhbarFuFuPL(int gO1, int , int ) const;
   std::complex<double> CpconjUHpmVWmVP(int gO2) const;
   std::complex<double> CpconjUHpmVZVWm(int gO2) const;
   std::complex<double> CpconjUHpmbargWmCgZ(int gO1) const;
   std::complex<double> CpUHpmgWmCbargZ(int gO2) const;
   std::complex<double> CpconjUHpmbargZgWm(int gO1) const;
   std::complex<double> CpUHpmgZbargWm(int gO2) const;
   std::complex<double> CpUHpmconjUHpmVZVZ(int gO1, int gO2) const;
   std::complex<double> CpUHpmconjUHpmconjSveLSveL(int gO1, int gO2) const;
   std::complex<double> CpUHpmconjUHpmconjSvmLSvmL(int gO1, int gO2) const;
   std::complex<double> CpUHpmconjUHpmconjSvtLSvtL(int gO1, int gO2) const;
   std::complex<double> CpUHpmconjUHpmconjVWmVWm(int gO1, int gO2) const;
   std::complex<double> CpUHpmconjUHpmAhAh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmconjSbSb(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmconjScSc(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmconjSdSd(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmconjSeSe(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmconjSmSm(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmconjSsSs(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmconjStSt(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmconjStauStau(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmconjSuSu(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpUHpmconjUHpmhhhh(int gO1, int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUHpmconjScSs(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUHpmconjStSb(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUHpmconjSuSd(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUHpmHpmAh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUHpmHpmhh(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUHpmChiChaPR(int gO2, int gI1, int gI2) const;
   std::complex<double> CpconjUHpmChiChaPL(int gO1, int gI1, int gI2) const;
   std::complex<double> CpconjUHpmconjSveLSe(int gO2, int gI2) const;
   std::complex<double> CpconjUHpmconjSvmLSm(int gO2, int gI2) const;
   std::complex<double> CpconjUHpmconjSvtLStau(int gO2, int gI2) const;
   std::complex<double> CpconjUHpmVWmAh(int gO2, int gI2) const;
   std::complex<double> CpconjUHpmVWmhh(int gO2, int gI2) const;
   std::complex<double> CpconjUHpmVPHpm(int gO2, int gI2) const;
   std::complex<double> CpconjUHpmVZHpm(int gO2, int gI2) const;
   double CpconjUHpmbarFveFePR(int gO2) const;
   double CpconjUHpmbarFveFePL(int ) const;
   double CpconjUHpmbarFvmFmPR(int gO2) const;
   double CpconjUHpmbarFvmFmPL(int ) const;
   double CpconjUHpmbarFvtFtauPR(int gO2) const;
   double CpconjUHpmbarFvtFtauPL(int ) const;
   double CpconjUHpmbarFcFsPR(int gO2, int , int ) const;
   double CpconjUHpmbarFcFsPL(int gO1, int , int ) const;
   double CpconjUHpmbarFtFbPR(int gO2, int , int ) const;
   double CpconjUHpmbarFtFbPL(int gO1, int , int ) const;
   double CpconjUHpmbarFuFdPR(int gO2, int , int ) const;
   double CpconjUHpmbarFuFdPL(int gO1, int , int ) const;
   std::complex<double> CpSveLconjSveLVZVZ() const;
   double CpSveLconjSveLconjSveLSveL() const;
   double CpSveLconjSveLconjSvmLSvmL() const;
   double CpSveLconjSveLconjSvtLSvtL() const;
   double CpSveLconjSveLconjVWmVWm() const;
   double CpconjSveLVZSveL() const;
   std::complex<double> CpSveLconjSveLAhAh(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLconjHpmHpm(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLconjSbSb(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLconjScSc(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLconjSdSd(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLconjSeSe(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLconjSmSm(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLconjSsSs(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLconjStSt(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLconjStauStau(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLconjSuSu(int gI1, int gI2) const;
   std::complex<double> CpSveLconjSveLhhhh(int gI1, int gI2) const;
   std::complex<double> CpconjSveLconjHpmSe(int gI1, int gI2) const;
   std::complex<double> CpconjSveLbarChaFePR(int gI1) const;
   std::complex<double> CpconjSveLbarChaFePL(int gI1) const;
   std::complex<double> CpconjSveLSveLhh(int gI2) const;
   std::complex<double> CpconjSveLconjVWmSe(int gI2) const;
   double CpconjSveLFveChiPR(int ) const;
   std::complex<double> CpconjSveLFveChiPL(int gI2) const;
   std::complex<double> CpSvmLconjSvmLVZVZ() const;
   double CpSvmLconjSvmLconjSveLSveL() const;
   double CpSvmLconjSvmLconjSvmLSvmL() const;
   double CpSvmLconjSvmLconjSvtLSvtL() const;
   double CpSvmLconjSvmLconjVWmVWm() const;
   double CpconjSvmLVZSvmL() const;
   std::complex<double> CpSvmLconjSvmLAhAh(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLconjHpmHpm(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLconjSbSb(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLconjScSc(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLconjSdSd(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLconjSeSe(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLconjSmSm(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLconjSsSs(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLconjStSt(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLconjStauStau(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLconjSuSu(int gI1, int gI2) const;
   std::complex<double> CpSvmLconjSvmLhhhh(int gI1, int gI2) const;
   std::complex<double> CpconjSvmLconjHpmSm(int gI1, int gI2) const;
   std::complex<double> CpconjSvmLbarChaFmPR(int gI1) const;
   std::complex<double> CpconjSvmLbarChaFmPL(int gI1) const;
   std::complex<double> CpconjSvmLSvmLhh(int gI2) const;
   std::complex<double> CpconjSvmLconjVWmSm(int gI2) const;
   double CpconjSvmLFvmChiPR(int ) const;
   std::complex<double> CpconjSvmLFvmChiPL(int gI2) const;
   std::complex<double> CpSvtLconjSvtLVZVZ() const;
   double CpSvtLconjSvtLconjSveLSveL() const;
   double CpSvtLconjSvtLconjSvmLSvmL() const;
   double CpSvtLconjSvtLconjSvtLSvtL() const;
   double CpSvtLconjSvtLconjVWmVWm() const;
   double CpconjSvtLVZSvtL() const;
   std::complex<double> CpSvtLconjSvtLAhAh(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLconjHpmHpm(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLconjSbSb(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLconjScSc(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLconjSdSd(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLconjSeSe(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLconjSmSm(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLconjSsSs(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLconjStSt(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLconjStauStau(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLconjSuSu(int gI1, int gI2) const;
   std::complex<double> CpSvtLconjSvtLhhhh(int gI1, int gI2) const;
   std::complex<double> CpconjSvtLconjHpmStau(int gI1, int gI2) const;
   std::complex<double> CpconjSvtLbarChaFtauPR(int gI1) const;
   std::complex<double> CpconjSvtLbarChaFtauPL(int gI1) const;
   std::complex<double> CpconjSvtLSvtLhh(int gI2) const;
   std::complex<double> CpconjSvtLconjVWmStau(int gI2) const;
   double CpconjSvtLFvtChiPR(int ) const;
   std::complex<double> CpconjSvtLFvtChiPL(int gI2) const;
   std::complex<double> CpVGVGVG() const;
   std::complex<double> CpVGbargGgG() const;
   std::complex<double> CpVGVGconjSbSb(int gI1, int gI2) const;
   std::complex<double> CpVGVGconjScSc(int gI1, int gI2) const;
   std::complex<double> CpVGVGconjSdSd(int gI1, int gI2) const;
   std::complex<double> CpVGVGconjSsSs(int gI1, int gI2) const;
   std::complex<double> CpVGVGconjStSt(int gI1, int gI2) const;
   std::complex<double> CpVGVGconjSuSu(int gI1, int gI2) const;
   double CpVGconjSbSb(int gI1, int gI2) const;
   double CpVGconjScSc(int gI1, int gI2) const;
   double CpVGconjSdSd(int gI1, int gI2) const;
   double CpVGconjSsSs(int gI1, int gI2) const;
   double CpVGconjStSt(int gI1, int gI2) const;
   double CpVGconjSuSu(int gI1, int gI2) const;
   double CpVGbarFbFbPL(int , int ) const;
   double CpVGbarFbFbPR(int , int ) const;
   double CpVGbarFcFcPL(int , int ) const;
   double CpVGbarFcFcPR(int , int ) const;
   double CpVGbarFdFdPL(int , int ) const;
   double CpVGbarFdFdPR(int , int ) const;
   double CpVGbarFsFsPL(int , int ) const;
   double CpVGbarFsFsPR(int , int ) const;
   double CpVGbarFtFtPL(int , int ) const;
   double CpVGbarFtFtPR(int , int ) const;
   double CpVGbarFuFuPL(int , int ) const;
   double CpVGbarFuFuPR(int , int ) const;
   double CpVGGluGluPL(int , int ) const;
   double CpVGGluGluPR(int , int ) const;
   double CpVGVGVGVG1() const;
   double CpVGVGVGVG2() const;
   double CpVGVGVGVG3() const;
   double CpVPbargWmgWm() const;
   double CpVPbargWmCgWmC() const;
   double CpVPconjVWmVWm() const;
   double CpVPbarFeFePL() const;
   double CpVPbarFeFePR() const;
   double CpVPbarFmFmPL() const;
   double CpVPbarFmFmPR() const;
   double CpVPbarFtauFtauPL() const;
   double CpVPbarFtauFtauPR() const;
   std::complex<double> CpVPVPconjHpmHpm(int gI1, int gI2) const;
   std::complex<double> CpVPVPconjSbSb(int gI1, int gI2) const;
   std::complex<double> CpVPVPconjScSc(int gI1, int gI2) const;
   std::complex<double> CpVPVPconjSdSd(int gI1, int gI2) const;
   std::complex<double> CpVPVPconjSeSe(int gI1, int gI2) const;
   std::complex<double> CpVPVPconjSmSm(int gI1, int gI2) const;
   std::complex<double> CpVPVPconjSsSs(int gI1, int gI2) const;
   std::complex<double> CpVPVPconjStSt(int gI1, int gI2) const;
   std::complex<double> CpVPVPconjStauStau(int gI1, int gI2) const;
   std::complex<double> CpVPVPconjSuSu(int gI1, int gI2) const;
   double CpVPconjHpmHpm(int gI1, int gI2) const;
   std::complex<double> CpVPconjSbSb(int gI1, int gI2) const;
   std::complex<double> CpVPconjScSc(int gI1, int gI2) const;
   std::complex<double> CpVPconjSdSd(int gI1, int gI2) const;
   std::complex<double> CpVPconjSeSe(int gI1, int gI2) const;
   std::complex<double> CpVPconjSmSm(int gI1, int gI2) const;
   std::complex<double> CpVPconjSsSs(int gI1, int gI2) const;
   std::complex<double> CpVPconjStSt(int gI1, int gI2) const;
   std::complex<double> CpVPconjStauStau(int gI1, int gI2) const;
   std::complex<double> CpVPconjSuSu(int gI1, int gI2) const;
   std::complex<double> CpVPbarChaChaPL(int gI1, int gI2) const;
   std::complex<double> CpVPbarChaChaPR(int gI1, int gI2) const;
   std::complex<double> CpVPconjVWmHpm(int gI2) const;
   double CpVPbarFbFbPL(int , int ) const;
   double CpVPbarFbFbPR(int , int ) const;
   double CpVPbarFcFcPL(int , int ) const;
   double CpVPbarFcFcPR(int , int ) const;
   double CpVPbarFdFdPL(int , int ) const;
   double CpVPbarFdFdPR(int , int ) const;
   double CpVPbarFsFsPL(int , int ) const;
   double CpVPbarFsFsPR(int , int ) const;
   double CpVPbarFtFtPL(int , int ) const;
   double CpVPbarFtFtPR(int , int ) const;
   double CpVPbarFuFuPL(int , int ) const;
   double CpVPbarFuFuPR(int , int ) const;
   double CpVPVPconjVWmVWm1() const;
   double CpVPVPconjVWmVWm2() const;
   double CpVPVPconjVWmVWm3() const;
   double CpVZbargWmgWm() const;
   double CpVZbargWmCgWmC() const;
   double CpVZconjSveLSveL() const;
   double CpVZconjSvmLSvmL() const;
   double CpVZconjSvtLSvtL() const;
   std::complex<double> CpVZVZconjSveLSveL() const;
   std::complex<double> CpVZVZconjSvmLSvmL() const;
   std::complex<double> CpVZVZconjSvtLSvtL() const;
   double CpVZconjVWmVWm() const;
   double CpVZbarFeFePL() const;
   double CpVZbarFeFePR() const;
   double CpVZbarFmFmPL() const;
   double CpVZbarFmFmPR() const;
   double CpVZbarFtauFtauPL() const;
   double CpVZbarFtauFtauPR() const;
   double CpVZbarFveFvePL() const;
   double CpVZbarFveFvePR() const;
   double CpVZbarFvmFvmPL() const;
   double CpVZbarFvmFvmPR() const;
   double CpVZbarFvtFvtPL() const;
   double CpVZbarFvtFvtPR() const;
   std::complex<double> CpVZVZAhAh(int gI1, int gI2) const;
   std::complex<double> CpVZVZconjHpmHpm(int gI1, int gI2) const;
   std::complex<double> CpVZVZconjSbSb(int gI1, int gI2) const;
   std::complex<double> CpVZVZconjScSc(int gI1, int gI2) const;
   std::complex<double> CpVZVZconjSdSd(int gI1, int gI2) const;
   std::complex<double> CpVZVZconjSeSe(int gI1, int gI2) const;
   std::complex<double> CpVZVZconjSmSm(int gI1, int gI2) const;
   std::complex<double> CpVZVZconjSsSs(int gI1, int gI2) const;
   std::complex<double> CpVZVZconjStSt(int gI1, int gI2) const;
   std::complex<double> CpVZVZconjStauStau(int gI1, int gI2) const;
   std::complex<double> CpVZVZconjSuSu(int gI1, int gI2) const;
   std::complex<double> CpVZVZhhhh(int gI1, int gI2) const;
   double CpVZconjHpmHpm(int gI1, int gI2) const;
   std::complex<double> CpVZconjSbSb(int gI1, int gI2) const;
   std::complex<double> CpVZconjScSc(int gI1, int gI2) const;
   std::complex<double> CpVZconjSdSd(int gI1, int gI2) const;
   std::complex<double> CpVZconjSeSe(int gI1, int gI2) const;
   std::complex<double> CpVZconjSmSm(int gI1, int gI2) const;
   std::complex<double> CpVZconjSsSs(int gI1, int gI2) const;
   std::complex<double> CpVZconjStSt(int gI1, int gI2) const;
   std::complex<double> CpVZconjStauStau(int gI1, int gI2) const;
   std::complex<double> CpVZconjSuSu(int gI1, int gI2) const;
   std::complex<double> CpVZhhAh(int gI1, int gI2) const;
   std::complex<double> CpVZbarChaChaPL(int gI1, int gI2) const;
   std::complex<double> CpVZbarChaChaPR(int gI1, int gI2) const;
   std::complex<double> CpVZChiChiPL(int gI1, int gI2) const;
   std::complex<double> CpVZChiChiPR(int gI1, int gI2) const;
   std::complex<double> CpVZVZhh(int gI2) const;
   std::complex<double> CpVZconjVWmHpm(int gI2) const;
   double CpVZbarFbFbPL(int , int ) const;
   double CpVZbarFbFbPR(int , int ) const;
   double CpVZbarFcFcPL(int , int ) const;
   double CpVZbarFcFcPR(int , int ) const;
   double CpVZbarFdFdPL(int , int ) const;
   double CpVZbarFdFdPR(int , int ) const;
   double CpVZbarFsFsPL(int , int ) const;
   double CpVZbarFsFsPR(int , int ) const;
   double CpVZbarFtFtPL(int , int ) const;
   double CpVZbarFtFtPR(int , int ) const;
   double CpVZbarFuFuPL(int , int ) const;
   double CpVZbarFuFuPR(int , int ) const;
   double CpVZVZconjVWmVWm1() const;
   double CpVZVZconjVWmVWm2() const;
   double CpVZVZconjVWmVWm3() const;
   double CpconjVWmbargPgWm() const;
   double CpconjVWmbargWmCgP() const;
   double CpconjVWmbargWmCgZ() const;
   double CpconjVWmbargZgWm() const;
   double CpVWmconjVWmconjSveLSveL() const;
   double CpVWmconjVWmconjSvmLSvmL() const;
   double CpVWmconjVWmconjSvtLSvtL() const;
   double CpconjVWmVWmVP() const;
   double CpconjVWmVZVWm() const;
   double CpconjVWmbarFveFePL() const;
   double CpconjVWmbarFveFePR() const;
   double CpconjVWmbarFvmFmPL() const;
   double CpconjVWmbarFvmFmPR() const;
   double CpconjVWmbarFvtFtauPL() const;
   double CpconjVWmbarFvtFtauPR() const;
   std::complex<double> CpVWmconjVWmAhAh(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmconjHpmHpm(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmconjSbSb(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmconjScSc(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmconjSdSd(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmconjSeSe(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmconjSmSm(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmconjSsSs(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmconjStSt(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmconjStauStau(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmconjSuSu(int gI1, int gI2) const;
   std::complex<double> CpVWmconjVWmhhhh(int gI1, int gI2) const;
   std::complex<double> CpconjVWmconjScSs(int gI1, int gI2) const;
   std::complex<double> CpconjVWmconjStSb(int gI1, int gI2) const;
   std::complex<double> CpconjVWmconjSuSd(int gI1, int gI2) const;
   std::complex<double> CpconjVWmHpmAh(int gI1, int gI2) const;
   std::complex<double> CpconjVWmHpmhh(int gI1, int gI2) const;
   std::complex<double> CpconjVWmChiChaPL(int gI1, int gI2) const;
   std::complex<double> CpconjVWmChiChaPR(int gI1, int gI2) const;
   std::complex<double> CpconjVWmVPHpm(int gI2) const;
   std::complex<double> CpconjVWmVWmhh(int gI2) const;
   std::complex<double> CpconjVWmVZHpm(int gI2) const;
   std::complex<double> CpconjVWmconjSveLSe(int gI2) const;
   std::complex<double> CpconjVWmconjSvmLSm(int gI2) const;
   std::complex<double> CpconjVWmconjSvtLStau(int gI2) const;
   double CpconjVWmbarFcFsPL(int , int ) const;
   double CpconjVWmbarFcFsPR(int , int ) const;
   double CpconjVWmbarFtFbPL(int , int ) const;
   double CpconjVWmbarFtFbPR(int , int ) const;
   double CpconjVWmbarFuFdPL(int , int ) const;
   double CpconjVWmbarFuFdPR(int , int ) const;
   double CpVWmconjVWmVPVP1() const;
   double CpVWmconjVWmVPVP2() const;
   double CpVWmconjVWmVPVP3() const;
   double CpVWmconjVWmVZVZ1() const;
   double CpVWmconjVWmVZVZ2() const;
   double CpVWmconjVWmVZVZ3() const;
   double CpVWmconjVWmconjVWmVWm1() const;
   double CpVWmconjVWmconjVWmVWm2() const;
   double CpVWmconjVWmconjVWmVWm3() const;
   std::complex<double> CpUChibarChaHpmPL(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUChibarChaHpmPR(int gO1, int gI1, int gI2) const;
   std::complex<double> CpUChiconjHpmChaPL(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUChiconjHpmChaPR(int gO1, int gI1, int gI2) const;
   std::complex<double> CpUChihhChiPL(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUChihhChiPR(int gO1, int gI1, int gI2) const;
   std::complex<double> CpUChibarChaVWmPR(int gO2, int gI1) const;
   std::complex<double> CpUChibarChaVWmPL(int gO1, int gI1) const;
   std::complex<double> CpUChiconjSbFbPL(int gO2, int gI1, int ) const;
   std::complex<double> CpUChiconjSbFbPR(int gO1, int gI1, int ) const;
   std::complex<double> CpUChiconjScFcPL(int gO2, int gI1, int ) const;
   std::complex<double> CpUChiconjScFcPR(int gO1, int gI1, int ) const;
   std::complex<double> CpUChiconjSdFdPL(int gO2, int gI1, int ) const;
   std::complex<double> CpUChiconjSdFdPR(int gO1, int gI1, int ) const;
   std::complex<double> CpUChiconjSeFePL(int gO2, int gI1) const;
   std::complex<double> CpUChiconjSeFePR(int gO1, int gI1) const;
   std::complex<double> CpUChiconjSmFmPL(int gO2, int gI1) const;
   std::complex<double> CpUChiconjSmFmPR(int gO1, int gI1) const;
   std::complex<double> CpUChiconjSsFsPL(int gO2, int gI1, int ) const;
   std::complex<double> CpUChiconjSsFsPR(int gO1, int gI1, int ) const;
   std::complex<double> CpUChiconjStFtPL(int gO2, int gI1, int ) const;
   std::complex<double> CpUChiconjStFtPR(int gO1, int gI1, int ) const;
   std::complex<double> CpUChiconjStauFtauPL(int gO2, int gI1) const;
   std::complex<double> CpUChiconjStauFtauPR(int gO1, int gI1) const;
   std::complex<double> CpUChiconjSuFuPL(int gO2, int gI1, int ) const;
   std::complex<double> CpUChiconjSuFuPR(int gO1, int gI1, int ) const;
   std::complex<double> CpUChiChiAhPL(int gO2, int gI1, int gI2) const;
   std::complex<double> CpUChiChiAhPR(int gO1, int gI1, int gI2) const;
   std::complex<double> CpUChibarFeSePL(int gO2, int gI2) const;
   std::complex<double> CpUChibarFeSePR(int gO1, int gI2) const;
   std::complex<double> CpUChibarFmSmPL(int gO2, int gI2) const;
   std::complex<double> CpUChibarFmSmPR(int gO1, int gI2) const;
   std::complex<double> CpUChibarFtauStauPL(int gO2, int gI2) const;
   std::complex<double> CpUChibarFtauStauPR(int gO1, int gI2) const;
   std::complex<double> CpUChibarFbSbPL(int gO2, int , int gI2) const;
   std::complex<double> CpUChibarFbSbPR(int gO1, int , int gI2) const;
   std::complex<double> CpUChibarFcScPL(int gO2, int , int gI2) const;
   std::complex<double> CpUChibarFcScPR(int gO1, int , int gI2) const;
   std::complex<double> CpUChibarFdSdPL(int gO2, int , int gI2) const;
   std::complex<double> CpUChibarFdSdPR(int gO1, int , int gI2) const;
   std::complex<double> CpUChibarFsSsPL(int gO2, int , int gI2) const;
   std::complex<double> CpUChibarFsSsPR(int gO1, int , int gI2) const;
   std::complex<double> CpUChibarFtStPL(int gO2, int , int gI2) const;
   std::complex<double> CpUChibarFtStPR(int gO1, int , int gI2) const;
   std::complex<double> CpUChibarFuSuPL(int gO2, int , int gI2) const;
   std::complex<double> CpUChibarFuSuPR(int gO1, int , int gI2) const;
   std::complex<double> CpUChiconjVWmChaPR(int gO2, int gI2) const;
   std::complex<double> CpUChiconjVWmChaPL(int gO1, int gI2) const;
   std::complex<double> CpUChiVZChiPR(int gO2, int gI2) const;
   std::complex<double> CpUChiVZChiPL(int gO1, int gI2) const;
   double CpUChibarFveSveLPL(int ) const;
   std::complex<double> CpUChibarFveSveLPR(int gO1) const;
   double CpUChibarFvmSvmLPL(int ) const;
   std::complex<double> CpUChibarFvmSvmLPR(int gO1) const;
   double CpUChibarFvtSvtLPL(int ) const;
   std::complex<double> CpUChibarFvtSvtLPR(int gO1) const;
   std::complex<double> CpUChiconjSveLFvePL(int gO2) const;
   double CpUChiconjSveLFvePR(int ) const;
   std::complex<double> CpUChiconjSvmLFvmPL(int gO2) const;
   double CpUChiconjSvmLFvmPR(int ) const;
   std::complex<double> CpUChiconjSvtLFvtPL(int gO2) const;
   double CpUChiconjSvtLFvtPR(int ) const;
   std::complex<double> CpbarUChaChaAhPL(int gO2, int gI1, int gI2) const;
   std::complex<double> CpbarUChaChaAhPR(int gO1, int gI1, int gI2) const;
   std::complex<double> CpbarUChahhChaPL(int gO2, int gI1, int gI2) const;
   std::complex<double> CpbarUChahhChaPR(int gO1, int gI1, int gI2) const;
   std::complex<double> CpbarUChaHpmChiPL(int gO2, int gI1, int gI2) const;
   std::complex<double> CpbarUChaHpmChiPR(int gO1, int gI1, int gI2) const;
   std::complex<double> CpbarUChaconjScFsPL(int gO2, int gI1, int ) const;
   std::complex<double> CpbarUChaconjScFsPR(int gO1, int gI1, int ) const;
   std::complex<double> CpbarUChaconjStFbPL(int gO2, int gI1, int ) const;
   std::complex<double> CpbarUChaconjStFbPR(int gO1, int gI1, int ) const;
   std::complex<double> CpbarUChaconjSuFdPL(int gO2, int gI1, int ) const;
   std::complex<double> CpbarUChaconjSuFdPR(int gO1, int gI1, int ) const;
   std::complex<double> CpbarUChaVPChaPR(int gO2, int gI2) const;
   std::complex<double> CpbarUChaVPChaPL(int gO1, int gI2) const;
   std::complex<double> CpbarUChaVZChaPR(int gO2, int gI2) const;
   std::complex<double> CpbarUChaVZChaPL(int gO1, int gI2) const;
   double CpbarUChabarFveSePL(int , int ) const;
   std::complex<double> CpbarUChabarFveSePR(int gO1, int gI2) const;
   double CpbarUChabarFvmSmPL(int , int ) const;
   std::complex<double> CpbarUChabarFvmSmPR(int gO1, int gI2) const;
   double CpbarUChabarFvtStauPL(int , int ) const;
   std::complex<double> CpbarUChabarFvtStauPR(int gO1, int gI2) const;
   std::complex<double> CpbarUChabarFcSsPL(int gO2, int , int gI2) const;
   std::complex<double> CpbarUChabarFcSsPR(int gO1, int , int gI2) const;
   std::complex<double> CpbarUChabarFtSbPL(int gO2, int , int gI2) const;
   std::complex<double> CpbarUChabarFtSbPR(int gO1, int , int gI2) const;
   std::complex<double> CpbarUChabarFuSdPL(int gO2, int , int gI2) const;
   std::complex<double> CpbarUChabarFuSdPR(int gO1, int , int gI2) const;
   std::complex<double> CpbarUChaVWmChiPR(int gO2, int gI2) const;
   std::complex<double> CpbarUChaVWmChiPL(int gO1, int gI2) const;
   double CpbarUChaconjSveLFePL(int gO2) const;
   double CpbarUChaconjSveLFePR(int gO1) const;
   double CpbarUChaconjSvmLFmPL(int gO2) const;
   double CpbarUChaconjSvmLFmPR(int gO1) const;
   double CpbarUChaconjSvtLFtauPL(int gO2) const;
   double CpbarUChaconjSvtLFtauPR(int gO1) const;
   std::complex<double> CpGluconjSbFbPL(int gI1, int ) const;
   std::complex<double> CpGluconjSbFbPR(int gI1, int ) const;
   std::complex<double> CpGluconjScFcPL(int gI1, int ) const;
   std::complex<double> CpGluconjScFcPR(int gI1, int ) const;
   std::complex<double> CpGluconjSdFdPL(int gI1, int ) const;
   std::complex<double> CpGluconjSdFdPR(int gI1, int ) const;
   std::complex<double> CpGluconjSsFsPL(int gI1, int ) const;
   std::complex<double> CpGluconjSsFsPR(int gI1, int ) const;
   std::complex<double> CpGluconjStFtPL(int gI1, int ) const;
   std::complex<double> CpGluconjStFtPR(int gI1, int ) const;
   std::complex<double> CpGluconjSuFuPL(int gI1, int ) const;
   std::complex<double> CpGluconjSuFuPR(int gI1, int ) const;
   std::complex<double> CpGlubarFbSbPL(int , int gI2) const;
   std::complex<double> CpGlubarFbSbPR(int , int gI2) const;
   std::complex<double> CpGlubarFcScPL(int , int gI2) const;
   std::complex<double> CpGlubarFcScPR(int , int gI2) const;
   std::complex<double> CpGlubarFdSdPL(int , int gI2) const;
   std::complex<double> CpGlubarFdSdPR(int , int gI2) const;
   std::complex<double> CpGlubarFsSsPL(int , int gI2) const;
   std::complex<double> CpGlubarFsSsPR(int , int gI2) const;
   std::complex<double> CpGlubarFtStPL(int , int gI2) const;
   std::complex<double> CpGlubarFtStPR(int , int gI2) const;
   std::complex<double> CpGlubarFuSuPL(int , int gI2) const;
   std::complex<double> CpGlubarFuSuPR(int , int gI2) const;
   std::complex<double> CpGluVGGluPR() const;
   std::complex<double> CpGluVGGluPL() const;
   std::complex<double> CpbarFdSuChaPL(int gI1, int gI2) const;
   std::complex<double> CpbarFdSuChaPR(int gI1, int gI2) const;
   std::complex<double> CpbarFdSdChiPL(int gI1, int gI2) const;
   std::complex<double> CpbarFdSdChiPR(int gI1, int gI2) const;
   std::complex<double> CpbarFdhhFdPL(int gI1) const;
   std::complex<double> CpbarFdhhFdPR(int gI1) const;
   std::complex<double> CpbarFdHpmFuPL(int gI1, int ) const;
   std::complex<double> CpbarFdHpmFuPR(int gI1, int ) const;
   std::complex<double> CpbarFdSdGluPL(int gI1, int ) const;
   std::complex<double> CpbarFdSdGluPR(int gI1, int ) const;
   std::complex<double> CpbarFdFdAhPL(int gI2) const;
   std::complex<double> CpbarFdFdAhPR(int gI2) const;
   double CpbarFdVGFdPR() const;
   double CpbarFdVGFdPL() const;
   double CpbarFdVPFdPR() const;
   double CpbarFdVPFdPL() const;
   double CpbarFdVWmFuPR(int ) const;
   double CpbarFdVWmFuPL(int ) const;
   double CpbarFdVZFdPR() const;
   double CpbarFdVZFdPL() const;
   std::complex<double> CpbarFsScChaPL(int gI1, int gI2) const;
   std::complex<double> CpbarFsScChaPR(int gI1, int gI2) const;
   std::complex<double> CpbarFsSsChiPL(int gI1, int gI2) const;
   std::complex<double> CpbarFsSsChiPR(int gI1, int gI2) const;
   std::complex<double> CpbarFshhFsPL(int gI1) const;
   std::complex<double> CpbarFshhFsPR(int gI1) const;
   std::complex<double> CpbarFsHpmFcPL(int gI1, int ) const;
   std::complex<double> CpbarFsHpmFcPR(int gI1, int ) const;
   std::complex<double> CpbarFsSsGluPL(int gI1, int ) const;
   std::complex<double> CpbarFsSsGluPR(int gI1, int ) const;
   std::complex<double> CpbarFsFsAhPL(int gI2) const;
   std::complex<double> CpbarFsFsAhPR(int gI2) const;
   double CpbarFsVGFsPR() const;
   double CpbarFsVGFsPL() const;
   double CpbarFsVPFsPR() const;
   double CpbarFsVPFsPL() const;
   double CpbarFsVWmFcPR(int ) const;
   double CpbarFsVWmFcPL(int ) const;
   double CpbarFsVZFsPR() const;
   double CpbarFsVZFsPL() const;
   std::complex<double> CpbarFbStChaPL(int gI1, int gI2) const;
   std::complex<double> CpbarFbStChaPR(int gI1, int gI2) const;
   std::complex<double> CpbarFbSbChiPL(int gI1, int gI2) const;
   std::complex<double> CpbarFbSbChiPR(int gI1, int gI2) const;
   std::complex<double> CpbarFbhhFbPL(int gI1) const;
   std::complex<double> CpbarFbhhFbPR(int gI1) const;
   std::complex<double> CpbarFbHpmFtPL(int gI1, int ) const;
   std::complex<double> CpbarFbHpmFtPR(int gI1, int ) const;
   std::complex<double> CpbarFbSbGluPL(int gI1, int ) const;
   std::complex<double> CpbarFbSbGluPR(int gI1, int ) const;
   std::complex<double> CpbarFbFbAhPL(int gI2) const;
   std::complex<double> CpbarFbFbAhPR(int gI2) const;
   double CpbarFbVGFbPR() const;
   double CpbarFbVGFbPL() const;
   double CpbarFbVPFbPR() const;
   double CpbarFbVPFbPL() const;
   double CpbarFbVWmFtPR(int ) const;
   double CpbarFbVWmFtPL(int ) const;
   double CpbarFbVZFbPR() const;
   double CpbarFbVZFbPL() const;
   std::complex<double> CpbarFubarChaSdPL(int gI1, int gI2) const;
   std::complex<double> CpbarFubarChaSdPR(int gI1, int gI2) const;
   std::complex<double> CpbarFuSuChiPL(int gI1, int gI2) const;
   std::complex<double> CpbarFuSuChiPR(int gI1, int gI2) const;
   std::complex<double> CpbarFuconjHpmFdPL(int gI1, int ) const;
   std::complex<double> CpbarFuconjHpmFdPR(int gI1, int ) const;
   std::complex<double> CpbarFuhhFuPL(int gI1) const;
   std::complex<double> CpbarFuhhFuPR(int gI1) const;
   std::complex<double> CpbarFuSuGluPL(int gI1, int ) const;
   std::complex<double> CpbarFuSuGluPR(int gI1, int ) const;
   std::complex<double> CpbarFuFuAhPL(int gI2) const;
   std::complex<double> CpbarFuFuAhPR(int gI2) const;
   double CpbarFuVGFuPR() const;
   double CpbarFuVGFuPL() const;
   double CpbarFuVPFuPR() const;
   double CpbarFuVPFuPL() const;
   double CpbarFuVZFuPR() const;
   double CpbarFuVZFuPL() const;
   double CpbarFuconjVWmFdPR(int ) const;
   double CpbarFuconjVWmFdPL(int ) const;
   std::complex<double> CpbarFcbarChaSsPL(int gI1, int gI2) const;
   std::complex<double> CpbarFcbarChaSsPR(int gI1, int gI2) const;
   std::complex<double> CpbarFcScChiPL(int gI1, int gI2) const;
   std::complex<double> CpbarFcScChiPR(int gI1, int gI2) const;
   std::complex<double> CpbarFcconjHpmFsPL(int gI1, int ) const;
   std::complex<double> CpbarFcconjHpmFsPR(int gI1, int ) const;
   std::complex<double> CpbarFchhFcPL(int gI1) const;
   std::complex<double> CpbarFchhFcPR(int gI1) const;
   std::complex<double> CpbarFcScGluPL(int gI1, int ) const;
   std::complex<double> CpbarFcScGluPR(int gI1, int ) const;
   std::complex<double> CpbarFcFcAhPL(int gI2) const;
   std::complex<double> CpbarFcFcAhPR(int gI2) const;
   double CpbarFcVGFcPR() const;
   double CpbarFcVGFcPL() const;
   double CpbarFcVPFcPR() const;
   double CpbarFcVPFcPL() const;
   double CpbarFcVZFcPR() const;
   double CpbarFcVZFcPL() const;
   double CpbarFcconjVWmFsPR(int ) const;
   double CpbarFcconjVWmFsPL(int ) const;
   std::complex<double> CpbarFtbarChaSbPL(int gI1, int gI2) const;
   std::complex<double> CpbarFtbarChaSbPR(int gI1, int gI2) const;
   std::complex<double> CpbarFtStChiPL(int gI1, int gI2) const;
   std::complex<double> CpbarFtStChiPR(int gI1, int gI2) const;
   std::complex<double> CpbarFtconjHpmFbPL(int gI1, int ) const;
   std::complex<double> CpbarFtconjHpmFbPR(int gI1, int ) const;
   std::complex<double> CpbarFthhFtPL(int gI1) const;
   std::complex<double> CpbarFthhFtPR(int gI1) const;
   std::complex<double> CpbarFtStGluPL(int gI1, int ) const;
   std::complex<double> CpbarFtStGluPR(int gI1, int ) const;
   std::complex<double> CpbarFtFtAhPL(int gI2) const;
   std::complex<double> CpbarFtFtAhPR(int gI2) const;
   double CpbarFtVGFtPR() const;
   double CpbarFtVGFtPL() const;
   double CpbarFtVPFtPR() const;
   double CpbarFtVPFtPL() const;
   double CpbarFtVZFtPR() const;
   double CpbarFtVZFtPL() const;
   double CpbarFtconjVWmFbPR(int ) const;
   double CpbarFtconjVWmFbPL(int ) const;
   double CpbarFvebarChaSePL(int , int ) const;
   std::complex<double> CpbarFvebarChaSePR(int gI1, int gI2) const;
   double CpbarFveconjHpmFePL(int ) const;
   std::complex<double> CpbarFveconjHpmFePR(int gI1) const;
   double CpbarFveSveLChiPL(int ) const;
   std::complex<double> CpbarFveSveLChiPR(int gI2) const;
   double CpbarFveVZFvePR() const;
   double CpbarFveVZFvePL() const;
   double CpbarFveconjVWmFePR() const;
   double CpbarFveconjVWmFePL() const;
   double CpbarFvmbarChaSmPL(int , int ) const;
   std::complex<double> CpbarFvmbarChaSmPR(int gI1, int gI2) const;
   double CpbarFvmconjHpmFmPL(int ) const;
   std::complex<double> CpbarFvmconjHpmFmPR(int gI1) const;
   double CpbarFvmSvmLChiPL(int ) const;
   std::complex<double> CpbarFvmSvmLChiPR(int gI2) const;
   double CpbarFvmVZFvmPR() const;
   double CpbarFvmVZFvmPL() const;
   double CpbarFvmconjVWmFmPR() const;
   double CpbarFvmconjVWmFmPL() const;
   double CpbarFvtbarChaStauPL(int , int ) const;
   std::complex<double> CpbarFvtbarChaStauPR(int gI1, int gI2) const;
   double CpbarFvtconjHpmFtauPL(int ) const;
   std::complex<double> CpbarFvtconjHpmFtauPR(int gI1) const;
   double CpbarFvtSvtLChiPL(int ) const;
   std::complex<double> CpbarFvtSvtLChiPR(int gI2) const;
   double CpbarFvtVZFvtPR() const;
   double CpbarFvtVZFvtPL() const;
   double CpbarFvtconjVWmFtauPR() const;
   double CpbarFvtconjVWmFtauPL() const;
   std::complex<double> CpbarFeSeChiPL(int gI1, int gI2) const;
   std::complex<double> CpbarFeSeChiPR(int gI1, int gI2) const;
   std::complex<double> CpbarFehhFePL(int gI1) const;
   std::complex<double> CpbarFehhFePR(int gI1) const;
   std::complex<double> CpbarFeHpmFvePL(int gI1) const;
   double CpbarFeHpmFvePR(int ) const;
   std::complex<double> CpbarFeFeAhPL(int gI2) const;
   std::complex<double> CpbarFeFeAhPR(int gI2) const;
   std::complex<double> CpbarFeSveLChaPL(int gI2) const;
   std::complex<double> CpbarFeSveLChaPR(int gI2) const;
   double CpbarFeVPFePR() const;
   double CpbarFeVPFePL() const;
   double CpbarFeVWmFvePR() const;
   double CpbarFeVWmFvePL() const;
   double CpbarFeVZFePR() const;
   double CpbarFeVZFePL() const;
   std::complex<double> CpbarFmSmChiPL(int gI1, int gI2) const;
   std::complex<double> CpbarFmSmChiPR(int gI1, int gI2) const;
   std::complex<double> CpbarFmhhFmPL(int gI1) const;
   std::complex<double> CpbarFmhhFmPR(int gI1) const;
   std::complex<double> CpbarFmHpmFvmPL(int gI1) const;
   double CpbarFmHpmFvmPR(int ) const;
   std::complex<double> CpbarFmFmAhPL(int gI2) const;
   std::complex<double> CpbarFmFmAhPR(int gI2) const;
   std::complex<double> CpbarFmSvmLChaPL(int gI2) const;
   std::complex<double> CpbarFmSvmLChaPR(int gI2) const;
   double CpbarFmVPFmPR() const;
   double CpbarFmVPFmPL() const;
   double CpbarFmVWmFvmPR() const;
   double CpbarFmVWmFvmPL() const;
   double CpbarFmVZFmPR() const;
   double CpbarFmVZFmPL() const;
   std::complex<double> CpbarFtauStauChiPL(int gI1, int gI2) const;
   std::complex<double> CpbarFtauStauChiPR(int gI1, int gI2) const;
   std::complex<double> CpbarFtauhhFtauPL(int gI1) const;
   std::complex<double> CpbarFtauhhFtauPR(int gI1) const;
   std::complex<double> CpbarFtauHpmFvtPL(int gI1) const;
   double CpbarFtauHpmFvtPR(int ) const;
   std::complex<double> CpbarFtauFtauAhPL(int gI2) const;
   std::complex<double> CpbarFtauFtauAhPR(int gI2) const;
   std::complex<double> CpbarFtauSvtLChaPL(int gI2) const;
   std::complex<double> CpbarFtauSvtLChaPR(int gI2) const;
   double CpbarFtauVPFtauPR() const;
   double CpbarFtauVPFtauPL() const;
   double CpbarFtauVWmFvtPR() const;
   double CpbarFtauVWmFvtPL() const;
   double CpbarFtauVZFtauPR() const;
   double CpbarFtauVZFtauPL() const;
   std::complex<double> self_energy_Sd_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Sd_1loop(double p) const;
   std::complex<double> self_energy_Su_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Su_1loop(double p) const;
   std::complex<double> self_energy_Se_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Se_1loop(double p) const;
   std::complex<double> self_energy_Sm_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Sm_1loop(double p) const;
   std::complex<double> self_energy_Stau_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Stau_1loop(double p) const;
   std::complex<double> self_energy_Ss_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Ss_1loop(double p) const;
   std::complex<double> self_energy_Sc_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Sc_1loop(double p) const;
   std::complex<double> self_energy_Sb_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Sb_1loop(double p) const;
   std::complex<double> self_energy_St_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_St_1loop(double p) const;
   std::complex<double> self_energy_hh_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_hh_1loop(double p) const;
   std::complex<double> self_energy_Ah_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Ah_1loop(double p) const;
   std::complex<double> self_energy_Hpm_1loop(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Hpm_1loop(double p) const;
   std::complex<double> self_energy_SveL_1loop(double p ) const;
   std::complex<double> self_energy_SvmL_1loop(double p ) const;
   std::complex<double> self_energy_SvtL_1loop(double p ) const;
   std::complex<double> self_energy_VG_1loop(double p ) const;
   std::complex<double> self_energy_VP_1loop(double p ) const;
   std::complex<double> self_energy_VZ_1loop(double p ) const;
   std::complex<double> self_energy_VWm_1loop(double p ) const;
   std::complex<double> self_energy_Chi_1loop_1(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,4,4> self_energy_Chi_1loop_1(double p) const;
   std::complex<double> self_energy_Chi_1loop_PR(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,4,4> self_energy_Chi_1loop_PR(double p) const;
   std::complex<double> self_energy_Chi_1loop_PL(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,4,4> self_energy_Chi_1loop_PL(double p) const;
   std::complex<double> self_energy_Cha_1loop_1(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Cha_1loop_1(double p) const;
   std::complex<double> self_energy_Cha_1loop_PR(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Cha_1loop_PR(double p) const;
   std::complex<double> self_energy_Cha_1loop_PL(double p , int gO1, int gO2) const;
   Eigen::Matrix<std::complex<double>,2,2> self_energy_Cha_1loop_PL(double p) const;
   std::complex<double> self_energy_Glu_1loop_1(double p ) const;
   std::complex<double> self_energy_Glu_1loop_PR(double p ) const;
   std::complex<double> self_energy_Glu_1loop_PL(double p ) const;
   std::complex<double> self_energy_Fd_1loop_1(double p ) const;
   std::complex<double> self_energy_Fd_1loop_PR(double p ) const;
   std::complex<double> self_energy_Fd_1loop_PL(double p ) const;
   std::complex<double> self_energy_Fs_1loop_1(double p ) const;
   std::complex<double> self_energy_Fs_1loop_PR(double p ) const;
   std::complex<double> self_energy_Fs_1loop_PL(double p ) const;
   std::complex<double> self_energy_Fb_1loop_1(double p ) const;
   std::complex<double> self_energy_Fb_1loop_PR(double p ) const;
   std::complex<double> self_energy_Fb_1loop_PL(double p ) const;
   std::complex<double> self_energy_Fu_1loop_1(double p ) const;
   std::complex<double> self_energy_Fu_1loop_PR(double p ) const;
   std::complex<double> self_energy_Fu_1loop_PL(double p ) const;
   std::complex<double> self_energy_Fc_1loop_1(double p ) const;
   std::complex<double> self_energy_Fc_1loop_PR(double p ) const;
   std::complex<double> self_energy_Fc_1loop_PL(double p ) const;
   std::complex<double> self_energy_Ft_1loop_1(double p ) const;
   std::complex<double> self_energy_Ft_1loop_PR(double p ) const;
   std::complex<double> self_energy_Ft_1loop_PL(double p ) const;
   std::complex<double> self_energy_Fve_1loop_1(double p ) const;
   std::complex<double> self_energy_Fve_1loop_PR(double p ) const;
   std::complex<double> self_energy_Fve_1loop_PL(double p ) const;
   std::complex<double> self_energy_Fvm_1loop_1(double p ) const;
   std::complex<double> self_energy_Fvm_1loop_PR(double p ) const;
   std::complex<double> self_energy_Fvm_1loop_PL(double p ) const;
   std::complex<double> self_energy_Fvt_1loop_1(double p ) const;
   std::complex<double> self_energy_Fvt_1loop_PR(double p ) const;
   std::complex<double> self_energy_Fvt_1loop_PL(double p ) const;
   std::complex<double> self_energy_Fe_1loop_1(double p ) const;
   std::complex<double> self_energy_Fe_1loop_PR(double p ) const;
   std::complex<double> self_energy_Fe_1loop_PL(double p ) const;
   std::complex<double> self_energy_Fm_1loop_1(double p ) const;
   std::complex<double> self_energy_Fm_1loop_PR(double p ) const;
   std::complex<double> self_energy_Fm_1loop_PL(double p ) const;
   std::complex<double> self_energy_Ftau_1loop_1(double p ) const;
   std::complex<double> self_energy_Ftau_1loop_PR(double p ) const;
   std::complex<double> self_energy_Ftau_1loop_PL(double p ) const;
   std::complex<double> self_energy_VZ_1loop_heavy(double p ) const;
   std::complex<double> self_energy_VWm_1loop_heavy(double p ) const;
   std::complex<double> self_energy_Fb_1loop_1_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Fb_1loop_PR_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Fb_1loop_PL_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Fe_1loop_1_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Fe_1loop_PR_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Fe_1loop_PL_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Fm_1loop_1_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Fm_1loop_PR_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Fm_1loop_PL_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Ftau_1loop_1_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Ftau_1loop_PR_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Ftau_1loop_PL_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Ft_1loop_1_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Ft_1loop_PR_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Ft_1loop_PL_heavy_rotated(double p ) const;
   std::complex<double> self_energy_Ft_1loop_1_heavy(double p ) const;
   std::complex<double> self_energy_Ft_1loop_PR_heavy(double p ) const;
   std::complex<double> self_energy_Ft_1loop_PL_heavy(double p ) const;
   std::complex<double> tadpole_hh_1loop(int gO1) const;


   using EWSB_vector_t = Eigen::Matrix<double,number_of_ewsb_equations,1>;

   /// calculates the tadpoles at current loop order
   void tadpole_equations(double[number_of_ewsb_equations]) const;
   /// calculates the tadpoles at current loop order
   EWSB_vector_t tadpole_equations() const;
   /// calculates the tadpoles divided by VEVs at current loop order
   EWSB_vector_t tadpole_equations_over_vevs() const;

   void calculate_MTopSquark_3rd_generation(double&, double&, double&) const;
   void calculate_MBottomSquark_3rd_generation(double&, double&, double&) const;
   void calculate_MSneutrino_3rd_generation(double&, double&, double&) const;
   void calculate_MSelectron_3rd_generation(double&, double&, double&) const;

   Eigen::Matrix<double,2,2> self_energy_hh_2loop() const;
   Eigen::Matrix<double,2,2> self_energy_Ah_2loop() const;

   Eigen::Matrix<double,2,1> tadpole_hh_2loop() const;

   Eigen::Matrix<double,2,2> self_energy_hh_3loop() const;

   void calculate_MVG_pole();
   void calculate_MGlu_pole();
   void calculate_MVP_pole();
   void calculate_MVZ_pole();
   void calculate_MFd_pole();
   void calculate_MFs_pole();
   void calculate_MFb_pole();
   void calculate_MFu_pole();
   void calculate_MFc_pole();
   void calculate_MFt_pole();
   void calculate_MFve_pole();
   void calculate_MFvm_pole();
   void calculate_MFvt_pole();
   void calculate_MFe_pole();
   void calculate_MFm_pole();
   void calculate_MFtau_pole();
   void calculate_MSveL_pole();
   void calculate_MSvmL_pole();
   void calculate_MSvtL_pole();
   void calculate_MSd_pole();
   void calculate_MSu_pole();
   void calculate_MSe_pole();
   void calculate_MSm_pole();
   void calculate_MStau_pole();
   void calculate_MSs_pole();
   void calculate_MSc_pole();
   void calculate_MSb_pole();
   void calculate_MSt_pole();
   void calculate_Mhh_pole();
   void calculate_MAh_pole();
   void calculate_MHpm_pole();
   void calculate_MChi_pole();
   void calculate_MCha_pole();
   void calculate_MVWm_pole();
   double calculate_MVWm_pole(double);
   double calculate_MVZ_pole(double);

   double calculate_MFve_DRbar(double) const;
   double calculate_MFvm_DRbar(double) const;
   double calculate_MFvt_DRbar(double) const;
   double calculate_MFe_DRbar(double) const;
   double calculate_MFm_DRbar(double) const;
   double calculate_MFtau_DRbar(double) const;
   double calculate_MFu_DRbar(double) const;
   double calculate_MFc_DRbar(double) const;
   double calculate_MFt_DRbar(double) const;
   double calculate_MFd_DRbar(double) const;
   double calculate_MFs_DRbar(double) const;
   double calculate_MFb_DRbar(double) const;
   double calculate_MVP_DRbar(double);
   double calculate_MVZ_DRbar(double);
   double calculate_MVWm_DRbar(double);

   double v() const;
   double Betax() const;
   double Alpha() const;
   double ThetaW() const;


private:
   class EEWSBStepFailed : public Error {
   public:
      virtual ~EEWSBStepFailed() = default;
      virtual std::string what() const override { return "Could not perform EWSB step."; }
   };

   int ewsb_loop_order;      ///< loop order for EWSB
   int pole_mass_loop_order; ///< loop order for pole masses
   bool calculate_sm_pole_masses; ///< switch to calculate the pole masses of the Standard Model particles
   bool calculate_bsm_pole_masses;///< switch to calculate the pole masses of the BSM particles
   bool force_output;             ///< switch to force output of pole masses
   double precision;              ///< RG running precision
   double ewsb_iteration_precision; ///< precision goal of EWSB solution
   NUHMSSMNoFVHimalaya_physical physical; ///< contains the pole masses and mixings
   Problems<NUHMSSMNoFVHimalaya_info::NUMBER_OF_PARTICLES, NUHMSSMNoFVHimalaya_info::NUMBER_OF_PARAMETERS> problems; ///< problems
   Loop_corrections loop_corrections; ///< used pole mass corrections
   Threshold_corrections threshold_corrections; ///< used threshold corrections

   int get_number_of_ewsb_iterations() const;
   int get_number_of_mass_iterations() const;
   void set_ewsb_solution(const EWSB_solver*);
   template <typename It> void set_best_ewsb_solution(It, It);
   int solve_ewsb_iteratively();
   int solve_ewsb_iteratively(int);
   int solve_ewsb_iteratively_with(EWSB_solver*, const EWSB_vector_t&);
   int solve_ewsb_tree_level_custom();
   EWSB_vector_t ewsb_initial_guess();
   EWSB_vector_t ewsb_step() const;
   void copy_DRbar_masses_to_pole_masses();

   // Passarino-Veltman loop functions
   double A0(double) const;
   double B0(double, double, double) const;
   double B1(double, double, double) const;
   double B00(double, double, double) const;
   double B22(double, double, double) const;
   double H0(double, double, double) const;
   double F0(double, double, double) const;
   double G0(double, double, double) const;

   // DR-bar masses
   double MVG{};
   double MGlu{};
   double MFd{};
   double MFs{};
   double MFb{};
   double MFu{};
   double MFc{};
   double MFt{};
   double MFve{};
   double MFvm{};
   double MFvt{};
   double MFe{};
   double MFm{};
   double MFtau{};
   double MSveL{};
   double MSvmL{};
   double MSvtL{};
   Eigen::Array<double,2,1> MSd{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> MSu{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> MSe{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> MSm{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> MStau{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> MSs{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> MSc{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> MSb{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> MSt{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> Mhh{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> MAh{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,2,1> MHpm{Eigen::Array<double,2,1>::Zero()};
   Eigen::Array<double,4,1> MChi{Eigen::Array<double,4,1>::Zero()};
   Eigen::Array<double,2,1> MCha{Eigen::Array<double,2,1>::Zero()};
   double MVWm{};
   double MVP{};
   double MVZ{};

   // DR-bar mixing matrices
   Eigen::Matrix<double,2,2> ZD{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZU{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZE{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZM{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZTau{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZS{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZC{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZB{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZT{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZH{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZA{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZP{Eigen::Matrix<double,2,2>::Zero()};
   Eigen::Matrix<std::complex<double>,4,4> ZN{Eigen::Matrix<std::complex<double>,4,4>::Zero()};
   Eigen::Matrix<std::complex<double>,2,2> UM{Eigen::Matrix<std::complex<double>,2,2>::Zero()};
   Eigen::Matrix<std::complex<double>,2,2> UP{Eigen::Matrix<std::complex<double>,2,2>::Zero()};
   Eigen::Matrix<double,2,2> ZZ{Eigen::Matrix<double,2,2>::Zero()};

   // phases
   std::complex<double> PhaseGlu{1.,0.};

   // extra parameters

};

std::ostream& operator<<(std::ostream&, const NUHMSSMNoFVHimalaya_mass_eigenstates&);

} // namespace flexiblesusy

#endif
