// ====================================================================
// This file is part of FlexibleSUSY.
//
// FlexibleSUSY is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License,
// or (at your option) any later version.
//
// FlexibleSUSY is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with FlexibleSUSY.  If not, see
// <http://www.gnu.org/licenses/>.
// ====================================================================

// File generated at Thu 17 Aug 2017 11:21:06

/**
 * @file NUHMSSMNoFVHimalaya_mass_eigenstates.cpp
 * @brief implementation of the NUHMSSMNoFVHimalaya model class
 *
 * Contains the definition of the NUHMSSMNoFVHimalaya model class methods
 * which solve EWSB and calculate pole masses and mixings from DRbar
 * parameters.
 *
 * This file was generated at Thu 17 Aug 2017 11:21:06 with FlexibleSUSY
 * 1.7.4-Himalaya (git commit: 20b32b788451d6463766eef390bf6d35c0983e74) and SARAH 4.12.0 .
 */

#include "NUHMSSMNoFVHimalaya_mass_eigenstates.hpp"
#include "eigen_utils.hpp"
#include "wrappers.hpp"
#include "linalg2.hpp"
#include "numerics2.hpp"
#include "logger.hpp"
#include "error.hpp"
#include "root_finder.hpp"
#include "fixed_point_iterator.hpp"
#include "config.h"
#include "pv.hpp"
#include "raii.hpp"
#include "thread_pool.hpp"
#include "functors.hpp"

#include "sfermions.hpp"
#include "mssm_twoloophiggs.hpp"

#ifdef ENABLE_HIMALAYA
#include "HierarchyCalculator.hpp"
#endif

#include "mssm_twoloop_mb.hpp"
#include "mssm_twoloop_mt.hpp"

#include <array>
#include <cmath>
#include <functional>
#include <iostream>
#include <memory>
#include <algorithm>
#include <stdexcept>

namespace flexiblesusy {

#define CLASSNAME NUHMSSMNoFVHimalaya_mass_eigenstates

#define PHYSICAL(parameter) physical.parameter
#define INPUT(parameter) model.get_input().parameter
#define LOCALINPUT(parameter) input.parameter
#define MODELPARAMETER(parameter) model.get_##parameter()
#define EXTRAPARAMETER(parameter) model.get_##parameter()

#define HIGGS_2LOOP_CORRECTION_AT_AS     loop_corrections.higgs_at_as
#define HIGGS_2LOOP_CORRECTION_AB_AS     loop_corrections.higgs_ab_as
#define HIGGS_2LOOP_CORRECTION_AT_AT     loop_corrections.higgs_at_at
#define HIGGS_2LOOP_CORRECTION_ATAU_ATAU loop_corrections.higgs_atau_atau
#define TOP_POLE_QCD_CORRECTION          loop_corrections.top_qcd
#define HIGGS_3LOOP_CORRECTION_AT_AS_AS  loop_corrections.higgs_at_as_as
#define HIGGS_3LOOP_CORRECTION_AB_AS_AS  loop_corrections.higgs_ab_as_as
#define HIGGS_3LOOP_MDR_SCHEME           loop_corrections.higgs_3L_mdr_scheme

CLASSNAME::NUHMSSMNoFVHimalaya_mass_eigenstates(const NUHMSSMNoFVHimalaya_input_parameters& input_)
   : NUHMSSMNoFVHimalaya_soft_parameters(input_)
   , ewsb_loop_order(2)
   , pole_mass_loop_order(2)
   , calculate_sm_pole_masses(false)
   , calculate_bsm_pole_masses(true)
   , force_output(false)
   , precision(1.0e-3)
   , ewsb_iteration_precision(1.0e-5)
   , physical()
   , problems(NUHMSSMNoFVHimalaya_info::particle_names, NUHMSSMNoFVHimalaya_info::parameter_names)
   , loop_corrections()
{
}

void CLASSNAME::do_calculate_sm_pole_masses(bool flag)
{
   calculate_sm_pole_masses = flag;
}

bool CLASSNAME::do_calculate_sm_pole_masses() const
{
   return calculate_sm_pole_masses;
}

void CLASSNAME::do_calculate_bsm_pole_masses(bool flag)
{
   calculate_bsm_pole_masses = flag;
}

bool CLASSNAME::do_calculate_bsm_pole_masses() const
{
   return calculate_bsm_pole_masses;
}

void CLASSNAME::do_force_output(bool flag)
{
   force_output = flag;
}

bool CLASSNAME::do_force_output() const
{
   return force_output;
}

void CLASSNAME::set_ewsb_loop_order(int loop_order)
{
   ewsb_loop_order = loop_order;
}

void CLASSNAME::set_loop_corrections(const Loop_corrections& loop_corrections_)
{
   loop_corrections = loop_corrections_;
}

const Loop_corrections& CLASSNAME::get_loop_corrections() const
{
   return loop_corrections;
}

void CLASSNAME::set_threshold_corrections(const Threshold_corrections& tc)
{
   threshold_corrections = tc;
}

const Threshold_corrections& CLASSNAME::get_threshold_corrections() const
{
   return threshold_corrections;
}

int CLASSNAME::get_number_of_ewsb_iterations() const
{
   return static_cast<int>(std::abs(-log10(ewsb_iteration_precision) * 10));
}

int CLASSNAME::get_number_of_mass_iterations() const
{
   return static_cast<int>(std::abs(-log10(precision) * 10));
}

void CLASSNAME::set_precision(double precision_)
{
   precision = precision_;
   ewsb_iteration_precision = precision_;
}

void CLASSNAME::set_pole_mass_loop_order(int loop_order)
{
   pole_mass_loop_order = loop_order;
}

int CLASSNAME::get_pole_mass_loop_order() const
{
   return pole_mass_loop_order;
}

void CLASSNAME::set_ewsb_iteration_precision(double precision)
{
   ewsb_iteration_precision = precision;
}

double CLASSNAME::get_ewsb_iteration_precision() const
{
   return ewsb_iteration_precision;
}

double CLASSNAME::get_precision() const
{
   return precision;
}

double CLASSNAME::get_ewsb_loop_order() const
{
   return ewsb_loop_order;
}

const NUHMSSMNoFVHimalaya_physical& CLASSNAME::get_physical() const
{
   return physical;
}

NUHMSSMNoFVHimalaya_physical& CLASSNAME::get_physical()
{
   return physical;
}

void CLASSNAME::set_physical(const NUHMSSMNoFVHimalaya_physical& physical_)
{
   physical = physical_;
}

const Problems<NUHMSSMNoFVHimalaya_info::NUMBER_OF_PARTICLES, NUHMSSMNoFVHimalaya_info::NUMBER_OF_PARAMETERS>& CLASSNAME::get_problems() const
{
   return problems;
}

Problems<NUHMSSMNoFVHimalaya_info::NUMBER_OF_PARTICLES, NUHMSSMNoFVHimalaya_info::NUMBER_OF_PARAMETERS>& CLASSNAME::get_problems()
{
   return problems;
}

/**
 * Method which calculates the tadpoles at the current loop order.
 *
 * @return array of tadpoles
 */
CLASSNAME::EWSB_vector_t CLASSNAME::tadpole_equations() const
{
   EWSB_vector_t tadpole(EWSB_vector_t::Zero());

   tadpole[0] = get_ewsb_eq_hh_1();
   tadpole[1] = get_ewsb_eq_hh_2();

   if (ewsb_loop_order > 0) {
      tadpole[0] -= Re(tadpole_hh_1loop(0));
      tadpole[1] -= Re(tadpole_hh_1loop(1));

      if (ewsb_loop_order > 1) {
         const auto tadpole_2l(tadpole_hh_2loop());
         tadpole[0] -= tadpole_2l(0);
         tadpole[1] -= tadpole_2l(1);

      }
   }

   return tadpole;
}

/**
 * Method which calculates the tadpoles at the current loop order.
 *
 * @param tadpole array of tadpole
 */
void CLASSNAME::tadpole_equations(double tadpole[number_of_ewsb_equations]) const
{
   const auto tadpole_(tadpole_equations());
   std::copy(tadpole_.data(), tadpole_.data() + number_of_ewsb_equations, tadpole);
}

/**
 * This function returns the vector of tadpoles, each divided by the
 * corresponding VEV.  Thus, the returned tadpoles have the dimension
 * GeV^2 each.
 *
 * @return vector of tadpoles
 */
CLASSNAME::EWSB_vector_t CLASSNAME::tadpole_equations_over_vevs() const
{
   auto tadpole = tadpole_equations();

   tadpole[0] /= vd;
   tadpole[1] /= vu;


   return tadpole;
}

/**
 * This method solves the EWSB conditions iteratively, trying several
 * root finding methods until a solution is found.
 */
int CLASSNAME::solve_ewsb_iteratively()
{
   auto model = *this;

   auto ewsb_stepper = [model](const EWSB_vector_t& ewsb_pars) mutable -> EWSB_vector_t {
      const double mHd2 = ewsb_pars(0);
      const double mHu2 = ewsb_pars(1);

      model.set_mHd2(mHd2);
      model.set_mHu2(mHu2);


      if (model.get_ewsb_loop_order() > 0)
         model.calculate_DRbar_masses();

      return model.ewsb_step();
   };

   auto tadpole_stepper = [model](const EWSB_vector_t& ewsb_pars) mutable -> EWSB_vector_t {
      const double mHd2 = ewsb_pars(0);
      const double mHu2 = ewsb_pars(1);

      model.set_mHd2(mHd2);
      model.set_mHu2(mHu2);


      if (model.get_ewsb_loop_order() > 0)
         model.calculate_DRbar_masses();

      return model.tadpole_equations();
   };

   std::unique_ptr<EWSB_solver> solvers[] = {
      std::unique_ptr<EWSB_solver>(new Fixed_point_iterator<number_of_ewsb_equations, fixed_point_iterator::Convergence_tester_relative>(ewsb_stepper, get_number_of_ewsb_iterations(), fixed_point_iterator::Convergence_tester_relative(ewsb_iteration_precision))),
      std::unique_ptr<EWSB_solver>(new Root_finder<number_of_ewsb_equations>(tadpole_stepper, get_number_of_ewsb_iterations(), ewsb_iteration_precision, Root_finder<number_of_ewsb_equations>::GSLHybridS)),
      std::unique_ptr<EWSB_solver>(new Root_finder<number_of_ewsb_equations>(tadpole_stepper, get_number_of_ewsb_iterations(), ewsb_iteration_precision, Root_finder<number_of_ewsb_equations>::GSLBroyden))
   };

   const auto x_init(ewsb_initial_guess());

   VERBOSE_MSG("\t\tSolving EWSB equations ...");
   VERBOSE_MSG("\t\tInitial guess: x_init = " << x_init.transpose());

   int status;
   for (auto& solver: solvers) {
      VERBOSE_MSG("\t\t\tStarting EWSB iteration using " << solver->name());
      status = solve_ewsb_iteratively_with(solver.get(), x_init);
      if (status == EWSB_solver::SUCCESS) {
         VERBOSE_MSG("\t\t\t" << solver->name() << " finished successfully!");
         break;
      }
#ifdef ENABLE_VERBOSE
      else {
         WARNING("\t\t\t" << solver->name() << " could not find a solution!"
                 " (requested precision: " << ewsb_iteration_precision << ")");
      }
#endif
   }

   if (status == EWSB_solver::SUCCESS) {
      problems.unflag_no_ewsb();
   } else {
      set_best_ewsb_solution(std::begin(solvers), std::end(solvers));
      problems.flag_no_ewsb();
#ifdef ENABLE_VERBOSE
      WARNING("\t\tCould not find a solution to the EWSB equations!"
              " (requested precision: " << ewsb_iteration_precision << ")");
#endif
   }

   return status;
}

/**
 * Solves EWSB equations with given EWSB solver
 *
 * @param solver EWSB solver
 * @param x_init initial values
 *
 * @return status of the EWSB solver
 */
int CLASSNAME::solve_ewsb_iteratively_with(
   EWSB_solver* solver, const EWSB_vector_t& x_init)
{
   const int status = solver->solve(x_init);

   if (status == EWSB_solver::SUCCESS)
      set_ewsb_solution(solver);

   return status;
}

/**
 * Sets EWSB output parameters from given solver.
 *
 * @param solver solver
 */
void CLASSNAME::set_ewsb_solution(const EWSB_solver* solver)
{
   const auto solution = solver->get_solution();

   mHd2 = solution(0);
   mHu2 = solution(1);


   calculate_DRbar_masses();
}

/**
 * Sets EWSB output parameters from the solver from the range [first,
 * last), which minimizes the tadpole equations at most.
 *
 * @param first iterator to first solver
 * @param last iterator to last solver
 */
template <typename It>
void CLASSNAME::set_best_ewsb_solution(It first, It last)
{
   auto ma(*this), mb(*this);

   const auto best_solver =
      std::min_element(first, last,
                       [this, &ma, &mb](const std::unique_ptr<EWSB_solver>& a, const std::unique_ptr<EWSB_solver>& b) {
                          ma.set_ewsb_solution(a.get());
                          mb.set_ewsb_solution(b.get());
                          return Total(Abs(Re(ma.tadpole_equations()))) < Total(Abs(Re(mb.tadpole_equations())));
                       });

   VERBOSE_MSG("\t\tUsing best solution from " << (*best_solver)->name());

   set_ewsb_solution(best_solver->get());
}

int CLASSNAME::solve_ewsb_iteratively(int loop_order)
{
   // temporarily set `ewsb_loop_order' to `loop_order' and do
   // iteration
   const auto save_loop_order_raii = make_raii_save(ewsb_loop_order);
   ewsb_loop_order = loop_order;
   return solve_ewsb_iteratively();
}


int CLASSNAME::solve_ewsb_tree_level()
{
   int error = 0;

   const double old_mHd2 = mHd2;
   const double old_mHu2 = mHu2;

   mHd2 = Re((0.025*(-40*vd*AbsSqr(Mu) + 20*vu*BMu + 20*vu*Conj(BMu) - 3*Cube(
      vd)*Sqr(g1) - 5*Cube(vd)*Sqr(g2) + 3*vd*Sqr(g1)*Sqr(vu) + 5*vd*Sqr(g2)*Sqr(
      vu)))/vd);
   mHu2 = Re((0.025*(-40*vu*AbsSqr(Mu) + 20*vd*BMu + 20*vd*Conj(BMu) - 3*Cube(
      vu)*Sqr(g1) - 5*Cube(vu)*Sqr(g2) + 3*vu*Sqr(g1)*Sqr(vd) + 5*vu*Sqr(g2)*Sqr(
      vd)))/vu);

   const bool is_finite = IsFinite(mHd2) && IsFinite(mHu2);

   if (!is_finite) {
      mHd2 = old_mHd2;
      mHu2 = old_mHu2;
      error = 1;
   }


   return error;
}

int CLASSNAME::solve_ewsb_tree_level_custom()
{
   int error = 0;

   const double old_mHd2 = mHd2;
   const double old_mHu2 = mHu2;

   mHd2 = Re((0.025*(-40*vd*AbsSqr(Mu) + 20*vu*BMu + 20*vu*Conj(BMu) - 3*Cube(
      vd)*Sqr(g1) - 5*Cube(vd)*Sqr(g2) + 3*vd*Sqr(g1)*Sqr(vu) + 5*vd*Sqr(g2)*Sqr(
      vu)))/vd);
   mHu2 = Re((0.025*(-40*vu*AbsSqr(Mu) + 20*vd*BMu + 20*vd*Conj(BMu) - 3*Cube(
      vu)*Sqr(g1) - 5*Cube(vu)*Sqr(g2) + 3*vu*Sqr(g1)*Sqr(vd) + 5*vu*Sqr(g2)*Sqr(
      vd)))/vu);

   const bool is_finite = IsFinite(mHd2) && IsFinite(mHu2);

   if (!is_finite) {
      mHd2 = old_mHd2;
      mHu2 = old_mHu2;
      error = 1;
   }


   return error;
}

int CLASSNAME::solve_ewsb_one_loop()
{
   return solve_ewsb_iteratively(1);
}

int CLASSNAME::solve_ewsb()
{
   VERBOSE_MSG("\t\tSolving NUHMSSMNoFVHimalaya EWSB at " << ewsb_loop_order << "-loop order");

   if (ewsb_loop_order == 0)
      return solve_ewsb_tree_level();

   return solve_ewsb_iteratively(ewsb_loop_order);
}

CLASSNAME::EWSB_vector_t CLASSNAME::ewsb_initial_guess()
{
   EWSB_vector_t x_init(EWSB_vector_t::Zero());

   x_init[0] = mHd2;
   x_init[1] = mHu2;


   return x_init;
}

/**
 * Calculates EWSB output parameters including loop-corrections.
 *
 * Throws exception of type EEWSBStepFailed if new EWSB parameters are
 * inf or nan.
 *
 * @return new set of EWSB output parameters
 */
CLASSNAME::EWSB_vector_t CLASSNAME::ewsb_step() const
{
   std::array<double, number_of_ewsb_equations> tadpole{};
   EWSB_vector_t ewsb_parameters(EWSB_vector_t::Zero());

   if (ewsb_loop_order > 0) {
      tadpole[0] += Re(tadpole_hh_1loop(0));
      tadpole[1] += Re(tadpole_hh_1loop(1));

      if (ewsb_loop_order > 1) {
         const auto tadpole_2l(tadpole_hh_2loop());
         tadpole[0] += tadpole_2l(0);
         tadpole[1] += tadpole_2l(1);

      }
   }

   double mHd2;
   double mHu2;

   mHd2 = Re((0.025*(-40*vd*AbsSqr(Mu) + 20*vu*BMu + 20*vu*Conj(BMu) + 40*
      tadpole[0] - 3*Cube(vd)*Sqr(g1) - 5*Cube(vd)*Sqr(g2) + 3*vd*Sqr(g1)*Sqr(vu)
      + 5*vd*Sqr(g2)*Sqr(vu)))/vd);
   mHu2 = Re((0.025*(-40*vu*AbsSqr(Mu) + 20*vd*BMu + 20*vd*Conj(BMu) + 40*
      tadpole[1] - 3*Cube(vu)*Sqr(g1) - 5*Cube(vu)*Sqr(g2) + 3*vu*Sqr(g1)*Sqr(vd)
      + 5*vu*Sqr(g2)*Sqr(vd)))/vu);

   const bool is_finite = IsFinite(mHd2) && IsFinite(mHu2);


   if (!is_finite)
      throw EEWSBStepFailed();

   ewsb_parameters[0] = mHd2;
   ewsb_parameters[1] = mHu2;


   return ewsb_parameters;
}

void CLASSNAME::print(std::ostream& ostr) const
{
   ostr << "========================================\n"
           "NUHMSSMNoFVHimalaya\n"
           "========================================\n";
   NUHMSSMNoFVHimalaya_soft_parameters::print(ostr);
   ostr << "----------------------------------------\n"
           "tree-level DRbar masses:\n"
           "----------------------------------------\n";
   ostr << "MVG = " << MVG << '\n';
   ostr << "MGlu = " << MGlu << '\n';
   ostr << "MFd = " << MFd << '\n';
   ostr << "MFs = " << MFs << '\n';
   ostr << "MFb = " << MFb << '\n';
   ostr << "MFu = " << MFu << '\n';
   ostr << "MFc = " << MFc << '\n';
   ostr << "MFt = " << MFt << '\n';
   ostr << "MFve = " << MFve << '\n';
   ostr << "MFvm = " << MFvm << '\n';
   ostr << "MFvt = " << MFvt << '\n';
   ostr << "MFe = " << MFe << '\n';
   ostr << "MFm = " << MFm << '\n';
   ostr << "MFtau = " << MFtau << '\n';
   ostr << "MSveL = " << MSveL << '\n';
   ostr << "MSvmL = " << MSvmL << '\n';
   ostr << "MSvtL = " << MSvtL << '\n';
   ostr << "MSd = " << MSd.transpose() << '\n';
   ostr << "MSu = " << MSu.transpose() << '\n';
   ostr << "MSe = " << MSe.transpose() << '\n';
   ostr << "MSm = " << MSm.transpose() << '\n';
   ostr << "MStau = " << MStau.transpose() << '\n';
   ostr << "MSs = " << MSs.transpose() << '\n';
   ostr << "MSc = " << MSc.transpose() << '\n';
   ostr << "MSb = " << MSb.transpose() << '\n';
   ostr << "MSt = " << MSt.transpose() << '\n';
   ostr << "Mhh = " << Mhh.transpose() << '\n';
   ostr << "MAh = " << MAh.transpose() << '\n';
   ostr << "MHpm = " << MHpm.transpose() << '\n';
   ostr << "MChi = " << MChi.transpose() << '\n';
   ostr << "MCha = " << MCha.transpose() << '\n';
   ostr << "MVWm = " << MVWm << '\n';
   ostr << "MVP = " << MVP << '\n';
   ostr << "MVZ = " << MVZ << '\n';

   ostr << "----------------------------------------\n"
           "tree-level DRbar mixing matrices:\n"
           "----------------------------------------\n";
   ostr << "ZD = " << ZD << '\n';
   ostr << "ZU = " << ZU << '\n';
   ostr << "ZE = " << ZE << '\n';
   ostr << "ZM = " << ZM << '\n';
   ostr << "ZTau = " << ZTau << '\n';
   ostr << "ZS = " << ZS << '\n';
   ostr << "ZC = " << ZC << '\n';
   ostr << "ZB = " << ZB << '\n';
   ostr << "ZT = " << ZT << '\n';
   ostr << "ZH = " << ZH << '\n';
   ostr << "ZA = " << ZA << '\n';
   ostr << "ZP = " << ZP << '\n';
   ostr << "ZN = " << ZN << '\n';
   ostr << "UM = " << UM << '\n';
   ostr << "UP = " << UP << '\n';
   ostr << "ZZ = " << ZZ << '\n';

   physical.print(ostr);
}

/**
 * wrapper routines for passarino Veltman functions
 * @note: They take squared arguments!
 */

double CLASSNAME::A0(double m) const
{
   return passarino_veltman::ReA0(m, Sqr(get_scale()));
}

double CLASSNAME::B0(double p, double m1, double m2) const
{
   return passarino_veltman::ReB0(p, m1, m2, Sqr(get_scale()));
}

double CLASSNAME::B1(double p, double m1, double m2) const
{
   return passarino_veltman::ReB1(p, m1, m2, Sqr(get_scale()));
}

double CLASSNAME::B00(double p, double m1, double m2) const
{
   return passarino_veltman::ReB00(p, m1, m2, Sqr(get_scale()));
}

double CLASSNAME::B22(double p, double m1, double m2) const
{
   return passarino_veltman::ReB22(p, m1, m2, Sqr(get_scale()));
}

double CLASSNAME::H0(double p, double m1, double m2) const
{
   return passarino_veltman::ReH0(p, m1, m2, Sqr(get_scale()));
}

double CLASSNAME::F0(double p, double m1, double m2) const
{
   return passarino_veltman::ReF0(p, m1, m2, Sqr(get_scale()));
}

double CLASSNAME::G0(double p, double m1, double m2) const
{
   return passarino_veltman::ReG0(p, m1, m2, Sqr(get_scale()));
}

/**
 * routine which finds the DRbar mass eigenstates and mixings.
 */
void CLASSNAME::calculate_DRbar_masses()
{
   const auto save_mHd2_raii = make_raii_save(mHd2);
   const auto save_mHu2_raii = make_raii_save(mHu2);

   solve_ewsb_tree_level_custom();

   calculate_MVPVZ();
   calculate_MVWm();
   calculate_MCha();
   calculate_MChi();
   calculate_MHpm();
   calculate_MAh();
   calculate_Mhh();
   calculate_MSt();
   calculate_MSb();
   calculate_MSc();
   calculate_MSs();
   calculate_MStau();
   calculate_MSm();
   calculate_MSe();
   calculate_MSu();
   calculate_MSd();
   calculate_MSvtL();
   calculate_MSvmL();
   calculate_MSveL();
   calculate_MFtau();
   calculate_MFm();
   calculate_MFe();
   calculate_MFvt();
   calculate_MFvm();
   calculate_MFve();
   calculate_MFt();
   calculate_MFc();
   calculate_MFu();
   calculate_MFb();
   calculate_MFs();
   calculate_MFd();
   calculate_MGlu();
   calculate_MVG();

}

/**
 * routine which finds the pole mass eigenstates and mixings.
 */
void CLASSNAME::calculate_pole_masses()
{
#ifdef ENABLE_THREADS
   Thread_pool tp(std::min(std::thread::hardware_concurrency(), 34u));

   if (calculate_bsm_pole_masses) {
      tp.run_task([this] () { calculate_MAh_pole(); });
      tp.run_task([this] () { calculate_MCha_pole(); });
      tp.run_task([this] () { calculate_MChi_pole(); });
      tp.run_task([this] () { calculate_MGlu_pole(); });
      tp.run_task([this] () { calculate_Mhh_pole(); });
      tp.run_task([this] () { calculate_MHpm_pole(); });
      tp.run_task([this] () { calculate_MSb_pole(); });
      tp.run_task([this] () { calculate_MSc_pole(); });
      tp.run_task([this] () { calculate_MSd_pole(); });
      tp.run_task([this] () { calculate_MSe_pole(); });
      tp.run_task([this] () { calculate_MSm_pole(); });
      tp.run_task([this] () { calculate_MSs_pole(); });
      tp.run_task([this] () { calculate_MSt_pole(); });
      tp.run_task([this] () { calculate_MStau_pole(); });
      tp.run_task([this] () { calculate_MSu_pole(); });
      tp.run_task([this] () { calculate_MSveL_pole(); });
      tp.run_task([this] () { calculate_MSvmL_pole(); });
      tp.run_task([this] () { calculate_MSvtL_pole(); });
   }

   if (calculate_sm_pole_masses) {
      tp.run_task([this] () { calculate_MVG_pole(); });
      tp.run_task([this] () { calculate_MVP_pole(); });
      tp.run_task([this] () { calculate_MVZ_pole(); });
      tp.run_task([this] () { calculate_MFd_pole(); });
      tp.run_task([this] () { calculate_MFs_pole(); });
      tp.run_task([this] () { calculate_MFb_pole(); });
      tp.run_task([this] () { calculate_MFu_pole(); });
      tp.run_task([this] () { calculate_MFc_pole(); });
      tp.run_task([this] () { calculate_MFt_pole(); });
      tp.run_task([this] () { calculate_MFve_pole(); });
      tp.run_task([this] () { calculate_MFvm_pole(); });
      tp.run_task([this] () { calculate_MFvt_pole(); });
      tp.run_task([this] () { calculate_MFe_pole(); });
      tp.run_task([this] () { calculate_MFm_pole(); });
      tp.run_task([this] () { calculate_MFtau_pole(); });
      tp.run_task([this] () { calculate_MVWm_pole(); });
   }

#else
   if (calculate_bsm_pole_masses) {
      calculate_MAh_pole();
      calculate_MCha_pole();
      calculate_MChi_pole();
      calculate_MGlu_pole();
      calculate_Mhh_pole();
      calculate_MHpm_pole();
      calculate_MSb_pole();
      calculate_MSc_pole();
      calculate_MSd_pole();
      calculate_MSe_pole();
      calculate_MSm_pole();
      calculate_MSs_pole();
      calculate_MSt_pole();
      calculate_MStau_pole();
      calculate_MSu_pole();
      calculate_MSveL_pole();
      calculate_MSvmL_pole();
      calculate_MSvtL_pole();
   }

   if (calculate_sm_pole_masses) {
      calculate_MVG_pole();
      calculate_MVP_pole();
      calculate_MVZ_pole();
      calculate_MFd_pole();
      calculate_MFs_pole();
      calculate_MFb_pole();
      calculate_MFu_pole();
      calculate_MFc_pole();
      calculate_MFt_pole();
      calculate_MFve_pole();
      calculate_MFvm_pole();
      calculate_MFvt_pole();
      calculate_MFe_pole();
      calculate_MFm_pole();
      calculate_MFtau_pole();
      calculate_MVWm_pole();
   }

#endif
}

void CLASSNAME::copy_DRbar_masses_to_pole_masses()
{
   PHYSICAL(MVG) = MVG;
   PHYSICAL(MGlu) = MGlu;
   PHYSICAL(MFd) = MFd;
   PHYSICAL(MFs) = MFs;
   PHYSICAL(MFb) = MFb;
   PHYSICAL(MFu) = MFu;
   PHYSICAL(MFc) = MFc;
   PHYSICAL(MFt) = MFt;
   PHYSICAL(MFve) = MFve;
   PHYSICAL(MFvm) = MFvm;
   PHYSICAL(MFvt) = MFvt;
   PHYSICAL(MFe) = MFe;
   PHYSICAL(MFm) = MFm;
   PHYSICAL(MFtau) = MFtau;
   PHYSICAL(MSveL) = MSveL;
   PHYSICAL(MSvmL) = MSvmL;
   PHYSICAL(MSvtL) = MSvtL;
   PHYSICAL(MSd) = MSd;
   PHYSICAL(ZD) = ZD;
   PHYSICAL(MSu) = MSu;
   PHYSICAL(ZU) = ZU;
   PHYSICAL(MSe) = MSe;
   PHYSICAL(ZE) = ZE;
   PHYSICAL(MSm) = MSm;
   PHYSICAL(ZM) = ZM;
   PHYSICAL(MStau) = MStau;
   PHYSICAL(ZTau) = ZTau;
   PHYSICAL(MSs) = MSs;
   PHYSICAL(ZS) = ZS;
   PHYSICAL(MSc) = MSc;
   PHYSICAL(ZC) = ZC;
   PHYSICAL(MSb) = MSb;
   PHYSICAL(ZB) = ZB;
   PHYSICAL(MSt) = MSt;
   PHYSICAL(ZT) = ZT;
   PHYSICAL(Mhh) = Mhh;
   PHYSICAL(ZH) = ZH;
   PHYSICAL(MAh) = MAh;
   PHYSICAL(ZA) = ZA;
   PHYSICAL(MHpm) = MHpm;
   PHYSICAL(ZP) = ZP;
   PHYSICAL(MChi) = MChi;
   PHYSICAL(ZN) = ZN;
   PHYSICAL(MCha) = MCha;
   PHYSICAL(UM) = UM;
   PHYSICAL(UP) = UP;
   PHYSICAL(MVWm) = MVWm;
   PHYSICAL(MVP) = MVP;
   PHYSICAL(MVZ) = MVZ;

}

/**
 * reorders DRbar masses so that golstones are placed at the index
 * specified in the model files definition of the associated
 * gauge boson (see Z-boson definition in default particles.m file
 * in the Models directory of your SARAH distribution for example)
 */
void CLASSNAME::reorder_DRbar_masses()
{
   move_goldstone_to(0, MVZ, MAh, ZA);
   move_goldstone_to(0, MVWm, MHpm, ZP);

}

/**
 * reorders pole masses so that golstones are placed at the index
 * specified in the model files definition of the associated
 * gauge boson (see Z-boson definition in default particles.m file
 * in the Models directory of your SARAH distribution for example)
 */
void CLASSNAME::reorder_pole_masses()
{
   move_goldstone_to(0, MVZ, PHYSICAL(MAh), PHYSICAL(ZA));
   move_goldstone_to(0, MVWm, PHYSICAL(MHpm), PHYSICAL(ZP));

}

/**
 * Checks the pole masses for tachyons
 */
void CLASSNAME::check_pole_masses_for_tachyons()
{
   if (PHYSICAL(MSveL) < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::SveL);
   if (PHYSICAL(MSvmL) < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::SvmL);
   if (PHYSICAL(MSvtL) < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::SvtL);
   if (PHYSICAL(MSd).tail<2>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::Sd);
   if (PHYSICAL(MSu).tail<2>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::Su);
   if (PHYSICAL(MSe).tail<2>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::Se);
   if (PHYSICAL(MSm).tail<2>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::Sm);
   if (PHYSICAL(MStau).tail<2>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::Stau);
   if (PHYSICAL(MSs).tail<2>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::Ss);
   if (PHYSICAL(MSc).tail<2>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::Sc);
   if (PHYSICAL(MSb).tail<2>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::Sb);
   if (PHYSICAL(MSt).tail<2>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::St);
   if (PHYSICAL(Mhh).tail<2>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::hh);
   if (PHYSICAL(MAh).tail<1>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::Ah);
   if (PHYSICAL(MHpm).tail<1>().minCoeff() < 0.) problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::Hpm);

}

/**
 * calculates spectrum for model once the DRbar parameters at
 * at low energies are known
 */
void CLASSNAME::calculate_spectrum()
{
   calculate_DRbar_masses();
   if (pole_mass_loop_order > 0)
      calculate_pole_masses();

   // move goldstone bosons to the front
   reorder_DRbar_masses();
   if (pole_mass_loop_order == 0)
      copy_DRbar_masses_to_pole_masses();
   else
      reorder_pole_masses();

   check_pole_masses_for_tachyons();

   if (problems.have_problem() && !force_output) {
      clear_DRbar_parameters();
      physical.clear();
   }
}

void CLASSNAME::clear_DRbar_parameters()
{
   MVG = 0.;
   MGlu = 0.;
   MFd = 0.;
   MFs = 0.;
   MFb = 0.;
   MFu = 0.;
   MFc = 0.;
   MFt = 0.;
   MFve = 0.;
   MFvm = 0.;
   MFvt = 0.;
   MFe = 0.;
   MFm = 0.;
   MFtau = 0.;
   MSveL = 0.;
   MSvmL = 0.;
   MSvtL = 0.;
   MSd = Eigen::Matrix<double,2,1>::Zero();
   ZD = Eigen::Matrix<double,2,2>::Zero();
   MSu = Eigen::Matrix<double,2,1>::Zero();
   ZU = Eigen::Matrix<double,2,2>::Zero();
   MSe = Eigen::Matrix<double,2,1>::Zero();
   ZE = Eigen::Matrix<double,2,2>::Zero();
   MSm = Eigen::Matrix<double,2,1>::Zero();
   ZM = Eigen::Matrix<double,2,2>::Zero();
   MStau = Eigen::Matrix<double,2,1>::Zero();
   ZTau = Eigen::Matrix<double,2,2>::Zero();
   MSs = Eigen::Matrix<double,2,1>::Zero();
   ZS = Eigen::Matrix<double,2,2>::Zero();
   MSc = Eigen::Matrix<double,2,1>::Zero();
   ZC = Eigen::Matrix<double,2,2>::Zero();
   MSb = Eigen::Matrix<double,2,1>::Zero();
   ZB = Eigen::Matrix<double,2,2>::Zero();
   MSt = Eigen::Matrix<double,2,1>::Zero();
   ZT = Eigen::Matrix<double,2,2>::Zero();
   Mhh = Eigen::Matrix<double,2,1>::Zero();
   ZH = Eigen::Matrix<double,2,2>::Zero();
   MAh = Eigen::Matrix<double,2,1>::Zero();
   ZA = Eigen::Matrix<double,2,2>::Zero();
   MHpm = Eigen::Matrix<double,2,1>::Zero();
   ZP = Eigen::Matrix<double,2,2>::Zero();
   MChi = Eigen::Matrix<double,4,1>::Zero();
   ZN = Eigen::Matrix<std::complex<double>,4,4>::Zero();
   MCha = Eigen::Matrix<double,2,1>::Zero();
   UM = Eigen::Matrix<std::complex<double>,2,2>::Zero();
   UP = Eigen::Matrix<std::complex<double>,2,2>::Zero();
   MVWm = 0.;
   MVP = 0.;
   MVZ = 0.;

   PhaseGlu = std::complex<double>(1.,0.);


}

void CLASSNAME::clear_problems()
{
   problems.unflag_all_tachyons();
}

void CLASSNAME::clear()
{
   NUHMSSMNoFVHimalaya_soft_parameters::clear();
   clear_DRbar_parameters();
   physical.clear();
   problems.clear();
}

void CLASSNAME::set_DRbar_masses(const Eigen::ArrayXd& pars)
{
   MVG = pars(0);
   MGlu = pars(1);
   MFd = pars(2);
   MFs = pars(3);
   MFb = pars(4);
   MFu = pars(5);
   MFc = pars(6);
   MFt = pars(7);
   MFve = pars(8);
   MFvm = pars(9);
   MFvt = pars(10);
   MFe = pars(11);
   MFm = pars(12);
   MFtau = pars(13);
   MSveL = pars(14);
   MSvmL = pars(15);
   MSvtL = pars(16);
   MSd(0) = pars(17);
   MSd(1) = pars(18);
   MSu(0) = pars(19);
   MSu(1) = pars(20);
   MSe(0) = pars(21);
   MSe(1) = pars(22);
   MSm(0) = pars(23);
   MSm(1) = pars(24);
   MStau(0) = pars(25);
   MStau(1) = pars(26);
   MSs(0) = pars(27);
   MSs(1) = pars(28);
   MSc(0) = pars(29);
   MSc(1) = pars(30);
   MSb(0) = pars(31);
   MSb(1) = pars(32);
   MSt(0) = pars(33);
   MSt(1) = pars(34);
   Mhh(0) = pars(35);
   Mhh(1) = pars(36);
   MAh(0) = pars(37);
   MAh(1) = pars(38);
   MHpm(0) = pars(39);
   MHpm(1) = pars(40);
   MChi(0) = pars(41);
   MChi(1) = pars(42);
   MChi(2) = pars(43);
   MChi(3) = pars(44);
   MCha(0) = pars(45);
   MCha(1) = pars(46);
   MVWm = pars(47);
   MVP = pars(48);
   MVZ = pars(49);

}

Eigen::ArrayXd CLASSNAME::get_DRbar_masses() const
{
   Eigen::ArrayXd pars(50);

   pars(0) = MVG;
   pars(1) = MGlu;
   pars(2) = MFd;
   pars(3) = MFs;
   pars(4) = MFb;
   pars(5) = MFu;
   pars(6) = MFc;
   pars(7) = MFt;
   pars(8) = MFve;
   pars(9) = MFvm;
   pars(10) = MFvt;
   pars(11) = MFe;
   pars(12) = MFm;
   pars(13) = MFtau;
   pars(14) = MSveL;
   pars(15) = MSvmL;
   pars(16) = MSvtL;
   pars(17) = MSd(0);
   pars(18) = MSd(1);
   pars(19) = MSu(0);
   pars(20) = MSu(1);
   pars(21) = MSe(0);
   pars(22) = MSe(1);
   pars(23) = MSm(0);
   pars(24) = MSm(1);
   pars(25) = MStau(0);
   pars(26) = MStau(1);
   pars(27) = MSs(0);
   pars(28) = MSs(1);
   pars(29) = MSc(0);
   pars(30) = MSc(1);
   pars(31) = MSb(0);
   pars(32) = MSb(1);
   pars(33) = MSt(0);
   pars(34) = MSt(1);
   pars(35) = Mhh(0);
   pars(36) = Mhh(1);
   pars(37) = MAh(0);
   pars(38) = MAh(1);
   pars(39) = MHpm(0);
   pars(40) = MHpm(1);
   pars(41) = MChi(0);
   pars(42) = MChi(1);
   pars(43) = MChi(2);
   pars(44) = MChi(3);
   pars(45) = MCha(0);
   pars(46) = MCha(1);
   pars(47) = MVWm;
   pars(48) = MVP;
   pars(49) = MVZ;

   return pars;
}

void CLASSNAME::set_DRbar_masses_and_mixings(const Eigen::ArrayXd& pars)
{
   set_DRbar_masses(pars);

   ZD(0,0) = pars(50);
   ZD(0,1) = pars(51);
   ZD(1,0) = pars(52);
   ZD(1,1) = pars(53);
   ZU(0,0) = pars(54);
   ZU(0,1) = pars(55);
   ZU(1,0) = pars(56);
   ZU(1,1) = pars(57);
   ZE(0,0) = pars(58);
   ZE(0,1) = pars(59);
   ZE(1,0) = pars(60);
   ZE(1,1) = pars(61);
   ZM(0,0) = pars(62);
   ZM(0,1) = pars(63);
   ZM(1,0) = pars(64);
   ZM(1,1) = pars(65);
   ZTau(0,0) = pars(66);
   ZTau(0,1) = pars(67);
   ZTau(1,0) = pars(68);
   ZTau(1,1) = pars(69);
   ZS(0,0) = pars(70);
   ZS(0,1) = pars(71);
   ZS(1,0) = pars(72);
   ZS(1,1) = pars(73);
   ZC(0,0) = pars(74);
   ZC(0,1) = pars(75);
   ZC(1,0) = pars(76);
   ZC(1,1) = pars(77);
   ZB(0,0) = pars(78);
   ZB(0,1) = pars(79);
   ZB(1,0) = pars(80);
   ZB(1,1) = pars(81);
   ZT(0,0) = pars(82);
   ZT(0,1) = pars(83);
   ZT(1,0) = pars(84);
   ZT(1,1) = pars(85);
   ZH(0,0) = pars(86);
   ZH(0,1) = pars(87);
   ZH(1,0) = pars(88);
   ZH(1,1) = pars(89);
   ZA(0,0) = pars(90);
   ZA(0,1) = pars(91);
   ZA(1,0) = pars(92);
   ZA(1,1) = pars(93);
   ZP(0,0) = pars(94);
   ZP(0,1) = pars(95);
   ZP(1,0) = pars(96);
   ZP(1,1) = pars(97);
   ZN(0,0) = std::complex<double>(pars(98), pars(99));
   ZN(0,1) = std::complex<double>(pars(100), pars(101));
   ZN(0,2) = std::complex<double>(pars(102), pars(103));
   ZN(0,3) = std::complex<double>(pars(104), pars(105));
   ZN(1,0) = std::complex<double>(pars(106), pars(107));
   ZN(1,1) = std::complex<double>(pars(108), pars(109));
   ZN(1,2) = std::complex<double>(pars(110), pars(111));
   ZN(1,3) = std::complex<double>(pars(112), pars(113));
   ZN(2,0) = std::complex<double>(pars(114), pars(115));
   ZN(2,1) = std::complex<double>(pars(116), pars(117));
   ZN(2,2) = std::complex<double>(pars(118), pars(119));
   ZN(2,3) = std::complex<double>(pars(120), pars(121));
   ZN(3,0) = std::complex<double>(pars(122), pars(123));
   ZN(3,1) = std::complex<double>(pars(124), pars(125));
   ZN(3,2) = std::complex<double>(pars(126), pars(127));
   ZN(3,3) = std::complex<double>(pars(128), pars(129));
   UM(0,0) = std::complex<double>(pars(130), pars(131));
   UM(0,1) = std::complex<double>(pars(132), pars(133));
   UM(1,0) = std::complex<double>(pars(134), pars(135));
   UM(1,1) = std::complex<double>(pars(136), pars(137));
   UP(0,0) = std::complex<double>(pars(138), pars(139));
   UP(0,1) = std::complex<double>(pars(140), pars(141));
   UP(1,0) = std::complex<double>(pars(142), pars(143));
   UP(1,1) = std::complex<double>(pars(144), pars(145));
   ZZ(0,0) = pars(146);
   ZZ(0,1) = pars(147);
   ZZ(1,0) = pars(148);
   ZZ(1,1) = pars(149);

}

Eigen::ArrayXd CLASSNAME::get_DRbar_masses_and_mixings() const
{
   Eigen::ArrayXd pars(get_DRbar_masses());

   pars.conservativeResize(150);

   pars(50) = ZD(0,0);
   pars(51) = ZD(0,1);
   pars(52) = ZD(1,0);
   pars(53) = ZD(1,1);
   pars(54) = ZU(0,0);
   pars(55) = ZU(0,1);
   pars(56) = ZU(1,0);
   pars(57) = ZU(1,1);
   pars(58) = ZE(0,0);
   pars(59) = ZE(0,1);
   pars(60) = ZE(1,0);
   pars(61) = ZE(1,1);
   pars(62) = ZM(0,0);
   pars(63) = ZM(0,1);
   pars(64) = ZM(1,0);
   pars(65) = ZM(1,1);
   pars(66) = ZTau(0,0);
   pars(67) = ZTau(0,1);
   pars(68) = ZTau(1,0);
   pars(69) = ZTau(1,1);
   pars(70) = ZS(0,0);
   pars(71) = ZS(0,1);
   pars(72) = ZS(1,0);
   pars(73) = ZS(1,1);
   pars(74) = ZC(0,0);
   pars(75) = ZC(0,1);
   pars(76) = ZC(1,0);
   pars(77) = ZC(1,1);
   pars(78) = ZB(0,0);
   pars(79) = ZB(0,1);
   pars(80) = ZB(1,0);
   pars(81) = ZB(1,1);
   pars(82) = ZT(0,0);
   pars(83) = ZT(0,1);
   pars(84) = ZT(1,0);
   pars(85) = ZT(1,1);
   pars(86) = ZH(0,0);
   pars(87) = ZH(0,1);
   pars(88) = ZH(1,0);
   pars(89) = ZH(1,1);
   pars(90) = ZA(0,0);
   pars(91) = ZA(0,1);
   pars(92) = ZA(1,0);
   pars(93) = ZA(1,1);
   pars(94) = ZP(0,0);
   pars(95) = ZP(0,1);
   pars(96) = ZP(1,0);
   pars(97) = ZP(1,1);
   pars(98) = Re(ZN(0,0));
   pars(99) = Im(ZN(0,0));
   pars(100) = Re(ZN(0,1));
   pars(101) = Im(ZN(0,1));
   pars(102) = Re(ZN(0,2));
   pars(103) = Im(ZN(0,2));
   pars(104) = Re(ZN(0,3));
   pars(105) = Im(ZN(0,3));
   pars(106) = Re(ZN(1,0));
   pars(107) = Im(ZN(1,0));
   pars(108) = Re(ZN(1,1));
   pars(109) = Im(ZN(1,1));
   pars(110) = Re(ZN(1,2));
   pars(111) = Im(ZN(1,2));
   pars(112) = Re(ZN(1,3));
   pars(113) = Im(ZN(1,3));
   pars(114) = Re(ZN(2,0));
   pars(115) = Im(ZN(2,0));
   pars(116) = Re(ZN(2,1));
   pars(117) = Im(ZN(2,1));
   pars(118) = Re(ZN(2,2));
   pars(119) = Im(ZN(2,2));
   pars(120) = Re(ZN(2,3));
   pars(121) = Im(ZN(2,3));
   pars(122) = Re(ZN(3,0));
   pars(123) = Im(ZN(3,0));
   pars(124) = Re(ZN(3,1));
   pars(125) = Im(ZN(3,1));
   pars(126) = Re(ZN(3,2));
   pars(127) = Im(ZN(3,2));
   pars(128) = Re(ZN(3,3));
   pars(129) = Im(ZN(3,3));
   pars(130) = Re(UM(0,0));
   pars(131) = Im(UM(0,0));
   pars(132) = Re(UM(0,1));
   pars(133) = Im(UM(0,1));
   pars(134) = Re(UM(1,0));
   pars(135) = Im(UM(1,0));
   pars(136) = Re(UM(1,1));
   pars(137) = Im(UM(1,1));
   pars(138) = Re(UP(0,0));
   pars(139) = Im(UP(0,0));
   pars(140) = Re(UP(0,1));
   pars(141) = Im(UP(0,1));
   pars(142) = Re(UP(1,0));
   pars(143) = Im(UP(1,0));
   pars(144) = Re(UP(1,1));
   pars(145) = Im(UP(1,1));
   pars(146) = ZZ(0,0);
   pars(147) = ZZ(0,1);
   pars(148) = ZZ(1,0);
   pars(149) = ZZ(1,1);


   return pars;
}

std::string CLASSNAME::name() const
{
   return "NUHMSSMNoFVHimalaya";
}

void CLASSNAME::run_to(double scale, double eps)
{
   if (eps < 0.0)
      eps = precision;
   NUHMSSMNoFVHimalaya_soft_parameters::run_to(scale, eps);
}


Eigen::Array<double,1,1> CLASSNAME::get_MChargedHiggs() const
{
   Eigen::Array<double,1,1> MHpm_goldstone;
   MHpm_goldstone(0) = MVWm;

   return remove_if_equal(MHpm, MHpm_goldstone);
}

Eigen::Array<double,1,1> CLASSNAME::get_MPseudoscalarHiggs() const
{
   Eigen::Array<double,1,1> MAh_goldstone;
   MAh_goldstone(0) = MVZ;

   return remove_if_equal(MAh, MAh_goldstone);
}




double CLASSNAME::get_mass_matrix_VG() const
{
   const double mass_matrix_VG = Re(0);

   return mass_matrix_VG;
}

void CLASSNAME::calculate_MVG()
{
   const auto mass_matrix_VG = get_mass_matrix_VG();
   MVG = mass_matrix_VG;
}

double CLASSNAME::get_mass_matrix_Glu() const
{
   const double mass_matrix_Glu = Re(MassG);

   return mass_matrix_Glu;
}

void CLASSNAME::calculate_MGlu()
{
   const auto mass_matrix_Glu = get_mass_matrix_Glu();
   MGlu = calculate_majorana_singlet_mass(mass_matrix_Glu, PhaseGlu);
}

double CLASSNAME::get_mass_matrix_Fd() const
{
   const double mass_matrix_Fd = Re(0.7071067811865475*vd*Yd(0,0));

   return mass_matrix_Fd;
}

void CLASSNAME::calculate_MFd()
{
   const auto mass_matrix_Fd = get_mass_matrix_Fd();
   MFd = calculate_singlet_mass(mass_matrix_Fd);
}

double CLASSNAME::get_mass_matrix_Fs() const
{
   const double mass_matrix_Fs = Re(0.7071067811865475*vd*Yd(1,1));

   return mass_matrix_Fs;
}

void CLASSNAME::calculate_MFs()
{
   const auto mass_matrix_Fs = get_mass_matrix_Fs();
   MFs = calculate_singlet_mass(mass_matrix_Fs);
}

double CLASSNAME::get_mass_matrix_Fb() const
{
   const double mass_matrix_Fb = Re(0.7071067811865475*vd*Yd(2,2));

   return mass_matrix_Fb;
}

void CLASSNAME::calculate_MFb()
{
   const auto mass_matrix_Fb = get_mass_matrix_Fb();
   MFb = calculate_singlet_mass(mass_matrix_Fb);
}

double CLASSNAME::get_mass_matrix_Fu() const
{
   const double mass_matrix_Fu = Re(0.7071067811865475*vu*Yu(0,0));

   return mass_matrix_Fu;
}

void CLASSNAME::calculate_MFu()
{
   const auto mass_matrix_Fu = get_mass_matrix_Fu();
   MFu = calculate_singlet_mass(mass_matrix_Fu);
}

double CLASSNAME::get_mass_matrix_Fc() const
{
   const double mass_matrix_Fc = Re(0.7071067811865475*vu*Yu(1,1));

   return mass_matrix_Fc;
}

void CLASSNAME::calculate_MFc()
{
   const auto mass_matrix_Fc = get_mass_matrix_Fc();
   MFc = calculate_singlet_mass(mass_matrix_Fc);
}

double CLASSNAME::get_mass_matrix_Ft() const
{
   const double mass_matrix_Ft = Re(0.7071067811865475*vu*Yu(2,2));

   return mass_matrix_Ft;
}

void CLASSNAME::calculate_MFt()
{
   const auto mass_matrix_Ft = get_mass_matrix_Ft();
   MFt = calculate_singlet_mass(mass_matrix_Ft);
}

double CLASSNAME::get_mass_matrix_Fve() const
{
   const double mass_matrix_Fve = Re(0);

   return mass_matrix_Fve;
}

void CLASSNAME::calculate_MFve()
{
   const auto mass_matrix_Fve = get_mass_matrix_Fve();
   MFve = calculate_singlet_mass(mass_matrix_Fve);
}

double CLASSNAME::get_mass_matrix_Fvm() const
{
   const double mass_matrix_Fvm = Re(0);

   return mass_matrix_Fvm;
}

void CLASSNAME::calculate_MFvm()
{
   const auto mass_matrix_Fvm = get_mass_matrix_Fvm();
   MFvm = calculate_singlet_mass(mass_matrix_Fvm);
}

double CLASSNAME::get_mass_matrix_Fvt() const
{
   const double mass_matrix_Fvt = Re(0);

   return mass_matrix_Fvt;
}

void CLASSNAME::calculate_MFvt()
{
   const auto mass_matrix_Fvt = get_mass_matrix_Fvt();
   MFvt = calculate_singlet_mass(mass_matrix_Fvt);
}

double CLASSNAME::get_mass_matrix_Fe() const
{
   const double mass_matrix_Fe = Re(0.7071067811865475*vd*Ye(0,0));

   return mass_matrix_Fe;
}

void CLASSNAME::calculate_MFe()
{
   const auto mass_matrix_Fe = get_mass_matrix_Fe();
   MFe = calculate_singlet_mass(mass_matrix_Fe);
}

double CLASSNAME::get_mass_matrix_Fm() const
{
   const double mass_matrix_Fm = Re(0.7071067811865475*vd*Ye(1,1));

   return mass_matrix_Fm;
}

void CLASSNAME::calculate_MFm()
{
   const auto mass_matrix_Fm = get_mass_matrix_Fm();
   MFm = calculate_singlet_mass(mass_matrix_Fm);
}

double CLASSNAME::get_mass_matrix_Ftau() const
{
   const double mass_matrix_Ftau = Re(0.7071067811865475*vd*Ye(2,2));

   return mass_matrix_Ftau;
}

void CLASSNAME::calculate_MFtau()
{
   const auto mass_matrix_Ftau = get_mass_matrix_Ftau();
   MFtau = calculate_singlet_mass(mass_matrix_Ftau);
}

double CLASSNAME::get_mass_matrix_SveL() const
{
   const double mass_matrix_SveL = Re(0.125*(8*ml2(0,0) - 0.6*Sqr(g1)*(
      -Sqr(vd) + Sqr(vu)) - Sqr(g2)*(-Sqr(vd) + Sqr(vu))));

   return mass_matrix_SveL;
}

void CLASSNAME::calculate_MSveL()
{
   const auto mass_matrix_SveL = get_mass_matrix_SveL();
   MSveL = mass_matrix_SveL;

   if (MSveL < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::SveL);
   }

   MSveL = AbsSqrt(MSveL);
}

double CLASSNAME::get_mass_matrix_SvmL() const
{
   const double mass_matrix_SvmL = Re(0.125*(8*ml2(1,1) - 0.6*Sqr(g1)*(
      -Sqr(vd) + Sqr(vu)) - Sqr(g2)*(-Sqr(vd) + Sqr(vu))));

   return mass_matrix_SvmL;
}

void CLASSNAME::calculate_MSvmL()
{
   const auto mass_matrix_SvmL = get_mass_matrix_SvmL();
   MSvmL = mass_matrix_SvmL;

   if (MSvmL < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::SvmL);
   }

   MSvmL = AbsSqrt(MSvmL);
}

double CLASSNAME::get_mass_matrix_SvtL() const
{
   const double mass_matrix_SvtL = Re(0.125*(8*ml2(2,2) - 0.6*Sqr(g1)*(
      -Sqr(vd) + Sqr(vu)) - Sqr(g2)*(-Sqr(vd) + Sqr(vu))));

   return mass_matrix_SvtL;
}

void CLASSNAME::calculate_MSvtL()
{
   const auto mass_matrix_SvtL = get_mass_matrix_SvtL();
   MSvtL = mass_matrix_SvtL;

   if (MSvtL < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::SvtL);
   }

   MSvtL = AbsSqrt(MSvtL);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Sd() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Sd;

   mass_matrix_Sd(0,0) = mq2(0,0) + 0.5*AbsSqr(Yd(0,0))*Sqr(vd) - 0.025*
      Sqr(g1)*Sqr(vd) - 0.125*Sqr(g2)*Sqr(vd) + 0.025*Sqr(g1)*Sqr(vu) + 0.125*
      Sqr(g2)*Sqr(vu);
   mass_matrix_Sd(0,1) = 0.7071067811865475*vd*Conj(TYd(0,0)) -
      0.7071067811865475*vu*Conj(Yd(0,0))*Mu;
   mass_matrix_Sd(1,1) = md2(0,0) + 0.5*AbsSqr(Yd(0,0))*Sqr(vd) - 0.05*
      Sqr(g1)*Sqr(vd) + 0.05*Sqr(g1)*Sqr(vu);

   Hermitianize(mass_matrix_Sd);

   return mass_matrix_Sd;
}

void CLASSNAME::calculate_MSd()
{
   const auto mass_matrix_Sd(get_mass_matrix_Sd());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_Sd, MSd, ZD, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Sd, eigenvalue_error
      > precision * Abs(MSd(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_Sd, MSd, ZD);
#endif


   if (MSd.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::Sd);
   }

   MSd = AbsSqrt(MSd);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Su() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Su;

   mass_matrix_Su(0,0) = mq2(0,0) - 0.025*Sqr(g1)*Sqr(vd) + 0.125*Sqr(g2)
      *Sqr(vd) + 0.5*AbsSqr(Yu(0,0))*Sqr(vu) + 0.025*Sqr(g1)*Sqr(vu) - 0.125*
      Sqr(g2)*Sqr(vu);
   mass_matrix_Su(0,1) = 0.7071067811865475*vu*Conj(TYu(0,0)) -
      0.7071067811865475*vd*Conj(Yu(0,0))*Mu;
   mass_matrix_Su(1,1) = mu2(0,0) + 0.1*Sqr(g1)*Sqr(vd) + 0.5*AbsSqr(Yu(0
      ,0))*Sqr(vu) - 0.1*Sqr(g1)*Sqr(vu);

   Hermitianize(mass_matrix_Su);

   return mass_matrix_Su;
}

void CLASSNAME::calculate_MSu()
{
   const auto mass_matrix_Su(get_mass_matrix_Su());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_Su, MSu, ZU, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Su, eigenvalue_error
      > precision * Abs(MSu(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_Su, MSu, ZU);
#endif


   if (MSu.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::Su);
   }

   MSu = AbsSqrt(MSu);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Se() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Se;

   mass_matrix_Se(0,0) = ml2(0,0) + 0.5*AbsSqr(Ye(0,0))*Sqr(vd) + 0.075*
      Sqr(g1)*Sqr(vd) - 0.125*Sqr(g2)*Sqr(vd) - 0.075*Sqr(g1)*Sqr(vu) + 0.125*
      Sqr(g2)*Sqr(vu);
   mass_matrix_Se(0,1) = 0.7071067811865475*vd*Conj(TYe(0,0)) -
      0.7071067811865475*vu*Conj(Ye(0,0))*Mu;
   mass_matrix_Se(1,1) = me2(0,0) + 0.5*AbsSqr(Ye(0,0))*Sqr(vd) - 0.15*
      Sqr(g1)*Sqr(vd) + 0.15*Sqr(g1)*Sqr(vu);

   Hermitianize(mass_matrix_Se);

   return mass_matrix_Se;
}

void CLASSNAME::calculate_MSe()
{
   const auto mass_matrix_Se(get_mass_matrix_Se());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_Se, MSe, ZE, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Se, eigenvalue_error
      > precision * Abs(MSe(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_Se, MSe, ZE);
#endif


   if (MSe.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::Se);
   }

   MSe = AbsSqrt(MSe);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Sm() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Sm;

   mass_matrix_Sm(0,0) = ml2(1,1) + 0.5*AbsSqr(Ye(1,1))*Sqr(vd) + 0.075*
      Sqr(g1)*Sqr(vd) - 0.125*Sqr(g2)*Sqr(vd) - 0.075*Sqr(g1)*Sqr(vu) + 0.125*
      Sqr(g2)*Sqr(vu);
   mass_matrix_Sm(0,1) = 0.7071067811865475*vd*Conj(TYe(1,1)) -
      0.7071067811865475*vu*Conj(Ye(1,1))*Mu;
   mass_matrix_Sm(1,1) = me2(1,1) + 0.5*AbsSqr(Ye(1,1))*Sqr(vd) - 0.15*
      Sqr(g1)*Sqr(vd) + 0.15*Sqr(g1)*Sqr(vu);

   Hermitianize(mass_matrix_Sm);

   return mass_matrix_Sm;
}

void CLASSNAME::calculate_MSm()
{
   const auto mass_matrix_Sm(get_mass_matrix_Sm());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_Sm, MSm, ZM, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Sm, eigenvalue_error
      > precision * Abs(MSm(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_Sm, MSm, ZM);
#endif


   if (MSm.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::Sm);
   }

   MSm = AbsSqrt(MSm);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Stau() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Stau;

   mass_matrix_Stau(0,0) = ml2(2,2) + 0.5*AbsSqr(Ye(2,2))*Sqr(vd) + 0.075
      *Sqr(g1)*Sqr(vd) - 0.125*Sqr(g2)*Sqr(vd) - 0.075*Sqr(g1)*Sqr(vu) + 0.125*
      Sqr(g2)*Sqr(vu);
   mass_matrix_Stau(0,1) = 0.7071067811865475*vd*Conj(TYe(2,2)) -
      0.7071067811865475*vu*Conj(Ye(2,2))*Mu;
   mass_matrix_Stau(1,1) = me2(2,2) + 0.5*AbsSqr(Ye(2,2))*Sqr(vd) - 0.15*
      Sqr(g1)*Sqr(vd) + 0.15*Sqr(g1)*Sqr(vu);

   Hermitianize(mass_matrix_Stau);

   return mass_matrix_Stau;
}

void CLASSNAME::calculate_MStau()
{
   const auto mass_matrix_Stau(get_mass_matrix_Stau());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_Stau, MStau, ZTau,
      eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Stau,
      eigenvalue_error > precision * Abs(MStau(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_Stau, MStau, ZTau);
#endif


   if (MStau.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::Stau);
   }

   MStau = AbsSqrt(MStau);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Ss() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Ss;

   mass_matrix_Ss(0,0) = mq2(1,1) + 0.5*AbsSqr(Yd(1,1))*Sqr(vd) - 0.025*
      Sqr(g1)*Sqr(vd) - 0.125*Sqr(g2)*Sqr(vd) + 0.025*Sqr(g1)*Sqr(vu) + 0.125*
      Sqr(g2)*Sqr(vu);
   mass_matrix_Ss(0,1) = 0.7071067811865475*vd*Conj(TYd(1,1)) -
      0.7071067811865475*vu*Conj(Yd(1,1))*Mu;
   mass_matrix_Ss(1,1) = md2(1,1) + 0.5*AbsSqr(Yd(1,1))*Sqr(vd) - 0.05*
      Sqr(g1)*Sqr(vd) + 0.05*Sqr(g1)*Sqr(vu);

   Hermitianize(mass_matrix_Ss);

   return mass_matrix_Ss;
}

void CLASSNAME::calculate_MSs()
{
   const auto mass_matrix_Ss(get_mass_matrix_Ss());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_Ss, MSs, ZS, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Ss, eigenvalue_error
      > precision * Abs(MSs(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_Ss, MSs, ZS);
#endif


   if (MSs.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::Ss);
   }

   MSs = AbsSqrt(MSs);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Sc() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Sc;

   mass_matrix_Sc(0,0) = mq2(1,1) - 0.025*Sqr(g1)*Sqr(vd) + 0.125*Sqr(g2)
      *Sqr(vd) + 0.5*AbsSqr(Yu(1,1))*Sqr(vu) + 0.025*Sqr(g1)*Sqr(vu) - 0.125*
      Sqr(g2)*Sqr(vu);
   mass_matrix_Sc(0,1) = 0.7071067811865475*vu*Conj(TYu(1,1)) -
      0.7071067811865475*vd*Conj(Yu(1,1))*Mu;
   mass_matrix_Sc(1,1) = mu2(1,1) + 0.1*Sqr(g1)*Sqr(vd) + 0.5*AbsSqr(Yu(1
      ,1))*Sqr(vu) - 0.1*Sqr(g1)*Sqr(vu);

   Hermitianize(mass_matrix_Sc);

   return mass_matrix_Sc;
}

void CLASSNAME::calculate_MSc()
{
   const auto mass_matrix_Sc(get_mass_matrix_Sc());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_Sc, MSc, ZC, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Sc, eigenvalue_error
      > precision * Abs(MSc(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_Sc, MSc, ZC);
#endif


   if (MSc.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::Sc);
   }

   MSc = AbsSqrt(MSc);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Sb() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Sb;

   mass_matrix_Sb(0,0) = mq2(2,2) + 0.5*AbsSqr(Yd(2,2))*Sqr(vd) - 0.025*
      Sqr(g1)*Sqr(vd) - 0.125*Sqr(g2)*Sqr(vd) + 0.025*Sqr(g1)*Sqr(vu) + 0.125*
      Sqr(g2)*Sqr(vu);
   mass_matrix_Sb(0,1) = 0.7071067811865475*vd*Conj(TYd(2,2)) -
      0.7071067811865475*vu*Conj(Yd(2,2))*Mu;
   mass_matrix_Sb(1,1) = md2(2,2) + 0.5*AbsSqr(Yd(2,2))*Sqr(vd) - 0.05*
      Sqr(g1)*Sqr(vd) + 0.05*Sqr(g1)*Sqr(vu);

   Hermitianize(mass_matrix_Sb);

   return mass_matrix_Sb;
}

void CLASSNAME::calculate_MSb()
{
   const auto mass_matrix_Sb(get_mass_matrix_Sb());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_Sb, MSb, ZB, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Sb, eigenvalue_error
      > precision * Abs(MSb(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_Sb, MSb, ZB);
#endif


   if (MSb.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::Sb);
   }

   MSb = AbsSqrt(MSb);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_St() const
{
   Eigen::Matrix<double,2,2> mass_matrix_St;

   mass_matrix_St(0,0) = mq2(2,2) - 0.025*Sqr(g1)*Sqr(vd) + 0.125*Sqr(g2)
      *Sqr(vd) + 0.5*AbsSqr(Yu(2,2))*Sqr(vu) + 0.025*Sqr(g1)*Sqr(vu) - 0.125*
      Sqr(g2)*Sqr(vu);
   mass_matrix_St(0,1) = 0.7071067811865475*vu*Conj(TYu(2,2)) -
      0.7071067811865475*vd*Conj(Yu(2,2))*Mu;
   mass_matrix_St(1,1) = mu2(2,2) + 0.1*Sqr(g1)*Sqr(vd) + 0.5*AbsSqr(Yu(2
      ,2))*Sqr(vu) - 0.1*Sqr(g1)*Sqr(vu);

   Hermitianize(mass_matrix_St);

   return mass_matrix_St;
}

void CLASSNAME::calculate_MSt()
{
   const auto mass_matrix_St(get_mass_matrix_St());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_St, MSt, ZT, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::St, eigenvalue_error
      > precision * Abs(MSt(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_St, MSt, ZT);
#endif


   if (MSt.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::St);
   }

   MSt = AbsSqrt(MSt);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_hh() const
{
   Eigen::Matrix<double,2,2> mass_matrix_hh;

   mass_matrix_hh(0,0) = mHd2 + AbsSqr(Mu) + 0.225*Sqr(g1)*Sqr(vd) +
      0.375*Sqr(g2)*Sqr(vd) - 0.075*Sqr(g1)*Sqr(vu) - 0.125*Sqr(g2)*Sqr(vu);
   mass_matrix_hh(0,1) = -0.5*BMu - 0.5*Conj(BMu) - 0.15*vd*vu*Sqr(g1) -
      0.25*vd*vu*Sqr(g2);
   mass_matrix_hh(1,1) = mHu2 + AbsSqr(Mu) - 0.075*Sqr(g1)*Sqr(vd) -
      0.125*Sqr(g2)*Sqr(vd) + 0.225*Sqr(g1)*Sqr(vu) + 0.375*Sqr(g2)*Sqr(vu);

   Symmetrize(mass_matrix_hh);

   return mass_matrix_hh;
}

void CLASSNAME::calculate_Mhh()
{
   const auto mass_matrix_hh(get_mass_matrix_hh());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_hh, Mhh, ZH, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::hh, eigenvalue_error
      > precision * Abs(Mhh(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_hh, Mhh, ZH);
#endif


   if (Mhh.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::hh);
   }

   Mhh = AbsSqrt(Mhh);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Ah() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Ah;

   mass_matrix_Ah(0,0) = mHd2 + AbsSqr(Mu) + 0.3872983346207417*g1*g2*Cos
      (ThetaW())*Sin(ThetaW())*Sqr(vd) + 0.075*Sqr(g1)*Sqr(vd) + 0.125*Sqr(g2)*
      Sqr(vd) - 0.075*Sqr(g1)*Sqr(vu) - 0.125*Sqr(g2)*Sqr(vu) + 0.25*Sqr(g2)*
      Sqr(vd)*Sqr(Cos(ThetaW())) + 0.15*Sqr(g1)*Sqr(vd)*Sqr(Sin(ThetaW()));
   mass_matrix_Ah(0,1) = 0.5*BMu + 0.5*Conj(BMu) - 0.3872983346207417*g1*
      g2*vd*vu*Cos(ThetaW())*Sin(ThetaW()) - 0.25*vd*vu*Sqr(g2)*Sqr(Cos(ThetaW(
      ))) - 0.15*vd*vu*Sqr(g1)*Sqr(Sin(ThetaW()));
   mass_matrix_Ah(1,1) = mHu2 + AbsSqr(Mu) - 0.075*Sqr(g1)*Sqr(vd) -
      0.125*Sqr(g2)*Sqr(vd) + 0.3872983346207417*g1*g2*Cos(ThetaW())*Sin(ThetaW
      ())*Sqr(vu) + 0.075*Sqr(g1)*Sqr(vu) + 0.125*Sqr(g2)*Sqr(vu) + 0.25*Sqr(g2
      )*Sqr(vu)*Sqr(Cos(ThetaW())) + 0.15*Sqr(g1)*Sqr(vu)*Sqr(Sin(ThetaW()));

   Symmetrize(mass_matrix_Ah);

   return mass_matrix_Ah;
}

void CLASSNAME::calculate_MAh()
{
   const auto mass_matrix_Ah(get_mass_matrix_Ah());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_Ah, MAh, ZA, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Ah, eigenvalue_error
      > precision * Abs(MAh(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_Ah, MAh, ZA);
#endif


   if (MAh.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::Ah);
   }

   MAh = AbsSqrt(MAh);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Hpm() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Hpm;

   mass_matrix_Hpm(0,0) = mHd2 + AbsSqr(Mu) + 0.075*Sqr(g1)*Sqr(vd) +
      0.375*Sqr(g2)*Sqr(vd) - 0.075*Sqr(g1)*Sqr(vu) + 0.125*Sqr(g2)*Sqr(vu);
   mass_matrix_Hpm(0,1) = Conj(BMu);
   mass_matrix_Hpm(1,1) = mHu2 + AbsSqr(Mu) - 0.075*Sqr(g1)*Sqr(vd) +
      0.125*Sqr(g2)*Sqr(vd) + 0.075*Sqr(g1)*Sqr(vu) + 0.375*Sqr(g2)*Sqr(vu);

   Hermitianize(mass_matrix_Hpm);

   return mass_matrix_Hpm;
}

void CLASSNAME::calculate_MHpm()
{
   const auto mass_matrix_Hpm(get_mass_matrix_Hpm());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_Hpm, MHpm, ZP, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Hpm, eigenvalue_error
      > precision * Abs(MHpm(0)));
#else
   fs_diagonalize_hermitian(mass_matrix_Hpm, MHpm, ZP);
#endif


   if (MHpm.minCoeff() < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::Hpm);
   }

   MHpm = AbsSqrt(MHpm);
}

Eigen::Matrix<double,4,4> CLASSNAME::get_mass_matrix_Chi() const
{
   Eigen::Matrix<double,4,4> mass_matrix_Chi;

   mass_matrix_Chi(0,0) = MassB;
   mass_matrix_Chi(0,1) = 0;
   mass_matrix_Chi(0,2) = -0.3872983346207417*g1*vd;
   mass_matrix_Chi(0,3) = 0.3872983346207417*g1*vu;
   mass_matrix_Chi(1,1) = MassWB;
   mass_matrix_Chi(1,2) = 0.5*g2*vd;
   mass_matrix_Chi(1,3) = -0.5*g2*vu;
   mass_matrix_Chi(2,2) = 0;
   mass_matrix_Chi(2,3) = -Mu;
   mass_matrix_Chi(3,3) = 0;

   Symmetrize(mass_matrix_Chi);

   return mass_matrix_Chi;
}

void CLASSNAME::calculate_MChi()
{
   const auto mass_matrix_Chi(get_mass_matrix_Chi());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_symmetric(mass_matrix_Chi, MChi, ZN, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Chi, eigenvalue_error
      > precision * Abs(MChi(0)));
#else
   fs_diagonalize_symmetric(mass_matrix_Chi, MChi, ZN);
#endif

}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_Cha() const
{
   Eigen::Matrix<double,2,2> mass_matrix_Cha;

   mass_matrix_Cha(0,0) = MassWB;
   mass_matrix_Cha(0,1) = 0.7071067811865475*g2*vu;
   mass_matrix_Cha(1,0) = 0.7071067811865475*g2*vd;
   mass_matrix_Cha(1,1) = Mu;

   return mass_matrix_Cha;
}

void CLASSNAME::calculate_MCha()
{
   const auto mass_matrix_Cha(get_mass_matrix_Cha());


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_svd(mass_matrix_Cha, MCha, UM, UP, eigenvalue_error);
   problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Cha, eigenvalue_error
      > precision * Abs(MCha(0)));
#else
   fs_svd(mass_matrix_Cha, MCha, UM, UP);
#endif

}

double CLASSNAME::get_mass_matrix_VWm() const
{
   const double mass_matrix_VWm = Re(0.25*Sqr(g2)*(Sqr(vd) + Sqr(vu)));

   return mass_matrix_VWm;
}

void CLASSNAME::calculate_MVWm()
{
   const auto mass_matrix_VWm = get_mass_matrix_VWm();
   MVWm = mass_matrix_VWm;

   if (MVWm < 0.) {
      problems.flag_running_tachyon(NUHMSSMNoFVHimalaya_info::VWm);
   }

   MVWm = AbsSqrt(MVWm);
}

Eigen::Matrix<double,2,2> CLASSNAME::get_mass_matrix_VPVZ() const
{
   Eigen::Matrix<double,2,2> mass_matrix_VPVZ;

   mass_matrix_VPVZ(0,0) = 0.15*Sqr(g1)*Sqr(vd) + 0.15*Sqr(g1)*Sqr(vu);
   mass_matrix_VPVZ(0,1) = -0.19364916731037085*g1*g2*Sqr(vd) -
      0.19364916731037085*g1*g2*Sqr(vu);
   mass_matrix_VPVZ(1,1) = 0.25*Sqr(g2)*Sqr(vd) + 0.25*Sqr(g2)*Sqr(vu);

   Symmetrize(mass_matrix_VPVZ);

   return mass_matrix_VPVZ;
}

void CLASSNAME::calculate_MVPVZ()
{
   const auto mass_matrix_VPVZ(get_mass_matrix_VPVZ());
   Eigen::Array<double,2,1> MVPVZ;


#ifdef CHECK_EIGENVALUE_ERROR
   double eigenvalue_error;
   fs_diagonalize_hermitian(mass_matrix_VPVZ, MVPVZ, ZZ, eigenvalue_error
      );
   ZZ.transposeInPlace();
#else
   fs_diagonalize_hermitian(mass_matrix_VPVZ, MVPVZ, ZZ);
   ZZ.transposeInPlace();
#endif


   MVPVZ = AbsSqrt(MVPVZ);

   MVP = 0.;
   MVZ = MVPVZ(1);
}


double CLASSNAME::get_ewsb_eq_hh_1() const
{
   double result = Re(mHd2*vd + vd*AbsSqr(Mu) - 0.5*vu*BMu - 0.5*vu*Conj(BMu) +
      0.075*Cube(vd)*Sqr(g1) + 0.125*Cube(vd)*Sqr(g2) - 0.075*vd*Sqr(g1)*Sqr(vu)
      - 0.125*vd*Sqr(g2)*Sqr(vu));

   return result;
}

double CLASSNAME::get_ewsb_eq_hh_2() const
{
   double result = Re(mHu2*vu + vu*AbsSqr(Mu) - 0.5*vd*BMu - 0.5*vd*Conj(BMu) +
      0.075*Cube(vu)*Sqr(g1) + 0.125*Cube(vu)*Sqr(g2) - 0.075*vu*Sqr(g1)*Sqr(vd)
      - 0.125*vu*Sqr(g2)*Sqr(vd));

   return result;
}



std::complex<double> CLASSNAME::CpUSdconjUSdVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.03333333333333333*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(g1*Sin(ThetaW())*(7.745966692414834*g2*Cos(
      ThetaW()) + g1*Sin(ThetaW())) + 15*Sqr(g2)*Sqr(Cos(ThetaW()))) + 4*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1)*Sqr(Sin(ThetaW())));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) + 5*Sqr(g2)));

   return result;
}

double CLASSNAME::CpUSdconjUSdconjVWmVWm(int gO1, int gO2) const
{
   const double result = 0.5*KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g2
      );

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Yd(0,0)) + Sqr(g1))*ZA(gI1,0)*ZA(gI2,0) -
      Sqr(g1)*ZA(gI1,1)*ZA(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((-20*AbsSqr(Yd(0,0)) + Sqr(g1) + 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) - (Sqr(g1)
      + 5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Yd(0,0)) + Sqr(g1))*ZP(gI1,0)*ZP(gI2,0) -
      Sqr(g1)*ZP(gI1,1)*ZP(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((Sqr(g1) - 5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) - (20*AbsSqr(Yu(0,0)) + Sqr(g1) -
      5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZB(gI2,1))*(30*Conj(Yd(2,2
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(0,0)*ZB(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZB(gI1,1)) - Conj(ZB(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZB(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZB(gI1,0) + 30*Conj(Yd(
      0,0))*KroneckerDelta(0,gO2)*Yd(2,2)*ZB(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZC(gI2,0))*(2*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZC(gI1,0)) + 4*Conj(ZC(gI2,1))
      *(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(-(Conj(ZD(gI2,1))*(
      8*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g3))*ZD(gI1,1
      ) + KroneckerDelta(0,gO1)*(KroneckerDelta(1,gO2)*(90*AbsSqr(Yd(0,0)) + Sqr(
      g1) - 40*Sqr(g3))*ZD(gI1,0) + 3*KroneckerDelta(0,gO2)*(10*AbsSqr(Yd(0,0)) +
      Sqr(g1))*ZD(gI1,1)))) - Conj(ZD(gI2,0))*(2*KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2) + 20*Sqr(g3))*ZD(gI1,0) +
      KroneckerDelta(1,gO1)*(3*KroneckerDelta(1,gO2)*(10*AbsSqr(Yd(0,0)) + Sqr(g1)
      )*ZD(gI1,0) + KroneckerDelta(0,gO2)*(90*AbsSqr(Yd(0,0)) + Sqr(g1) - 40*Sqr(
      g3))*ZD(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZE(gI2,1))*(10*Conj(Ye(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(0,0)*ZE(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZE(gI1,1)) + Conj(ZE(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZE(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZE(gI1,0) - 10*Conj(Yd(
      0,0))*KroneckerDelta(0,gO2)*Ye(0,0)*ZE(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZM(gI2,1))*(10*Conj(Ye(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(0,0)*ZM(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZM(gI1,1)) + Conj(ZM(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZM(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZM(gI1,0) - 10*Conj(Yd(
      0,0))*KroneckerDelta(0,gO2)*Ye(1,1)*ZM(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZS(gI2,1))*(30*Conj(Yd(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(0,0)*ZS(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZS(gI1,1)) - Conj(ZS(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZS(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZS(gI1,0) + 30*Conj(Yd(
      0,0))*KroneckerDelta(0,gO2)*Yd(1,1)*ZS(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZT(gI2,0))*(2*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZT(gI1,0)) + 4*Conj(ZT(gI2,1))
      *(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZTau(gI2,1))*(10*Conj(Ye(2
      ,2))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(0,0)*ZTau(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZTau(gI1,1)) + Conj(ZTau(gI2,0))*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZTau(gI1,0
      ) + 2*KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZTau(gI1,0) - 10*
      Conj(Yd(0,0))*KroneckerDelta(0,gO2)*Ye(2,2)*ZTau(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZU(gI2,0))*(2*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(10*AbsSqr(Yd(0,0)) + Sqr(g1)) +
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)))*ZU(gI1,0
      )) + 4*Conj(ZU(gI2,1))*(2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1
      ) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-5*AbsSqr(Yu(0,0)) + Sqr(g1
      )))*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSdconjUSdhhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Yd(0,0)) + Sqr(g1))*ZH(gI1,0)*ZH(gI2,0) -
      Sqr(g1)*ZH(gI1,1)*ZH(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((-20*AbsSqr(Yd(0,0)) + Sqr(g1) + 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) - (Sqr(g1)
      + 5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdSdAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZD(gI1,1))*KroneckerDelta(0,gO2)*(Conj(TYd(0,0))*
      ZA(gI2,0) + Conj(Yd(0,0))*Mu*ZA(gI2,1)) - Conj(ZD(gI1,0))*KroneckerDelta(1,
      gO2)*(Conj(Mu)*Yd(0,0)*ZA(gI2,1) + ZA(gI2,0)*TYd(0,0)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdSdhh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZD(gI1,1))*(
      7.0710678118654755*Conj(TYd(0,0))*KroneckerDelta(0,gO2)*ZH(gI2,0) -
      7.0710678118654755*Conj(Yd(0,0))*KroneckerDelta(0,gO2)*Mu*ZH(gI2,1) +
      KroneckerDelta(1,gO2)*(-(vd*(-10*AbsSqr(Yd(0,0)) + Sqr(g1))*ZH(gI2,0)) + vu*
      Sqr(g1)*ZH(gI2,1))) + Conj(ZD(gI1,0))*(KroneckerDelta(0,gO2)*(vd*(-20*AbsSqr
      (Yd(0,0)) + Sqr(g1) + 5*Sqr(g2))*ZH(gI2,0) - vu*(Sqr(g1) + 5*Sqr(g2))*ZH(gI2
      ,1)) + 14.142135623730951*KroneckerDelta(1,gO2)*(Conj(Mu)*Yd(0,0)*ZH(gI2,1)
      - ZH(gI2,0)*TYd(0,0))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdSuHpm(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZU(gI1,1))*(2*Conj(TYu(0,0))*
      KroneckerDelta(0,gO2)*ZP(gI2,1) + Conj(Yu(0,0))*(2*KroneckerDelta(0,gO2)*Mu*
      ZP(gI2,0) + 1.4142135623730951*KroneckerDelta(1,gO2)*Yd(0,0)*(vu*ZP(gI2,0) +
      vd*ZP(gI2,1)))) - 0.25*Conj(ZU(gI1,0))*(1.4142135623730951*KroneckerDelta(0
      ,gO2)*(vd*(-2*AbsSqr(Yd(0,0)) + Sqr(g2))*ZP(gI2,0) + vu*(-2*AbsSqr(Yu(0,0))
      + Sqr(g2))*ZP(gI2,1)) - 4*KroneckerDelta(1,gO2)*(Conj(Mu)*Yd(0,0)*ZP(gI2,1)
      + ZP(gI2,0)*TYd(0,0)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdVGSd(int gO2, int gI2) const
{
   const std::complex<double> result = IF(gI2 < 2,g3*Conj(ZD(gI2,gO2)),0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdVPSd(int gO2, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(-7.745966692414834*
      g1*Conj(ZD(gI2,1))*Cos(ThetaW())*KroneckerDelta(1,gO2) + Conj(ZD(gI2,0))*
      KroneckerDelta(0,gO2)*(3.872983346207417*g1*Cos(ThetaW()) - 15*g2*Sin(ThetaW
      ())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdVZSd(int gO2, int gI2) const
{
   const std::complex<double> result = 0.2581988897471611*g1*Conj(ZD(gI2,1))*
      KroneckerDelta(1,gO2)*Sin(ThetaW()) - 0.03333333333333333*Conj(ZD(gI2,0))*
      KroneckerDelta(0,gO2)*(15*g2*Cos(ThetaW()) + 3.872983346207417*g1*Sin(ThetaW
      ()));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdVWmSu(int gO2, int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZU(gI2,0))*
      KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdFuChaPR(int gO2, int , int gI2) const
{
   const std::complex<double> result = Conj(Yu(0,0))*KroneckerDelta(0,gO2)*UP(
      gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdFuChaPL(int gO1, int , int gI2) const
{
   const std::complex<double> result = -(g2*Conj(UM(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(UM(gI2,1))*KroneckerDelta(1,gO1)*Yd(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdFdChiPR(int gO2, int , int gI2) const
{
   const std::complex<double> result = -0.3651483716701107*g1*KroneckerDelta(1,
      gO2)*ZN(gI2,0) - Conj(Yd(0,0))*KroneckerDelta(0,gO2)*ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdFdChiPL(int gO1, int , int gI2) const
{
   const std::complex<double> result = -0.18257418583505536*g1*Conj(ZN(gI2,0))*
      KroneckerDelta(0,gO1) + 0.7071067811865475*g2*Conj(ZN(gI2,1))*KroneckerDelta
      (0,gO1) - Conj(ZN(gI2,2))*KroneckerDelta(1,gO1)*Yd(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdGluFdPR(int gO2, int , int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*
      KroneckerDelta(1,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSdGluFdPL(int gO1, int , int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*
      KroneckerDelta(0,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.03333333333333333*(16*KroneckerDelta(1
      ,gO1)*KroneckerDelta(1,gO2)*Sqr(g1)*Sqr(Sin(ThetaW())) + KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(-7.745966692414834*g1*g2*Cos(ThetaW())*Sin(
      ThetaW()) + 15*Sqr(g2)*Sqr(Cos(ThetaW())) + Sqr(g1)*Sqr(Sin(ThetaW()))));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) - 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) - 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) - 5*Sqr(g2)));

   return result;
}

double CLASSNAME::CpUSuconjUSuconjVWmVWm(int gO1, int gO2) const
{
   const double result = 0.5*KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g2
      );

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(Sqr(g1)*ZA(gI1,0)*ZA(gI2,0) - (-5*AbsSqr(Yu(0,0)) +
      Sqr(g1))*ZA(gI1,1)*ZA(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((Sqr(g1) - 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) - (20*AbsSqr(Yu(0,0)) + Sqr(g1) -
      5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(Sqr(g1)*ZP(gI1,0)*ZP(gI2,0) - (-5*AbsSqr(Yu(0,0)) +
      Sqr(g1))*ZP(gI1,1)*ZP(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((-20*AbsSqr(Yd(0,0)) + Sqr(g1) + 5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) - (Sqr(g1)
      + 5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZB(gI2,0))*(4*KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) - KroneckerDelta(0,gO1)*KroneckerDelta(
      0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZB(gI1,0) - 2*Conj(ZB(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(4*Conj(ZC(gI2,1))*(-15*Conj(Yu(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yu(0,0)*ZC(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZC(gI1,1)) - Conj(ZC(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZC(gI1,0) - 4*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZC(gI1,0) - 15*Conj(Yu(
      0,0))*KroneckerDelta(0,gO2)*Yu(1,1)*ZC(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZD(gI2,0))*(4*KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(0,0)) + Sqr(g1)) - KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)))*ZD(gI1,0) - 2*Conj(ZD(
      gI2,1))*(-4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) +
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(10*AbsSqr(Yd(0,0)) + Sqr(g1)))*
      ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZE(gI2,0))*(-4*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZE(gI1,0) - 2*Conj(ZE(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZM(gI2,0))*(-4*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZM(gI1,0) - 2*Conj(ZM(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZS(gI2,0))*(4*KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) - KroneckerDelta(0,gO1)*KroneckerDelta(
      0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZS(gI1,0) - 2*Conj(ZS(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(4*Conj(ZT(gI2,1))*(-15*Conj(Yu(2,2
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yu(0,0)*ZT(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZT(gI1,1)) - Conj(ZT(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZT(gI1,0) - 4*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZT(gI1,0) - 15*Conj(Yu(
      0,0))*KroneckerDelta(0,gO2)*Yu(2,2)*ZT(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZTau(gI2,0))*(-4*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZTau(gI1,0) - 2*Conj(ZTau(gI2,1
      ))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.06666666666666667*(Conj(ZU(gI2,1))*(-4
      *KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(4*Sqr(g1) + 5*Sqr(g3))*ZU(gI1,
      1) + KroneckerDelta(0,gO1)*(KroneckerDelta(1,gO2)*(-45*AbsSqr(Yu(0,0)) + Sqr
      (g1) + 20*Sqr(g3))*ZU(gI1,0) + 3*KroneckerDelta(0,gO2)*(-5*AbsSqr(Yu(0,0)) +
      Sqr(g1))*ZU(gI1,1))) - Conj(ZU(gI2,0))*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2) + 20*Sqr(g3))*ZU(gI1,0) -
      KroneckerDelta(1,gO1)*(3*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(0,0)) + Sqr(g1)
      )*ZU(gI1,0) + KroneckerDelta(0,gO2)*(-45*AbsSqr(Yu(0,0)) + Sqr(g1) + 20*Sqr(
      g3))*ZU(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSuconjUSuhhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(Sqr(g1)*ZH(gI1,0)*ZH(gI2,0) - (-5*AbsSqr(Yu(0,0)) +
      Sqr(g1))*ZH(gI1,1)*ZH(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((Sqr(g1) - 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) - (20*AbsSqr(Yu(0,0)) + Sqr(g1) -
      5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuconjHpmSd(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZD(gI2,1))*(2*Conj(TYd(0,0))*
      KroneckerDelta(0,gO2)*ZP(gI1,0) + Conj(Yd(0,0))*(2*KroneckerDelta(0,gO2)*Mu*
      ZP(gI1,1) + 1.4142135623730951*KroneckerDelta(1,gO2)*Yu(0,0)*(vu*ZP(gI1,0) +
      vd*ZP(gI1,1)))) - 0.25*Conj(ZD(gI2,0))*(1.4142135623730951*KroneckerDelta(0
      ,gO2)*(vd*(-2*AbsSqr(Yd(0,0)) + Sqr(g2))*ZP(gI1,0) + vu*(-2*AbsSqr(Yu(0,0))
      + Sqr(g2))*ZP(gI1,1)) - 4*KroneckerDelta(1,gO2)*(Conj(Mu)*Yu(0,0)*ZP(gI1,0)
      + ZP(gI1,1)*TYu(0,0)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuSuAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(Yu(0,0))*Conj(ZU(gI1,1))*KroneckerDelta(0,gO2)*Mu*
      ZA(gI2,0) - Conj(Mu)*Conj(ZU(gI1,0))*KroneckerDelta(1,gO2)*Yu(0,0)*ZA(gI2,0)
      + ZA(gI2,1)*(Conj(ZU(gI1,1))*Conj(TYu(0,0))*KroneckerDelta(0,gO2) - Conj(ZU
      (gI1,0))*KroneckerDelta(1,gO2)*TYu(0,0)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuSuhh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZU(gI1,1))*(
      -7.0710678118654755*Conj(TYu(0,0))*KroneckerDelta(0,gO2)*ZH(gI2,1) + 2*
      KroneckerDelta(1,gO2)*Sqr(g1)*(-(vd*ZH(gI2,0)) + vu*ZH(gI2,1)) + 5*Conj(Yu(0
      ,0))*(1.4142135623730951*KroneckerDelta(0,gO2)*Mu*ZH(gI2,0) - 2*vu*
      KroneckerDelta(1,gO2)*Yu(0,0)*ZH(gI2,1))) + Conj(ZU(gI1,0))*(KroneckerDelta(
      0,gO2)*(vd*(Sqr(g1) - 5*Sqr(g2))*ZH(gI2,0) - vu*(20*AbsSqr(Yu(0,0)) + Sqr(g1
      ) - 5*Sqr(g2))*ZH(gI2,1)) + 14.142135623730951*KroneckerDelta(1,gO2)*(Conj(
      Mu)*Yu(0,0)*ZH(gI2,0) - ZH(gI2,1)*TYu(0,0))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSubarChaFdPR(int gO2, int gI1, int ) const
{
   const std::complex<double> result = Conj(Yd(0,0))*KroneckerDelta(0,gO2)*UM(
      gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSubarChaFdPL(int gO1, int gI1, int ) const
{
   const std::complex<double> result = -(g2*Conj(UP(gI1,0))*KroneckerDelta(0,
      gO1)) + Conj(UP(gI1,1))*KroneckerDelta(1,gO1)*Yu(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuconjVWmSd(int gO2, int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZD(gI2,0))*
      KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuVGSu(int gO2, int gI2) const
{
   const std::complex<double> result = IF(gI2 < 2,g3*Conj(ZU(gI2,gO2)),0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuVPSu(int gO2, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(15.491933384829668*
      g1*Conj(ZU(gI2,1))*Cos(ThetaW())*KroneckerDelta(1,gO2) + Conj(ZU(gI2,0))*
      KroneckerDelta(0,gO2)*(3.872983346207417*g1*Cos(ThetaW()) + 15*g2*Sin(ThetaW
      ())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuVZSu(int gO2, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(-15.491933384829668
      *g1*Conj(ZU(gI2,1))*KroneckerDelta(1,gO2)*Sin(ThetaW()) + Conj(ZU(gI2,0))*
      KroneckerDelta(0,gO2)*(15*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW
      ())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuFuChiPR(int gO2, int , int gI2) const
{
   const std::complex<double> result = 0.7302967433402214*g1*KroneckerDelta(1,
      gO2)*ZN(gI2,0) - Conj(Yu(0,0))*KroneckerDelta(0,gO2)*ZN(gI2,3);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuFuChiPL(int gO1, int , int gI2) const
{
   const std::complex<double> result = -0.18257418583505536*g1*Conj(ZN(gI2,0))*
      KroneckerDelta(0,gO1) - 0.7071067811865475*g2*Conj(ZN(gI2,1))*KroneckerDelta
      (0,gO1) - Conj(ZN(gI2,3))*KroneckerDelta(1,gO1)*Yu(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuGluFuPR(int gO2, int , int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*
      KroneckerDelta(1,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSuGluFuPL(int gO1, int , int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*
      KroneckerDelta(0,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.1*(12*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1)*Sqr(Sin(ThetaW())) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-7.745966692414834*g1*g2*Cos(ThetaW())*Sin(ThetaW())
      + 5*Sqr(g2)*Sqr(Cos(ThetaW())) + 3*Sqr(g1)*Sqr(Sin(ThetaW()))));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(-10*AbsSqr(Ye(0,0)) + 3*Sqr(g1)) - KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(6*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(6*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

double CLASSNAME::CpUSeconjUSeconjVWmVWm(int gO1, int gO2) const
{
   const double result = 0.5*KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g2
      );

   return result;
}

double CLASSNAME::CpconjUSeVWmSveL(int gO2) const
{
   const double result = -0.7071067811865475*g2*KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Ye(0,0)) + 3*Sqr(g1))*ZA(gI1,0)*ZA(gI2,0)
      - 3*Sqr(g1)*ZA(gI1,1)*ZA(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,
      gO2)*((-20*AbsSqr(Ye(0,0)) - 3*Sqr(g1) + 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) + (3
      *Sqr(g1) - 5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*(ZP(gI1,0)*ZP(gI2,0) - ZP(gI1,
      1)*ZP(gI2,1))) + 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*((-10*AbsSqr(
      Ye(0,0)) + 3*Sqr(g1))*ZP(gI1,0)*ZP(gI2,0) - 3*Sqr(g1)*ZP(gI1,1)*ZP(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZB(gI2,1))*(-10*Conj(Yd(2,2
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(0,0)*ZB(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZB(gI1,1)) + Conj(ZB(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZB(gI1,0) - 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZB(gI1,0) + 10*Conj(Ye(
      0,0))*KroneckerDelta(0,gO2)*Yd(2,2)*ZB(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZC(gI2,0))*(-2*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZC(gI1,0) - 4*Conj(ZC(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZD(gI2,1))*(-10*Conj(Yd(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(0,0)*ZD(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZD(gI1,1)) + Conj(ZD(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZD(gI1,0) - 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZD(gI1,0) + 10*Conj(Ye(
      0,0))*KroneckerDelta(0,gO2)*Yd(0,0)*ZD(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZE(gI2,1))*(-12*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1)*ZE(gI1,1) + KroneckerDelta(0,gO1)*(-10
      *AbsSqr(Ye(0,0)) + 3*Sqr(g1))*(KroneckerDelta(1,gO2)*ZE(gI1,0) +
      KroneckerDelta(0,gO2)*ZE(gI1,1))) - Conj(ZE(gI2,0))*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*ZE(gI1,0) - KroneckerDelta(1,
      gO1)*(-10*AbsSqr(Ye(0,0)) + 3*Sqr(g1))*(KroneckerDelta(1,gO2)*ZE(gI1,0) +
      KroneckerDelta(0,gO2)*ZE(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZM(gI2,1))*(10*Conj(Ye(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(0,0)*ZM(gI1,0) - 3*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZM(gI1,1)) - Conj(ZM(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*ZM(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(-3*KroneckerDelta(1,gO2)*Sqr(g1)*ZM(gI1,0) + 10*Conj(
      Ye(0,0))*KroneckerDelta(0,gO2)*Ye(1,1)*ZM(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZS(gI2,1))*(-10*Conj(Yd(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(0,0)*ZS(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZS(gI1,1)) + Conj(ZS(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZS(gI1,0) - 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZS(gI1,0) + 10*Conj(Ye(
      0,0))*KroneckerDelta(0,gO2)*Yd(1,1)*ZS(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZT(gI2,0))*(-2*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZT(gI1,0) - 4*Conj(ZT(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZTau(gI2,1))*(10*Conj(Ye(2
      ,2))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(0,0)*ZTau(gI1,0) - 3*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZTau(gI1,1)) - Conj(ZTau(gI2,0))*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*ZTau(gI1
      ,0) + 2*KroneckerDelta(1,gO1)*(-3*KroneckerDelta(1,gO2)*Sqr(g1)*ZTau(gI1,0)
      + 10*Conj(Ye(0,0))*KroneckerDelta(0,gO2)*Ye(2,2)*ZTau(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSeconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZU(gI2,0))*(-2*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZU(gI1,0) - 4*Conj(ZU(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSeconjUSehhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Ye(0,0)) + 3*Sqr(g1))*ZH(gI1,0)*ZH(gI2,0)
      - 3*Sqr(g1)*ZH(gI1,1)*ZH(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,
      gO2)*((-20*AbsSqr(Ye(0,0)) - 3*Sqr(g1) + 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) + (3
      *Sqr(g1) - 5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSeSeAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZE(gI1,1))*KroneckerDelta(0,gO2)*(Conj(TYe(0,0))*
      ZA(gI2,0) + Conj(Ye(0,0))*Mu*ZA(gI2,1)) - Conj(ZE(gI1,0))*KroneckerDelta(1,
      gO2)*(Conj(Mu)*Ye(0,0)*ZA(gI2,1) + ZA(gI2,0)*TYe(0,0)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSeSehh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZE(gI1,1))*(
      7.0710678118654755*Conj(TYe(0,0))*KroneckerDelta(0,gO2)*ZH(gI2,0) -
      7.0710678118654755*Conj(Ye(0,0))*KroneckerDelta(0,gO2)*Mu*ZH(gI2,1) +
      KroneckerDelta(1,gO2)*((10*vd*AbsSqr(Ye(0,0)) - 3*vd*Sqr(g1))*ZH(gI2,0) + 3*
      vu*Sqr(g1)*ZH(gI2,1))) + Conj(ZE(gI1,0))*(KroneckerDelta(0,gO2)*(vd*(-20*
      AbsSqr(Ye(0,0)) - 3*Sqr(g1) + 5*Sqr(g2))*ZH(gI2,0) + vu*(3*Sqr(g1) - 5*Sqr(
      g2))*ZH(gI2,1)) + 14.142135623730951*KroneckerDelta(1,gO2)*(Conj(Mu)*Ye(0,0)
      *ZH(gI2,1) - ZH(gI2,0)*TYe(0,0))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSeSveLHpm(int gO2, int gI2) const
{
   const std::complex<double> result = 0.25*(-1.4142135623730951*KroneckerDelta
      (0,gO2)*(vd*(-2*AbsSqr(Ye(0,0)) + Sqr(g2))*ZP(gI2,0) + vu*Sqr(g2)*ZP(gI2,1))
      + 4*KroneckerDelta(1,gO2)*(Conj(Mu)*Ye(0,0)*ZP(gI2,1) + ZP(gI2,0)*TYe(0,0))
      );

   return result;
}

std::complex<double> CLASSNAME::CpconjUSeVPSe(int gO2, int gI2) const
{
   const std::complex<double> result = 0.1*(-7.745966692414834*g1*Conj(ZE(gI2,1
      ))*Cos(ThetaW())*KroneckerDelta(1,gO2) - Conj(ZE(gI2,0))*KroneckerDelta(0,
      gO2)*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(ThetaW())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSeVZSe(int gO2, int gI2) const
{
   const std::complex<double> result = 0.1*(7.745966692414834*g1*Conj(ZE(gI2,1)
      )*KroneckerDelta(1,gO2)*Sin(ThetaW()) - Conj(ZE(gI2,0))*KroneckerDelta(0,gO2
      )*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW())));

   return result;
}

double CLASSNAME::CpconjUSeFveChaPR(int , int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpconjUSeFveChaPL(int gO1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(UM(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(UM(gI2,1))*KroneckerDelta(1,gO1)*Ye(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSeFeChiPR(int gO2, int gI2) const
{
   const std::complex<double> result = -1.0954451150103321*g1*KroneckerDelta(1,
      gO2)*ZN(gI2,0) - Conj(Ye(0,0))*KroneckerDelta(0,gO2)*ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSeFeChiPL(int gO1, int gI2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*Conj(ZN(gI2,0))*
      KroneckerDelta(0,gO1) + 0.7071067811865475*g2*Conj(ZN(gI2,1))*KroneckerDelta
      (0,gO1) - Conj(ZN(gI2,2))*KroneckerDelta(1,gO1)*Ye(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.1*(12*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1)*Sqr(Sin(ThetaW())) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-7.745966692414834*g1*g2*Cos(ThetaW())*Sin(ThetaW())
      + 5*Sqr(g2)*Sqr(Cos(ThetaW())) + 3*Sqr(g1)*Sqr(Sin(ThetaW()))));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(6*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(-10*AbsSqr(Ye(1,1)) + 3*Sqr(g1)) - KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(6*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

double CLASSNAME::CpUSmconjUSmconjVWmVWm(int gO1, int gO2) const
{
   const double result = 0.5*KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g2
      );

   return result;
}

double CLASSNAME::CpconjUSmVWmSvmL(int gO2) const
{
   const double result = -0.7071067811865475*g2*KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Ye(1,1)) + 3*Sqr(g1))*ZA(gI1,0)*ZA(gI2,0)
      - 3*Sqr(g1)*ZA(gI1,1)*ZA(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,
      gO2)*((-20*AbsSqr(Ye(1,1)) - 3*Sqr(g1) + 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) + (3
      *Sqr(g1) - 5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*(ZP(gI1,0)*ZP(gI2,0) - ZP(gI1,
      1)*ZP(gI2,1))) + 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*((-10*AbsSqr(
      Ye(1,1)) + 3*Sqr(g1))*ZP(gI1,0)*ZP(gI2,0) - 3*Sqr(g1)*ZP(gI1,1)*ZP(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZB(gI2,1))*(-10*Conj(Yd(2,2
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(1,1)*ZB(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZB(gI1,1)) + Conj(ZB(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZB(gI1,0) - 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZB(gI1,0) + 10*Conj(Ye(
      1,1))*KroneckerDelta(0,gO2)*Yd(2,2)*ZB(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZC(gI2,0))*(-2*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZC(gI1,0) - 4*Conj(ZC(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZD(gI2,1))*(-10*Conj(Yd(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(1,1)*ZD(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZD(gI1,1)) + Conj(ZD(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZD(gI1,0) - 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZD(gI1,0) + 10*Conj(Ye(
      1,1))*KroneckerDelta(0,gO2)*Yd(0,0)*ZD(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZE(gI2,1))*(10*Conj(Ye(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(1,1)*ZE(gI1,0) - 3*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZE(gI1,1)) - Conj(ZE(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*ZE(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(-3*KroneckerDelta(1,gO2)*Sqr(g1)*ZE(gI1,0) + 10*Conj(
      Ye(1,1))*KroneckerDelta(0,gO2)*Ye(0,0)*ZE(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZM(gI2,1))*(-12*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1)*ZM(gI1,1) + KroneckerDelta(0,gO1)*(-10
      *AbsSqr(Ye(1,1)) + 3*Sqr(g1))*(KroneckerDelta(1,gO2)*ZM(gI1,0) +
      KroneckerDelta(0,gO2)*ZM(gI1,1))) - Conj(ZM(gI2,0))*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*ZM(gI1,0) - KroneckerDelta(1,
      gO1)*(-10*AbsSqr(Ye(1,1)) + 3*Sqr(g1))*(KroneckerDelta(1,gO2)*ZM(gI1,0) +
      KroneckerDelta(0,gO2)*ZM(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZS(gI2,1))*(-10*Conj(Yd(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(1,1)*ZS(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZS(gI1,1)) + Conj(ZS(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZS(gI1,0) - 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZS(gI1,0) + 10*Conj(Ye(
      1,1))*KroneckerDelta(0,gO2)*Yd(1,1)*ZS(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZT(gI2,0))*(-2*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZT(gI1,0) - 4*Conj(ZT(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZTau(gI2,1))*(10*Conj(Ye(2
      ,2))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(1,1)*ZTau(gI1,0) - 3*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZTau(gI1,1)) - Conj(ZTau(gI2,0))*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*ZTau(gI1
      ,0) + 2*KroneckerDelta(1,gO1)*(-3*KroneckerDelta(1,gO2)*Sqr(g1)*ZTau(gI1,0)
      + 10*Conj(Ye(1,1))*KroneckerDelta(0,gO2)*Ye(2,2)*ZTau(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZU(gI2,0))*(-2*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZU(gI1,0) - 4*Conj(ZU(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSmconjUSmhhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Ye(1,1)) + 3*Sqr(g1))*ZH(gI1,0)*ZH(gI2,0)
      - 3*Sqr(g1)*ZH(gI1,1)*ZH(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,
      gO2)*((-20*AbsSqr(Ye(1,1)) - 3*Sqr(g1) + 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) + (3
      *Sqr(g1) - 5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSmSmAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZM(gI1,1))*KroneckerDelta(0,gO2)*(Conj(TYe(1,1))*
      ZA(gI2,0) + Conj(Ye(1,1))*Mu*ZA(gI2,1)) - Conj(ZM(gI1,0))*KroneckerDelta(1,
      gO2)*(Conj(Mu)*Ye(1,1)*ZA(gI2,1) + ZA(gI2,0)*TYe(1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSmSmhh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZM(gI1,1))*(
      7.0710678118654755*Conj(TYe(1,1))*KroneckerDelta(0,gO2)*ZH(gI2,0) -
      7.0710678118654755*Conj(Ye(1,1))*KroneckerDelta(0,gO2)*Mu*ZH(gI2,1) +
      KroneckerDelta(1,gO2)*((10*vd*AbsSqr(Ye(1,1)) - 3*vd*Sqr(g1))*ZH(gI2,0) + 3*
      vu*Sqr(g1)*ZH(gI2,1))) + Conj(ZM(gI1,0))*(KroneckerDelta(0,gO2)*(vd*(-20*
      AbsSqr(Ye(1,1)) - 3*Sqr(g1) + 5*Sqr(g2))*ZH(gI2,0) + vu*(3*Sqr(g1) - 5*Sqr(
      g2))*ZH(gI2,1)) + 14.142135623730951*KroneckerDelta(1,gO2)*(Conj(Mu)*Ye(1,1)
      *ZH(gI2,1) - ZH(gI2,0)*TYe(1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSmSvmLHpm(int gO2, int gI2) const
{
   const std::complex<double> result = 0.25*(-1.4142135623730951*KroneckerDelta
      (0,gO2)*(vd*(-2*AbsSqr(Ye(1,1)) + Sqr(g2))*ZP(gI2,0) + vu*Sqr(g2)*ZP(gI2,1))
      + 4*KroneckerDelta(1,gO2)*(Conj(Mu)*Ye(1,1)*ZP(gI2,1) + ZP(gI2,0)*TYe(1,1))
      );

   return result;
}

std::complex<double> CLASSNAME::CpconjUSmVPSm(int gO2, int gI2) const
{
   const std::complex<double> result = 0.1*(-7.745966692414834*g1*Conj(ZM(gI2,1
      ))*Cos(ThetaW())*KroneckerDelta(1,gO2) - Conj(ZM(gI2,0))*KroneckerDelta(0,
      gO2)*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(ThetaW())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSmVZSm(int gO2, int gI2) const
{
   const std::complex<double> result = 0.1*(7.745966692414834*g1*Conj(ZM(gI2,1)
      )*KroneckerDelta(1,gO2)*Sin(ThetaW()) - Conj(ZM(gI2,0))*KroneckerDelta(0,gO2
      )*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW())));

   return result;
}

double CLASSNAME::CpconjUSmFvmChaPR(int , int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpconjUSmFvmChaPL(int gO1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(UM(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(UM(gI2,1))*KroneckerDelta(1,gO1)*Ye(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSmFmChiPR(int gO2, int gI2) const
{
   const std::complex<double> result = -1.0954451150103321*g1*KroneckerDelta(1,
      gO2)*ZN(gI2,0) - Conj(Ye(1,1))*KroneckerDelta(0,gO2)*ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSmFmChiPL(int gO1, int gI2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*Conj(ZN(gI2,0))*
      KroneckerDelta(0,gO1) + 0.7071067811865475*g2*Conj(ZN(gI2,1))*KroneckerDelta
      (0,gO1) - Conj(ZN(gI2,2))*KroneckerDelta(1,gO1)*Ye(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.1*(12*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1)*Sqr(Sin(ThetaW())) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-7.745966692414834*g1*g2*Cos(ThetaW())*Sin(ThetaW())
      + 5*Sqr(g2)*Sqr(Cos(ThetaW())) + 3*Sqr(g1)*Sqr(Sin(ThetaW()))));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(6*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(6*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(-10*AbsSqr(Ye(2,2)) + 3*Sqr(g1)) - KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

double CLASSNAME::CpUStauconjUStauconjVWmVWm(int gO1, int gO2) const
{
   const double result = 0.5*KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g2
      );

   return result;
}

double CLASSNAME::CpconjUStauVWmSvtL(int gO2) const
{
   const double result = -0.7071067811865475*g2*KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Ye(2,2)) + 3*Sqr(g1))*ZA(gI1,0)*ZA(gI2,0)
      - 3*Sqr(g1)*ZA(gI1,1)*ZA(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,
      gO2)*((-20*AbsSqr(Ye(2,2)) - 3*Sqr(g1) + 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) + (3
      *Sqr(g1) - 5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*(ZP(gI1,0)*ZP(gI2,0) - ZP(gI1,
      1)*ZP(gI2,1))) + 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*((-10*AbsSqr(
      Ye(2,2)) + 3*Sqr(g1))*ZP(gI1,0)*ZP(gI2,0) - 3*Sqr(g1)*ZP(gI1,1)*ZP(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZB(gI2,1))*(-10*Conj(Yd(2,2
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(2,2)*ZB(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZB(gI1,1)) + Conj(ZB(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZB(gI1,0) - 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZB(gI1,0) + 10*Conj(Ye(
      2,2))*KroneckerDelta(0,gO2)*Yd(2,2)*ZB(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZC(gI2,0))*(-2*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZC(gI1,0) - 4*Conj(ZC(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZD(gI2,1))*(-10*Conj(Yd(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(2,2)*ZD(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZD(gI1,1)) + Conj(ZD(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZD(gI1,0) - 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZD(gI1,0) + 10*Conj(Ye(
      2,2))*KroneckerDelta(0,gO2)*Yd(0,0)*ZD(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZE(gI2,1))*(10*Conj(Ye(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(2,2)*ZE(gI1,0) - 3*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZE(gI1,1)) - Conj(ZE(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*ZE(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(-3*KroneckerDelta(1,gO2)*Sqr(g1)*ZE(gI1,0) + 10*Conj(
      Ye(2,2))*KroneckerDelta(0,gO2)*Ye(0,0)*ZE(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZM(gI2,1))*(10*Conj(Ye(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(2,2)*ZM(gI1,0) - 3*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZM(gI1,1)) - Conj(ZM(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*ZM(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(-3*KroneckerDelta(1,gO2)*Sqr(g1)*ZM(gI1,0) + 10*Conj(
      Ye(2,2))*KroneckerDelta(0,gO2)*Ye(1,1)*ZM(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZS(gI2,1))*(-10*Conj(Yd(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Ye(2,2)*ZS(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZS(gI1,1)) + Conj(ZS(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZS(gI1,0) - 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZS(gI1,0) + 10*Conj(Ye(
      2,2))*KroneckerDelta(0,gO2)*Yd(1,1)*ZS(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZT(gI2,0))*(-2*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZT(gI1,0) - 4*Conj(ZT(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZTau(gI2,1))*(-12*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1)*ZTau(gI1,1) +
      KroneckerDelta(0,gO1)*(-10*AbsSqr(Ye(2,2)) + 3*Sqr(g1))*(KroneckerDelta(1,
      gO2)*ZTau(gI1,0) + KroneckerDelta(0,gO2)*ZTau(gI1,1))) - Conj(ZTau(gI2,0))*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(3*Sqr(g1) + 5*Sqr(g2))*ZTau(gI1
      ,0) - KroneckerDelta(1,gO1)*(-10*AbsSqr(Ye(2,2)) + 3*Sqr(g1))*(
      KroneckerDelta(1,gO2)*ZTau(gI1,0) + KroneckerDelta(0,gO2)*ZTau(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZU(gI2,0))*(-2*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZU(gI1,0) - 4*Conj(ZU(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUStauconjUStauhhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Ye(2,2)) + 3*Sqr(g1))*ZH(gI1,0)*ZH(gI2,0)
      - 3*Sqr(g1)*ZH(gI1,1)*ZH(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,
      gO2)*((-20*AbsSqr(Ye(2,2)) - 3*Sqr(g1) + 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) + (3
      *Sqr(g1) - 5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUStauStauAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZTau(gI1,1))*KroneckerDelta(0,gO2)*(Conj(TYe(2,2))
      *ZA(gI2,0) + Conj(Ye(2,2))*Mu*ZA(gI2,1)) - Conj(ZTau(gI1,0))*KroneckerDelta(
      1,gO2)*(Conj(Mu)*Ye(2,2)*ZA(gI2,1) + ZA(gI2,0)*TYe(2,2)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUStauStauhh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZTau(gI1,1))*(
      7.0710678118654755*Conj(TYe(2,2))*KroneckerDelta(0,gO2)*ZH(gI2,0) -
      7.0710678118654755*Conj(Ye(2,2))*KroneckerDelta(0,gO2)*Mu*ZH(gI2,1) +
      KroneckerDelta(1,gO2)*((10*vd*AbsSqr(Ye(2,2)) - 3*vd*Sqr(g1))*ZH(gI2,0) + 3*
      vu*Sqr(g1)*ZH(gI2,1))) + Conj(ZTau(gI1,0))*(KroneckerDelta(0,gO2)*(vd*(-20*
      AbsSqr(Ye(2,2)) - 3*Sqr(g1) + 5*Sqr(g2))*ZH(gI2,0) + vu*(3*Sqr(g1) - 5*Sqr(
      g2))*ZH(gI2,1)) + 14.142135623730951*KroneckerDelta(1,gO2)*(Conj(Mu)*Ye(2,2)
      *ZH(gI2,1) - ZH(gI2,0)*TYe(2,2))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUStauSvtLHpm(int gO2, int gI2) const
{
   const std::complex<double> result = 0.25*(-1.4142135623730951*KroneckerDelta
      (0,gO2)*(vd*(-2*AbsSqr(Ye(2,2)) + Sqr(g2))*ZP(gI2,0) + vu*Sqr(g2)*ZP(gI2,1))
      + 4*KroneckerDelta(1,gO2)*(Conj(Mu)*Ye(2,2)*ZP(gI2,1) + ZP(gI2,0)*TYe(2,2))
      );

   return result;
}

std::complex<double> CLASSNAME::CpconjUStauVPStau(int gO2, int gI2) const
{
   const std::complex<double> result = 0.1*(-7.745966692414834*g1*Conj(ZTau(gI2
      ,1))*Cos(ThetaW())*KroneckerDelta(1,gO2) - Conj(ZTau(gI2,0))*KroneckerDelta(
      0,gO2)*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(ThetaW())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUStauVZStau(int gO2, int gI2) const
{
   const std::complex<double> result = 0.1*(7.745966692414834*g1*Conj(ZTau(gI2,
      1))*KroneckerDelta(1,gO2)*Sin(ThetaW()) - Conj(ZTau(gI2,0))*KroneckerDelta(0
      ,gO2)*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW())));

   return result;
}

double CLASSNAME::CpconjUStauFvtChaPR(int , int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpconjUStauFvtChaPL(int gO1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(UM(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(UM(gI2,1))*KroneckerDelta(1,gO1)*Ye(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUStauFtauChiPR(int gO2, int gI2) const
{
   const std::complex<double> result = -1.0954451150103321*g1*KroneckerDelta(1,
      gO2)*ZN(gI2,0) - Conj(Ye(2,2))*KroneckerDelta(0,gO2)*ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUStauFtauChiPL(int gO1, int gI2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*Conj(ZN(gI2,0))*
      KroneckerDelta(0,gO1) + 0.7071067811865475*g2*Conj(ZN(gI2,1))*KroneckerDelta
      (0,gO1) - Conj(ZN(gI2,2))*KroneckerDelta(1,gO1)*Ye(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.03333333333333333*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(g1*Sin(ThetaW())*(7.745966692414834*g2*Cos(
      ThetaW()) + g1*Sin(ThetaW())) + 15*Sqr(g2)*Sqr(Cos(ThetaW()))) + 4*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1)*Sqr(Sin(ThetaW())));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) + 5*Sqr(g2)));

   return result;
}

double CLASSNAME::CpUSsconjUSsconjVWmVWm(int gO1, int gO2) const
{
   const double result = 0.5*KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g2
      );

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Yd(1,1)) + Sqr(g1))*ZA(gI1,0)*ZA(gI2,0) -
      Sqr(g1)*ZA(gI1,1)*ZA(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((-20*AbsSqr(Yd(1,1)) + Sqr(g1) + 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) - (Sqr(g1)
      + 5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Yd(1,1)) + Sqr(g1))*ZP(gI1,0)*ZP(gI2,0) -
      Sqr(g1)*ZP(gI1,1)*ZP(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((Sqr(g1) - 5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) - (20*AbsSqr(Yu(1,1)) + Sqr(g1) -
      5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZB(gI2,1))*(30*Conj(Yd(2,2
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(1,1)*ZB(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZB(gI1,1)) - Conj(ZB(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZB(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZB(gI1,0) + 30*Conj(Yd(
      1,1))*KroneckerDelta(0,gO2)*Yd(2,2)*ZB(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZC(gI2,0))*(2*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(10*AbsSqr(Yd(1,1)) + Sqr(g1)) +
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)))*ZC(gI1,0
      )) + 4*Conj(ZC(gI2,1))*(2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1
      ) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-5*AbsSqr(Yu(1,1)) + Sqr(g1
      )))*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZD(gI2,1))*(30*Conj(Yd(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(1,1)*ZD(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZD(gI1,1)) - Conj(ZD(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZD(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZD(gI1,0) + 30*Conj(Yd(
      1,1))*KroneckerDelta(0,gO2)*Yd(0,0)*ZD(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZE(gI2,1))*(10*Conj(Ye(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(1,1)*ZE(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZE(gI1,1)) + Conj(ZE(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZE(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZE(gI1,0) - 10*Conj(Yd(
      1,1))*KroneckerDelta(0,gO2)*Ye(0,0)*ZE(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZM(gI2,1))*(10*Conj(Ye(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(1,1)*ZM(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZM(gI1,1)) + Conj(ZM(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZM(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZM(gI1,0) - 10*Conj(Yd(
      1,1))*KroneckerDelta(0,gO2)*Ye(1,1)*ZM(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(-(Conj(ZS(gI2,1))*(
      8*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g3))*ZS(gI1,1
      ) + KroneckerDelta(0,gO1)*(KroneckerDelta(1,gO2)*(90*AbsSqr(Yd(1,1)) + Sqr(
      g1) - 40*Sqr(g3))*ZS(gI1,0) + 3*KroneckerDelta(0,gO2)*(10*AbsSqr(Yd(1,1)) +
      Sqr(g1))*ZS(gI1,1)))) - Conj(ZS(gI2,0))*(2*KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2) + 20*Sqr(g3))*ZS(gI1,0) +
      KroneckerDelta(1,gO1)*(3*KroneckerDelta(1,gO2)*(10*AbsSqr(Yd(1,1)) + Sqr(g1)
      )*ZS(gI1,0) + KroneckerDelta(0,gO2)*(90*AbsSqr(Yd(1,1)) + Sqr(g1) - 40*Sqr(
      g3))*ZS(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZT(gI2,0))*(2*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZT(gI1,0)) + 4*Conj(ZT(gI2,1))
      *(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZTau(gI2,1))*(10*Conj(Ye(2
      ,2))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(1,1)*ZTau(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZTau(gI1,1)) + Conj(ZTau(gI2,0))*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZTau(gI1,0
      ) + 2*KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZTau(gI1,0) - 10*
      Conj(Yd(1,1))*KroneckerDelta(0,gO2)*Ye(2,2)*ZTau(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSsconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZU(gI2,0))*(2*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZU(gI1,0)) + 4*Conj(ZU(gI2,1))
      *(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSsconjUSshhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Yd(1,1)) + Sqr(g1))*ZH(gI1,0)*ZH(gI2,0) -
      Sqr(g1)*ZH(gI1,1)*ZH(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((-20*AbsSqr(Yd(1,1)) + Sqr(g1) + 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) - (Sqr(g1)
      + 5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsScHpm(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZC(gI1,1))*(2*Conj(TYu(1,1))*
      KroneckerDelta(0,gO2)*ZP(gI2,1) + Conj(Yu(1,1))*(2*KroneckerDelta(0,gO2)*Mu*
      ZP(gI2,0) + 1.4142135623730951*KroneckerDelta(1,gO2)*Yd(1,1)*(vu*ZP(gI2,0) +
      vd*ZP(gI2,1)))) - 0.25*Conj(ZC(gI1,0))*(1.4142135623730951*KroneckerDelta(0
      ,gO2)*(vd*(-2*AbsSqr(Yd(1,1)) + Sqr(g2))*ZP(gI2,0) + vu*(-2*AbsSqr(Yu(1,1))
      + Sqr(g2))*ZP(gI2,1)) - 4*KroneckerDelta(1,gO2)*(Conj(Mu)*Yd(1,1)*ZP(gI2,1)
      + ZP(gI2,0)*TYd(1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsSsAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZS(gI1,1))*KroneckerDelta(0,gO2)*(Conj(TYd(1,1))*
      ZA(gI2,0) + Conj(Yd(1,1))*Mu*ZA(gI2,1)) - Conj(ZS(gI1,0))*KroneckerDelta(1,
      gO2)*(Conj(Mu)*Yd(1,1)*ZA(gI2,1) + ZA(gI2,0)*TYd(1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsSshh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZS(gI1,1))*(
      7.0710678118654755*Conj(TYd(1,1))*KroneckerDelta(0,gO2)*ZH(gI2,0) -
      7.0710678118654755*Conj(Yd(1,1))*KroneckerDelta(0,gO2)*Mu*ZH(gI2,1) +
      KroneckerDelta(1,gO2)*(-(vd*(-10*AbsSqr(Yd(1,1)) + Sqr(g1))*ZH(gI2,0)) + vu*
      Sqr(g1)*ZH(gI2,1))) + Conj(ZS(gI1,0))*(KroneckerDelta(0,gO2)*(vd*(-20*AbsSqr
      (Yd(1,1)) + Sqr(g1) + 5*Sqr(g2))*ZH(gI2,0) - vu*(Sqr(g1) + 5*Sqr(g2))*ZH(gI2
      ,1)) + 14.142135623730951*KroneckerDelta(1,gO2)*(Conj(Mu)*Yd(1,1)*ZH(gI2,1)
      - ZH(gI2,0)*TYd(1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsVWmSc(int gO2, int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZC(gI2,0))*
      KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsVGSs(int gO2, int gI2) const
{
   const std::complex<double> result = IF(gI2 < 2,g3*Conj(ZS(gI2,gO2)),0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsVPSs(int gO2, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(-7.745966692414834*
      g1*Conj(ZS(gI2,1))*Cos(ThetaW())*KroneckerDelta(1,gO2) + Conj(ZS(gI2,0))*
      KroneckerDelta(0,gO2)*(3.872983346207417*g1*Cos(ThetaW()) - 15*g2*Sin(ThetaW
      ())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsVZSs(int gO2, int gI2) const
{
   const std::complex<double> result = 0.2581988897471611*g1*Conj(ZS(gI2,1))*
      KroneckerDelta(1,gO2)*Sin(ThetaW()) - 0.03333333333333333*Conj(ZS(gI2,0))*
      KroneckerDelta(0,gO2)*(15*g2*Cos(ThetaW()) + 3.872983346207417*g1*Sin(ThetaW
      ()));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsFcChaPR(int gO2, int , int gI2) const
{
   const std::complex<double> result = Conj(Yu(1,1))*KroneckerDelta(0,gO2)*UP(
      gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsFcChaPL(int gO1, int , int gI2) const
{
   const std::complex<double> result = -(g2*Conj(UM(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(UM(gI2,1))*KroneckerDelta(1,gO1)*Yd(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsFsChiPR(int gO2, int , int gI2) const
{
   const std::complex<double> result = -0.3651483716701107*g1*KroneckerDelta(1,
      gO2)*ZN(gI2,0) - Conj(Yd(1,1))*KroneckerDelta(0,gO2)*ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsFsChiPL(int gO1, int , int gI2) const
{
   const std::complex<double> result = -0.18257418583505536*g1*Conj(ZN(gI2,0))*
      KroneckerDelta(0,gO1) + 0.7071067811865475*g2*Conj(ZN(gI2,1))*KroneckerDelta
      (0,gO1) - Conj(ZN(gI2,2))*KroneckerDelta(1,gO1)*Yd(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsGluFsPR(int gO2, int , int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*
      KroneckerDelta(1,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSsGluFsPL(int gO1, int , int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*
      KroneckerDelta(0,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.03333333333333333*(16*KroneckerDelta(1
      ,gO1)*KroneckerDelta(1,gO2)*Sqr(g1)*Sqr(Sin(ThetaW())) + KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(-7.745966692414834*g1*g2*Cos(ThetaW())*Sin(
      ThetaW()) + 15*Sqr(g2)*Sqr(Cos(ThetaW())) + Sqr(g1)*Sqr(Sin(ThetaW()))));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) - 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) - 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) - 5*Sqr(g2)));

   return result;
}

double CLASSNAME::CpUScconjUScconjVWmVWm(int gO1, int gO2) const
{
   const double result = 0.5*KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g2
      );

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(Sqr(g1)*ZA(gI1,0)*ZA(gI2,0) - (-5*AbsSqr(Yu(1,1)) +
      Sqr(g1))*ZA(gI1,1)*ZA(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((Sqr(g1) - 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) - (20*AbsSqr(Yu(1,1)) + Sqr(g1) -
      5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(Sqr(g1)*ZP(gI1,0)*ZP(gI2,0) - (-5*AbsSqr(Yu(1,1)) +
      Sqr(g1))*ZP(gI1,1)*ZP(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((-20*AbsSqr(Yd(1,1)) + Sqr(g1) + 5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) - (Sqr(g1)
      + 5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZB(gI2,0))*(4*KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) - KroneckerDelta(0,gO1)*KroneckerDelta(
      0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZB(gI1,0) - 2*Conj(ZB(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.06666666666666667*(Conj(ZC(gI2,1))*(-4
      *KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(4*Sqr(g1) + 5*Sqr(g3))*ZC(gI1,
      1) + KroneckerDelta(0,gO1)*(KroneckerDelta(1,gO2)*(-45*AbsSqr(Yu(1,1)) + Sqr
      (g1) + 20*Sqr(g3))*ZC(gI1,0) + 3*KroneckerDelta(0,gO2)*(-5*AbsSqr(Yu(1,1)) +
      Sqr(g1))*ZC(gI1,1))) - Conj(ZC(gI2,0))*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2) + 20*Sqr(g3))*ZC(gI1,0) -
      KroneckerDelta(1,gO1)*(3*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(1,1)) + Sqr(g1)
      )*ZC(gI1,0) + KroneckerDelta(0,gO2)*(-45*AbsSqr(Yu(1,1)) + Sqr(g1) + 20*Sqr(
      g3))*ZC(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZD(gI2,0))*(4*KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) - KroneckerDelta(0,gO1)*KroneckerDelta(
      0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZD(gI1,0) - 2*Conj(ZD(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZE(gI2,0))*(-4*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZE(gI1,0) - 2*Conj(ZE(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZM(gI2,0))*(-4*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZM(gI1,0) - 2*Conj(ZM(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZS(gI2,0))*(4*KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(1,1)) + Sqr(g1)) - KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)))*ZS(gI1,0) - 2*Conj(ZS(
      gI2,1))*(-4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) +
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(10*AbsSqr(Yd(1,1)) + Sqr(g1)))*
      ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(4*Conj(ZT(gI2,1))*(-15*Conj(Yu(2,2
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yu(1,1)*ZT(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZT(gI1,1)) - Conj(ZT(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZT(gI1,0) - 4*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZT(gI1,0) - 15*Conj(Yu(
      1,1))*KroneckerDelta(0,gO2)*Yu(2,2)*ZT(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZTau(gI2,0))*(-4*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZTau(gI1,0) - 2*Conj(ZTau(gI2,1
      ))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUScconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(4*Conj(ZU(gI2,1))*(-15*Conj(Yu(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yu(1,1)*ZU(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZU(gI1,1)) - Conj(ZU(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZU(gI1,0) - 4*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZU(gI1,0) - 15*Conj(Yu(
      1,1))*KroneckerDelta(0,gO2)*Yu(0,0)*ZU(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUScconjUSchhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(Sqr(g1)*ZH(gI1,0)*ZH(gI2,0) - (-5*AbsSqr(Yu(1,1)) +
      Sqr(g1))*ZH(gI1,1)*ZH(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((Sqr(g1) - 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) - (20*AbsSqr(Yu(1,1)) + Sqr(g1) -
      5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUScconjHpmSs(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZS(gI2,1))*(2*Conj(TYd(1,1))*
      KroneckerDelta(0,gO2)*ZP(gI1,0) + Conj(Yd(1,1))*(2*KroneckerDelta(0,gO2)*Mu*
      ZP(gI1,1) + 1.4142135623730951*KroneckerDelta(1,gO2)*Yu(1,1)*(vu*ZP(gI1,0) +
      vd*ZP(gI1,1)))) - 0.25*Conj(ZS(gI2,0))*(1.4142135623730951*KroneckerDelta(0
      ,gO2)*(vd*(-2*AbsSqr(Yd(1,1)) + Sqr(g2))*ZP(gI1,0) + vu*(-2*AbsSqr(Yu(1,1))
      + Sqr(g2))*ZP(gI1,1)) - 4*KroneckerDelta(1,gO2)*(Conj(Mu)*Yu(1,1)*ZP(gI1,0)
      + ZP(gI1,1)*TYu(1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUScScAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(Yu(1,1))*Conj(ZC(gI1,1))*KroneckerDelta(0,gO2)*Mu*
      ZA(gI2,0) - Conj(Mu)*Conj(ZC(gI1,0))*KroneckerDelta(1,gO2)*Yu(1,1)*ZA(gI2,0)
      + ZA(gI2,1)*(Conj(ZC(gI1,1))*Conj(TYu(1,1))*KroneckerDelta(0,gO2) - Conj(ZC
      (gI1,0))*KroneckerDelta(1,gO2)*TYu(1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUScSchh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZC(gI1,1))*(
      -7.0710678118654755*Conj(TYu(1,1))*KroneckerDelta(0,gO2)*ZH(gI2,1) + 2*
      KroneckerDelta(1,gO2)*Sqr(g1)*(-(vd*ZH(gI2,0)) + vu*ZH(gI2,1)) + 5*Conj(Yu(1
      ,1))*(1.4142135623730951*KroneckerDelta(0,gO2)*Mu*ZH(gI2,0) - 2*vu*
      KroneckerDelta(1,gO2)*Yu(1,1)*ZH(gI2,1))) + Conj(ZC(gI1,0))*(KroneckerDelta(
      0,gO2)*(vd*(Sqr(g1) - 5*Sqr(g2))*ZH(gI2,0) - vu*(20*AbsSqr(Yu(1,1)) + Sqr(g1
      ) - 5*Sqr(g2))*ZH(gI2,1)) + 14.142135623730951*KroneckerDelta(1,gO2)*(Conj(
      Mu)*Yu(1,1)*ZH(gI2,0) - ZH(gI2,1)*TYu(1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUScbarChaFsPR(int gO2, int gI1, int ) const
{
   const std::complex<double> result = Conj(Yd(1,1))*KroneckerDelta(0,gO2)*UM(
      gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUScbarChaFsPL(int gO1, int gI1, int ) const
{
   const std::complex<double> result = -(g2*Conj(UP(gI1,0))*KroneckerDelta(0,
      gO1)) + Conj(UP(gI1,1))*KroneckerDelta(1,gO1)*Yu(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUScVGSc(int gO2, int gI2) const
{
   const std::complex<double> result = IF(gI2 < 2,g3*Conj(ZC(gI2,gO2)),0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUScVPSc(int gO2, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(15.491933384829668*
      g1*Conj(ZC(gI2,1))*Cos(ThetaW())*KroneckerDelta(1,gO2) + Conj(ZC(gI2,0))*
      KroneckerDelta(0,gO2)*(3.872983346207417*g1*Cos(ThetaW()) + 15*g2*Sin(ThetaW
      ())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUScVZSc(int gO2, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(-15.491933384829668
      *g1*Conj(ZC(gI2,1))*KroneckerDelta(1,gO2)*Sin(ThetaW()) + Conj(ZC(gI2,0))*
      KroneckerDelta(0,gO2)*(15*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW
      ())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUScconjVWmSs(int gO2, int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZS(gI2,0))*
      KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUScFcChiPR(int gO2, int , int gI2) const
{
   const std::complex<double> result = 0.7302967433402214*g1*KroneckerDelta(1,
      gO2)*ZN(gI2,0) - Conj(Yu(1,1))*KroneckerDelta(0,gO2)*ZN(gI2,3);

   return result;
}

std::complex<double> CLASSNAME::CpconjUScFcChiPL(int gO1, int , int gI2) const
{
   const std::complex<double> result = -0.18257418583505536*g1*Conj(ZN(gI2,0))*
      KroneckerDelta(0,gO1) - 0.7071067811865475*g2*Conj(ZN(gI2,1))*KroneckerDelta
      (0,gO1) - Conj(ZN(gI2,3))*KroneckerDelta(1,gO1)*Yu(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUScGluFcPR(int gO2, int , int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*
      KroneckerDelta(1,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUScGluFcPL(int gO1, int , int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*
      KroneckerDelta(0,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.03333333333333333*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(g1*Sin(ThetaW())*(7.745966692414834*g2*Cos(
      ThetaW()) + g1*Sin(ThetaW())) + 15*Sqr(g2)*Sqr(Cos(ThetaW()))) + 4*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1)*Sqr(Sin(ThetaW())));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) + 5*Sqr(g2)));

   return result;
}

double CLASSNAME::CpUSbconjUSbconjVWmVWm(int gO1, int gO2) const
{
   const double result = 0.5*KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g2
      );

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Yd(2,2)) + Sqr(g1))*ZA(gI1,0)*ZA(gI2,0) -
      Sqr(g1)*ZA(gI1,1)*ZA(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((-20*AbsSqr(Yd(2,2)) + Sqr(g1) + 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) - (Sqr(g1)
      + 5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Yd(2,2)) + Sqr(g1))*ZP(gI1,0)*ZP(gI2,0) -
      Sqr(g1)*ZP(gI1,1)*ZP(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((Sqr(g1) - 5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) - (20*AbsSqr(Yu(2,2)) + Sqr(g1) -
      5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(-(Conj(ZB(gI2,1))*(
      8*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g3))*ZB(gI1,1
      ) + KroneckerDelta(0,gO1)*(KroneckerDelta(1,gO2)*(90*AbsSqr(Yd(2,2)) + Sqr(
      g1) - 40*Sqr(g3))*ZB(gI1,0) + 3*KroneckerDelta(0,gO2)*(10*AbsSqr(Yd(2,2)) +
      Sqr(g1))*ZB(gI1,1)))) - Conj(ZB(gI2,0))*(2*KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2) + 20*Sqr(g3))*ZB(gI1,0) +
      KroneckerDelta(1,gO1)*(3*KroneckerDelta(1,gO2)*(10*AbsSqr(Yd(2,2)) + Sqr(g1)
      )*ZB(gI1,0) + KroneckerDelta(0,gO2)*(90*AbsSqr(Yd(2,2)) + Sqr(g1) - 40*Sqr(
      g3))*ZB(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZC(gI2,0))*(2*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZC(gI1,0)) + 4*Conj(ZC(gI2,1))
      *(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZD(gI2,1))*(30*Conj(Yd(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(2,2)*ZD(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZD(gI1,1)) - Conj(ZD(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZD(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZD(gI1,0) + 30*Conj(Yd(
      2,2))*KroneckerDelta(0,gO2)*Yd(0,0)*ZD(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZE(gI2,1))*(10*Conj(Ye(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(2,2)*ZE(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZE(gI1,1)) + Conj(ZE(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZE(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZE(gI1,0) - 10*Conj(Yd(
      2,2))*KroneckerDelta(0,gO2)*Ye(0,0)*ZE(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZM(gI2,1))*(10*Conj(Ye(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(2,2)*ZM(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZM(gI1,1)) + Conj(ZM(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZM(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZM(gI1,0) - 10*Conj(Yd(
      2,2))*KroneckerDelta(0,gO2)*Ye(1,1)*ZM(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZS(gI2,1))*(30*Conj(Yd(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(2,2)*ZS(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZS(gI1,1)) - Conj(ZS(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZS(gI1,0) + 2*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZS(gI1,0) + 30*Conj(Yd(
      2,2))*KroneckerDelta(0,gO2)*Yd(1,1)*ZS(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZT(gI2,0))*(2*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(10*AbsSqr(Yd(2,2)) + Sqr(g1)) +
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)))*ZT(gI1,0
      )) + 4*Conj(ZT(gI2,1))*(2*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1
      ) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-5*AbsSqr(Yu(2,2)) + Sqr(g1
      )))*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZTau(gI2,1))*(10*Conj(Ye(2
      ,2))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yd(2,2)*ZTau(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZTau(gI1,1)) + Conj(ZTau(gI2,0))*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2))*ZTau(gI1,0
      ) + 2*KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZTau(gI1,0) - 10*
      Conj(Yd(2,2))*KroneckerDelta(0,gO2)*Ye(2,2)*ZTau(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZU(gI2,0))*(2*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZU(gI1,0)) + 4*Conj(ZU(gI2,1))
      *(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) + 2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUSbconjUSbhhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*((-10*AbsSqr(Yd(2,2)) + Sqr(g1))*ZH(gI1,0)*ZH(gI2,0) -
      Sqr(g1)*ZH(gI1,1)*ZH(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((-20*AbsSqr(Yd(2,2)) + Sqr(g1) + 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) - (Sqr(g1)
      + 5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbSbAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZB(gI1,1))*KroneckerDelta(0,gO2)*(Conj(TYd(2,2))*
      ZA(gI2,0) + Conj(Yd(2,2))*Mu*ZA(gI2,1)) - Conj(ZB(gI1,0))*KroneckerDelta(1,
      gO2)*(Conj(Mu)*Yd(2,2)*ZA(gI2,1) + ZA(gI2,0)*TYd(2,2)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbSbhh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZB(gI1,1))*(
      7.0710678118654755*Conj(TYd(2,2))*KroneckerDelta(0,gO2)*ZH(gI2,0) -
      7.0710678118654755*Conj(Yd(2,2))*KroneckerDelta(0,gO2)*Mu*ZH(gI2,1) +
      KroneckerDelta(1,gO2)*(-(vd*(-10*AbsSqr(Yd(2,2)) + Sqr(g1))*ZH(gI2,0)) + vu*
      Sqr(g1)*ZH(gI2,1))) + Conj(ZB(gI1,0))*(KroneckerDelta(0,gO2)*(vd*(-20*AbsSqr
      (Yd(2,2)) + Sqr(g1) + 5*Sqr(g2))*ZH(gI2,0) - vu*(Sqr(g1) + 5*Sqr(g2))*ZH(gI2
      ,1)) + 14.142135623730951*KroneckerDelta(1,gO2)*(Conj(Mu)*Yd(2,2)*ZH(gI2,1)
      - ZH(gI2,0)*TYd(2,2))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbStHpm(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZT(gI1,1))*(2*Conj(TYu(2,2))*
      KroneckerDelta(0,gO2)*ZP(gI2,1) + Conj(Yu(2,2))*(2*KroneckerDelta(0,gO2)*Mu*
      ZP(gI2,0) + 1.4142135623730951*KroneckerDelta(1,gO2)*Yd(2,2)*(vu*ZP(gI2,0) +
      vd*ZP(gI2,1)))) - 0.25*Conj(ZT(gI1,0))*(1.4142135623730951*KroneckerDelta(0
      ,gO2)*(vd*(-2*AbsSqr(Yd(2,2)) + Sqr(g2))*ZP(gI2,0) + vu*(-2*AbsSqr(Yu(2,2))
      + Sqr(g2))*ZP(gI2,1)) - 4*KroneckerDelta(1,gO2)*(Conj(Mu)*Yd(2,2)*ZP(gI2,1)
      + ZP(gI2,0)*TYd(2,2)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbVGSb(int gO2, int gI2) const
{
   const std::complex<double> result = IF(gI2 < 2,g3*Conj(ZB(gI2,gO2)),0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbVPSb(int gO2, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(-7.745966692414834*
      g1*Conj(ZB(gI2,1))*Cos(ThetaW())*KroneckerDelta(1,gO2) + Conj(ZB(gI2,0))*
      KroneckerDelta(0,gO2)*(3.872983346207417*g1*Cos(ThetaW()) - 15*g2*Sin(ThetaW
      ())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbVZSb(int gO2, int gI2) const
{
   const std::complex<double> result = 0.2581988897471611*g1*Conj(ZB(gI2,1))*
      KroneckerDelta(1,gO2)*Sin(ThetaW()) - 0.03333333333333333*Conj(ZB(gI2,0))*
      KroneckerDelta(0,gO2)*(15*g2*Cos(ThetaW()) + 3.872983346207417*g1*Sin(ThetaW
      ()));

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbVWmSt(int gO2, int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZT(gI2,0))*
      KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbFtChaPR(int gO2, int , int gI2) const
{
   const std::complex<double> result = Conj(Yu(2,2))*KroneckerDelta(0,gO2)*UP(
      gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbFtChaPL(int gO1, int , int gI2) const
{
   const std::complex<double> result = -(g2*Conj(UM(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(UM(gI2,1))*KroneckerDelta(1,gO1)*Yd(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbFbChiPR(int gO2, int , int gI2) const
{
   const std::complex<double> result = -0.3651483716701107*g1*KroneckerDelta(1,
      gO2)*ZN(gI2,0) - Conj(Yd(2,2))*KroneckerDelta(0,gO2)*ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbFbChiPL(int gO1, int , int gI2) const
{
   const std::complex<double> result = -0.18257418583505536*g1*Conj(ZN(gI2,0))*
      KroneckerDelta(0,gO1) + 0.7071067811865475*g2*Conj(ZN(gI2,1))*KroneckerDelta
      (0,gO1) - Conj(ZN(gI2,2))*KroneckerDelta(1,gO1)*Yd(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbGluFbPR(int gO2, int , int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*
      KroneckerDelta(1,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUSbGluFbPL(int gO1, int , int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*
      KroneckerDelta(0,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.03333333333333333*(16*KroneckerDelta(1
      ,gO1)*KroneckerDelta(1,gO2)*Sqr(g1)*Sqr(Sin(ThetaW())) + KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(-7.745966692414834*g1*g2*Cos(ThetaW())*Sin(
      ThetaW()) + 15*Sqr(g2)*Sqr(Cos(ThetaW())) + Sqr(g1)*Sqr(Sin(ThetaW()))));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) - 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) - 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      (Sqr(g1) - 5*Sqr(g2)));

   return result;
}

double CLASSNAME::CpUStconjUStconjVWmVWm(int gO1, int gO2) const
{
   const double result = 0.5*KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g2
      );

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(Sqr(g1)*ZA(gI1,0)*ZA(gI2,0) - (-5*AbsSqr(Yu(2,2)) +
      Sqr(g1))*ZA(gI1,1)*ZA(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((Sqr(g1) - 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) - (20*AbsSqr(Yu(2,2)) + Sqr(g1) -
      5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(Sqr(g1)*ZP(gI1,0)*ZP(gI2,0) - (-5*AbsSqr(Yu(2,2)) +
      Sqr(g1))*ZP(gI1,1)*ZP(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((-20*AbsSqr(Yd(2,2)) + Sqr(g1) + 5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) - (Sqr(g1)
      + 5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZB(gI2,0))*(4*KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(2,2)) + Sqr(g1)) - KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)))*ZB(gI1,0) - 2*Conj(ZB(
      gI2,1))*(-4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) +
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(10*AbsSqr(Yd(2,2)) + Sqr(g1)))*
      ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(4*Conj(ZC(gI2,1))*(-15*Conj(Yu(1,1
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yu(2,2)*ZC(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZC(gI1,1)) - Conj(ZC(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZC(gI1,0) - 4*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZC(gI1,0) - 15*Conj(Yu(
      2,2))*KroneckerDelta(0,gO2)*Yu(1,1)*ZC(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZD(gI2,0))*(4*KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) - KroneckerDelta(0,gO1)*KroneckerDelta(
      0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZD(gI1,0) - 2*Conj(ZD(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZE(gI2,0))*(-4*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZE(gI1,0) - 2*Conj(ZE(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZM(gI2,0))*(-4*KroneckerDelta
      (1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta
      (0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZM(gI1,0) - 2*Conj(ZM(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZS(gI2,0))*(4*KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) - KroneckerDelta(0,gO1)*KroneckerDelta(
      0,gO2)*(Sqr(g1) - 15*Sqr(g2)))*ZS(gI1,0) - 2*Conj(ZS(gI2,1))*(KroneckerDelta
      (0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2
      ))*Sqr(g1)*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.06666666666666667*(Conj(ZT(gI2,1))*(-4
      *KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(4*Sqr(g1) + 5*Sqr(g3))*ZT(gI1,
      1) + KroneckerDelta(0,gO1)*(KroneckerDelta(1,gO2)*(-45*AbsSqr(Yu(2,2)) + Sqr
      (g1) + 20*Sqr(g3))*ZT(gI1,0) + 3*KroneckerDelta(0,gO2)*(-5*AbsSqr(Yu(2,2)) +
      Sqr(g1))*ZT(gI1,1))) - Conj(ZT(gI2,0))*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2) + 20*Sqr(g3))*ZT(gI1,0) -
      KroneckerDelta(1,gO1)*(3*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(2,2)) + Sqr(g1)
      )*ZT(gI1,0) + KroneckerDelta(0,gO2)*(-45*AbsSqr(Yu(2,2)) + Sqr(g1) + 20*Sqr(
      g3))*ZT(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZTau(gI2,0))*(-4*
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(Sqr(g1) + 5*Sqr(g2)))*ZTau(gI1,0) - 2*Conj(ZTau(gI2,1
      ))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUStconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(4*Conj(ZU(gI2,1))*(-15*Conj(Yu(0,0
      ))*KroneckerDelta(0,gO1)*KroneckerDelta(1,gO2)*Yu(2,2)*ZU(gI1,0) + (
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - 4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*Sqr(g1)*ZU(gI1,1)) - Conj(ZU(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) + 15*Sqr(g2))*ZU(gI1,0) - 4*
      KroneckerDelta(1,gO1)*(KroneckerDelta(1,gO2)*Sqr(g1)*ZU(gI1,0) - 15*Conj(Yu(
      2,2))*KroneckerDelta(0,gO2)*Yu(0,0)*ZU(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUStconjUSthhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-4*KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(Sqr(g1)*ZH(gI1,0)*ZH(gI2,0) - (-5*AbsSqr(Yu(2,2)) +
      Sqr(g1))*ZH(gI1,1)*ZH(gI2,1)) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*
      ((Sqr(g1) - 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) - (20*AbsSqr(Yu(2,2)) + Sqr(g1) -
      5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUStconjHpmSb(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZB(gI2,1))*(2*Conj(TYd(2,2))*
      KroneckerDelta(0,gO2)*ZP(gI1,0) + Conj(Yd(2,2))*(2*KroneckerDelta(0,gO2)*Mu*
      ZP(gI1,1) + 1.4142135623730951*KroneckerDelta(1,gO2)*Yu(2,2)*(vu*ZP(gI1,0) +
      vd*ZP(gI1,1)))) - 0.25*Conj(ZB(gI2,0))*(1.4142135623730951*KroneckerDelta(0
      ,gO2)*(vd*(-2*AbsSqr(Yd(2,2)) + Sqr(g2))*ZP(gI1,0) + vu*(-2*AbsSqr(Yu(2,2))
      + Sqr(g2))*ZP(gI1,1)) - 4*KroneckerDelta(1,gO2)*(Conj(Mu)*Yu(2,2)*ZP(gI1,0)
      + ZP(gI1,1)*TYu(2,2)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUStStAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(Yu(2,2))*Conj(ZT(gI1,1))*KroneckerDelta(0,gO2)*Mu*
      ZA(gI2,0) - Conj(Mu)*Conj(ZT(gI1,0))*KroneckerDelta(1,gO2)*Yu(2,2)*ZA(gI2,0)
      + ZA(gI2,1)*(Conj(ZT(gI1,1))*Conj(TYu(2,2))*KroneckerDelta(0,gO2) - Conj(ZT
      (gI1,0))*KroneckerDelta(1,gO2)*TYu(2,2)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUStSthh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZT(gI1,1))*(
      -7.0710678118654755*Conj(TYu(2,2))*KroneckerDelta(0,gO2)*ZH(gI2,1) + 2*
      KroneckerDelta(1,gO2)*Sqr(g1)*(-(vd*ZH(gI2,0)) + vu*ZH(gI2,1)) + 5*Conj(Yu(2
      ,2))*(1.4142135623730951*KroneckerDelta(0,gO2)*Mu*ZH(gI2,0) - 2*vu*
      KroneckerDelta(1,gO2)*Yu(2,2)*ZH(gI2,1))) + Conj(ZT(gI1,0))*(KroneckerDelta(
      0,gO2)*(vd*(Sqr(g1) - 5*Sqr(g2))*ZH(gI2,0) - vu*(20*AbsSqr(Yu(2,2)) + Sqr(g1
      ) - 5*Sqr(g2))*ZH(gI2,1)) + 14.142135623730951*KroneckerDelta(1,gO2)*(Conj(
      Mu)*Yu(2,2)*ZH(gI2,0) - ZH(gI2,1)*TYu(2,2))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUStbarChaFbPR(int gO2, int gI1, int ) const
{
   const std::complex<double> result = Conj(Yd(2,2))*KroneckerDelta(0,gO2)*UM(
      gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUStbarChaFbPL(int gO1, int gI1, int ) const
{
   const std::complex<double> result = -(g2*Conj(UP(gI1,0))*KroneckerDelta(0,
      gO1)) + Conj(UP(gI1,1))*KroneckerDelta(1,gO1)*Yu(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUStconjVWmSb(int gO2, int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZB(gI2,0))*
      KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUStVGSt(int gO2, int gI2) const
{
   const std::complex<double> result = IF(gI2 < 2,g3*Conj(ZT(gI2,gO2)),0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUStVPSt(int gO2, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(15.491933384829668*
      g1*Conj(ZT(gI2,1))*Cos(ThetaW())*KroneckerDelta(1,gO2) + Conj(ZT(gI2,0))*
      KroneckerDelta(0,gO2)*(3.872983346207417*g1*Cos(ThetaW()) + 15*g2*Sin(ThetaW
      ())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUStVZSt(int gO2, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(-15.491933384829668
      *g1*Conj(ZT(gI2,1))*KroneckerDelta(1,gO2)*Sin(ThetaW()) + Conj(ZT(gI2,0))*
      KroneckerDelta(0,gO2)*(15*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW
      ())));

   return result;
}

std::complex<double> CLASSNAME::CpconjUStFtChiPR(int gO2, int , int gI2) const
{
   const std::complex<double> result = 0.7302967433402214*g1*KroneckerDelta(1,
      gO2)*ZN(gI2,0) - Conj(Yu(2,2))*KroneckerDelta(0,gO2)*ZN(gI2,3);

   return result;
}

std::complex<double> CLASSNAME::CpconjUStFtChiPL(int gO1, int , int gI2) const
{
   const std::complex<double> result = -0.18257418583505536*g1*Conj(ZN(gI2,0))*
      KroneckerDelta(0,gO1) - 0.7071067811865475*g2*Conj(ZN(gI2,1))*KroneckerDelta
      (0,gO1) - Conj(ZN(gI2,3))*KroneckerDelta(1,gO1)*Yu(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUStGluFtPR(int gO2, int , int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*
      KroneckerDelta(1,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpconjUStGluFtPL(int gO1, int , int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*
      KroneckerDelta(0,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUhhVZVZ(int gO2) const
{
   const std::complex<double> result = 0.05*(vd*KroneckerDelta(0,gO2) + vu*
      KroneckerDelta(1,gO2))*(7.745966692414834*g1*g2*Sin(2*ThetaW()) + 3*Sqr(g1)
      + 5*Sqr(g2) + Cos(2*ThetaW())*(-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjSveLSveL(int gO2) const
{
   const std::complex<double> result = -0.05*(vd*KroneckerDelta(0,gO2) - vu*
      KroneckerDelta(1,gO2))*(3*Sqr(g1) + 5*Sqr(g2));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjSvmLSvmL(int gO2) const
{
   const std::complex<double> result = -0.05*(vd*KroneckerDelta(0,gO2) - vu*
      KroneckerDelta(1,gO2))*(3*Sqr(g1) + 5*Sqr(g2));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjSvtLSvtL(int gO2) const
{
   const std::complex<double> result = -0.05*(vd*KroneckerDelta(0,gO2) - vu*
      KroneckerDelta(1,gO2))*(3*Sqr(g1) + 5*Sqr(g2));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjVWmVWm(int gO2) const
{
   const std::complex<double> result = 0.5*(vd*KroneckerDelta(0,gO2) + vu*
      KroneckerDelta(1,gO2))*Sqr(g2);

   return result;
}

std::complex<double> CLASSNAME::CpUhhbargWmgWm(int gO1) const
{
   const std::complex<double> result = -0.25*(vd*KroneckerDelta(0,gO1) + vu*
      KroneckerDelta(1,gO1))*Sqr(g2);

   return result;
}

std::complex<double> CLASSNAME::CpUhhbargWmCgWmC(int gO1) const
{
   const std::complex<double> result = -0.25*(vd*KroneckerDelta(0,gO1) + vu*
      KroneckerDelta(1,gO1))*Sqr(g2);

   return result;
}

std::complex<double> CLASSNAME::CpUhhbargZgZ(int gO1) const
{
   const std::complex<double> result = -0.025*(vd*KroneckerDelta(0,gO1) + vu*
      KroneckerDelta(1,gO1))*(7.745966692414834*g1*g2*Sin(2*ThetaW()) + 3*Sqr(g1)
      + 5*Sqr(g2) + Cos(2*ThetaW())*(-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) + KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(
      7.745966692414834*g1*g2*Sin(2*ThetaW()) + 3*Sqr(g1) + 5*Sqr(g2) + Cos(2*
      ThetaW())*(-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = -0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(3*Sqr(
      g1) + 5*Sqr(g2));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = -0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(3*Sqr(
      g1) + 5*Sqr(g2));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = -0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(3*Sqr(
      g1) + 5*Sqr(g2));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjVWmVWm(int gO1, int gO2) const
{
   const std::complex<double> result = 0.5*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) + KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*Sqr(g2)
      ;

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = -0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(3*Sqr(
      g1) + 5*Sqr(g2))*(ZA(gI1,0)*ZA(gI2,0) - ZA(gI1,1)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(KroneckerDelta(0,gO1)*(5*
      KroneckerDelta(1,gO2)*Sqr(g2)*(ZP(gI1,1)*ZP(gI2,0) + ZP(gI1,0)*ZP(gI2,1)) +
      KroneckerDelta(0,gO2)*((3*Sqr(g1) + 5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) + (-3*Sqr
      (g1) + 5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1)))) + KroneckerDelta(1,gO1)*(-5*
      KroneckerDelta(0,gO2)*Sqr(g2)*(ZP(gI1,1)*ZP(gI2,0) + ZP(gI1,0)*ZP(gI2,1)) +
      KroneckerDelta(1,gO2)*((3*Sqr(g1) - 5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) - (3*Sqr(
      g1) + 5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZB(gI2,0))*(-(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g2))) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Yd(2,2)) + Sqr(g1) + 5*Sqr(g2)))*ZB(gI1,0)
      + 2*Conj(ZB(gI2,1))*(-(KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1))
      + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Yd(2,2)) + Sqr(g1
      )))*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZC(gI2,0))*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)) - KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(20*AbsSqr(Yu(1,1)) + Sqr(g1) - 5*Sqr(g2)))*ZC(gI1,0)
      - 4*Conj(ZC(gI2,1))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g1) -
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(1,1)) + Sqr(g1)))*
      ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZD(gI2,0))*(-(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g2))) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Yd(0,0)) + Sqr(g1) + 5*Sqr(g2)))*ZD(gI1,0)
      + 2*Conj(ZD(gI2,1))*(-(KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1))
      + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Yd(0,0)) + Sqr(g1
      )))*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZE(gI2,0))*(KroneckerDelta(1,
      gO1)*KroneckerDelta(1,gO2)*(3*Sqr(g1) - 5*Sqr(g2)) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Ye(0,0)) - 3*Sqr(g1) + 5*Sqr(g2)))*ZE(gI1,
      0) + 2*Conj(ZE(gI2,1))*(-3*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(
      g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Ye(0,0)) + 3*
      Sqr(g1)))*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZM(gI2,0))*(KroneckerDelta(1,
      gO1)*KroneckerDelta(1,gO2)*(3*Sqr(g1) - 5*Sqr(g2)) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Ye(1,1)) - 3*Sqr(g1) + 5*Sqr(g2)))*ZM(gI1,
      0) + 2*Conj(ZM(gI2,1))*(-3*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(
      g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Ye(1,1)) + 3*
      Sqr(g1)))*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZS(gI2,0))*(-(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g2))) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Yd(1,1)) + Sqr(g1) + 5*Sqr(g2)))*ZS(gI1,0)
      + 2*Conj(ZS(gI2,1))*(-(KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1))
      + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Yd(1,1)) + Sqr(g1
      )))*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZT(gI2,0))*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)) - KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(20*AbsSqr(Yu(2,2)) + Sqr(g1) - 5*Sqr(g2)))*ZT(gI1,0)
      - 4*Conj(ZT(gI2,1))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g1) -
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(2,2)) + Sqr(g1)))*
      ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZTau(gI2,0))*(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(3*Sqr(g1) - 5*Sqr(g2)) + KroneckerDelta(0,gO1)
      *KroneckerDelta(0,gO2)*(-20*AbsSqr(Ye(2,2)) - 3*Sqr(g1) + 5*Sqr(g2)))*ZTau(
      gI1,0) + 2*Conj(ZTau(gI2,1))*(-3*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)
      *Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Ye(2,2))
      + 3*Sqr(g1)))*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZU(gI2,0))*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)) - KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(20*AbsSqr(Yu(0,0)) + Sqr(g1) - 5*Sqr(g2)))*ZU(gI1,0)
      - 4*Conj(ZU(gI2,1))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g1) -
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(0,0)) + Sqr(g1)))*
      ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhUhhhhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(3*Sqr(g1) + 5*Sqr(g2))*(
      KroneckerDelta(1,gO1)*(KroneckerDelta(0,gO2)*(ZH(gI1,1)*ZH(gI2,0) + ZH(gI1,0
      )*ZH(gI2,1)) + KroneckerDelta(1,gO2)*(ZH(gI1,0)*ZH(gI2,0) - 3*ZH(gI1,1)*ZH(
      gI2,1))) + KroneckerDelta(0,gO1)*(KroneckerDelta(1,gO2)*(ZH(gI1,1)*ZH(gI2,0)
      + ZH(gI1,0)*ZH(gI2,1)) + KroneckerDelta(0,gO2)*(-3*ZH(gI1,0)*ZH(gI2,0) + ZH
      (gI1,1)*ZH(gI2,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhAhAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = -0.05*(vd*KroneckerDelta(0,gO2) - vu*
      KroneckerDelta(1,gO2))*(3*Sqr(g1) + 5*Sqr(g2))*(ZA(gI1,0)*ZA(gI2,0) - ZA(gI1
      ,1)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjHpmHpm(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(KroneckerDelta(0,gO2)*(ZP(gI1,0)
      *(vd*(3*Sqr(g1) + 5*Sqr(g2))*ZP(gI2,0) + 5*vu*Sqr(g2)*ZP(gI2,1)) + ZP(gI1,1)
      *(5*vu*Sqr(g2)*ZP(gI2,0) + vd*(-3*Sqr(g1) + 5*Sqr(g2))*ZP(gI2,1)))) +
      KroneckerDelta(1,gO2)*(ZP(gI1,0)*(vu*(3*Sqr(g1) - 5*Sqr(g2))*ZP(gI2,0) - 5*
      vd*Sqr(g2)*ZP(gI2,1)) - ZP(gI1,1)*(5*vd*Sqr(g2)*ZP(gI2,0) + vu*(3*Sqr(g1) +
      5*Sqr(g2))*ZP(gI2,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjSbSb(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZB(gI2,1))*(
      7.0710678118654755*Conj(TYd(2,2))*KroneckerDelta(0,gO2)*ZB(gI1,0) + (-(vd*
      KroneckerDelta(0,gO2)) + vu*KroneckerDelta(1,gO2))*Sqr(g1)*ZB(gI1,1) - 5*
      Conj(Yd(2,2))*(1.4142135623730951*KroneckerDelta(1,gO2)*Mu*ZB(gI1,0) - 2*vd*
      KroneckerDelta(0,gO2)*Yd(2,2)*ZB(gI1,1))) + Conj(ZB(gI2,0))*(-(
      KroneckerDelta(1,gO2)*(vu*(Sqr(g1) + 5*Sqr(g2))*ZB(gI1,0) -
      14.142135623730951*Conj(Mu)*Yd(2,2)*ZB(gI1,1))) + KroneckerDelta(0,gO2)*(vd*
      (-20*AbsSqr(Yd(2,2)) + Sqr(g1) + 5*Sqr(g2))*ZB(gI1,0) - 14.142135623730951*
      ZB(gI1,1)*TYd(2,2))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjScSc(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZC(gI2,1))*(
      -7.0710678118654755*Conj(TYu(1,1))*KroneckerDelta(1,gO2)*ZC(gI1,0) + 2*(-(vd
      *KroneckerDelta(0,gO2)) + vu*KroneckerDelta(1,gO2))*Sqr(g1)*ZC(gI1,1) + 5*
      Conj(Yu(1,1))*(1.4142135623730951*KroneckerDelta(0,gO2)*Mu*ZC(gI1,0) - 2*vu*
      KroneckerDelta(1,gO2)*Yu(1,1)*ZC(gI1,1))) + Conj(ZC(gI2,0))*(KroneckerDelta(
      0,gO2)*(vd*(Sqr(g1) - 5*Sqr(g2))*ZC(gI1,0) + 14.142135623730951*Conj(Mu)*Yu(
      1,1)*ZC(gI1,1)) - KroneckerDelta(1,gO2)*(vu*(20*AbsSqr(Yu(1,1)) + Sqr(g1) -
      5*Sqr(g2))*ZC(gI1,0) + 14.142135623730951*ZC(gI1,1)*TYu(1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjSdSd(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZD(gI2,1))*(
      7.0710678118654755*Conj(TYd(0,0))*KroneckerDelta(0,gO2)*ZD(gI1,0) + (-(vd*
      KroneckerDelta(0,gO2)) + vu*KroneckerDelta(1,gO2))*Sqr(g1)*ZD(gI1,1) - 5*
      Conj(Yd(0,0))*(1.4142135623730951*KroneckerDelta(1,gO2)*Mu*ZD(gI1,0) - 2*vd*
      KroneckerDelta(0,gO2)*Yd(0,0)*ZD(gI1,1))) + Conj(ZD(gI2,0))*(-(
      KroneckerDelta(1,gO2)*(vu*(Sqr(g1) + 5*Sqr(g2))*ZD(gI1,0) -
      14.142135623730951*Conj(Mu)*Yd(0,0)*ZD(gI1,1))) + KroneckerDelta(0,gO2)*(vd*
      (-20*AbsSqr(Yd(0,0)) + Sqr(g1) + 5*Sqr(g2))*ZD(gI1,0) - 14.142135623730951*
      ZD(gI1,1)*TYd(0,0))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjSeSe(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZE(gI2,1))*(
      7.0710678118654755*Conj(TYe(0,0))*KroneckerDelta(0,gO2)*ZE(gI1,0) + 3*(-(vd*
      KroneckerDelta(0,gO2)) + vu*KroneckerDelta(1,gO2))*Sqr(g1)*ZE(gI1,1) - 5*
      Conj(Ye(0,0))*(1.4142135623730951*KroneckerDelta(1,gO2)*Mu*ZE(gI1,0) - 2*vd*
      KroneckerDelta(0,gO2)*Ye(0,0)*ZE(gI1,1))) + Conj(ZE(gI2,0))*(KroneckerDelta(
      1,gO2)*(vu*(3*Sqr(g1) - 5*Sqr(g2))*ZE(gI1,0) + 14.142135623730951*Conj(Mu)*
      Ye(0,0)*ZE(gI1,1)) - KroneckerDelta(0,gO2)*(vd*(20*AbsSqr(Ye(0,0)) + 3*Sqr(
      g1) - 5*Sqr(g2))*ZE(gI1,0) + 14.142135623730951*ZE(gI1,1)*TYe(0,0))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjSmSm(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZM(gI2,1))*(
      7.0710678118654755*Conj(TYe(1,1))*KroneckerDelta(0,gO2)*ZM(gI1,0) + 3*(-(vd*
      KroneckerDelta(0,gO2)) + vu*KroneckerDelta(1,gO2))*Sqr(g1)*ZM(gI1,1) - 5*
      Conj(Ye(1,1))*(1.4142135623730951*KroneckerDelta(1,gO2)*Mu*ZM(gI1,0) - 2*vd*
      KroneckerDelta(0,gO2)*Ye(1,1)*ZM(gI1,1))) + Conj(ZM(gI2,0))*(KroneckerDelta(
      1,gO2)*(vu*(3*Sqr(g1) - 5*Sqr(g2))*ZM(gI1,0) + 14.142135623730951*Conj(Mu)*
      Ye(1,1)*ZM(gI1,1)) - KroneckerDelta(0,gO2)*(vd*(20*AbsSqr(Ye(1,1)) + 3*Sqr(
      g1) - 5*Sqr(g2))*ZM(gI1,0) + 14.142135623730951*ZM(gI1,1)*TYe(1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjSsSs(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZS(gI2,1))*(
      7.0710678118654755*Conj(TYd(1,1))*KroneckerDelta(0,gO2)*ZS(gI1,0) + (-(vd*
      KroneckerDelta(0,gO2)) + vu*KroneckerDelta(1,gO2))*Sqr(g1)*ZS(gI1,1) - 5*
      Conj(Yd(1,1))*(1.4142135623730951*KroneckerDelta(1,gO2)*Mu*ZS(gI1,0) - 2*vd*
      KroneckerDelta(0,gO2)*Yd(1,1)*ZS(gI1,1))) + Conj(ZS(gI2,0))*(-(
      KroneckerDelta(1,gO2)*(vu*(Sqr(g1) + 5*Sqr(g2))*ZS(gI1,0) -
      14.142135623730951*Conj(Mu)*Yd(1,1)*ZS(gI1,1))) + KroneckerDelta(0,gO2)*(vd*
      (-20*AbsSqr(Yd(1,1)) + Sqr(g1) + 5*Sqr(g2))*ZS(gI1,0) - 14.142135623730951*
      ZS(gI1,1)*TYd(1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjStSt(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZT(gI2,1))*(
      -7.0710678118654755*Conj(TYu(2,2))*KroneckerDelta(1,gO2)*ZT(gI1,0) + 2*(-(vd
      *KroneckerDelta(0,gO2)) + vu*KroneckerDelta(1,gO2))*Sqr(g1)*ZT(gI1,1) + 5*
      Conj(Yu(2,2))*(1.4142135623730951*KroneckerDelta(0,gO2)*Mu*ZT(gI1,0) - 2*vu*
      KroneckerDelta(1,gO2)*Yu(2,2)*ZT(gI1,1))) + Conj(ZT(gI2,0))*(KroneckerDelta(
      0,gO2)*(vd*(Sqr(g1) - 5*Sqr(g2))*ZT(gI1,0) + 14.142135623730951*Conj(Mu)*Yu(
      2,2)*ZT(gI1,1)) - KroneckerDelta(1,gO2)*(vu*(20*AbsSqr(Yu(2,2)) + Sqr(g1) -
      5*Sqr(g2))*ZT(gI1,0) + 14.142135623730951*ZT(gI1,1)*TYu(2,2))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjStauStau(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-2*Conj(ZTau(gI2,1))*(
      7.0710678118654755*Conj(TYe(2,2))*KroneckerDelta(0,gO2)*ZTau(gI1,0) + 3*(-(
      vd*KroneckerDelta(0,gO2)) + vu*KroneckerDelta(1,gO2))*Sqr(g1)*ZTau(gI1,1) -
      5*Conj(Ye(2,2))*(1.4142135623730951*KroneckerDelta(1,gO2)*Mu*ZTau(gI1,0) - 2
      *vd*KroneckerDelta(0,gO2)*Ye(2,2)*ZTau(gI1,1))) + Conj(ZTau(gI2,0))*(
      KroneckerDelta(1,gO2)*(vu*(3*Sqr(g1) - 5*Sqr(g2))*ZTau(gI1,0) +
      14.142135623730951*Conj(Mu)*Ye(2,2)*ZTau(gI1,1)) - KroneckerDelta(0,gO2)*(vd
      *(20*AbsSqr(Ye(2,2)) + 3*Sqr(g1) - 5*Sqr(g2))*ZTau(gI1,0) +
      14.142135623730951*ZTau(gI1,1)*TYe(2,2))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjSuSu(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(2*Conj(ZU(gI2,1))*(
      -7.0710678118654755*Conj(TYu(0,0))*KroneckerDelta(1,gO2)*ZU(gI1,0) + 2*(-(vd
      *KroneckerDelta(0,gO2)) + vu*KroneckerDelta(1,gO2))*Sqr(g1)*ZU(gI1,1) + 5*
      Conj(Yu(0,0))*(1.4142135623730951*KroneckerDelta(0,gO2)*Mu*ZU(gI1,0) - 2*vu*
      KroneckerDelta(1,gO2)*Yu(0,0)*ZU(gI1,1))) + Conj(ZU(gI2,0))*(KroneckerDelta(
      0,gO2)*(vd*(Sqr(g1) - 5*Sqr(g2))*ZU(gI1,0) + 14.142135623730951*Conj(Mu)*Yu(
      0,0)*ZU(gI1,1)) - KroneckerDelta(1,gO2)*(vu*(20*AbsSqr(Yu(0,0)) + Sqr(g1) -
      5*Sqr(g2))*ZU(gI1,0) + 14.142135623730951*ZU(gI1,1)*TYu(0,0))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhhhhh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(3*Sqr(g1) + 5*Sqr(g2))*(
      KroneckerDelta(1,gO2)*(ZH(gI1,0)*(vu*ZH(gI2,0) + vd*ZH(gI2,1)) + ZH(gI1,1)*(
      vd*ZH(gI2,0) - 3*vu*ZH(gI2,1))) + KroneckerDelta(0,gO2)*(ZH(gI1,1)*(vu*ZH(
      gI2,0) + vd*ZH(gI2,1)) + ZH(gI1,0)*(-3*vd*ZH(gI2,0) + vu*ZH(gI2,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUhhbarChaChaPR(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = -0.7071067811865475*g2*(KroneckerDelta(0
      ,gO2)*UM(gI1,1)*UP(gI2,0) + KroneckerDelta(1,gO2)*UM(gI1,0)*UP(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhbarChaChaPL(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = -0.7071067811865475*g2*(Conj(UM(gI2,1))*
      Conj(UP(gI1,0))*KroneckerDelta(0,gO1) + Conj(UM(gI2,0))*Conj(UP(gI1,1))*
      KroneckerDelta(1,gO1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhChiChiPR(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(KroneckerDelta(0,gO2)*(ZN(gI1,2)*(
      3.872983346207417*g1*ZN(gI2,0) - 5*g2*ZN(gI2,1)) + (3.872983346207417*g1*ZN(
      gI1,0) - 5*g2*ZN(gI1,1))*ZN(gI2,2)) - KroneckerDelta(1,gO2)*(ZN(gI1,3)*(
      3.872983346207417*g1*ZN(gI2,0) - 5*g2*ZN(gI2,1)) + (3.872983346207417*g1*ZN(
      gI1,0) - 5*g2*ZN(gI1,1))*ZN(gI2,3)));

   return result;
}

std::complex<double> CLASSNAME::CpUhhChiChiPL(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZN(gI1,2))*(3.872983346207417*
      g1*Conj(ZN(gI2,0)) - 5*g2*Conj(ZN(gI2,1)))*KroneckerDelta(0,gO1) - 5*g2*Conj
      (ZN(gI1,1))*Conj(ZN(gI2,2))*KroneckerDelta(0,gO1) - 3.872983346207417*g1*
      Conj(ZN(gI1,3))*Conj(ZN(gI2,0))*KroneckerDelta(1,gO1) + 5*g2*Conj(ZN(gI1,3))
      *Conj(ZN(gI2,1))*KroneckerDelta(1,gO1) + 5*g2*Conj(ZN(gI1,1))*Conj(ZN(gI2,3)
      )*KroneckerDelta(1,gO1) + 3.872983346207417*g1*Conj(ZN(gI1,0))*(Conj(ZN(gI2,
      2))*KroneckerDelta(0,gO1) - Conj(ZN(gI2,3))*KroneckerDelta(1,gO1)));

   return result;
}

std::complex<double> CLASSNAME::CpUhhVZAh(int gO2, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,-0.1)*(5*g2*Cos(
      ThetaW()) + 3.872983346207417*g1*Sin(ThetaW()))*(KroneckerDelta(0,gO2)*ZA(
      gI2,0) - KroneckerDelta(1,gO2)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpUhhconjVWmHpm(int gO2, int gI2) const
{
   const std::complex<double> result = 0.5*g2*(KroneckerDelta(0,gO2)*ZP(gI2,0)
      - KroneckerDelta(1,gO2)*ZP(gI2,1));

   return result;
}

double CLASSNAME::CpUhhbarFeFePR(int gO2) const
{
   const double result = -0.7071067811865475*Conj(Ye(0,0))*KroneckerDelta(0,gO2
      );

   return result;
}

double CLASSNAME::CpUhhbarFeFePL(int gO1) const
{
   const double result = -0.7071067811865475*KroneckerDelta(0,gO1)*Ye(0,0);

   return result;
}

double CLASSNAME::CpUhhbarFmFmPR(int gO2) const
{
   const double result = -0.7071067811865475*Conj(Ye(1,1))*KroneckerDelta(0,gO2
      );

   return result;
}

double CLASSNAME::CpUhhbarFmFmPL(int gO1) const
{
   const double result = -0.7071067811865475*KroneckerDelta(0,gO1)*Ye(1,1);

   return result;
}

double CLASSNAME::CpUhhbarFtauFtauPR(int gO2) const
{
   const double result = -0.7071067811865475*Conj(Ye(2,2))*KroneckerDelta(0,gO2
      );

   return result;
}

double CLASSNAME::CpUhhbarFtauFtauPL(int gO1) const
{
   const double result = -0.7071067811865475*KroneckerDelta(0,gO1)*Ye(2,2);

   return result;
}

double CLASSNAME::CpUhhbarFbFbPR(int gO2, int , int ) const
{
   const double result = -0.7071067811865475*Conj(Yd(2,2))*KroneckerDelta(0,gO2
      );

   return result;
}

double CLASSNAME::CpUhhbarFbFbPL(int gO1, int , int ) const
{
   const double result = -0.7071067811865475*KroneckerDelta(0,gO1)*Yd(2,2);

   return result;
}

double CLASSNAME::CpUhhbarFcFcPR(int gO2, int , int ) const
{
   const double result = -0.7071067811865475*Conj(Yu(1,1))*KroneckerDelta(1,gO2
      );

   return result;
}

double CLASSNAME::CpUhhbarFcFcPL(int gO1, int , int ) const
{
   const double result = -0.7071067811865475*KroneckerDelta(1,gO1)*Yu(1,1);

   return result;
}

double CLASSNAME::CpUhhbarFdFdPR(int gO2, int , int ) const
{
   const double result = -0.7071067811865475*Conj(Yd(0,0))*KroneckerDelta(0,gO2
      );

   return result;
}

double CLASSNAME::CpUhhbarFdFdPL(int gO1, int , int ) const
{
   const double result = -0.7071067811865475*KroneckerDelta(0,gO1)*Yd(0,0);

   return result;
}

double CLASSNAME::CpUhhbarFsFsPR(int gO2, int , int ) const
{
   const double result = -0.7071067811865475*Conj(Yd(1,1))*KroneckerDelta(0,gO2
      );

   return result;
}

double CLASSNAME::CpUhhbarFsFsPL(int gO1, int , int ) const
{
   const double result = -0.7071067811865475*KroneckerDelta(0,gO1)*Yd(1,1);

   return result;
}

double CLASSNAME::CpUhhbarFtFtPR(int gO2, int , int ) const
{
   const double result = -0.7071067811865475*Conj(Yu(2,2))*KroneckerDelta(1,gO2
      );

   return result;
}

double CLASSNAME::CpUhhbarFtFtPL(int gO1, int , int ) const
{
   const double result = -0.7071067811865475*KroneckerDelta(1,gO1)*Yu(2,2);

   return result;
}

double CLASSNAME::CpUhhbarFuFuPR(int gO2, int , int ) const
{
   const double result = -0.7071067811865475*Conj(Yu(0,0))*KroneckerDelta(1,gO2
      );

   return result;
}

double CLASSNAME::CpUhhbarFuFuPL(int gO1, int , int ) const
{
   const double result = -0.7071067811865475*KroneckerDelta(1,gO1)*Yu(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbargWmgWm(int gO1) const
{
   const std::complex<double> result = std::complex<double>(0,-0.25)*(vd*
      KroneckerDelta(0,gO1) - vu*KroneckerDelta(1,gO1))*Sqr(g2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbargWmCgWmC(int gO1) const
{
   const std::complex<double> result = std::complex<double>(0,0.25)*(vd*
      KroneckerDelta(0,gO1) - vu*KroneckerDelta(1,gO1))*Sqr(g2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) + KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(
      7.745966692414834*g1*g2*Sin(2*ThetaW()) + 3*Sqr(g1) + 5*Sqr(g2) + Cos(2*
      ThetaW())*(-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = -0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(3*Sqr(
      g1) + 5*Sqr(g2));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = -0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(3*Sqr(
      g1) + 5*Sqr(g2));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = -0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(3*Sqr(
      g1) + 5*Sqr(g2));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjVWmVWm(int gO1, int gO2) const
{
   const std::complex<double> result = 0.5*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) + KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*Sqr(g2)
      ;

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(3*Sqr(g1) + 5*Sqr(g2))*(
      KroneckerDelta(1,gO1)*(KroneckerDelta(0,gO2)*(ZA(gI1,1)*ZA(gI2,0) + ZA(gI1,0
      )*ZA(gI2,1)) + KroneckerDelta(1,gO2)*(ZA(gI1,0)*ZA(gI2,0) - 3*ZA(gI1,1)*ZA(
      gI2,1))) + KroneckerDelta(0,gO1)*(KroneckerDelta(1,gO2)*(ZA(gI1,1)*ZA(gI2,0)
      + ZA(gI1,0)*ZA(gI2,1)) + KroneckerDelta(0,gO2)*(-3*ZA(gI1,0)*ZA(gI2,0) + ZA
      (gI1,1)*ZA(gI2,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(KroneckerDelta(0,gO1)*(5*
      KroneckerDelta(1,gO2)*Sqr(g2)*(ZP(gI1,1)*ZP(gI2,0) + ZP(gI1,0)*ZP(gI2,1)) -
      KroneckerDelta(0,gO2)*((3*Sqr(g1) + 5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) + (-3*Sqr
      (g1) + 5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1))) + KroneckerDelta(1,gO1)*(5*
      KroneckerDelta(0,gO2)*Sqr(g2)*(ZP(gI1,1)*ZP(gI2,0) + ZP(gI1,0)*ZP(gI2,1)) +
      KroneckerDelta(1,gO2)*((3*Sqr(g1) - 5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) - (3*Sqr(
      g1) + 5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZB(gI2,0))*(-(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g2))) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Yd(2,2)) + Sqr(g1) + 5*Sqr(g2)))*ZB(gI1,0)
      + 2*Conj(ZB(gI2,1))*(-(KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1))
      + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Yd(2,2)) + Sqr(g1
      )))*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZC(gI2,0))*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)) - KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(20*AbsSqr(Yu(1,1)) + Sqr(g1) - 5*Sqr(g2)))*ZC(gI1,0)
      - 4*Conj(ZC(gI2,1))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g1) -
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(1,1)) + Sqr(g1)))*
      ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZD(gI2,0))*(-(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g2))) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Yd(0,0)) + Sqr(g1) + 5*Sqr(g2)))*ZD(gI1,0)
      + 2*Conj(ZD(gI2,1))*(-(KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1))
      + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Yd(0,0)) + Sqr(g1
      )))*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZE(gI2,0))*(KroneckerDelta(1,
      gO1)*KroneckerDelta(1,gO2)*(3*Sqr(g1) - 5*Sqr(g2)) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Ye(0,0)) - 3*Sqr(g1) + 5*Sqr(g2)))*ZE(gI1,
      0) + 2*Conj(ZE(gI2,1))*(-3*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(
      g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Ye(0,0)) + 3*
      Sqr(g1)))*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZM(gI2,0))*(KroneckerDelta(1,
      gO1)*KroneckerDelta(1,gO2)*(3*Sqr(g1) - 5*Sqr(g2)) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Ye(1,1)) - 3*Sqr(g1) + 5*Sqr(g2)))*ZM(gI1,
      0) + 2*Conj(ZM(gI2,1))*(-3*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(
      g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Ye(1,1)) + 3*
      Sqr(g1)))*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZS(gI2,0))*(-(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g2))) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Yd(1,1)) + Sqr(g1) + 5*Sqr(g2)))*ZS(gI1,0)
      + 2*Conj(ZS(gI2,1))*(-(KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1))
      + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Yd(1,1)) + Sqr(g1
      )))*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZT(gI2,0))*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)) - KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(20*AbsSqr(Yu(2,2)) + Sqr(g1) - 5*Sqr(g2)))*ZT(gI1,0)
      - 4*Conj(ZT(gI2,1))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g1) -
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(2,2)) + Sqr(g1)))*
      ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZTau(gI2,0))*(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(3*Sqr(g1) - 5*Sqr(g2)) + KroneckerDelta(0,gO1)
      *KroneckerDelta(0,gO2)*(-20*AbsSqr(Ye(2,2)) - 3*Sqr(g1) + 5*Sqr(g2)))*ZTau(
      gI1,0) + 2*Conj(ZTau(gI2,1))*(-3*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)
      *Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Ye(2,2))
      + 3*Sqr(g1)))*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZU(gI2,0))*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)) - KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(20*AbsSqr(Yu(0,0)) + Sqr(g1) - 5*Sqr(g2)))*ZU(gI1,0)
      - 4*Conj(ZU(gI2,1))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g1) -
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(0,0)) + Sqr(g1)))*
      ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhUAhhhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = -0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(3*Sqr(
      g1) + 5*Sqr(g2))*(ZH(gI1,0)*ZH(gI2,0) - ZH(gI1,1)*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjHpmHpm(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,0.25)*(vu*
      KroneckerDelta(0,gO2) + vd*KroneckerDelta(1,gO2))*Sqr(g2)*(ZP(gI1,1)*ZP(gI2,
      0) - ZP(gI1,0)*ZP(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjSbSb(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZB(gI2,1))*(Conj(TYd(2,2))*KroneckerDelta(0,gO2) +
      Conj(Yd(2,2))*KroneckerDelta(1,gO2)*Mu)*ZB(gI1,0) - Conj(ZB(gI2,0))*ZB(gI1,
      1)*(Conj(Mu)*KroneckerDelta(1,gO2)*Yd(2,2) + KroneckerDelta(0,gO2)*TYd(2,2))
      );

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjScSc(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZC(gI2,1))*Conj(TYu(1,1))*KroneckerDelta(1,gO2)*ZC
      (gI1,0) + Conj(Yu(1,1))*Conj(ZC(gI2,1))*KroneckerDelta(0,gO2)*Mu*ZC(gI1,0) -
      Conj(ZC(gI2,0))*ZC(gI1,1)*(Conj(Mu)*KroneckerDelta(0,gO2)*Yu(1,1) +
      KroneckerDelta(1,gO2)*TYu(1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjSdSd(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZD(gI2,1))*(Conj(TYd(0,0))*KroneckerDelta(0,gO2) +
      Conj(Yd(0,0))*KroneckerDelta(1,gO2)*Mu)*ZD(gI1,0) - Conj(ZD(gI2,0))*ZD(gI1,
      1)*(Conj(Mu)*KroneckerDelta(1,gO2)*Yd(0,0) + KroneckerDelta(0,gO2)*TYd(0,0))
      );

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjSeSe(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZE(gI2,1))*(Conj(TYe(0,0))*KroneckerDelta(0,gO2) +
      Conj(Ye(0,0))*KroneckerDelta(1,gO2)*Mu)*ZE(gI1,0) - Conj(ZE(gI2,0))*ZE(gI1,
      1)*(Conj(Mu)*KroneckerDelta(1,gO2)*Ye(0,0) + KroneckerDelta(0,gO2)*TYe(0,0))
      );

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjSmSm(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZM(gI2,1))*(Conj(TYe(1,1))*KroneckerDelta(0,gO2) +
      Conj(Ye(1,1))*KroneckerDelta(1,gO2)*Mu)*ZM(gI1,0) - Conj(ZM(gI2,0))*ZM(gI1,
      1)*(Conj(Mu)*KroneckerDelta(1,gO2)*Ye(1,1) + KroneckerDelta(0,gO2)*TYe(1,1))
      );

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjSsSs(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZS(gI2,1))*(Conj(TYd(1,1))*KroneckerDelta(0,gO2) +
      Conj(Yd(1,1))*KroneckerDelta(1,gO2)*Mu)*ZS(gI1,0) - Conj(ZS(gI2,0))*ZS(gI1,
      1)*(Conj(Mu)*KroneckerDelta(1,gO2)*Yd(1,1) + KroneckerDelta(0,gO2)*TYd(1,1))
      );

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjStSt(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZT(gI2,1))*Conj(TYu(2,2))*KroneckerDelta(1,gO2)*ZT
      (gI1,0) + Conj(Yu(2,2))*Conj(ZT(gI2,1))*KroneckerDelta(0,gO2)*Mu*ZT(gI1,0) -
      Conj(ZT(gI2,0))*ZT(gI1,1)*(Conj(Mu)*KroneckerDelta(0,gO2)*Yu(2,2) +
      KroneckerDelta(1,gO2)*TYu(2,2)));

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjStauStau(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZTau(gI2,1))*(Conj(TYe(2,2))*KroneckerDelta(0,gO2)
      + Conj(Ye(2,2))*KroneckerDelta(1,gO2)*Mu)*ZTau(gI1,0) - Conj(ZTau(gI2,0))*
      ZTau(gI1,1)*(Conj(Mu)*KroneckerDelta(1,gO2)*Ye(2,2) + KroneckerDelta(0,gO2)*
      TYe(2,2)));

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjSuSu(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*(Conj(ZU(gI2,1))*Conj(TYu(0,0))*KroneckerDelta(1,gO2)*ZU
      (gI1,0) + Conj(Yu(0,0))*Conj(ZU(gI2,1))*KroneckerDelta(0,gO2)*Mu*ZU(gI1,0) -
      Conj(ZU(gI2,0))*ZU(gI1,1)*(Conj(Mu)*KroneckerDelta(0,gO2)*Yu(0,0) +
      KroneckerDelta(1,gO2)*TYu(0,0)));

   return result;
}

std::complex<double> CLASSNAME::CpUAhhhAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = -0.05*(3*Sqr(g1) + 5*Sqr(g2))*(
      KroneckerDelta(0,gO2)*ZA(gI2,0) - KroneckerDelta(1,gO2)*ZA(gI2,1))*(vd*ZH(
      gI1,0) - vu*ZH(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarChaChaPR(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*g2*(KroneckerDelta(0,gO2)*UM(gI1,1)*UP(gI2,0) +
      KroneckerDelta(1,gO2)*UM(gI1,0)*UP(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarChaChaPL(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*g2*(Conj(UM(gI2,1))*Conj(UP(gI1,0))*KroneckerDelta(0,gO1
      ) + Conj(UM(gI2,0))*Conj(UP(gI1,1))*KroneckerDelta(1,gO1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhChiChiPR(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,0.1)*(
      KroneckerDelta(0,gO2)*(ZN(gI1,2)*(3.872983346207417*g1*ZN(gI2,0) - 5*g2*ZN(
      gI2,1)) + (3.872983346207417*g1*ZN(gI1,0) - 5*g2*ZN(gI1,1))*ZN(gI2,2)) -
      KroneckerDelta(1,gO2)*(ZN(gI1,3)*(3.872983346207417*g1*ZN(gI2,0) - 5*g2*ZN(
      gI2,1)) + (3.872983346207417*g1*ZN(gI1,0) - 5*g2*ZN(gI1,1))*ZN(gI2,3)));

   return result;
}

std::complex<double> CLASSNAME::CpUAhChiChiPL(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,-0.1)*(Conj(ZN(
      gI1,2))*(3.872983346207417*g1*Conj(ZN(gI2,0)) - 5*g2*Conj(ZN(gI2,1)))*
      KroneckerDelta(0,gO1) - 5*g2*Conj(ZN(gI1,1))*Conj(ZN(gI2,2))*KroneckerDelta(
      0,gO1) - 3.872983346207417*g1*Conj(ZN(gI1,3))*Conj(ZN(gI2,0))*KroneckerDelta
      (1,gO1) + 5*g2*Conj(ZN(gI1,3))*Conj(ZN(gI2,1))*KroneckerDelta(1,gO1) + 5*g2*
      Conj(ZN(gI1,1))*Conj(ZN(gI2,3))*KroneckerDelta(1,gO1) + 3.872983346207417*g1
      *Conj(ZN(gI1,0))*(Conj(ZN(gI2,2))*KroneckerDelta(0,gO1) - Conj(ZN(gI2,3))*
      KroneckerDelta(1,gO1)));

   return result;
}

std::complex<double> CLASSNAME::CpUAhVZhh(int gO2, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,-0.1)*(5*g2*Cos(
      ThetaW()) + 3.872983346207417*g1*Sin(ThetaW()))*(KroneckerDelta(0,gO2)*ZH(
      gI2,0) - KroneckerDelta(1,gO2)*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhconjVWmHpm(int gO2, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,-0.5)*g2*(
      KroneckerDelta(0,gO2)*ZP(gI2,0) + KroneckerDelta(1,gO2)*ZP(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFeFePR(int gO2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Ye(0,0))*KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFeFePL(int gO1) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*KroneckerDelta(0,gO1)*Ye(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFmFmPR(int gO2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Ye(1,1))*KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFmFmPL(int gO1) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*KroneckerDelta(0,gO1)*Ye(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFtauFtauPR(int gO2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Ye(2,2))*KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFtauFtauPL(int gO1) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*KroneckerDelta(0,gO1)*Ye(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFbFbPR(int gO2, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yd(2,2))*KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFbFbPL(int gO1, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*KroneckerDelta(0,gO1)*Yd(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFcFcPR(int gO2, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yu(1,1))*KroneckerDelta(1,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFcFcPL(int gO1, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*KroneckerDelta(1,gO1)*Yu(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFdFdPR(int gO2, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yd(0,0))*KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFdFdPL(int gO1, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*KroneckerDelta(0,gO1)*Yd(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFsFsPR(int gO2, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yd(1,1))*KroneckerDelta(0,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFsFsPL(int gO1, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*KroneckerDelta(0,gO1)*Yd(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFtFtPR(int gO2, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yu(2,2))*KroneckerDelta(1,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFtFtPL(int gO1, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*KroneckerDelta(1,gO1)*Yu(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFuFuPR(int gO2, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yu(0,0))*KroneckerDelta(1,gO2);

   return result;
}

std::complex<double> CLASSNAME::CpUAhbarFuFuPL(int gO1, int , int ) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*KroneckerDelta(1,gO1)*Yu(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmVWmVP(int gO2) const
{
   const std::complex<double> result = -0.3872983346207417*g1*g2*Cos(ThetaW())*
      (vd*KroneckerDelta(0,gO2) - vu*KroneckerDelta(1,gO2));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmVZVWm(int gO2) const
{
   const std::complex<double> result = 0.3872983346207417*g1*g2*(vd*
      KroneckerDelta(0,gO2) - vu*KroneckerDelta(1,gO2))*Sin(ThetaW());

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmbargWmCgZ(int gO1) const
{
   const std::complex<double> result = 0.05*g2*(vd*KroneckerDelta(0,gO1) - vu*
      KroneckerDelta(1,gO1))*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW
      ()));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmgWmCbargZ(int gO2) const
{
   const std::complex<double> result = -0.05*g2*(vd*KroneckerDelta(0,gO2) - vu*
      KroneckerDelta(1,gO2))*(5*g2*Cos(ThetaW()) + 3.872983346207417*g1*Sin(ThetaW
      ()));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmbargZgWm(int gO1) const
{
   const std::complex<double> result = -0.05*g2*(vd*KroneckerDelta(0,gO1) - vu*
      KroneckerDelta(1,gO1))*(5*g2*Cos(ThetaW()) + 3.872983346207417*g1*Sin(ThetaW
      ()));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmgZbargWm(int gO2) const
{
   const std::complex<double> result = 0.05*g2*(vd*KroneckerDelta(0,gO2) - vu*
      KroneckerDelta(1,gO2))*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW
      ()));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmVZVZ(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) + KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*(
      -7.745966692414834*g1*g2*Sin(2*ThetaW()) + 3*Sqr(g1) + 5*Sqr(g2) + Cos(2*
      ThetaW())*(-3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjSveLSveL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(3*Sqr(g1) - 5*Sqr(g2)) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Ye(0,0)) - 3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjSvmLSvmL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(3*Sqr(g1) - 5*Sqr(g2)) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Ye(1,1)) - 3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjSvtLSvtL(int gO1, int gO2) const
{
   const std::complex<double> result = 0.05*(KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(3*Sqr(g1) - 5*Sqr(g2)) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Ye(2,2)) - 3*Sqr(g1) + 5*Sqr(g2)));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjVWmVWm(int gO1, int gO2) const
{
   const std::complex<double> result = 0.5*(KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2) + KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*Sqr(g2)
      ;

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmAhAh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(KroneckerDelta(0,gO1)*(5*
      KroneckerDelta(1,gO2)*Sqr(g2)*(ZA(gI1,1)*ZA(gI2,0) + ZA(gI1,0)*ZA(gI2,1)) -
      KroneckerDelta(0,gO2)*((3*Sqr(g1) + 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) + (-3*Sqr
      (g1) + 5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1))) + KroneckerDelta(1,gO1)*(5*
      KroneckerDelta(0,gO2)*Sqr(g2)*(ZA(gI1,1)*ZA(gI2,0) + ZA(gI1,0)*ZA(gI2,1)) +
      KroneckerDelta(1,gO2)*((3*Sqr(g1) - 5*Sqr(g2))*ZA(gI1,0)*ZA(gI2,0) - (3*Sqr(
      g1) + 5*Sqr(g2))*ZA(gI1,1)*ZA(gI2,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjHpmHpm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(3*Sqr(g1) + 5*Sqr(g2))*(
      KroneckerDelta(1,gO1)*(KroneckerDelta(0,gO2)*ZP(gI1,1)*ZP(gI2,0) +
      KroneckerDelta(1,gO2)*(ZP(gI1,0)*ZP(gI2,0) - 2*ZP(gI1,1)*ZP(gI2,1))) +
      KroneckerDelta(0,gO1)*(KroneckerDelta(1,gO2)*ZP(gI1,0)*ZP(gI2,1) +
      KroneckerDelta(0,gO2)*(-2*ZP(gI1,0)*ZP(gI2,0) + ZP(gI1,1)*ZP(gI2,1))));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjSbSb(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZB(gI2,0))*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)) - KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(20*AbsSqr(Yu(2,2)) + Sqr(g1) - 5*Sqr(g2)))*ZB(gI1,0)
      + 2*Conj(ZB(gI2,1))*(-(KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1))
      + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Yd(2,2)) + Sqr(g1)
      ))*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjScSc(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZC(gI2,0))*(-(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g2))) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Yd(1,1)) + Sqr(g1) + 5*Sqr(g2)))*ZC(gI1,0)
      - 4*Conj(ZC(gI2,1))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g1) -
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(1,1)) + Sqr(g1)))*
      ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjSdSd(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZD(gI2,0))*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)) - KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(20*AbsSqr(Yu(0,0)) + Sqr(g1) - 5*Sqr(g2)))*ZD(gI1,0)
      + 2*Conj(ZD(gI2,1))*(-(KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1))
      + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Yd(0,0)) + Sqr(g1)
      ))*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjSeSe(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZE(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*
      (3*Sqr(g1) + 5*Sqr(g2))*ZE(gI1,0)) + 2*Conj(ZE(gI2,1))*(-3*KroneckerDelta(1,
      gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,
      gO2)*(-10*AbsSqr(Ye(0,0)) + 3*Sqr(g1)))*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjSmSm(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZM(gI2,0))*(KroneckerDelta(
      0,gO1)*KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2))*
      (3*Sqr(g1) + 5*Sqr(g2))*ZM(gI1,0)) + 2*Conj(ZM(gI2,1))*(-3*KroneckerDelta(1,
      gO1)*KroneckerDelta(1,gO2)*Sqr(g1) + KroneckerDelta(0,gO1)*KroneckerDelta(0,
      gO2)*(-10*AbsSqr(Ye(1,1)) + 3*Sqr(g1)))*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjSsSs(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZS(gI2,0))*(KroneckerDelta(0,
      gO1)*KroneckerDelta(0,gO2)*(Sqr(g1) - 5*Sqr(g2)) - KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2)*(20*AbsSqr(Yu(1,1)) + Sqr(g1) - 5*Sqr(g2)))*ZS(gI1,0)
      + 2*Conj(ZS(gI2,1))*(-(KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1))
      + KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Yd(1,1)) + Sqr(g1)
      ))*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjStSt(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZT(gI2,0))*(-(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g2))) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Yd(2,2)) + Sqr(g1) + 5*Sqr(g2)))*ZT(gI1,0)
      - 4*Conj(ZT(gI2,1))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g1) -
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(2,2)) + Sqr(g1)))*
      ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjStauStau(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZTau(gI2,0))*(
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2) - KroneckerDelta(1,gO1)*
      KroneckerDelta(1,gO2))*(3*Sqr(g1) + 5*Sqr(g2))*ZTau(gI1,0)) + 2*Conj(ZTau(
      gI2,1))*(-3*KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*Sqr(g1) +
      KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*(-10*AbsSqr(Ye(2,2)) + 3*Sqr(g1)
      ))*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmconjSuSu(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZU(gI2,0))*(-(KroneckerDelta(
      1,gO1)*KroneckerDelta(1,gO2)*(Sqr(g1) + 5*Sqr(g2))) + KroneckerDelta(0,gO1)*
      KroneckerDelta(0,gO2)*(-20*AbsSqr(Yd(0,0)) + Sqr(g1) + 5*Sqr(g2)))*ZU(gI1,0)
      - 4*Conj(ZU(gI2,1))*(KroneckerDelta(0,gO1)*KroneckerDelta(0,gO2)*Sqr(g1) -
      KroneckerDelta(1,gO1)*KroneckerDelta(1,gO2)*(-5*AbsSqr(Yu(0,0)) + Sqr(g1)))*
      ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpUHpmconjUHpmhhhh(int gO1, int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(KroneckerDelta(0,gO1)*(5*
      KroneckerDelta(1,gO2)*Sqr(g2)*(ZH(gI1,1)*ZH(gI2,0) + ZH(gI1,0)*ZH(gI2,1)) +
      KroneckerDelta(0,gO2)*((3*Sqr(g1) + 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) + (-3*Sqr
      (g1) + 5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1)))) + KroneckerDelta(1,gO1)*(-5*
      KroneckerDelta(0,gO2)*Sqr(g2)*(ZH(gI1,1)*ZH(gI2,0) + ZH(gI1,0)*ZH(gI2,1)) +
      KroneckerDelta(1,gO2)*((3*Sqr(g1) - 5*Sqr(g2))*ZH(gI1,0)*ZH(gI2,0) - (3*Sqr(
      g1) + 5*Sqr(g2))*ZH(gI1,1)*ZH(gI2,1))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmconjScSs(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZS(gI2,1))*(2*Conj(TYd(1,1))*
      KroneckerDelta(0,gO2)*ZC(gI1,0) + Conj(Yd(1,1))*(1.4142135623730951*vu*
      KroneckerDelta(0,gO2)*Yu(1,1)*ZC(gI1,1) + KroneckerDelta(1,gO2)*(2*Mu*ZC(gI1
      ,0) + 1.4142135623730951*vd*Yu(1,1)*ZC(gI1,1)))) - 0.25*Conj(ZS(gI2,0))*(
      KroneckerDelta(0,gO2)*(1.4142135623730951*vd*(-2*AbsSqr(Yd(1,1)) + Sqr(g2))*
      ZC(gI1,0) - 4*Conj(Mu)*Yu(1,1)*ZC(gI1,1)) + KroneckerDelta(1,gO2)*(
      1.4142135623730951*vu*(-2*AbsSqr(Yu(1,1)) + Sqr(g2))*ZC(gI1,0) - 4*ZC(gI1,1)
      *TYu(1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmconjStSb(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZB(gI2,1))*(2*Conj(TYd(2,2))*
      KroneckerDelta(0,gO2)*ZT(gI1,0) + Conj(Yd(2,2))*(1.4142135623730951*vu*
      KroneckerDelta(0,gO2)*Yu(2,2)*ZT(gI1,1) + KroneckerDelta(1,gO2)*(2*Mu*ZT(gI1
      ,0) + 1.4142135623730951*vd*Yu(2,2)*ZT(gI1,1)))) - 0.25*Conj(ZB(gI2,0))*(
      KroneckerDelta(0,gO2)*(1.4142135623730951*vd*(-2*AbsSqr(Yd(2,2)) + Sqr(g2))*
      ZT(gI1,0) - 4*Conj(Mu)*Yu(2,2)*ZT(gI1,1)) + KroneckerDelta(1,gO2)*(
      1.4142135623730951*vu*(-2*AbsSqr(Yu(2,2)) + Sqr(g2))*ZT(gI1,0) - 4*ZT(gI1,1)
      *TYu(2,2)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmconjSuSd(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZD(gI2,1))*(2*Conj(TYd(0,0))*
      KroneckerDelta(0,gO2)*ZU(gI1,0) + Conj(Yd(0,0))*(1.4142135623730951*vu*
      KroneckerDelta(0,gO2)*Yu(0,0)*ZU(gI1,1) + KroneckerDelta(1,gO2)*(2*Mu*ZU(gI1
      ,0) + 1.4142135623730951*vd*Yu(0,0)*ZU(gI1,1)))) - 0.25*Conj(ZD(gI2,0))*(
      KroneckerDelta(0,gO2)*(1.4142135623730951*vd*(-2*AbsSqr(Yd(0,0)) + Sqr(g2))*
      ZU(gI1,0) - 4*Conj(Mu)*Yu(0,0)*ZU(gI1,1)) + KroneckerDelta(1,gO2)*(
      1.4142135623730951*vu*(-2*AbsSqr(Yu(0,0)) + Sqr(g2))*ZU(gI1,0) - 4*ZU(gI1,1)
      *TYu(0,0)));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmHpmAh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,0.25)*Sqr(g2)*(vu
      *ZA(gI2,0) + vd*ZA(gI2,1))*(KroneckerDelta(1,gO2)*ZP(gI1,0) - KroneckerDelta
      (0,gO2)*ZP(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmHpmhh(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(KroneckerDelta(0,gO2)*(ZH(gI2,1)
      *((-3*vu*Sqr(g1) + 5*vu*Sqr(g2))*ZP(gI1,0) + 5*vd*Sqr(g2)*ZP(gI1,1)) + ZH(
      gI2,0)*(vd*(3*Sqr(g1) + 5*Sqr(g2))*ZP(gI1,0) + 5*vu*Sqr(g2)*ZP(gI1,1)))) -
      KroneckerDelta(1,gO2)*(ZH(gI2,0)*(5*vu*Sqr(g2)*ZP(gI1,0) + vd*(-3*Sqr(g1) +
      5*Sqr(g2))*ZP(gI1,1)) + ZH(gI2,1)*(5*vd*Sqr(g2)*ZP(gI1,0) + vu*(3*Sqr(g1) +
      5*Sqr(g2))*ZP(gI1,1))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmChiChaPR(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = -0.1*KroneckerDelta(1,gO2)*(UP(gI2,1)*(
      5.477225575051661*g1*ZN(gI1,0) + 7.0710678118654755*g2*ZN(gI1,1)) + 10*g2*UP
      (gI2,0)*ZN(gI1,3));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmChiChaPL(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(UM(gI2,1))*(5.477225575051661*
      g1*Conj(ZN(gI1,0)) + 7.0710678118654755*g2*Conj(ZN(gI1,1))) - 10*g2*Conj(UM(
      gI2,0))*Conj(ZN(gI1,2)))*KroneckerDelta(0,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmconjSveLSe(int gO2, int gI2) const
{
   const std::complex<double> result = 0.25*(4*Conj(ZE(gI2,1))*(Conj(TYe(0,0))*
      KroneckerDelta(0,gO2) + Conj(Ye(0,0))*KroneckerDelta(1,gO2)*Mu) -
      1.4142135623730951*Conj(ZE(gI2,0))*(vu*KroneckerDelta(1,gO2)*Sqr(g2) + vd*
      KroneckerDelta(0,gO2)*(-2*AbsSqr(Ye(0,0)) + Sqr(g2))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmconjSvmLSm(int gO2, int gI2) const
{
   const std::complex<double> result = 0.25*(4*Conj(ZM(gI2,1))*(Conj(TYe(1,1))*
      KroneckerDelta(0,gO2) + Conj(Ye(1,1))*KroneckerDelta(1,gO2)*Mu) -
      1.4142135623730951*Conj(ZM(gI2,0))*(vu*KroneckerDelta(1,gO2)*Sqr(g2) + vd*
      KroneckerDelta(0,gO2)*(-2*AbsSqr(Ye(1,1)) + Sqr(g2))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmconjSvtLStau(int gO2, int gI2) const
{
   const std::complex<double> result = 0.25*(4*Conj(ZTau(gI2,1))*(Conj(TYe(2,2)
      )*KroneckerDelta(0,gO2) + Conj(Ye(2,2))*KroneckerDelta(1,gO2)*Mu) -
      1.4142135623730951*Conj(ZTau(gI2,0))*(vu*KroneckerDelta(1,gO2)*Sqr(g2) + vd*
      KroneckerDelta(0,gO2)*(-2*AbsSqr(Ye(2,2)) + Sqr(g2))));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmVWmAh(int gO2, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,-0.5)*g2*(
      KroneckerDelta(0,gO2)*ZA(gI2,0) + KroneckerDelta(1,gO2)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmVWmhh(int gO2, int gI2) const
{
   const std::complex<double> result = 0.5*g2*(KroneckerDelta(0,gO2)*ZH(gI2,0)
      - KroneckerDelta(1,gO2)*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmVPHpm(int gO2, int gI2) const
{
   const std::complex<double> result = IF(gI2 < 2,-0.3872983346207417*g1*Cos(
      ThetaW())*ZP(gI2,gO2),0) + IF(gI2 < 2,-0.5*g2*Sin(ThetaW())*ZP(gI2,gO2),0);

   return result;
}

std::complex<double> CLASSNAME::CpconjUHpmVZHpm(int gO2, int gI2) const
{
   const std::complex<double> result = IF(gI2 < 2,-0.5*g2*Cos(ThetaW())*ZP(gI2,
      gO2),0) + IF(gI2 < 2,0.3872983346207417*g1*Sin(ThetaW())*ZP(gI2,gO2),0);

   return result;
}

double CLASSNAME::CpconjUHpmbarFveFePR(int gO2) const
{
   const double result = Conj(Ye(0,0))*KroneckerDelta(0,gO2);

   return result;
}

double CLASSNAME::CpconjUHpmbarFveFePL(int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpconjUHpmbarFvmFmPR(int gO2) const
{
   const double result = Conj(Ye(1,1))*KroneckerDelta(0,gO2);

   return result;
}

double CLASSNAME::CpconjUHpmbarFvmFmPL(int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpconjUHpmbarFvtFtauPR(int gO2) const
{
   const double result = Conj(Ye(2,2))*KroneckerDelta(0,gO2);

   return result;
}

double CLASSNAME::CpconjUHpmbarFvtFtauPL(int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpconjUHpmbarFcFsPR(int gO2, int , int ) const
{
   const double result = Conj(Yd(1,1))*KroneckerDelta(0,gO2);

   return result;
}

double CLASSNAME::CpconjUHpmbarFcFsPL(int gO1, int , int ) const
{
   const double result = KroneckerDelta(1,gO1)*Yu(1,1);

   return result;
}

double CLASSNAME::CpconjUHpmbarFtFbPR(int gO2, int , int ) const
{
   const double result = Conj(Yd(2,2))*KroneckerDelta(0,gO2);

   return result;
}

double CLASSNAME::CpconjUHpmbarFtFbPL(int gO1, int , int ) const
{
   const double result = KroneckerDelta(1,gO1)*Yu(2,2);

   return result;
}

double CLASSNAME::CpconjUHpmbarFuFdPR(int gO2, int , int ) const
{
   const double result = Conj(Yd(0,0))*KroneckerDelta(0,gO2);

   return result;
}

double CLASSNAME::CpconjUHpmbarFuFdPL(int gO1, int , int ) const
{
   const double result = KroneckerDelta(1,gO1)*Yu(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLVZVZ() const
{
   const std::complex<double> result = 0.1*(g1*Sin(ThetaW())*(7.745966692414834
      *g2*Cos(ThetaW()) + 3*g1*Sin(ThetaW())) + 5*Sqr(g2)*Sqr(Cos(ThetaW())));

   return result;
}

double CLASSNAME::CpSveLconjSveLconjSveLSveL() const
{
   const double result = 0.1*(-3*Sqr(g1) - 5*Sqr(g2));

   return result;
}

double CLASSNAME::CpSveLconjSveLconjSvmLSvmL() const
{
   const double result = 0.05*(-3*Sqr(g1) - 5*Sqr(g2));

   return result;
}

double CLASSNAME::CpSveLconjSveLconjSvtLSvtL() const
{
   const double result = 0.05*(-3*Sqr(g1) - 5*Sqr(g2));

   return result;
}

double CLASSNAME::CpSveLconjSveLconjVWmVWm() const
{
   const double result = 0.5*Sqr(g2);

   return result;
}

double CLASSNAME::CpconjSveLVZSveL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLAhAh(int gI1, int gI2) const
{
   const std::complex<double> result = -0.05*(3*Sqr(g1) + 5*Sqr(g2))*(ZA(gI1,0)
      *ZA(gI2,0) - ZA(gI1,1)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLconjHpmHpm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*((-20*AbsSqr(Ye(0,0)) - 3*Sqr(g1) +
      5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) + (3*Sqr(g1) - 5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1
      ));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLconjSbSb(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZB(gI2,0))*(Sqr(g1) + 5*Sqr(
      g2))*ZB(gI1,0) + 2*Conj(ZB(gI2,1))*Sqr(g1)*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLconjScSc(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZC(gI2,0))*(Sqr(g1) - 5*Sqr(
      g2))*ZC(gI1,0) - 4*Conj(ZC(gI2,1))*Sqr(g1)*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLconjSdSd(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZD(gI2,0))*(Sqr(g1) + 5*Sqr(
      g2))*ZD(gI1,0) + 2*Conj(ZD(gI2,1))*Sqr(g1)*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLconjSeSe(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZE(gI2,0))*(3*Sqr(g1) + 5*
      Sqr(g2))*ZE(gI1,0)) + 2*Conj(ZE(gI2,1))*(-10*AbsSqr(Ye(0,0)) + 3*Sqr(g1))*ZE
      (gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLconjSmSm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZM(gI2,0))*(-3*Sqr(g1) + 5*
      Sqr(g2))*ZM(gI1,0) + 6*Conj(ZM(gI2,1))*Sqr(g1)*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLconjSsSs(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZS(gI2,0))*(Sqr(g1) + 5*Sqr(
      g2))*ZS(gI1,0) + 2*Conj(ZS(gI2,1))*Sqr(g1)*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLconjStSt(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZT(gI2,0))*(Sqr(g1) - 5*Sqr(
      g2))*ZT(gI1,0) - 4*Conj(ZT(gI2,1))*Sqr(g1)*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLconjStauStau(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZTau(gI2,0))*(-3*Sqr(g1) + 5*
      Sqr(g2))*ZTau(gI1,0) + 6*Conj(ZTau(gI2,1))*Sqr(g1)*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLconjSuSu(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZU(gI2,0))*(Sqr(g1) - 5*Sqr(
      g2))*ZU(gI1,0) - 4*Conj(ZU(gI2,1))*Sqr(g1)*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSveLconjSveLhhhh(int gI1, int gI2) const
{
   const std::complex<double> result = -0.05*(3*Sqr(g1) + 5*Sqr(g2))*(ZH(gI1,0)
      *ZH(gI2,0) - ZH(gI1,1)*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjSveLconjHpmSe(int gI1, int gI2) const
{
   const std::complex<double> result = 0.25*(4*Conj(ZE(gI2,1))*(Conj(TYe(0,0))*
      ZP(gI1,0) + Conj(Ye(0,0))*Mu*ZP(gI1,1)) - 1.4142135623730951*Conj(ZE(gI2,0))
      *(vd*(-2*AbsSqr(Ye(0,0)) + Sqr(g2))*ZP(gI1,0) + vu*Sqr(g2)*ZP(gI1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjSveLbarChaFePR(int gI1) const
{
   const std::complex<double> result = Conj(Ye(0,0))*UM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjSveLbarChaFePL(int gI1) const
{
   const std::complex<double> result = -(g2*Conj(UP(gI1,0)));

   return result;
}

std::complex<double> CLASSNAME::CpconjSveLSveLhh(int gI2) const
{
   const std::complex<double> result = -0.05*(3*Sqr(g1) + 5*Sqr(g2))*(vd*ZH(gI2
      ,0) - vu*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjSveLconjVWmSe(int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZE(gI2,0));

   return result;
}

double CLASSNAME::CpconjSveLFveChiPR(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpconjSveLFveChiPL(int gI2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*Conj(ZN(gI2,0)) -
      0.7071067811865475*g2*Conj(ZN(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLVZVZ() const
{
   const std::complex<double> result = 0.1*(g1*Sin(ThetaW())*(7.745966692414834
      *g2*Cos(ThetaW()) + 3*g1*Sin(ThetaW())) + 5*Sqr(g2)*Sqr(Cos(ThetaW())));

   return result;
}

double CLASSNAME::CpSvmLconjSvmLconjSveLSveL() const
{
   const double result = 0.05*(-3*Sqr(g1) - 5*Sqr(g2));

   return result;
}

double CLASSNAME::CpSvmLconjSvmLconjSvmLSvmL() const
{
   const double result = 0.1*(-3*Sqr(g1) - 5*Sqr(g2));

   return result;
}

double CLASSNAME::CpSvmLconjSvmLconjSvtLSvtL() const
{
   const double result = 0.05*(-3*Sqr(g1) - 5*Sqr(g2));

   return result;
}

double CLASSNAME::CpSvmLconjSvmLconjVWmVWm() const
{
   const double result = 0.5*Sqr(g2);

   return result;
}

double CLASSNAME::CpconjSvmLVZSvmL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLAhAh(int gI1, int gI2) const
{
   const std::complex<double> result = -0.05*(3*Sqr(g1) + 5*Sqr(g2))*(ZA(gI1,0)
      *ZA(gI2,0) - ZA(gI1,1)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLconjHpmHpm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*((-20*AbsSqr(Ye(1,1)) - 3*Sqr(g1) +
      5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) + (3*Sqr(g1) - 5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1
      ));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLconjSbSb(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZB(gI2,0))*(Sqr(g1) + 5*Sqr(
      g2))*ZB(gI1,0) + 2*Conj(ZB(gI2,1))*Sqr(g1)*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLconjScSc(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZC(gI2,0))*(Sqr(g1) - 5*Sqr(
      g2))*ZC(gI1,0) - 4*Conj(ZC(gI2,1))*Sqr(g1)*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLconjSdSd(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZD(gI2,0))*(Sqr(g1) + 5*Sqr(
      g2))*ZD(gI1,0) + 2*Conj(ZD(gI2,1))*Sqr(g1)*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLconjSeSe(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZE(gI2,0))*(-3*Sqr(g1) + 5*
      Sqr(g2))*ZE(gI1,0) + 6*Conj(ZE(gI2,1))*Sqr(g1)*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLconjSmSm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZM(gI2,0))*(3*Sqr(g1) + 5*
      Sqr(g2))*ZM(gI1,0)) + 2*Conj(ZM(gI2,1))*(-10*AbsSqr(Ye(1,1)) + 3*Sqr(g1))*ZM
      (gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLconjSsSs(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZS(gI2,0))*(Sqr(g1) + 5*Sqr(
      g2))*ZS(gI1,0) + 2*Conj(ZS(gI2,1))*Sqr(g1)*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLconjStSt(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZT(gI2,0))*(Sqr(g1) - 5*Sqr(
      g2))*ZT(gI1,0) - 4*Conj(ZT(gI2,1))*Sqr(g1)*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLconjStauStau(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZTau(gI2,0))*(-3*Sqr(g1) + 5*
      Sqr(g2))*ZTau(gI1,0) + 6*Conj(ZTau(gI2,1))*Sqr(g1)*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLconjSuSu(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZU(gI2,0))*(Sqr(g1) - 5*Sqr(
      g2))*ZU(gI1,0) - 4*Conj(ZU(gI2,1))*Sqr(g1)*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvmLconjSvmLhhhh(int gI1, int gI2) const
{
   const std::complex<double> result = -0.05*(3*Sqr(g1) + 5*Sqr(g2))*(ZH(gI1,0)
      *ZH(gI2,0) - ZH(gI1,1)*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjSvmLconjHpmSm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.25*(4*Conj(ZM(gI2,1))*(Conj(TYe(1,1))*
      ZP(gI1,0) + Conj(Ye(1,1))*Mu*ZP(gI1,1)) - 1.4142135623730951*Conj(ZM(gI2,0))
      *(vd*(-2*AbsSqr(Ye(1,1)) + Sqr(g2))*ZP(gI1,0) + vu*Sqr(g2)*ZP(gI1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjSvmLbarChaFmPR(int gI1) const
{
   const std::complex<double> result = Conj(Ye(1,1))*UM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjSvmLbarChaFmPL(int gI1) const
{
   const std::complex<double> result = -(g2*Conj(UP(gI1,0)));

   return result;
}

std::complex<double> CLASSNAME::CpconjSvmLSvmLhh(int gI2) const
{
   const std::complex<double> result = -0.05*(3*Sqr(g1) + 5*Sqr(g2))*(vd*ZH(gI2
      ,0) - vu*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjSvmLconjVWmSm(int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZM(gI2,0));

   return result;
}

double CLASSNAME::CpconjSvmLFvmChiPR(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpconjSvmLFvmChiPL(int gI2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*Conj(ZN(gI2,0)) -
      0.7071067811865475*g2*Conj(ZN(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLVZVZ() const
{
   const std::complex<double> result = 0.1*(g1*Sin(ThetaW())*(7.745966692414834
      *g2*Cos(ThetaW()) + 3*g1*Sin(ThetaW())) + 5*Sqr(g2)*Sqr(Cos(ThetaW())));

   return result;
}

double CLASSNAME::CpSvtLconjSvtLconjSveLSveL() const
{
   const double result = 0.05*(-3*Sqr(g1) - 5*Sqr(g2));

   return result;
}

double CLASSNAME::CpSvtLconjSvtLconjSvmLSvmL() const
{
   const double result = 0.05*(-3*Sqr(g1) - 5*Sqr(g2));

   return result;
}

double CLASSNAME::CpSvtLconjSvtLconjSvtLSvtL() const
{
   const double result = 0.1*(-3*Sqr(g1) - 5*Sqr(g2));

   return result;
}

double CLASSNAME::CpSvtLconjSvtLconjVWmVWm() const
{
   const double result = 0.5*Sqr(g2);

   return result;
}

double CLASSNAME::CpconjSvtLVZSvtL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLAhAh(int gI1, int gI2) const
{
   const std::complex<double> result = -0.05*(3*Sqr(g1) + 5*Sqr(g2))*(ZA(gI1,0)
      *ZA(gI2,0) - ZA(gI1,1)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLconjHpmHpm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*((-20*AbsSqr(Ye(2,2)) - 3*Sqr(g1) +
      5*Sqr(g2))*ZP(gI1,0)*ZP(gI2,0) + (3*Sqr(g1) - 5*Sqr(g2))*ZP(gI1,1)*ZP(gI2,1
      ));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLconjSbSb(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZB(gI2,0))*(Sqr(g1) + 5*Sqr(
      g2))*ZB(gI1,0) + 2*Conj(ZB(gI2,1))*Sqr(g1)*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLconjScSc(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZC(gI2,0))*(Sqr(g1) - 5*Sqr(
      g2))*ZC(gI1,0) - 4*Conj(ZC(gI2,1))*Sqr(g1)*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLconjSdSd(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZD(gI2,0))*(Sqr(g1) + 5*Sqr(
      g2))*ZD(gI1,0) + 2*Conj(ZD(gI2,1))*Sqr(g1)*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLconjSeSe(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZE(gI2,0))*(-3*Sqr(g1) + 5*
      Sqr(g2))*ZE(gI1,0) + 6*Conj(ZE(gI2,1))*Sqr(g1)*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLconjSmSm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZM(gI2,0))*(-3*Sqr(g1) + 5*
      Sqr(g2))*ZM(gI1,0) + 6*Conj(ZM(gI2,1))*Sqr(g1)*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLconjSsSs(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZS(gI2,0))*(Sqr(g1) + 5*Sqr(
      g2))*ZS(gI1,0) + 2*Conj(ZS(gI2,1))*Sqr(g1)*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLconjStSt(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZT(gI2,0))*(Sqr(g1) - 5*Sqr(
      g2))*ZT(gI1,0) - 4*Conj(ZT(gI2,1))*Sqr(g1)*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLconjStauStau(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-(Conj(ZTau(gI2,0))*(3*Sqr(g1) + 5
      *Sqr(g2))*ZTau(gI1,0)) + 2*Conj(ZTau(gI2,1))*(-10*AbsSqr(Ye(2,2)) + 3*Sqr(g1
      ))*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLconjSuSu(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(Conj(ZU(gI2,0))*(Sqr(g1) - 5*Sqr(
      g2))*ZU(gI1,0) - 4*Conj(ZU(gI2,1))*Sqr(g1)*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpSvtLconjSvtLhhhh(int gI1, int gI2) const
{
   const std::complex<double> result = -0.05*(3*Sqr(g1) + 5*Sqr(g2))*(ZH(gI1,0)
      *ZH(gI2,0) - ZH(gI1,1)*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjSvtLconjHpmStau(int gI1, int gI2) const
{
   const std::complex<double> result = 0.25*(4*Conj(ZTau(gI2,1))*(Conj(TYe(2,2)
      )*ZP(gI1,0) + Conj(Ye(2,2))*Mu*ZP(gI1,1)) - 1.4142135623730951*Conj(ZTau(gI2
      ,0))*(vd*(-2*AbsSqr(Ye(2,2)) + Sqr(g2))*ZP(gI1,0) + vu*Sqr(g2)*ZP(gI1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpconjSvtLbarChaFtauPR(int gI1) const
{
   const std::complex<double> result = Conj(Ye(2,2))*UM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjSvtLbarChaFtauPL(int gI1) const
{
   const std::complex<double> result = -(g2*Conj(UP(gI1,0)));

   return result;
}

std::complex<double> CLASSNAME::CpconjSvtLSvtLhh(int gI2) const
{
   const std::complex<double> result = -0.05*(3*Sqr(g1) + 5*Sqr(g2))*(vd*ZH(gI2
      ,0) - vu*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjSvtLconjVWmStau(int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZTau(gI2,0));

   return result;
}

double CLASSNAME::CpconjSvtLFvtChiPR(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpconjSvtLFvtChiPL(int gI2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*Conj(ZN(gI2,0)) -
      0.7071067811865475*g2*Conj(ZN(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpVGVGVG() const
{
   const std::complex<double> result = std::complex<double>(0,-1)*g3;

   return result;
}

std::complex<double> CLASSNAME::CpVGbargGgG() const
{
   const std::complex<double> result = std::complex<double>(0,-1)*g3;

   return result;
}

std::complex<double> CLASSNAME::CpVGVGconjSbSb(int gI1, int gI2) const
{
   const std::complex<double> result = 6*Sqr(g3)*(Conj(ZB(gI2,0))*ZB(gI1,0) +
      Conj(ZB(gI2,1))*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVGVGconjScSc(int gI1, int gI2) const
{
   const std::complex<double> result = 6*Sqr(g3)*(Conj(ZC(gI2,0))*ZC(gI1,0) +
      Conj(ZC(gI2,1))*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVGVGconjSdSd(int gI1, int gI2) const
{
   const std::complex<double> result = 6*Sqr(g3)*(Conj(ZD(gI2,0))*ZD(gI1,0) +
      Conj(ZD(gI2,1))*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVGVGconjSsSs(int gI1, int gI2) const
{
   const std::complex<double> result = 6*Sqr(g3)*(Conj(ZS(gI2,0))*ZS(gI1,0) +
      Conj(ZS(gI2,1))*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVGVGconjStSt(int gI1, int gI2) const
{
   const std::complex<double> result = 6*Sqr(g3)*(Conj(ZT(gI2,0))*ZT(gI1,0) +
      Conj(ZT(gI2,1))*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVGVGconjSuSu(int gI1, int gI2) const
{
   const std::complex<double> result = 6*Sqr(g3)*(Conj(ZU(gI2,0))*ZU(gI1,0) +
      Conj(ZU(gI2,1))*ZU(gI1,1));

   return result;
}

double CLASSNAME::CpVGconjSbSb(int gI1, int gI2) const
{
   const double result = g3*KroneckerDelta(gI1,gI2);

   return result;
}

double CLASSNAME::CpVGconjScSc(int gI1, int gI2) const
{
   const double result = g3*KroneckerDelta(gI1,gI2);

   return result;
}

double CLASSNAME::CpVGconjSdSd(int gI1, int gI2) const
{
   const double result = g3*KroneckerDelta(gI1,gI2);

   return result;
}

double CLASSNAME::CpVGconjSsSs(int gI1, int gI2) const
{
   const double result = g3*KroneckerDelta(gI1,gI2);

   return result;
}

double CLASSNAME::CpVGconjStSt(int gI1, int gI2) const
{
   const double result = g3*KroneckerDelta(gI1,gI2);

   return result;
}

double CLASSNAME::CpVGconjSuSu(int gI1, int gI2) const
{
   const double result = g3*KroneckerDelta(gI1,gI2);

   return result;
}

double CLASSNAME::CpVGbarFbFbPL(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFbFbPR(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFcFcPL(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFcFcPR(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFdFdPL(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFdFdPR(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFsFsPL(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFsFsPR(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFtFtPL(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFtFtPR(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFuFuPL(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGbarFuFuPR(int , int ) const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpVGGluGluPL(int , int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpVGGluGluPR(int , int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpVGVGVGVG1() const
{
   const double result = -16*Sqr(g3);

   return result;
}

double CLASSNAME::CpVGVGVGVG2() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpVGVGVGVG3() const
{
   const double result = 16*Sqr(g3);

   return result;
}

double CLASSNAME::CpVPbargWmgWm() const
{
   const double result = -(g2*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVPbargWmCgWmC() const
{
   const double result = g2*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpVPconjVWmVWm() const
{
   const double result = -(g2*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVPbarFeFePL() const
{
   const double result = 0.1*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVPbarFeFePR() const
{
   const double result = 0.7745966692414834*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFmFmPL() const
{
   const double result = 0.1*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVPbarFmFmPR() const
{
   const double result = 0.7745966692414834*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFtauFtauPL() const
{
   const double result = 0.1*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVPbarFtauFtauPR() const
{
   const double result = 0.7745966692414834*g1*Cos(ThetaW());

   return result;
}

std::complex<double> CLASSNAME::CpVPVPconjHpmHpm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(7.745966692414834*g1*g2*Sin(2*
      ThetaW()) + 3*Sqr(g1) + Cos(2*ThetaW())*(3*Sqr(g1) - 5*Sqr(g2)) + 5*Sqr(g2))
      *(ZP(gI1,0)*ZP(gI2,0) + ZP(gI1,1)*ZP(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPVPconjSbSb(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZB(gI2,0))*(
      -7.745966692414834*g1*g2*Cos(ThetaW())*Sin(ThetaW()) + Sqr(g1)*Sqr(Cos(
      ThetaW())) + 15*Sqr(g2)*Sqr(Sin(ThetaW())))*ZB(gI1,0) + 4*Conj(ZB(gI2,1))*
      Sqr(g1)*Sqr(Cos(ThetaW()))*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPVPconjScSc(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZC(gI2,0))*(g2
      *Sin(ThetaW())*(7.745966692414834*g1*Cos(ThetaW()) + 15*g2*Sin(ThetaW())) +
      Sqr(g1)*Sqr(Cos(ThetaW())))*ZC(gI1,0) + 16*Conj(ZC(gI2,1))*Sqr(g1)*Sqr(Cos(
      ThetaW()))*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPVPconjSdSd(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZD(gI2,0))*(
      -7.745966692414834*g1*g2*Cos(ThetaW())*Sin(ThetaW()) + Sqr(g1)*Sqr(Cos(
      ThetaW())) + 15*Sqr(g2)*Sqr(Sin(ThetaW())))*ZD(gI1,0) + 4*Conj(ZD(gI2,1))*
      Sqr(g1)*Sqr(Cos(ThetaW()))*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPVPconjSeSe(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZE(gI2,0))*(g2*Sin(ThetaW())*(
      7.745966692414834*g1*Cos(ThetaW()) + 5*g2*Sin(ThetaW())) + 3*Sqr(g1)*Sqr(Cos
      (ThetaW())))*ZE(gI1,0) + 12*Conj(ZE(gI2,1))*Sqr(g1)*Sqr(Cos(ThetaW()))*ZE(
      gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPVPconjSmSm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZM(gI2,0))*(g2*Sin(ThetaW())*(
      7.745966692414834*g1*Cos(ThetaW()) + 5*g2*Sin(ThetaW())) + 3*Sqr(g1)*Sqr(Cos
      (ThetaW())))*ZM(gI1,0) + 12*Conj(ZM(gI2,1))*Sqr(g1)*Sqr(Cos(ThetaW()))*ZM(
      gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPVPconjSsSs(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZS(gI2,0))*(
      -7.745966692414834*g1*g2*Cos(ThetaW())*Sin(ThetaW()) + Sqr(g1)*Sqr(Cos(
      ThetaW())) + 15*Sqr(g2)*Sqr(Sin(ThetaW())))*ZS(gI1,0) + 4*Conj(ZS(gI2,1))*
      Sqr(g1)*Sqr(Cos(ThetaW()))*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPVPconjStSt(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZT(gI2,0))*(g2
      *Sin(ThetaW())*(7.745966692414834*g1*Cos(ThetaW()) + 15*g2*Sin(ThetaW())) +
      Sqr(g1)*Sqr(Cos(ThetaW())))*ZT(gI1,0) + 16*Conj(ZT(gI2,1))*Sqr(g1)*Sqr(Cos(
      ThetaW()))*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPVPconjStauStau(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZTau(gI2,0))*(g2*Sin(ThetaW())
      *(7.745966692414834*g1*Cos(ThetaW()) + 5*g2*Sin(ThetaW())) + 3*Sqr(g1)*Sqr(
      Cos(ThetaW())))*ZTau(gI1,0) + 12*Conj(ZTau(gI2,1))*Sqr(g1)*Sqr(Cos(ThetaW())
      )*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPVPconjSuSu(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZU(gI2,0))*(g2
      *Sin(ThetaW())*(7.745966692414834*g1*Cos(ThetaW()) + 15*g2*Sin(ThetaW())) +
      Sqr(g1)*Sqr(Cos(ThetaW())))*ZU(gI1,0) + 16*Conj(ZU(gI2,1))*Sqr(g1)*Sqr(Cos(
      ThetaW()))*ZU(gI1,1));

   return result;
}

double CLASSNAME::CpVPconjHpmHpm(int gI1, int gI2) const
{
   const double result = -0.1*KroneckerDelta(gI1,gI2)*(3.872983346207417*g1*Cos
      (ThetaW()) + 5*g2*Sin(ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpVPconjSbSb(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZB(gI2,0))*(
      3.872983346207417*g1*Cos(ThetaW()) - 15*g2*Sin(ThetaW()))*ZB(gI1,0) -
      7.745966692414834*g1*Conj(ZB(gI2,1))*Cos(ThetaW())*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPconjScSc(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZC(gI2,0))*(
      3.872983346207417*g1*Cos(ThetaW()) + 15*g2*Sin(ThetaW()))*ZC(gI1,0) +
      15.491933384829668*g1*Conj(ZC(gI2,1))*Cos(ThetaW())*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPconjSdSd(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZD(gI2,0))*(
      3.872983346207417*g1*Cos(ThetaW()) - 15*g2*Sin(ThetaW()))*ZD(gI1,0) -
      7.745966692414834*g1*Conj(ZD(gI2,1))*Cos(ThetaW())*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPconjSeSe(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(-(Conj(ZE(gI2,0))*(
      3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(ThetaW()))*ZE(gI1,0)) -
      7.745966692414834*g1*Conj(ZE(gI2,1))*Cos(ThetaW())*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPconjSmSm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(-(Conj(ZM(gI2,0))*(
      3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(ThetaW()))*ZM(gI1,0)) -
      7.745966692414834*g1*Conj(ZM(gI2,1))*Cos(ThetaW())*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPconjSsSs(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZS(gI2,0))*(
      3.872983346207417*g1*Cos(ThetaW()) - 15*g2*Sin(ThetaW()))*ZS(gI1,0) -
      7.745966692414834*g1*Conj(ZS(gI2,1))*Cos(ThetaW())*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPconjStSt(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZT(gI2,0))*(
      3.872983346207417*g1*Cos(ThetaW()) + 15*g2*Sin(ThetaW()))*ZT(gI1,0) +
      15.491933384829668*g1*Conj(ZT(gI2,1))*Cos(ThetaW())*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPconjStauStau(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(-(Conj(ZTau(gI2,0))*(
      3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(ThetaW()))*ZTau(gI1,0)) -
      7.745966692414834*g1*Conj(ZTau(gI2,1))*Cos(ThetaW())*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPconjSuSu(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZU(gI2,0))*(
      3.872983346207417*g1*Cos(ThetaW()) + 15*g2*Sin(ThetaW()))*ZU(gI1,0) +
      15.491933384829668*g1*Conj(ZU(gI2,1))*Cos(ThetaW())*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVPbarChaChaPL(int gI1, int gI2) const
{
   const std::complex<double> result = g2*Conj(UM(gI2,0))*Sin(ThetaW())*UM(gI1,
      0) + 0.1*Conj(UM(gI2,1))*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(
      ThetaW()))*UM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpVPbarChaChaPR(int gI1, int gI2) const
{
   const std::complex<double> result = g2*Conj(UP(gI1,0))*Sin(ThetaW())*UP(gI2,
      0) + 0.1*Conj(UP(gI1,1))*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(
      ThetaW()))*UP(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpVPconjVWmHpm(int gI2) const
{
   const std::complex<double> result = -0.3872983346207417*g1*g2*Cos(ThetaW())*
      (vd*ZP(gI2,0) - vu*ZP(gI2,1));

   return result;
}

double CLASSNAME::CpVPbarFbFbPL(int , int ) const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) + 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFbFbPR(int , int ) const
{
   const double result = 0.2581988897471611*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFcFcPL(int , int ) const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) - 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFcFcPR(int , int ) const
{
   const double result = -0.5163977794943222*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFdFdPL(int , int ) const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) + 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFdFdPR(int , int ) const
{
   const double result = 0.2581988897471611*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFsFsPL(int , int ) const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) + 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFsFsPR(int , int ) const
{
   const double result = 0.2581988897471611*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFtFtPL(int , int ) const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) - 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFtFtPR(int , int ) const
{
   const double result = -0.5163977794943222*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFuFuPL(int , int ) const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) - 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpVPbarFuFuPR(int , int ) const
{
   const double result = -0.5163977794943222*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpVPVPconjVWmVWm1() const
{
   const double result = -2*Sqr(g2)*Sqr(Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVPVPconjVWmVWm2() const
{
   const double result = Sqr(g2)*Sqr(Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVPVPconjVWmVWm3() const
{
   const double result = Sqr(g2)*Sqr(Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVZbargWmgWm() const
{
   const double result = -(g2*Cos(ThetaW()));

   return result;
}

double CLASSNAME::CpVZbargWmCgWmC() const
{
   const double result = g2*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpVZconjSveLSveL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVZconjSvmLSvmL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVZconjSvtLSvtL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjSveLSveL() const
{
   const std::complex<double> result = 0.1*(g1*Sin(ThetaW())*(7.745966692414834
      *g2*Cos(ThetaW()) + 3*g1*Sin(ThetaW())) + 5*Sqr(g2)*Sqr(Cos(ThetaW())));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjSvmLSvmL() const
{
   const std::complex<double> result = 0.1*(g1*Sin(ThetaW())*(7.745966692414834
      *g2*Cos(ThetaW()) + 3*g1*Sin(ThetaW())) + 5*Sqr(g2)*Sqr(Cos(ThetaW())));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjSvtLSvtL() const
{
   const std::complex<double> result = 0.1*(g1*Sin(ThetaW())*(7.745966692414834
      *g2*Cos(ThetaW()) + 3*g1*Sin(ThetaW())) + 5*Sqr(g2)*Sqr(Cos(ThetaW())));

   return result;
}

double CLASSNAME::CpVZconjVWmVWm() const
{
   const double result = -(g2*Cos(ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFeFePL() const
{
   const double result = 0.1*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFeFePR() const
{
   const double result = -0.7745966692414834*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpVZbarFmFmPL() const
{
   const double result = 0.1*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFmFmPR() const
{
   const double result = -0.7745966692414834*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpVZbarFtauFtauPL() const
{
   const double result = 0.1*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFtauFtauPR() const
{
   const double result = -0.7745966692414834*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpVZbarFveFvePL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFveFvePR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpVZbarFvmFvmPL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFvmFvmPR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpVZbarFvtFvtPL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFvtFvtPR() const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpVZVZAhAh(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(7.745966692414834*g1*g2*Sin(2*
      ThetaW()) + 3*Sqr(g1) + 5*Sqr(g2) + Cos(2*ThetaW())*(-3*Sqr(g1) + 5*Sqr(g2))
      )*(ZA(gI1,0)*ZA(gI2,0) + ZA(gI1,1)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjHpmHpm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(-7.745966692414834*g1*g2*Sin(2*
      ThetaW()) + 3*Sqr(g1) + 5*Sqr(g2) + Cos(2*ThetaW())*(-3*Sqr(g1) + 5*Sqr(g2))
      )*(ZP(gI1,0)*ZP(gI2,0) + ZP(gI1,1)*ZP(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjSbSb(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZB(gI2,0))*(g1
      *Sin(ThetaW())*(7.745966692414834*g2*Cos(ThetaW()) + g1*Sin(ThetaW())) + 15*
      Sqr(g2)*Sqr(Cos(ThetaW())))*ZB(gI1,0) + 4*Conj(ZB(gI2,1))*Sqr(g1)*Sqr(Sin(
      ThetaW()))*ZB(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjScSc(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZC(gI2,0))*(
      -7.745966692414834*g1*g2*Cos(ThetaW())*Sin(ThetaW()) + 15*Sqr(g2)*Sqr(Cos(
      ThetaW())) + Sqr(g1)*Sqr(Sin(ThetaW())))*ZC(gI1,0) + 16*Conj(ZC(gI2,1))*Sqr(
      g1)*Sqr(Sin(ThetaW()))*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjSdSd(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZD(gI2,0))*(g1
      *Sin(ThetaW())*(7.745966692414834*g2*Cos(ThetaW()) + g1*Sin(ThetaW())) + 15*
      Sqr(g2)*Sqr(Cos(ThetaW())))*ZD(gI1,0) + 4*Conj(ZD(gI2,1))*Sqr(g1)*Sqr(Sin(
      ThetaW()))*ZD(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjSeSe(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZE(gI2,0))*(-7.745966692414834
      *g1*g2*Cos(ThetaW())*Sin(ThetaW()) + 5*Sqr(g2)*Sqr(Cos(ThetaW())) + 3*Sqr(g1
      )*Sqr(Sin(ThetaW())))*ZE(gI1,0) + 12*Conj(ZE(gI2,1))*Sqr(g1)*Sqr(Sin(ThetaW(
      )))*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjSmSm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZM(gI2,0))*(-7.745966692414834
      *g1*g2*Cos(ThetaW())*Sin(ThetaW()) + 5*Sqr(g2)*Sqr(Cos(ThetaW())) + 3*Sqr(g1
      )*Sqr(Sin(ThetaW())))*ZM(gI1,0) + 12*Conj(ZM(gI2,1))*Sqr(g1)*Sqr(Sin(ThetaW(
      )))*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjSsSs(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZS(gI2,0))*(g1
      *Sin(ThetaW())*(7.745966692414834*g2*Cos(ThetaW()) + g1*Sin(ThetaW())) + 15*
      Sqr(g2)*Sqr(Cos(ThetaW())))*ZS(gI1,0) + 4*Conj(ZS(gI2,1))*Sqr(g1)*Sqr(Sin(
      ThetaW()))*ZS(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjStSt(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZT(gI2,0))*(
      -7.745966692414834*g1*g2*Cos(ThetaW())*Sin(ThetaW()) + 15*Sqr(g2)*Sqr(Cos(
      ThetaW())) + Sqr(g1)*Sqr(Sin(ThetaW())))*ZT(gI1,0) + 16*Conj(ZT(gI2,1))*Sqr(
      g1)*Sqr(Sin(ThetaW()))*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjStauStau(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZTau(gI2,0))*(
      -7.745966692414834*g1*g2*Cos(ThetaW())*Sin(ThetaW()) + 5*Sqr(g2)*Sqr(Cos(
      ThetaW())) + 3*Sqr(g1)*Sqr(Sin(ThetaW())))*ZTau(gI1,0) + 12*Conj(ZTau(gI2,1)
      )*Sqr(g1)*Sqr(Sin(ThetaW()))*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZconjSuSu(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZU(gI2,0))*(
      -7.745966692414834*g1*g2*Cos(ThetaW())*Sin(ThetaW()) + 15*Sqr(g2)*Sqr(Cos(
      ThetaW())) + Sqr(g1)*Sqr(Sin(ThetaW())))*ZU(gI1,0) + 16*Conj(ZU(gI2,1))*Sqr(
      g1)*Sqr(Sin(ThetaW()))*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZhhhh(int gI1, int gI2) const
{
   const std::complex<double> result = 0.05*(7.745966692414834*g1*g2*Sin(2*
      ThetaW()) + 3*Sqr(g1) + 5*Sqr(g2) + Cos(2*ThetaW())*(-3*Sqr(g1) + 5*Sqr(g2))
      )*(ZH(gI1,0)*ZH(gI2,0) + ZH(gI1,1)*ZH(gI2,1));

   return result;
}

double CLASSNAME::CpVZconjHpmHpm(int gI1, int gI2) const
{
   const double result = -0.1*KroneckerDelta(gI1,gI2)*(5*g2*Cos(ThetaW()) -
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpVZconjSbSb(int gI1, int gI2) const
{
   const std::complex<double> result = -0.03333333333333333*Conj(ZB(gI2,0))*(15
      *g2*Cos(ThetaW()) + 3.872983346207417*g1*Sin(ThetaW()))*ZB(gI1,0) +
      0.2581988897471611*g1*Conj(ZB(gI2,1))*Sin(ThetaW())*ZB(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpVZconjScSc(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZC(gI2,0))*(15
      *g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW()))*ZC(gI1,0) -
      15.491933384829668*g1*Conj(ZC(gI2,1))*Sin(ThetaW())*ZC(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZconjSdSd(int gI1, int gI2) const
{
   const std::complex<double> result = -0.03333333333333333*Conj(ZD(gI2,0))*(15
      *g2*Cos(ThetaW()) + 3.872983346207417*g1*Sin(ThetaW()))*ZD(gI1,0) +
      0.2581988897471611*g1*Conj(ZD(gI2,1))*Sin(ThetaW())*ZD(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpVZconjSeSe(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(-(Conj(ZE(gI2,0))*(5*g2*Cos(ThetaW(
      )) - 3.872983346207417*g1*Sin(ThetaW()))*ZE(gI1,0)) + 7.745966692414834*g1*
      Conj(ZE(gI2,1))*Sin(ThetaW())*ZE(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZconjSmSm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(-(Conj(ZM(gI2,0))*(5*g2*Cos(ThetaW(
      )) - 3.872983346207417*g1*Sin(ThetaW()))*ZM(gI1,0)) + 7.745966692414834*g1*
      Conj(ZM(gI2,1))*Sin(ThetaW())*ZM(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZconjSsSs(int gI1, int gI2) const
{
   const std::complex<double> result = -0.03333333333333333*Conj(ZS(gI2,0))*(15
      *g2*Cos(ThetaW()) + 3.872983346207417*g1*Sin(ThetaW()))*ZS(gI1,0) +
      0.2581988897471611*g1*Conj(ZS(gI2,1))*Sin(ThetaW())*ZS(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpVZconjStSt(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZT(gI2,0))*(15
      *g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW()))*ZT(gI1,0) -
      15.491933384829668*g1*Conj(ZT(gI2,1))*Sin(ThetaW())*ZT(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZconjStauStau(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(-(Conj(ZTau(gI2,0))*(5*g2*Cos(
      ThetaW()) - 3.872983346207417*g1*Sin(ThetaW()))*ZTau(gI1,0)) +
      7.745966692414834*g1*Conj(ZTau(gI2,1))*Sin(ThetaW())*ZTau(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZconjSuSu(int gI1, int gI2) const
{
   const std::complex<double> result = 0.03333333333333333*(Conj(ZU(gI2,0))*(15
      *g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(ThetaW()))*ZU(gI1,0) -
      15.491933384829668*g1*Conj(ZU(gI2,1))*Sin(ThetaW())*ZU(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZhhAh(int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,-0.1)*(5*g2*Cos(
      ThetaW()) + 3.872983346207417*g1*Sin(ThetaW()))*(ZA(gI2,0)*ZH(gI1,0) - ZA(
      gI2,1)*ZH(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZbarChaChaPL(int gI1, int gI2) const
{
   const std::complex<double> result = g2*Conj(UM(gI2,0))*Cos(ThetaW())*UM(gI1,
      0) + 0.1*Conj(UM(gI2,1))*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()))*UM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpVZbarChaChaPR(int gI1, int gI2) const
{
   const std::complex<double> result = g2*Conj(UP(gI1,0))*Cos(ThetaW())*UP(gI2,
      0) + 0.1*Conj(UP(gI1,1))*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()))*UP(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpVZChiChiPL(int gI1, int gI2) const
{
   const std::complex<double> result = -0.1*(5*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()))*(Conj(ZN(gI2,2))*ZN(gI1,2) - Conj(ZN(gI2
      ,3))*ZN(gI1,3));

   return result;
}

std::complex<double> CLASSNAME::CpVZChiChiPR(int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(5*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()))*(Conj(ZN(gI1,2))*ZN(gI2,2) - Conj(ZN(gI1
      ,3))*ZN(gI2,3));

   return result;
}

std::complex<double> CLASSNAME::CpVZVZhh(int gI2) const
{
   const std::complex<double> result = 0.05*(7.745966692414834*g1*g2*Sin(2*
      ThetaW()) + 3*Sqr(g1) + 5*Sqr(g2) + Cos(2*ThetaW())*(-3*Sqr(g1) + 5*Sqr(g2))
      )*(vd*ZH(gI2,0) + vu*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpVZconjVWmHpm(int gI2) const
{
   const std::complex<double> result = 0.3872983346207417*g1*g2*Sin(ThetaW())*(
      vd*ZP(gI2,0) - vu*ZP(gI2,1));

   return result;
}

double CLASSNAME::CpVZbarFbFbPL(int , int ) const
{
   const double result = 0.03333333333333333*(15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFbFbPR(int , int ) const
{
   const double result = -0.2581988897471611*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpVZbarFcFcPL(int , int ) const
{
   const double result = 0.03333333333333333*(-15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFcFcPR(int , int ) const
{
   const double result = 0.5163977794943222*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpVZbarFdFdPL(int , int ) const
{
   const double result = 0.03333333333333333*(15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFdFdPR(int , int ) const
{
   const double result = -0.2581988897471611*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpVZbarFsFsPL(int , int ) const
{
   const double result = 0.03333333333333333*(15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFsFsPR(int , int ) const
{
   const double result = -0.2581988897471611*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpVZbarFtFtPL(int , int ) const
{
   const double result = 0.03333333333333333*(-15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFtFtPR(int , int ) const
{
   const double result = 0.5163977794943222*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpVZbarFuFuPL(int , int ) const
{
   const double result = 0.03333333333333333*(-15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVZbarFuFuPR(int , int ) const
{
   const double result = 0.5163977794943222*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpVZVZconjVWmVWm1() const
{
   const double result = -2*Sqr(g2)*Sqr(Cos(ThetaW()));

   return result;
}

double CLASSNAME::CpVZVZconjVWmVWm2() const
{
   const double result = Sqr(g2)*Sqr(Cos(ThetaW()));

   return result;
}

double CLASSNAME::CpVZVZconjVWmVWm3() const
{
   const double result = Sqr(g2)*Sqr(Cos(ThetaW()));

   return result;
}

double CLASSNAME::CpconjVWmbargPgWm() const
{
   const double result = g2*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpconjVWmbargWmCgP() const
{
   const double result = -(g2*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpconjVWmbargWmCgZ() const
{
   const double result = -(g2*Cos(ThetaW()));

   return result;
}

double CLASSNAME::CpconjVWmbargZgWm() const
{
   const double result = g2*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpVWmconjVWmconjSveLSveL() const
{
   const double result = 0.5*Sqr(g2);

   return result;
}

double CLASSNAME::CpVWmconjVWmconjSvmLSvmL() const
{
   const double result = 0.5*Sqr(g2);

   return result;
}

double CLASSNAME::CpVWmconjVWmconjSvtLSvtL() const
{
   const double result = 0.5*Sqr(g2);

   return result;
}

double CLASSNAME::CpconjVWmVWmVP() const
{
   const double result = -(g2*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpconjVWmVZVWm() const
{
   const double result = g2*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpconjVWmbarFveFePL() const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpconjVWmbarFveFePR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpconjVWmbarFvmFmPL() const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpconjVWmbarFvmFmPR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpconjVWmbarFvtFtauPL() const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpconjVWmbarFvtFtauPR() const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmAhAh(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Sqr(g2)*(ZA(gI1,0)*ZA(gI2,0) + ZA(
      gI1,1)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmconjHpmHpm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Sqr(g2)*(ZP(gI1,0)*ZP(gI2,0) + ZP(
      gI1,1)*ZP(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmconjSbSb(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZB(gI2,0))*Sqr(g2)*ZB(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmconjScSc(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZC(gI2,0))*Sqr(g2)*ZC(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmconjSdSd(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZD(gI2,0))*Sqr(g2)*ZD(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmconjSeSe(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZE(gI2,0))*Sqr(g2)*ZE(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmconjSmSm(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZM(gI2,0))*Sqr(g2)*ZM(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmconjSsSs(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZS(gI2,0))*Sqr(g2)*ZS(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmconjStSt(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZT(gI2,0))*Sqr(g2)*ZT(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmconjStauStau(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZTau(gI2,0))*Sqr(g2)*ZTau(gI1,0
      );

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmconjSuSu(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Conj(ZU(gI2,0))*Sqr(g2)*ZU(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpVWmconjVWmhhhh(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*Sqr(g2)*(ZH(gI1,0)*ZH(gI2,0) + ZH(
      gI1,1)*ZH(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmconjScSs(int gI1, int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZS(gI2,0))*ZC
      (gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmconjStSb(int gI1, int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZB(gI2,0))*ZT
      (gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmconjSuSd(int gI1, int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZD(gI2,0))*ZU
      (gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmHpmAh(int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,-0.5)*g2*(ZA(gI2,
      0)*ZP(gI1,0) + ZA(gI2,1)*ZP(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmHpmhh(int gI1, int gI2) const
{
   const std::complex<double> result = 0.5*g2*(ZH(gI2,0)*ZP(gI1,0) - ZH(gI2,1)*
      ZP(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmChiChaPL(int gI1, int gI2) const
{
   const std::complex<double> result = -0.5*g2*(2*Conj(UM(gI2,0))*ZN(gI1,1) +
      1.4142135623730951*Conj(UM(gI2,1))*ZN(gI1,2));

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmChiChaPR(int gI1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZN(gI1,1))*UP(gI2,0)) +
      0.7071067811865475*g2*Conj(ZN(gI1,3))*UP(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmVPHpm(int gI2) const
{
   const std::complex<double> result = -0.3872983346207417*g1*g2*Cos(ThetaW())*
      (vd*ZP(gI2,0) - vu*ZP(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmVWmhh(int gI2) const
{
   const std::complex<double> result = 0.5*Sqr(g2)*(vd*ZH(gI2,0) + vu*ZH(gI2,1)
      );

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmVZHpm(int gI2) const
{
   const std::complex<double> result = 0.3872983346207417*g1*g2*Sin(ThetaW())*(
      vd*ZP(gI2,0) - vu*ZP(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmconjSveLSe(int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZE(gI2,0));

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmconjSvmLSm(int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZM(gI2,0));

   return result;
}

std::complex<double> CLASSNAME::CpconjVWmconjSvtLStau(int gI2) const
{
   const std::complex<double> result = 0.7071067811865475*g2*Conj(ZTau(gI2,0));

   return result;
}

double CLASSNAME::CpconjVWmbarFcFsPL(int , int ) const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpconjVWmbarFcFsPR(int , int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpconjVWmbarFtFbPL(int , int ) const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpconjVWmbarFtFbPR(int , int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpconjVWmbarFuFdPL(int , int ) const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpconjVWmbarFuFdPR(int , int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpVWmconjVWmVPVP1() const
{
   const double result = -2*Sqr(g2)*Sqr(Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVWmconjVWmVPVP2() const
{
   const double result = Sqr(g2)*Sqr(Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVWmconjVWmVPVP3() const
{
   const double result = Sqr(g2)*Sqr(Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpVWmconjVWmVZVZ1() const
{
   const double result = -2*Sqr(g2)*Sqr(Cos(ThetaW()));

   return result;
}

double CLASSNAME::CpVWmconjVWmVZVZ2() const
{
   const double result = Sqr(g2)*Sqr(Cos(ThetaW()));

   return result;
}

double CLASSNAME::CpVWmconjVWmVZVZ3() const
{
   const double result = Sqr(g2)*Sqr(Cos(ThetaW()));

   return result;
}

double CLASSNAME::CpVWmconjVWmconjVWmVWm1() const
{
   const double result = -Sqr(g2);

   return result;
}

double CLASSNAME::CpVWmconjVWmconjVWmVWm2() const
{
   const double result = -Sqr(g2);

   return result;
}

double CLASSNAME::CpVWmconjVWmconjVWmVWm3() const
{
   const double result = 2*Sqr(g2);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarChaHpmPL(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = -0.1*(Conj(UP(gI1,1))*(5.477225575051661
      *g1*KroneckerDelta(0,gO2) + 7.0710678118654755*g2*KroneckerDelta(1,gO2)) +
      10*g2*Conj(UP(gI1,0))*KroneckerDelta(3,gO2))*ZP(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarChaHpmPR(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(-10*g2*KroneckerDelta(2,gO1)*UM(gI1
      ,0) + 1.4142135623730951*(3.872983346207417*g1*KroneckerDelta(0,gO1) + 5*g2*
      KroneckerDelta(1,gO1))*UM(gI1,1))*ZP(gI2,0);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjHpmChaPL(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(UM(gI2,1))*(5.477225575051661*
      g1*KroneckerDelta(0,gO2) + 7.0710678118654755*g2*KroneckerDelta(1,gO2)) - 10
      *g2*Conj(UM(gI2,0))*KroneckerDelta(2,gO2))*ZP(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjHpmChaPR(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = -0.1*(10*g2*KroneckerDelta(3,gO1)*UP(gI2
      ,0) + 1.4142135623730951*(3.872983346207417*g1*KroneckerDelta(0,gO1) + 5*g2*
      KroneckerDelta(1,gO1))*UP(gI2,1))*ZP(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChihhChiPL(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(Conj(ZN(gI2,2))*(3.872983346207417*
      g1*KroneckerDelta(0,gO2) - 5*g2*KroneckerDelta(1,gO2))*ZH(gI1,0) - 5*g2*Conj
      (ZN(gI2,1))*KroneckerDelta(2,gO2)*ZH(gI1,0) - 3.872983346207417*g1*Conj(ZN(
      gI2,3))*KroneckerDelta(0,gO2)*ZH(gI1,1) + 5*g2*Conj(ZN(gI2,3))*
      KroneckerDelta(1,gO2)*ZH(gI1,1) + 5*g2*Conj(ZN(gI2,1))*KroneckerDelta(3,gO2)
      *ZH(gI1,1) + 3.872983346207417*g1*Conj(ZN(gI2,0))*(KroneckerDelta(2,gO2)*ZH(
      gI1,0) - KroneckerDelta(3,gO2)*ZH(gI1,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUChihhChiPR(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(KroneckerDelta(2,gO1)*ZH(gI1,0)*(
      3.872983346207417*g1*ZN(gI2,0) - 5*g2*ZN(gI2,1)) + KroneckerDelta(3,gO1)*ZH(
      gI1,1)*(-3.872983346207417*g1*ZN(gI2,0) + 5*g2*ZN(gI2,1)) + (
      3.872983346207417*g1*KroneckerDelta(0,gO1) - 5*g2*KroneckerDelta(1,gO1))*(ZH
      (gI1,0)*ZN(gI2,2) - ZH(gI1,1)*ZN(gI2,3)));

   return result;
}

std::complex<double> CLASSNAME::CpUChibarChaVWmPR(int gO2, int gI1) const
{
   const std::complex<double> result = g2*KroneckerDelta(1,gO2)*UM(gI1,0) +
      0.7071067811865475*g2*KroneckerDelta(2,gO2)*UM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarChaVWmPL(int gO1, int gI1) const
{
   const std::complex<double> result = g2*Conj(UP(gI1,0))*KroneckerDelta(1,gO1)
      - 0.7071067811865475*g2*Conj(UP(gI1,1))*KroneckerDelta(3,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSbFbPL(int gO2, int gI1, int ) const
{
   const std::complex<double> result = -0.18257418583505536*g1*KroneckerDelta(0
      ,gO2)*ZB(gI1,0) + 0.7071067811865475*g2*KroneckerDelta(1,gO2)*ZB(gI1,0) -
      KroneckerDelta(2,gO2)*Yd(2,2)*ZB(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSbFbPR(int gO1, int gI1, int ) const
{
   const std::complex<double> result = -(Conj(Yd(2,2))*KroneckerDelta(2,gO1)*ZB
      (gI1,0)) - 0.3651483716701107*g1*KroneckerDelta(0,gO1)*ZB(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjScFcPL(int gO2, int gI1, int ) const
{
   const std::complex<double> result = -0.18257418583505536*g1*KroneckerDelta(0
      ,gO2)*ZC(gI1,0) - 0.7071067811865475*g2*KroneckerDelta(1,gO2)*ZC(gI1,0) -
      KroneckerDelta(3,gO2)*Yu(1,1)*ZC(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjScFcPR(int gO1, int gI1, int ) const
{
   const std::complex<double> result = -(Conj(Yu(1,1))*KroneckerDelta(3,gO1)*ZC
      (gI1,0)) + 0.7302967433402214*g1*KroneckerDelta(0,gO1)*ZC(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSdFdPL(int gO2, int gI1, int ) const
{
   const std::complex<double> result = -0.18257418583505536*g1*KroneckerDelta(0
      ,gO2)*ZD(gI1,0) + 0.7071067811865475*g2*KroneckerDelta(1,gO2)*ZD(gI1,0) -
      KroneckerDelta(2,gO2)*Yd(0,0)*ZD(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSdFdPR(int gO1, int gI1, int ) const
{
   const std::complex<double> result = -(Conj(Yd(0,0))*KroneckerDelta(2,gO1)*ZD
      (gI1,0)) - 0.3651483716701107*g1*KroneckerDelta(0,gO1)*ZD(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSeFePL(int gO2, int gI1) const
{
   const std::complex<double> result = 0.5477225575051661*g1*KroneckerDelta(0,
      gO2)*ZE(gI1,0) + 0.7071067811865475*g2*KroneckerDelta(1,gO2)*ZE(gI1,0) -
      KroneckerDelta(2,gO2)*Ye(0,0)*ZE(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSeFePR(int gO1, int gI1) const
{
   const std::complex<double> result = -(Conj(Ye(0,0))*KroneckerDelta(2,gO1)*ZE
      (gI1,0)) - 1.0954451150103321*g1*KroneckerDelta(0,gO1)*ZE(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSmFmPL(int gO2, int gI1) const
{
   const std::complex<double> result = 0.5477225575051661*g1*KroneckerDelta(0,
      gO2)*ZM(gI1,0) + 0.7071067811865475*g2*KroneckerDelta(1,gO2)*ZM(gI1,0) -
      KroneckerDelta(2,gO2)*Ye(1,1)*ZM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSmFmPR(int gO1, int gI1) const
{
   const std::complex<double> result = -(Conj(Ye(1,1))*KroneckerDelta(2,gO1)*ZM
      (gI1,0)) - 1.0954451150103321*g1*KroneckerDelta(0,gO1)*ZM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSsFsPL(int gO2, int gI1, int ) const
{
   const std::complex<double> result = -0.18257418583505536*g1*KroneckerDelta(0
      ,gO2)*ZS(gI1,0) + 0.7071067811865475*g2*KroneckerDelta(1,gO2)*ZS(gI1,0) -
      KroneckerDelta(2,gO2)*Yd(1,1)*ZS(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSsFsPR(int gO1, int gI1, int ) const
{
   const std::complex<double> result = -(Conj(Yd(1,1))*KroneckerDelta(2,gO1)*ZS
      (gI1,0)) - 0.3651483716701107*g1*KroneckerDelta(0,gO1)*ZS(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjStFtPL(int gO2, int gI1, int ) const
{
   const std::complex<double> result = -0.18257418583505536*g1*KroneckerDelta(0
      ,gO2)*ZT(gI1,0) - 0.7071067811865475*g2*KroneckerDelta(1,gO2)*ZT(gI1,0) -
      KroneckerDelta(3,gO2)*Yu(2,2)*ZT(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjStFtPR(int gO1, int gI1, int ) const
{
   const std::complex<double> result = -(Conj(Yu(2,2))*KroneckerDelta(3,gO1)*ZT
      (gI1,0)) + 0.7302967433402214*g1*KroneckerDelta(0,gO1)*ZT(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjStauFtauPL(int gO2, int gI1) const
{
   const std::complex<double> result = 0.5477225575051661*g1*KroneckerDelta(0,
      gO2)*ZTau(gI1,0) + 0.7071067811865475*g2*KroneckerDelta(1,gO2)*ZTau(gI1,0) -
      KroneckerDelta(2,gO2)*Ye(2,2)*ZTau(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjStauFtauPR(int gO1, int gI1) const
{
   const std::complex<double> result = -(Conj(Ye(2,2))*KroneckerDelta(2,gO1)*
      ZTau(gI1,0)) - 1.0954451150103321*g1*KroneckerDelta(0,gO1)*ZTau(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSuFuPL(int gO2, int gI1, int ) const
{
   const std::complex<double> result = -0.18257418583505536*g1*KroneckerDelta(0
      ,gO2)*ZU(gI1,0) - 0.7071067811865475*g2*KroneckerDelta(1,gO2)*ZU(gI1,0) -
      KroneckerDelta(3,gO2)*Yu(0,0)*ZU(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSuFuPR(int gO1, int gI1, int ) const
{
   const std::complex<double> result = -(Conj(Yu(0,0))*KroneckerDelta(3,gO1)*ZU
      (gI1,0)) + 0.7302967433402214*g1*KroneckerDelta(0,gO1)*ZU(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiChiAhPL(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,-0.1)*(Conj(ZN(
      gI1,2))*(3.872983346207417*g1*KroneckerDelta(0,gO2) - 5*g2*KroneckerDelta(1,
      gO2))*ZA(gI2,0) - 5*g2*Conj(ZN(gI1,1))*KroneckerDelta(2,gO2)*ZA(gI2,0) -
      3.872983346207417*g1*Conj(ZN(gI1,3))*KroneckerDelta(0,gO2)*ZA(gI2,1) + 5*g2*
      Conj(ZN(gI1,3))*KroneckerDelta(1,gO2)*ZA(gI2,1) + 5*g2*Conj(ZN(gI1,1))*
      KroneckerDelta(3,gO2)*ZA(gI2,1) + 3.872983346207417*g1*Conj(ZN(gI1,0))*(
      KroneckerDelta(2,gO2)*ZA(gI2,0) - KroneckerDelta(3,gO2)*ZA(gI2,1)));

   return result;
}

std::complex<double> CLASSNAME::CpUChiChiAhPR(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0,0.1)*(
      KroneckerDelta(2,gO1)*ZA(gI2,0)*(3.872983346207417*g1*ZN(gI1,0) - 5*g2*ZN(
      gI1,1)) + KroneckerDelta(3,gO1)*ZA(gI2,1)*(-3.872983346207417*g1*ZN(gI1,0) +
      5*g2*ZN(gI1,1)) + (3.872983346207417*g1*KroneckerDelta(0,gO1) - 5*g2*
      KroneckerDelta(1,gO1))*(ZA(gI2,0)*ZN(gI1,2) - ZA(gI2,1)*ZN(gI1,3)));

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFeSePL(int gO2, int gI2) const
{
   const std::complex<double> result = -1.0954451150103321*g1*Conj(ZE(gI2,1))*
      KroneckerDelta(0,gO2) - Conj(ZE(gI2,0))*KroneckerDelta(2,gO2)*Ye(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFeSePR(int gO1, int gI2) const
{
   const std::complex<double> result = Conj(ZE(gI2,0))*(0.5477225575051661*g1*
      KroneckerDelta(0,gO1) + 0.7071067811865475*g2*KroneckerDelta(1,gO1)) - Conj(
      Ye(0,0))*Conj(ZE(gI2,1))*KroneckerDelta(2,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFmSmPL(int gO2, int gI2) const
{
   const std::complex<double> result = -1.0954451150103321*g1*Conj(ZM(gI2,1))*
      KroneckerDelta(0,gO2) - Conj(ZM(gI2,0))*KroneckerDelta(2,gO2)*Ye(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFmSmPR(int gO1, int gI2) const
{
   const std::complex<double> result = Conj(ZM(gI2,0))*(0.5477225575051661*g1*
      KroneckerDelta(0,gO1) + 0.7071067811865475*g2*KroneckerDelta(1,gO1)) - Conj(
      Ye(1,1))*Conj(ZM(gI2,1))*KroneckerDelta(2,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFtauStauPL(int gO2, int gI2) const
{
   const std::complex<double> result = -1.0954451150103321*g1*Conj(ZTau(gI2,1))
      *KroneckerDelta(0,gO2) - Conj(ZTau(gI2,0))*KroneckerDelta(2,gO2)*Ye(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFtauStauPR(int gO1, int gI2) const
{
   const std::complex<double> result = Conj(ZTau(gI2,0))*(0.5477225575051661*g1
      *KroneckerDelta(0,gO1) + 0.7071067811865475*g2*KroneckerDelta(1,gO1)) - Conj
      (Ye(2,2))*Conj(ZTau(gI2,1))*KroneckerDelta(2,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFbSbPL(int gO2, int , int gI2) const
{
   const std::complex<double> result = -0.3651483716701107*g1*Conj(ZB(gI2,1))*
      KroneckerDelta(0,gO2) - Conj(ZB(gI2,0))*KroneckerDelta(2,gO2)*Yd(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFbSbPR(int gO1, int , int gI2) const
{
   const std::complex<double> result = Conj(ZB(gI2,0))*(-0.18257418583505536*g1
      *KroneckerDelta(0,gO1) + 0.7071067811865475*g2*KroneckerDelta(1,gO1)) - Conj
      (Yd(2,2))*Conj(ZB(gI2,1))*KroneckerDelta(2,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFcScPL(int gO2, int , int gI2) const
{
   const std::complex<double> result = 0.7302967433402214*g1*Conj(ZC(gI2,1))*
      KroneckerDelta(0,gO2) - Conj(ZC(gI2,0))*KroneckerDelta(3,gO2)*Yu(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFcScPR(int gO1, int , int gI2) const
{
   const std::complex<double> result = Conj(ZC(gI2,0))*(-0.18257418583505536*g1
      *KroneckerDelta(0,gO1) - 0.7071067811865475*g2*KroneckerDelta(1,gO1)) - Conj
      (Yu(1,1))*Conj(ZC(gI2,1))*KroneckerDelta(3,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFdSdPL(int gO2, int , int gI2) const
{
   const std::complex<double> result = -0.3651483716701107*g1*Conj(ZD(gI2,1))*
      KroneckerDelta(0,gO2) - Conj(ZD(gI2,0))*KroneckerDelta(2,gO2)*Yd(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFdSdPR(int gO1, int , int gI2) const
{
   const std::complex<double> result = Conj(ZD(gI2,0))*(-0.18257418583505536*g1
      *KroneckerDelta(0,gO1) + 0.7071067811865475*g2*KroneckerDelta(1,gO1)) - Conj
      (Yd(0,0))*Conj(ZD(gI2,1))*KroneckerDelta(2,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFsSsPL(int gO2, int , int gI2) const
{
   const std::complex<double> result = -0.3651483716701107*g1*Conj(ZS(gI2,1))*
      KroneckerDelta(0,gO2) - Conj(ZS(gI2,0))*KroneckerDelta(2,gO2)*Yd(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFsSsPR(int gO1, int , int gI2) const
{
   const std::complex<double> result = Conj(ZS(gI2,0))*(-0.18257418583505536*g1
      *KroneckerDelta(0,gO1) + 0.7071067811865475*g2*KroneckerDelta(1,gO1)) - Conj
      (Yd(1,1))*Conj(ZS(gI2,1))*KroneckerDelta(2,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFtStPL(int gO2, int , int gI2) const
{
   const std::complex<double> result = 0.7302967433402214*g1*Conj(ZT(gI2,1))*
      KroneckerDelta(0,gO2) - Conj(ZT(gI2,0))*KroneckerDelta(3,gO2)*Yu(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFtStPR(int gO1, int , int gI2) const
{
   const std::complex<double> result = Conj(ZT(gI2,0))*(-0.18257418583505536*g1
      *KroneckerDelta(0,gO1) - 0.7071067811865475*g2*KroneckerDelta(1,gO1)) - Conj
      (Yu(2,2))*Conj(ZT(gI2,1))*KroneckerDelta(3,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFuSuPL(int gO2, int , int gI2) const
{
   const std::complex<double> result = 0.7302967433402214*g1*Conj(ZU(gI2,1))*
      KroneckerDelta(0,gO2) - Conj(ZU(gI2,0))*KroneckerDelta(3,gO2)*Yu(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFuSuPR(int gO1, int , int gI2) const
{
   const std::complex<double> result = Conj(ZU(gI2,0))*(-0.18257418583505536*g1
      *KroneckerDelta(0,gO1) - 0.7071067811865475*g2*KroneckerDelta(1,gO1)) - Conj
      (Yu(0,0))*Conj(ZU(gI2,1))*KroneckerDelta(3,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjVWmChaPR(int gO2, int gI2) const
{
   const std::complex<double> result = -(g2*KroneckerDelta(1,gO2)*UP(gI2,0)) +
      0.7071067811865475*g2*KroneckerDelta(3,gO2)*UP(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjVWmChaPL(int gO1, int gI2) const
{
   const std::complex<double> result = -0.5*g2*(2*Conj(UM(gI2,0))*
      KroneckerDelta(1,gO1) + 1.4142135623730951*Conj(UM(gI2,1))*KroneckerDelta(2,
      gO1));

   return result;
}

std::complex<double> CLASSNAME::CpUChiVZChiPR(int gO2, int gI2) const
{
   const std::complex<double> result = 0.1*(5*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()))*(KroneckerDelta(2,gO2)*ZN(gI2,2) -
      KroneckerDelta(3,gO2)*ZN(gI2,3));

   return result;
}

std::complex<double> CLASSNAME::CpUChiVZChiPL(int gO1, int gI2) const
{
   const std::complex<double> result = -0.1*(Conj(ZN(gI2,2))*KroneckerDelta(2,
      gO1) - Conj(ZN(gI2,3))*KroneckerDelta(3,gO1))*(5*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpUChibarFveSveLPL(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFveSveLPR(int gO1) const
{
   const std::complex<double> result = 0.5477225575051661*g1*KroneckerDelta(0,
      gO1) - 0.7071067811865475*g2*KroneckerDelta(1,gO1);

   return result;
}

double CLASSNAME::CpUChibarFvmSvmLPL(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFvmSvmLPR(int gO1) const
{
   const std::complex<double> result = 0.5477225575051661*g1*KroneckerDelta(0,
      gO1) - 0.7071067811865475*g2*KroneckerDelta(1,gO1);

   return result;
}

double CLASSNAME::CpUChibarFvtSvtLPL(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpUChibarFvtSvtLPR(int gO1) const
{
   const std::complex<double> result = 0.5477225575051661*g1*KroneckerDelta(0,
      gO1) - 0.7071067811865475*g2*KroneckerDelta(1,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSveLFvePL(int gO2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*KroneckerDelta(0,
      gO2) - 0.7071067811865475*g2*KroneckerDelta(1,gO2);

   return result;
}

double CLASSNAME::CpUChiconjSveLFvePR(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSvmLFvmPL(int gO2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*KroneckerDelta(0,
      gO2) - 0.7071067811865475*g2*KroneckerDelta(1,gO2);

   return result;
}

double CLASSNAME::CpUChiconjSvmLFvmPR(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpUChiconjSvtLFvtPL(int gO2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*KroneckerDelta(0,
      gO2) - 0.7071067811865475*g2*KroneckerDelta(1,gO2);

   return result;
}

double CLASSNAME::CpUChiconjSvtLFvtPR(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaChaAhPL(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*g2*(Conj(UM(gI1,1))*KroneckerDelta(0,gO2)*ZA(gI2,0) +
      Conj(UM(gI1,0))*KroneckerDelta(1,gO2)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaChaAhPR(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*g2*(KroneckerDelta(1,gO1)*UP(gI1,0)*ZA(gI2,0) +
      KroneckerDelta(0,gO1)*UP(gI1,1)*ZA(gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpbarUChahhChaPL(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = -0.7071067811865475*g2*(Conj(UM(gI2,1))*
      KroneckerDelta(0,gO2)*ZH(gI1,0) + Conj(UM(gI2,0))*KroneckerDelta(1,gO2)*ZH(
      gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpbarUChahhChaPR(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = -0.7071067811865475*g2*(KroneckerDelta(1
      ,gO1)*UP(gI2,0)*ZH(gI1,0) + KroneckerDelta(0,gO1)*UP(gI2,1)*ZH(gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaHpmChiPL(int gO2, int gI1, int gI2) const
{
   const std::complex<double> result = -0.1*(10*g2*Conj(ZN(gI2,3))*
      KroneckerDelta(0,gO2) + 1.4142135623730951*(3.872983346207417*g1*Conj(ZN(gI2
      ,0)) + 5*g2*Conj(ZN(gI2,1)))*KroneckerDelta(1,gO2))*ZP(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaHpmChiPR(int gO1, int gI1, int gI2) const
{
   const std::complex<double> result = 0.1*(KroneckerDelta(1,gO1)*(
      5.477225575051661*g1*ZN(gI2,0) + 7.0710678118654755*g2*ZN(gI2,1)) - 10*g2*
      KroneckerDelta(0,gO1)*ZN(gI2,2))*ZP(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaconjScFsPL(int gO2, int gI1, int ) const
{
   const std::complex<double> result = -(g2*KroneckerDelta(0,gO2)*ZC(gI1,0)) +
      KroneckerDelta(1,gO2)*Yu(1,1)*ZC(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaconjScFsPR(int gO1, int gI1, int ) const
{
   const std::complex<double> result = Conj(Yd(1,1))*KroneckerDelta(1,gO1)*ZC(
      gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaconjStFbPL(int gO2, int gI1, int ) const
{
   const std::complex<double> result = -(g2*KroneckerDelta(0,gO2)*ZT(gI1,0)) +
      KroneckerDelta(1,gO2)*Yu(2,2)*ZT(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaconjStFbPR(int gO1, int gI1, int ) const
{
   const std::complex<double> result = Conj(Yd(2,2))*KroneckerDelta(1,gO1)*ZT(
      gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaconjSuFdPL(int gO2, int gI1, int ) const
{
   const std::complex<double> result = -(g2*KroneckerDelta(0,gO2)*ZU(gI1,0)) +
      KroneckerDelta(1,gO2)*Yu(0,0)*ZU(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaconjSuFdPR(int gO1, int gI1, int ) const
{
   const std::complex<double> result = Conj(Yd(0,0))*KroneckerDelta(1,gO1)*ZU(
      gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaVPChaPR(int gO2, int gI2) const
{
   const std::complex<double> result = g2*KroneckerDelta(0,gO2)*Sin(ThetaW())*
      UP(gI2,0) + 0.1*KroneckerDelta(1,gO2)*(3.872983346207417*g1*Cos(ThetaW()) +
      5*g2*Sin(ThetaW()))*UP(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaVPChaPL(int gO1, int gI2) const
{
   const std::complex<double> result = g2*Conj(UM(gI2,0))*KroneckerDelta(0,gO1)
      *Sin(ThetaW()) + 0.1*Conj(UM(gI2,1))*KroneckerDelta(1,gO1)*(
      3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaVZChaPR(int gO2, int gI2) const
{
   const std::complex<double> result = g2*Cos(ThetaW())*KroneckerDelta(0,gO2)*
      UP(gI2,0) + 0.1*KroneckerDelta(1,gO2)*(5*g2*Cos(ThetaW()) -
      3.872983346207417*g1*Sin(ThetaW()))*UP(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaVZChaPL(int gO1, int gI2) const
{
   const std::complex<double> result = g2*Conj(UM(gI2,0))*Cos(ThetaW())*
      KroneckerDelta(0,gO1) + 0.1*Conj(UM(gI2,1))*KroneckerDelta(1,gO1)*(5*g2*Cos(
      ThetaW()) - 3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpbarUChabarFveSePL(int , int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarUChabarFveSePR(int gO1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZE(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(Ye(0,0))*Conj(ZE(gI2,1))*KroneckerDelta(1,gO1);

   return result;
}

double CLASSNAME::CpbarUChabarFvmSmPL(int , int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarUChabarFvmSmPR(int gO1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZM(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(Ye(1,1))*Conj(ZM(gI2,1))*KroneckerDelta(1,gO1);

   return result;
}

double CLASSNAME::CpbarUChabarFvtStauPL(int , int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarUChabarFvtStauPR(int gO1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZTau(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(Ye(2,2))*Conj(ZTau(gI2,1))*KroneckerDelta(1,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChabarFcSsPL(int gO2, int , int gI2) const
{
   const std::complex<double> result = Conj(ZS(gI2,0))*KroneckerDelta(1,gO2)*Yu
      (1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChabarFcSsPR(int gO1, int , int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZS(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(Yd(1,1))*Conj(ZS(gI2,1))*KroneckerDelta(1,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChabarFtSbPL(int gO2, int , int gI2) const
{
   const std::complex<double> result = Conj(ZB(gI2,0))*KroneckerDelta(1,gO2)*Yu
      (2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChabarFtSbPR(int gO1, int , int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZB(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(Yd(2,2))*Conj(ZB(gI2,1))*KroneckerDelta(1,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChabarFuSdPL(int gO2, int , int gI2) const
{
   const std::complex<double> result = Conj(ZD(gI2,0))*KroneckerDelta(1,gO2)*Yu
      (0,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChabarFuSdPR(int gO1, int , int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZD(gI2,0))*KroneckerDelta(0,
      gO1)) + Conj(Yd(0,0))*Conj(ZD(gI2,1))*KroneckerDelta(1,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaVWmChiPR(int gO2, int gI2) const
{
   const std::complex<double> result = -(g2*KroneckerDelta(0,gO2)*ZN(gI2,1)) +
      0.7071067811865475*g2*KroneckerDelta(1,gO2)*ZN(gI2,3);

   return result;
}

std::complex<double> CLASSNAME::CpbarUChaVWmChiPL(int gO1, int gI2) const
{
   const std::complex<double> result = -0.5*g2*(2*Conj(ZN(gI2,1))*
      KroneckerDelta(0,gO1) + 1.4142135623730951*Conj(ZN(gI2,2))*KroneckerDelta(1,
      gO1));

   return result;
}

double CLASSNAME::CpbarUChaconjSveLFePL(int gO2) const
{
   const double result = -(g2*KroneckerDelta(0,gO2));

   return result;
}

double CLASSNAME::CpbarUChaconjSveLFePR(int gO1) const
{
   const double result = Conj(Ye(0,0))*KroneckerDelta(1,gO1);

   return result;
}

double CLASSNAME::CpbarUChaconjSvmLFmPL(int gO2) const
{
   const double result = -(g2*KroneckerDelta(0,gO2));

   return result;
}

double CLASSNAME::CpbarUChaconjSvmLFmPR(int gO1) const
{
   const double result = Conj(Ye(1,1))*KroneckerDelta(1,gO1);

   return result;
}

double CLASSNAME::CpbarUChaconjSvtLFtauPL(int gO2) const
{
   const double result = -(g2*KroneckerDelta(0,gO2));

   return result;
}

double CLASSNAME::CpbarUChaconjSvtLFtauPR(int gO1) const
{
   const double result = Conj(Ye(2,2))*KroneckerDelta(1,gO1);

   return result;
}

std::complex<double> CLASSNAME::CpGluconjSbFbPL(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*ZB(gI1,0
      );

   return result;
}

std::complex<double> CLASSNAME::CpGluconjSbFbPR(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*ZB(
      gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpGluconjScFcPL(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*ZC(gI1,0
      );

   return result;
}

std::complex<double> CLASSNAME::CpGluconjScFcPR(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*ZC(
      gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpGluconjSdFdPL(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*ZD(gI1,0
      );

   return result;
}

std::complex<double> CLASSNAME::CpGluconjSdFdPR(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*ZD(
      gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpGluconjSsFsPL(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*ZS(gI1,0
      );

   return result;
}

std::complex<double> CLASSNAME::CpGluconjSsFsPR(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*ZS(
      gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpGluconjStFtPL(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*ZT(gI1,0
      );

   return result;
}

std::complex<double> CLASSNAME::CpGluconjStFtPR(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*ZT(
      gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpGluconjSuFuPL(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*PhaseGlu*ZU(gI1,0
      );

   return result;
}

std::complex<double> CLASSNAME::CpGluconjSuFuPR(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*Conj(PhaseGlu)*ZU(
      gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFbSbPL(int , int gI2) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZB(
      gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFbSbPR(int , int gI2) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZB(gI2,0));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFcScPL(int , int gI2) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZC(
      gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFcScPR(int , int gI2) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZC(gI2,0));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFdSdPL(int , int gI2) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZD(
      gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFdSdPR(int , int gI2) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZD(gI2,0));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFsSsPL(int , int gI2) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZS(
      gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFsSsPR(int , int gI2) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZS(gI2,0));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFtStPL(int , int gI2) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZT(
      gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFtStPR(int , int gI2) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZT(gI2,0));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFuSuPL(int , int gI2) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZU(
      gI2,1));

   return result;
}

std::complex<double> CLASSNAME::CpGlubarFuSuPR(int , int gI2) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZU(gI2,0));

   return result;
}

std::complex<double> CLASSNAME::CpGluVGGluPR() const
{
   const std::complex<double> result = std::complex<double>(0,-1)*g3*AbsSqr(
      PhaseGlu);

   return result;
}

std::complex<double> CLASSNAME::CpGluVGGluPL() const
{
   const std::complex<double> result = std::complex<double>(0,-1)*g3*AbsSqr(
      PhaseGlu);

   return result;
}

std::complex<double> CLASSNAME::CpbarFdSuChaPL(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(UM(gI2,1))*Conj(ZU(gI1,0))*Yd(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFdSuChaPR(int gI1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZU(gI1,0))*UP(gI2,0)) + Conj(
      Yu(0,0))*Conj(ZU(gI1,1))*UP(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFdSdChiPL(int gI1, int gI2) const
{
   const std::complex<double> result = -0.3651483716701107*g1*Conj(ZD(gI1,1))*
      Conj(ZN(gI2,0)) - Conj(ZD(gI1,0))*Conj(ZN(gI2,2))*Yd(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFdSdChiPR(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(ZD(gI1,0))*(-0.18257418583505536*g1
      *ZN(gI2,0) + 0.7071067811865475*g2*ZN(gI2,1)) - Conj(Yd(0,0))*Conj(ZD(gI1,1)
      )*ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFdhhFdPL(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Yd(0,0)*ZH(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFdhhFdPR(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Conj(Yd(0,0))*ZH(gI1
      ,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFdHpmFuPL(int gI1, int ) const
{
   const std::complex<double> result = Yd(0,0)*ZP(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFdHpmFuPR(int gI1, int ) const
{
   const std::complex<double> result = Conj(Yu(0,0))*ZP(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFdSdGluPL(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZD(
      gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpbarFdSdGluPR(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZD(gI1,0));

   return result;
}

std::complex<double> CLASSNAME::CpbarFdFdAhPL(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*Yd(0,0)*ZA(gI2,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFdFdAhPR(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yd(0,0))*ZA(gI2,0);

   return result;
}

double CLASSNAME::CpbarFdVGFdPR() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFdVGFdPL() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFdVPFdPR() const
{
   const double result = 0.2581988897471611*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpbarFdVPFdPL() const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) + 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpbarFdVWmFuPR(int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFdVWmFuPL(int ) const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpbarFdVZFdPR() const
{
   const double result = -0.2581988897471611*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpbarFdVZFdPL() const
{
   const double result = 0.03333333333333333*(15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpbarFsScChaPL(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(UM(gI2,1))*Conj(ZC(gI1,0))*Yd(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFsScChaPR(int gI1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZC(gI1,0))*UP(gI2,0)) + Conj(
      Yu(1,1))*Conj(ZC(gI1,1))*UP(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFsSsChiPL(int gI1, int gI2) const
{
   const std::complex<double> result = -0.3651483716701107*g1*Conj(ZN(gI2,0))*
      Conj(ZS(gI1,1)) - Conj(ZN(gI2,2))*Conj(ZS(gI1,0))*Yd(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFsSsChiPR(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(ZS(gI1,0))*(-0.18257418583505536*g1
      *ZN(gI2,0) + 0.7071067811865475*g2*ZN(gI2,1)) - Conj(Yd(1,1))*Conj(ZS(gI1,1)
      )*ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFshhFsPL(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Yd(1,1)*ZH(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFshhFsPR(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Conj(Yd(1,1))*ZH(gI1
      ,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFsHpmFcPL(int gI1, int ) const
{
   const std::complex<double> result = Yd(1,1)*ZP(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFsHpmFcPR(int gI1, int ) const
{
   const std::complex<double> result = Conj(Yu(1,1))*ZP(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFsSsGluPL(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZS(
      gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpbarFsSsGluPR(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZS(gI1,0));

   return result;
}

std::complex<double> CLASSNAME::CpbarFsFsAhPL(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*Yd(1,1)*ZA(gI2,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFsFsAhPR(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yd(1,1))*ZA(gI2,0);

   return result;
}

double CLASSNAME::CpbarFsVGFsPR() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFsVGFsPL() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFsVPFsPR() const
{
   const double result = 0.2581988897471611*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpbarFsVPFsPL() const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) + 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpbarFsVWmFcPR(int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFsVWmFcPL(int ) const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpbarFsVZFsPR() const
{
   const double result = -0.2581988897471611*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpbarFsVZFsPL() const
{
   const double result = 0.03333333333333333*(15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpbarFbStChaPL(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(UM(gI2,1))*Conj(ZT(gI1,0))*Yd(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFbStChaPR(int gI1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZT(gI1,0))*UP(gI2,0)) + Conj(
      Yu(2,2))*Conj(ZT(gI1,1))*UP(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFbSbChiPL(int gI1, int gI2) const
{
   const std::complex<double> result = -0.3651483716701107*g1*Conj(ZB(gI1,1))*
      Conj(ZN(gI2,0)) - Conj(ZB(gI1,0))*Conj(ZN(gI2,2))*Yd(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFbSbChiPR(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(ZB(gI1,0))*(-0.18257418583505536*g1
      *ZN(gI2,0) + 0.7071067811865475*g2*ZN(gI2,1)) - Conj(Yd(2,2))*Conj(ZB(gI1,1)
      )*ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFbhhFbPL(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Yd(2,2)*ZH(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFbhhFbPR(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Conj(Yd(2,2))*ZH(gI1
      ,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFbHpmFtPL(int gI1, int ) const
{
   const std::complex<double> result = Yd(2,2)*ZP(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFbHpmFtPR(int gI1, int ) const
{
   const std::complex<double> result = Conj(Yu(2,2))*ZP(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFbSbGluPL(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZB(
      gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpbarFbSbGluPR(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZB(gI1,0));

   return result;
}

std::complex<double> CLASSNAME::CpbarFbFbAhPL(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*Yd(2,2)*ZA(gI2,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFbFbAhPR(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yd(2,2))*ZA(gI2,0);

   return result;
}

double CLASSNAME::CpbarFbVGFbPR() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFbVGFbPL() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFbVPFbPR() const
{
   const double result = 0.2581988897471611*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpbarFbVPFbPL() const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) + 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpbarFbVWmFtPR(int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFbVWmFtPL(int ) const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpbarFbVZFbPR() const
{
   const double result = -0.2581988897471611*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpbarFbVZFbPL() const
{
   const double result = 0.03333333333333333*(15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpbarFubarChaSdPL(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(UP(gI1,1))*Conj(ZD(gI2,0))*Yu(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFubarChaSdPR(int gI1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZD(gI2,0))*UM(gI1,0)) + Conj(
      Yd(0,0))*Conj(ZD(gI2,1))*UM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFuSuChiPL(int gI1, int gI2) const
{
   const std::complex<double> result = 0.7302967433402214*g1*Conj(ZN(gI2,0))*
      Conj(ZU(gI1,1)) - Conj(ZN(gI2,3))*Conj(ZU(gI1,0))*Yu(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFuSuChiPR(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(ZU(gI1,0))*(-0.18257418583505536*g1
      *ZN(gI2,0) - 0.7071067811865475*g2*ZN(gI2,1)) - Conj(Yu(0,0))*Conj(ZU(gI1,1)
      )*ZN(gI2,3);

   return result;
}

std::complex<double> CLASSNAME::CpbarFuconjHpmFdPL(int gI1, int ) const
{
   const std::complex<double> result = Yu(0,0)*ZP(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFuconjHpmFdPR(int gI1, int ) const
{
   const std::complex<double> result = Conj(Yd(0,0))*ZP(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFuhhFuPL(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Yu(0,0)*ZH(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFuhhFuPR(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Conj(Yu(0,0))*ZH(gI1
      ,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFuSuGluPL(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZU(
      gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpbarFuSuGluPR(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZU(gI1,0));

   return result;
}

std::complex<double> CLASSNAME::CpbarFuFuAhPL(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*Yu(0,0)*ZA(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFuFuAhPR(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yu(0,0))*ZA(gI2,1);

   return result;
}

double CLASSNAME::CpbarFuVGFuPR() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFuVGFuPL() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFuVPFuPR() const
{
   const double result = -0.5163977794943222*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpbarFuVPFuPL() const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) - 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpbarFuVZFuPR() const
{
   const double result = 0.5163977794943222*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpbarFuVZFuPL() const
{
   const double result = 0.03333333333333333*(-15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpbarFuconjVWmFdPR(int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFuconjVWmFdPL(int ) const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

std::complex<double> CLASSNAME::CpbarFcbarChaSsPL(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(UP(gI1,1))*Conj(ZS(gI2,0))*Yu(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFcbarChaSsPR(int gI1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZS(gI2,0))*UM(gI1,0)) + Conj(
      Yd(1,1))*Conj(ZS(gI2,1))*UM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFcScChiPL(int gI1, int gI2) const
{
   const std::complex<double> result = 0.7302967433402214*g1*Conj(ZC(gI1,1))*
      Conj(ZN(gI2,0)) - Conj(ZC(gI1,0))*Conj(ZN(gI2,3))*Yu(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFcScChiPR(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(ZC(gI1,0))*(-0.18257418583505536*g1
      *ZN(gI2,0) - 0.7071067811865475*g2*ZN(gI2,1)) - Conj(Yu(1,1))*Conj(ZC(gI1,1)
      )*ZN(gI2,3);

   return result;
}

std::complex<double> CLASSNAME::CpbarFcconjHpmFsPL(int gI1, int ) const
{
   const std::complex<double> result = Yu(1,1)*ZP(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFcconjHpmFsPR(int gI1, int ) const
{
   const std::complex<double> result = Conj(Yd(1,1))*ZP(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFchhFcPL(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Yu(1,1)*ZH(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFchhFcPR(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Conj(Yu(1,1))*ZH(gI1
      ,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFcScGluPL(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZC(
      gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpbarFcScGluPR(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZC(gI1,0));

   return result;
}

std::complex<double> CLASSNAME::CpbarFcFcAhPL(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*Yu(1,1)*ZA(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFcFcAhPR(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yu(1,1))*ZA(gI2,1);

   return result;
}

double CLASSNAME::CpbarFcVGFcPR() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFcVGFcPL() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFcVPFcPR() const
{
   const double result = -0.5163977794943222*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpbarFcVPFcPL() const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) - 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpbarFcVZFcPR() const
{
   const double result = 0.5163977794943222*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpbarFcVZFcPL() const
{
   const double result = 0.03333333333333333*(-15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpbarFcconjVWmFsPR(int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFcconjVWmFsPL(int ) const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

std::complex<double> CLASSNAME::CpbarFtbarChaSbPL(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(UP(gI1,1))*Conj(ZB(gI2,0))*Yu(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtbarChaSbPR(int gI1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZB(gI2,0))*UM(gI1,0)) + Conj(
      Yd(2,2))*Conj(ZB(gI2,1))*UM(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtStChiPL(int gI1, int gI2) const
{
   const std::complex<double> result = 0.7302967433402214*g1*Conj(ZN(gI2,0))*
      Conj(ZT(gI1,1)) - Conj(ZN(gI2,3))*Conj(ZT(gI1,0))*Yu(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtStChiPR(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(ZT(gI1,0))*(-0.18257418583505536*g1
      *ZN(gI2,0) - 0.7071067811865475*g2*ZN(gI2,1)) - Conj(Yu(2,2))*Conj(ZT(gI1,1)
      )*ZN(gI2,3);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtconjHpmFbPL(int gI1, int ) const
{
   const std::complex<double> result = Yu(2,2)*ZP(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtconjHpmFbPR(int gI1, int ) const
{
   const std::complex<double> result = Conj(Yd(2,2))*ZP(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFthhFtPL(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Yu(2,2)*ZH(gI1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFthhFtPR(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Conj(Yu(2,2))*ZH(gI1
      ,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtStGluPL(int gI1, int ) const
{
   const std::complex<double> result = 1.4142135623730951*g3*PhaseGlu*Conj(ZT(
      gI1,1));

   return result;
}

std::complex<double> CLASSNAME::CpbarFtStGluPR(int gI1, int ) const
{
   const std::complex<double> result = -1.4142135623730951*g3*Conj(PhaseGlu)*
      Conj(ZT(gI1,0));

   return result;
}

std::complex<double> CLASSNAME::CpbarFtFtAhPL(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*Yu(2,2)*ZA(gI2,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtFtAhPR(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Yu(2,2))*ZA(gI2,1);

   return result;
}

double CLASSNAME::CpbarFtVGFtPR() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFtVGFtPL() const
{
   const double result = -g3;

   return result;
}

double CLASSNAME::CpbarFtVPFtPR() const
{
   const double result = -0.5163977794943222*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpbarFtVPFtPL() const
{
   const double result = -0.12909944487358055*g1*Cos(ThetaW()) - 0.5*g2*Sin(
      ThetaW());

   return result;
}

double CLASSNAME::CpbarFtVZFtPR() const
{
   const double result = 0.5163977794943222*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpbarFtVZFtPL() const
{
   const double result = 0.03333333333333333*(-15*g2*Cos(ThetaW()) +
      3.872983346207417*g1*Sin(ThetaW()));

   return result;
}

double CLASSNAME::CpbarFtconjVWmFbPR(int ) const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFtconjVWmFbPL(int ) const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpbarFvebarChaSePL(int , int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFvebarChaSePR(int gI1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZE(gI2,0))*UM(gI1,0)) + Conj(
      Ye(0,0))*Conj(ZE(gI2,1))*UM(gI1,1);

   return result;
}

double CLASSNAME::CpbarFveconjHpmFePL(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFveconjHpmFePR(int gI1) const
{
   const std::complex<double> result = Conj(Ye(0,0))*ZP(gI1,0);

   return result;
}

double CLASSNAME::CpbarFveSveLChiPL(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFveSveLChiPR(int gI2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*ZN(gI2,0) -
      0.7071067811865475*g2*ZN(gI2,1);

   return result;
}

double CLASSNAME::CpbarFveVZFvePR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFveVZFvePL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpbarFveconjVWmFePR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFveconjVWmFePL() const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpbarFvmbarChaSmPL(int , int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFvmbarChaSmPR(int gI1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZM(gI2,0))*UM(gI1,0)) + Conj(
      Ye(1,1))*Conj(ZM(gI2,1))*UM(gI1,1);

   return result;
}

double CLASSNAME::CpbarFvmconjHpmFmPL(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFvmconjHpmFmPR(int gI1) const
{
   const std::complex<double> result = Conj(Ye(1,1))*ZP(gI1,0);

   return result;
}

double CLASSNAME::CpbarFvmSvmLChiPL(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFvmSvmLChiPR(int gI2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*ZN(gI2,0) -
      0.7071067811865475*g2*ZN(gI2,1);

   return result;
}

double CLASSNAME::CpbarFvmVZFvmPR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFvmVZFvmPL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpbarFvmconjVWmFmPR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFvmconjVWmFmPL() const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpbarFvtbarChaStauPL(int , int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFvtbarChaStauPR(int gI1, int gI2) const
{
   const std::complex<double> result = -(g2*Conj(ZTau(gI2,0))*UM(gI1,0)) + Conj
      (Ye(2,2))*Conj(ZTau(gI2,1))*UM(gI1,1);

   return result;
}

double CLASSNAME::CpbarFvtconjHpmFtauPL(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFvtconjHpmFtauPR(int gI1) const
{
   const std::complex<double> result = Conj(Ye(2,2))*ZP(gI1,0);

   return result;
}

double CLASSNAME::CpbarFvtSvtLChiPL(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFvtSvtLChiPR(int gI2) const
{
   const std::complex<double> result = 0.5477225575051661*g1*ZN(gI2,0) -
      0.7071067811865475*g2*ZN(gI2,1);

   return result;
}

double CLASSNAME::CpbarFvtVZFvtPR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFvtVZFvtPL() const
{
   const double result = 0.1*(-5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpbarFvtconjVWmFtauPR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFvtconjVWmFtauPL() const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

std::complex<double> CLASSNAME::CpbarFeSeChiPL(int gI1, int gI2) const
{
   const std::complex<double> result = -1.0954451150103321*g1*Conj(ZE(gI1,1))*
      Conj(ZN(gI2,0)) - Conj(ZE(gI1,0))*Conj(ZN(gI2,2))*Ye(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFeSeChiPR(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(ZE(gI1,0))*(0.5477225575051661*g1*
      ZN(gI2,0) + 0.7071067811865475*g2*ZN(gI2,1)) - Conj(Ye(0,0))*Conj(ZE(gI1,1))
      *ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFehhFePL(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Ye(0,0)*ZH(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFehhFePR(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Conj(Ye(0,0))*ZH(gI1
      ,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFeHpmFvePL(int gI1) const
{
   const std::complex<double> result = Ye(0,0)*ZP(gI1,0);

   return result;
}

double CLASSNAME::CpbarFeHpmFvePR(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFeFeAhPL(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*Ye(0,0)*ZA(gI2,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFeFeAhPR(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Ye(0,0))*ZA(gI2,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFeSveLChaPL(int gI2) const
{
   const std::complex<double> result = Conj(UM(gI2,1))*Ye(0,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFeSveLChaPR(int gI2) const
{
   const std::complex<double> result = -(g2*UP(gI2,0));

   return result;
}

double CLASSNAME::CpbarFeVPFePR() const
{
   const double result = 0.7745966692414834*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpbarFeVPFePL() const
{
   const double result = 0.1*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpbarFeVWmFvePR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFeVWmFvePL() const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpbarFeVZFePR() const
{
   const double result = -0.7745966692414834*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpbarFeVZFePL() const
{
   const double result = 0.1*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpbarFmSmChiPL(int gI1, int gI2) const
{
   const std::complex<double> result = -1.0954451150103321*g1*Conj(ZM(gI1,1))*
      Conj(ZN(gI2,0)) - Conj(ZM(gI1,0))*Conj(ZN(gI2,2))*Ye(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFmSmChiPR(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(ZM(gI1,0))*(0.5477225575051661*g1*
      ZN(gI2,0) + 0.7071067811865475*g2*ZN(gI2,1)) - Conj(Ye(1,1))*Conj(ZM(gI1,1))
      *ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFmhhFmPL(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Ye(1,1)*ZH(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFmhhFmPR(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Conj(Ye(1,1))*ZH(gI1
      ,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFmHpmFvmPL(int gI1) const
{
   const std::complex<double> result = Ye(1,1)*ZP(gI1,0);

   return result;
}

double CLASSNAME::CpbarFmHpmFvmPR(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFmFmAhPL(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*Ye(1,1)*ZA(gI2,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFmFmAhPR(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Ye(1,1))*ZA(gI2,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFmSvmLChaPL(int gI2) const
{
   const std::complex<double> result = Conj(UM(gI2,1))*Ye(1,1);

   return result;
}

std::complex<double> CLASSNAME::CpbarFmSvmLChaPR(int gI2) const
{
   const std::complex<double> result = -(g2*UP(gI2,0));

   return result;
}

double CLASSNAME::CpbarFmVPFmPR() const
{
   const double result = 0.7745966692414834*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpbarFmVPFmPL() const
{
   const double result = 0.1*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpbarFmVWmFvmPR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFmVWmFvmPL() const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpbarFmVZFmPR() const
{
   const double result = -0.7745966692414834*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpbarFmVZFmPL() const
{
   const double result = 0.1*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}

std::complex<double> CLASSNAME::CpbarFtauStauChiPL(int gI1, int gI2) const
{
   const std::complex<double> result = -1.0954451150103321*g1*Conj(ZN(gI2,0))*
      Conj(ZTau(gI1,1)) - Conj(ZN(gI2,2))*Conj(ZTau(gI1,0))*Ye(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtauStauChiPR(int gI1, int gI2) const
{
   const std::complex<double> result = Conj(ZTau(gI1,0))*(0.5477225575051661*g1
      *ZN(gI2,0) + 0.7071067811865475*g2*ZN(gI2,1)) - Conj(Ye(2,2))*Conj(ZTau(gI1,
      1))*ZN(gI2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtauhhFtauPL(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Ye(2,2)*ZH(gI1,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtauhhFtauPR(int gI1) const
{
   const std::complex<double> result = -0.7071067811865475*Conj(Ye(2,2))*ZH(gI1
      ,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtauHpmFvtPL(int gI1) const
{
   const std::complex<double> result = Ye(2,2)*ZP(gI1,0);

   return result;
}

double CLASSNAME::CpbarFtauHpmFvtPR(int ) const
{
   const double result = 0;

   return result;
}

std::complex<double> CLASSNAME::CpbarFtauFtauAhPL(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      -0.7071067811865475)*Ye(2,2)*ZA(gI2,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtauFtauAhPR(int gI2) const
{
   const std::complex<double> result = std::complex<double>(0.,
      0.7071067811865475)*Conj(Ye(2,2))*ZA(gI2,0);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtauSvtLChaPL(int gI2) const
{
   const std::complex<double> result = Conj(UM(gI2,1))*Ye(2,2);

   return result;
}

std::complex<double> CLASSNAME::CpbarFtauSvtLChaPR(int gI2) const
{
   const std::complex<double> result = -(g2*UP(gI2,0));

   return result;
}

double CLASSNAME::CpbarFtauVPFtauPR() const
{
   const double result = 0.7745966692414834*g1*Cos(ThetaW());

   return result;
}

double CLASSNAME::CpbarFtauVPFtauPL() const
{
   const double result = 0.1*(3.872983346207417*g1*Cos(ThetaW()) + 5*g2*Sin(
      ThetaW()));

   return result;
}

double CLASSNAME::CpbarFtauVWmFvtPR() const
{
   const double result = 0;

   return result;
}

double CLASSNAME::CpbarFtauVWmFvtPL() const
{
   const double result = -0.7071067811865475*g2;

   return result;
}

double CLASSNAME::CpbarFtauVZFtauPR() const
{
   const double result = -0.7745966692414834*g1*Sin(ThetaW());

   return result;
}

double CLASSNAME::CpbarFtauVZFtauPL() const
{
   const double result = 0.1*(5*g2*Cos(ThetaW()) - 3.872983346207417*g1*Sin(
      ThetaW()));

   return result;
}


std::complex<double> CLASSNAME::self_energy_Sd_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpUSdconjUSdconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUSdconjUSdconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUSdconjUSdconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUSdconjUSdconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUSdconjUSdVZVZ(gO1,gO2);
   result += 1.3333333333333333*(Conj(CpconjUSdGluFdPL(gO2,1,1))*
      CpconjUSdGluFdPL(gO1,1,1) + Conj(CpconjUSdGluFdPR(gO2,1,1))*CpconjUSdGluFdPR
      (gO1,1,1))*G0(Sqr(p),Sqr(MGlu),Sqr(MFd));
   result += -2.6666666666666665*MFd*MGlu*B0(Sqr(p),Sqr(MGlu),Sqr(MFd))*(Conj(
      CpconjUSdGluFdPR(gO2,1,1))*CpconjUSdGluFdPL(gO1,1,1) + Conj(CpconjUSdGluFdPL
      (gO2,1,1))*CpconjUSdGluFdPR(gO1,1,1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUSdconjUSdAhAh(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUSdconjUSdconjHpmHpm(gO1,gO2,gI1
      ,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUSdconjUSdconjSbSb(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUSdconjUSdconjScSc(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUSdconjUSdconjSdSd(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUSdconjUSdconjSeSe(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUSdconjUSdconjSmSm(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUSdconjUSdconjSsSs(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUSdconjUSdconjStauStau(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUSdconjUSdconjStSt(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUSdconjUSdconjSuSu(gO1,gO2,gI1,
      gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUSdconjUSdhhhh(gO1,gO2,gI1,
      gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSd(gI1)),Sqr(MAh(gI2)))*
      Conj(CpconjUSdSdAh(gO2,gI1,gI2))*CpconjUSdSdAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSd(gI1)),Sqr(Mhh(gI2)))*
      Conj(CpconjUSdSdhh(gO2,gI1,gI2))*CpconjUSdSdhh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSu(gI1)),Sqr(MHpm(gI2)))*
      Conj(CpconjUSdSuHpm(gO2,gI1,gI2))*CpconjUSdSuHpm(gO1,gI1,gI2)));
   result += 1.3333333333333333*SUM(gI2,0,1,Conj(CpconjUSdVGSd(gO2,gI2))*
      CpconjUSdVGSd(gO1,gI2)*F0(Sqr(p),Sqr(MSd(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUSdVPSd(gO2,gI2))*CpconjUSdVPSd(gO1,gI2)*F0
      (Sqr(p),Sqr(MSd(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUSdVZSd(gO2,gI2))*CpconjUSdVZSd(gO1,gI2)*F0
      (Sqr(p),Sqr(MSd(gI2)),Sqr(MVZ)));
   result += SUM(gI2,0,1,Conj(CpconjUSdVWmSu(gO2,gI2))*CpconjUSdVWmSu(gO1,gI2)*
      F0(Sqr(p),Sqr(MSu(gI2)),Sqr(MVWm)));
   result += SUM(gI2,0,1,(Conj(CpconjUSdFuChaPL(gO2,1,gI2))*CpconjUSdFuChaPL(
      gO1,1,gI2) + Conj(CpconjUSdFuChaPR(gO2,1,gI2))*CpconjUSdFuChaPR(gO1,1,gI2))*
      G0(Sqr(p),Sqr(MFu),Sqr(MCha(gI2))));
   result += -2*MFu*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFu),Sqr(MCha(gI2)))*(Conj(
      CpconjUSdFuChaPR(gO2,1,gI2))*CpconjUSdFuChaPL(gO1,1,gI2) + Conj(
      CpconjUSdFuChaPL(gO2,1,gI2))*CpconjUSdFuChaPR(gO1,1,gI2))*MCha(gI2));
   result += SUM(gI2,0,3,(Conj(CpconjUSdFdChiPL(gO2,1,gI2))*CpconjUSdFdChiPL(
      gO1,1,gI2) + Conj(CpconjUSdFdChiPR(gO2,1,gI2))*CpconjUSdFdChiPR(gO1,1,gI2))*
      G0(Sqr(p),Sqr(MFd),Sqr(MChi(gI2))));
   result += -2*MFd*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFd),Sqr(MChi(gI2)))*(Conj(
      CpconjUSdFdChiPR(gO2,1,gI2))*CpconjUSdFdChiPL(gO1,1,gI2) + Conj(
      CpconjUSdFdChiPL(gO2,1,gI2))*CpconjUSdFdChiPR(gO1,1,gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Sd_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_Sd_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Su_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpUSuconjUSuconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUSuconjUSuconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUSuconjUSuconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUSuconjUSuconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUSuconjUSuVZVZ(gO1,gO2);
   result += 1.3333333333333333*(Conj(CpconjUSuGluFuPL(gO2,1,1))*
      CpconjUSuGluFuPL(gO1,1,1) + Conj(CpconjUSuGluFuPR(gO2,1,1))*CpconjUSuGluFuPR
      (gO1,1,1))*G0(Sqr(p),Sqr(MGlu),Sqr(MFu));
   result += -2.6666666666666665*MFu*MGlu*B0(Sqr(p),Sqr(MGlu),Sqr(MFu))*(Conj(
      CpconjUSuGluFuPR(gO2,1,1))*CpconjUSuGluFuPL(gO1,1,1) + Conj(CpconjUSuGluFuPL
      (gO2,1,1))*CpconjUSuGluFuPR(gO1,1,1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUSuconjUSuAhAh(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUSuconjUSuconjHpmHpm(gO1,gO2,gI1
      ,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUSuconjUSuconjSbSb(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUSuconjUSuconjScSc(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUSuconjUSuconjSdSd(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUSuconjUSuconjSeSe(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUSuconjUSuconjSmSm(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUSuconjUSuconjSsSs(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUSuconjUSuconjStauStau(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUSuconjUSuconjStSt(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUSuconjUSuconjSuSu(gO1,gO2,gI1,
      gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUSuconjUSuhhhh(gO1,gO2,gI1,
      gI1));
   result += SUM(gI1,0,1,(Conj(CpconjUSubarChaFdPL(gO2,gI1,1))*
      CpconjUSubarChaFdPL(gO1,gI1,1) + Conj(CpconjUSubarChaFdPR(gO2,gI1,1))*
      CpconjUSubarChaFdPR(gO1,gI1,1))*G0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFd)));
   result += -2*MFd*SUM(gI1,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFd))*(Conj(
      CpconjUSubarChaFdPR(gO2,gI1,1))*CpconjUSubarChaFdPL(gO1,gI1,1) + Conj(
      CpconjUSubarChaFdPL(gO2,gI1,1))*CpconjUSubarChaFdPR(gO1,gI1,1))*MCha(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MHpm(gI1)),Sqr(MSd(gI2)))*
      Conj(CpconjUSuconjHpmSd(gO2,gI1,gI2))*CpconjUSuconjHpmSd(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSu(gI1)),Sqr(MAh(gI2)))*
      Conj(CpconjUSuSuAh(gO2,gI1,gI2))*CpconjUSuSuAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSu(gI1)),Sqr(Mhh(gI2)))*
      Conj(CpconjUSuSuhh(gO2,gI1,gI2))*CpconjUSuSuhh(gO1,gI1,gI2)));
   result += SUM(gI2,0,1,Conj(CpconjUSuconjVWmSd(gO2,gI2))*CpconjUSuconjVWmSd(
      gO1,gI2)*F0(Sqr(p),Sqr(MSd(gI2)),Sqr(MVWm)));
   result += 1.3333333333333333*SUM(gI2,0,1,Conj(CpconjUSuVGSu(gO2,gI2))*
      CpconjUSuVGSu(gO1,gI2)*F0(Sqr(p),Sqr(MSu(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUSuVPSu(gO2,gI2))*CpconjUSuVPSu(gO1,gI2)*F0
      (Sqr(p),Sqr(MSu(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUSuVZSu(gO2,gI2))*CpconjUSuVZSu(gO1,gI2)*F0
      (Sqr(p),Sqr(MSu(gI2)),Sqr(MVZ)));
   result += SUM(gI2,0,3,(Conj(CpconjUSuFuChiPL(gO2,1,gI2))*CpconjUSuFuChiPL(
      gO1,1,gI2) + Conj(CpconjUSuFuChiPR(gO2,1,gI2))*CpconjUSuFuChiPR(gO1,1,gI2))*
      G0(Sqr(p),Sqr(MFu),Sqr(MChi(gI2))));
   result += -2*MFu*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFu),Sqr(MChi(gI2)))*(Conj(
      CpconjUSuFuChiPR(gO2,1,gI2))*CpconjUSuFuChiPL(gO1,1,gI2) + Conj(
      CpconjUSuFuChiPL(gO2,1,gI2))*CpconjUSuFuChiPR(gO1,1,gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Su_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_Su_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Se_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpUSeconjUSeconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUSeconjUSeconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUSeconjUSeconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUSeconjUSeconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUSeconjUSeVZVZ(gO1,gO2);
   result += Conj(CpconjUSeVWmSveL(gO2))*CpconjUSeVWmSveL(gO1)*F0(Sqr(p),Sqr(
      MSveL),Sqr(MVWm));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUSeconjUSeAhAh(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUSeconjUSeconjHpmHpm(gO1,gO2,gI1
      ,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUSeconjUSeconjSbSb(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUSeconjUSeconjScSc(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUSeconjUSeconjSdSd(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUSeconjUSeconjSeSe(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUSeconjUSeconjSmSm(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUSeconjUSeconjSsSs(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUSeconjUSeconjStauStau(gO1,gO2,
      gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUSeconjUSeconjStSt(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUSeconjUSeconjSuSu(gO1,gO2,gI1,
      gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUSeconjUSehhhh(gO1,gO2,gI1,
      gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSe(gI1)),Sqr(MAh(gI2)))*
      Conj(CpconjUSeSeAh(gO2,gI1,gI2))*CpconjUSeSeAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSe(gI1)),Sqr(Mhh(gI2)))*
      Conj(CpconjUSeSehh(gO2,gI1,gI2))*CpconjUSeSehh(gO1,gI1,gI2)));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MSveL),Sqr(MHpm(gI2)))*Conj(
      CpconjUSeSveLHpm(gO2,gI2))*CpconjUSeSveLHpm(gO1,gI2));
   result += SUM(gI2,0,1,Conj(CpconjUSeVPSe(gO2,gI2))*CpconjUSeVPSe(gO1,gI2)*F0
      (Sqr(p),Sqr(MSe(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUSeVZSe(gO2,gI2))*CpconjUSeVZSe(gO1,gI2)*F0
      (Sqr(p),Sqr(MSe(gI2)),Sqr(MVZ)));
   result += SUM(gI2,0,1,(Conj(CpconjUSeFveChaPL(gO2,gI2))*CpconjUSeFveChaPL(
      gO1,gI2) + Conj(CpconjUSeFveChaPR(gO2,gI2))*CpconjUSeFveChaPR(gO1,gI2))*G0(
      Sqr(p),Sqr(MFve),Sqr(MCha(gI2))));
   result += -2*MFve*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFve),Sqr(MCha(gI2)))*(Conj(
      CpconjUSeFveChaPR(gO2,gI2))*CpconjUSeFveChaPL(gO1,gI2) + Conj(
      CpconjUSeFveChaPL(gO2,gI2))*CpconjUSeFveChaPR(gO1,gI2))*MCha(gI2));
   result += SUM(gI2,0,3,(Conj(CpconjUSeFeChiPL(gO2,gI2))*CpconjUSeFeChiPL(gO1,
      gI2) + Conj(CpconjUSeFeChiPR(gO2,gI2))*CpconjUSeFeChiPR(gO1,gI2))*G0(Sqr(p),
      Sqr(MFe),Sqr(MChi(gI2))));
   result += -2*MFe*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFe),Sqr(MChi(gI2)))*(Conj(
      CpconjUSeFeChiPR(gO2,gI2))*CpconjUSeFeChiPL(gO1,gI2) + Conj(CpconjUSeFeChiPL
      (gO2,gI2))*CpconjUSeFeChiPR(gO1,gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Se_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_Se_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Sm_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpUSmconjUSmconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUSmconjUSmconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUSmconjUSmconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUSmconjUSmconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUSmconjUSmVZVZ(gO1,gO2);
   result += Conj(CpconjUSmVWmSvmL(gO2))*CpconjUSmVWmSvmL(gO1)*F0(Sqr(p),Sqr(
      MSvmL),Sqr(MVWm));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUSmconjUSmAhAh(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUSmconjUSmconjHpmHpm(gO1,gO2,gI1
      ,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUSmconjUSmconjSbSb(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUSmconjUSmconjScSc(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUSmconjUSmconjSdSd(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUSmconjUSmconjSeSe(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUSmconjUSmconjSmSm(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUSmconjUSmconjSsSs(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUSmconjUSmconjStauStau(gO1,gO2,
      gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUSmconjUSmconjStSt(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUSmconjUSmconjSuSu(gO1,gO2,gI1,
      gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUSmconjUSmhhhh(gO1,gO2,gI1,
      gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSm(gI1)),Sqr(MAh(gI2)))*
      Conj(CpconjUSmSmAh(gO2,gI1,gI2))*CpconjUSmSmAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSm(gI1)),Sqr(Mhh(gI2)))*
      Conj(CpconjUSmSmhh(gO2,gI1,gI2))*CpconjUSmSmhh(gO1,gI1,gI2)));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MSvmL),Sqr(MHpm(gI2)))*Conj(
      CpconjUSmSvmLHpm(gO2,gI2))*CpconjUSmSvmLHpm(gO1,gI2));
   result += SUM(gI2,0,1,Conj(CpconjUSmVPSm(gO2,gI2))*CpconjUSmVPSm(gO1,gI2)*F0
      (Sqr(p),Sqr(MSm(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUSmVZSm(gO2,gI2))*CpconjUSmVZSm(gO1,gI2)*F0
      (Sqr(p),Sqr(MSm(gI2)),Sqr(MVZ)));
   result += SUM(gI2,0,1,(Conj(CpconjUSmFvmChaPL(gO2,gI2))*CpconjUSmFvmChaPL(
      gO1,gI2) + Conj(CpconjUSmFvmChaPR(gO2,gI2))*CpconjUSmFvmChaPR(gO1,gI2))*G0(
      Sqr(p),Sqr(MFvm),Sqr(MCha(gI2))));
   result += -2*MFvm*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFvm),Sqr(MCha(gI2)))*(Conj(
      CpconjUSmFvmChaPR(gO2,gI2))*CpconjUSmFvmChaPL(gO1,gI2) + Conj(
      CpconjUSmFvmChaPL(gO2,gI2))*CpconjUSmFvmChaPR(gO1,gI2))*MCha(gI2));
   result += SUM(gI2,0,3,(Conj(CpconjUSmFmChiPL(gO2,gI2))*CpconjUSmFmChiPL(gO1,
      gI2) + Conj(CpconjUSmFmChiPR(gO2,gI2))*CpconjUSmFmChiPR(gO1,gI2))*G0(Sqr(p),
      Sqr(MFm),Sqr(MChi(gI2))));
   result += -2*MFm*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFm),Sqr(MChi(gI2)))*(Conj(
      CpconjUSmFmChiPR(gO2,gI2))*CpconjUSmFmChiPL(gO1,gI2) + Conj(CpconjUSmFmChiPL
      (gO2,gI2))*CpconjUSmFmChiPR(gO1,gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Sm_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_Sm_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Stau_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpUStauconjUStauconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUStauconjUStauconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUStauconjUStauconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUStauconjUStauconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUStauconjUStauVZVZ(gO1,gO2);
   result += Conj(CpconjUStauVWmSvtL(gO2))*CpconjUStauVWmSvtL(gO1)*F0(Sqr(p),
      Sqr(MSvtL),Sqr(MVWm));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUStauconjUStauAhAh(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUStauconjUStauconjHpmHpm(gO1,gO2
      ,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUStauconjUStauconjSbSb(gO1,gO2,
      gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUStauconjUStauconjScSc(gO1,gO2,
      gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUStauconjUStauconjSdSd(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUStauconjUStauconjSeSe(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUStauconjUStauconjSmSm(gO1,gO2,
      gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUStauconjUStauconjSsSs(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUStauconjUStauconjStauStau(gO1,
      gO2,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUStauconjUStauconjStSt(gO1,gO2,
      gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUStauconjUStauconjSuSu(gO1,gO2,
      gI1,gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUStauconjUStauhhhh(gO1,gO2,
      gI1,gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MStau(gI1)),Sqr(MAh(gI2)))*
      Conj(CpconjUStauStauAh(gO2,gI1,gI2))*CpconjUStauStauAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MStau(gI1)),Sqr(Mhh(gI2)))*
      Conj(CpconjUStauStauhh(gO2,gI1,gI2))*CpconjUStauStauhh(gO1,gI1,gI2)));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MSvtL),Sqr(MHpm(gI2)))*Conj(
      CpconjUStauSvtLHpm(gO2,gI2))*CpconjUStauSvtLHpm(gO1,gI2));
   result += SUM(gI2,0,1,Conj(CpconjUStauVPStau(gO2,gI2))*CpconjUStauVPStau(gO1
      ,gI2)*F0(Sqr(p),Sqr(MStau(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUStauVZStau(gO2,gI2))*CpconjUStauVZStau(gO1
      ,gI2)*F0(Sqr(p),Sqr(MStau(gI2)),Sqr(MVZ)));
   result += SUM(gI2,0,1,(Conj(CpconjUStauFvtChaPL(gO2,gI2))*
      CpconjUStauFvtChaPL(gO1,gI2) + Conj(CpconjUStauFvtChaPR(gO2,gI2))*
      CpconjUStauFvtChaPR(gO1,gI2))*G0(Sqr(p),Sqr(MFvt),Sqr(MCha(gI2))));
   result += -2*MFvt*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFvt),Sqr(MCha(gI2)))*(Conj(
      CpconjUStauFvtChaPR(gO2,gI2))*CpconjUStauFvtChaPL(gO1,gI2) + Conj(
      CpconjUStauFvtChaPL(gO2,gI2))*CpconjUStauFvtChaPR(gO1,gI2))*MCha(gI2));
   result += SUM(gI2,0,3,(Conj(CpconjUStauFtauChiPL(gO2,gI2))*
      CpconjUStauFtauChiPL(gO1,gI2) + Conj(CpconjUStauFtauChiPR(gO2,gI2))*
      CpconjUStauFtauChiPR(gO1,gI2))*G0(Sqr(p),Sqr(MFtau),Sqr(MChi(gI2))));
   result += -2*MFtau*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFtau),Sqr(MChi(gI2)))*(Conj(
      CpconjUStauFtauChiPR(gO2,gI2))*CpconjUStauFtauChiPL(gO1,gI2) + Conj(
      CpconjUStauFtauChiPL(gO2,gI2))*CpconjUStauFtauChiPR(gO1,gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Stau_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_Stau_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Ss_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpUSsconjUSsconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUSsconjUSsconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUSsconjUSsconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUSsconjUSsconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUSsconjUSsVZVZ(gO1,gO2);
   result += 1.3333333333333333*(Conj(CpconjUSsGluFsPL(gO2,1,1))*
      CpconjUSsGluFsPL(gO1,1,1) + Conj(CpconjUSsGluFsPR(gO2,1,1))*CpconjUSsGluFsPR
      (gO1,1,1))*G0(Sqr(p),Sqr(MGlu),Sqr(MFs));
   result += -2.6666666666666665*MFs*MGlu*B0(Sqr(p),Sqr(MGlu),Sqr(MFs))*(Conj(
      CpconjUSsGluFsPR(gO2,1,1))*CpconjUSsGluFsPL(gO1,1,1) + Conj(CpconjUSsGluFsPL
      (gO2,1,1))*CpconjUSsGluFsPR(gO1,1,1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUSsconjUSsAhAh(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUSsconjUSsconjHpmHpm(gO1,gO2,gI1
      ,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUSsconjUSsconjSbSb(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUSsconjUSsconjScSc(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUSsconjUSsconjSdSd(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUSsconjUSsconjSeSe(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUSsconjUSsconjSmSm(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUSsconjUSsconjSsSs(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUSsconjUSsconjStauStau(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUSsconjUSsconjStSt(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUSsconjUSsconjSuSu(gO1,gO2,gI1,
      gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUSsconjUSshhhh(gO1,gO2,gI1,
      gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSc(gI1)),Sqr(MHpm(gI2)))*
      Conj(CpconjUSsScHpm(gO2,gI1,gI2))*CpconjUSsScHpm(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSs(gI1)),Sqr(MAh(gI2)))*
      Conj(CpconjUSsSsAh(gO2,gI1,gI2))*CpconjUSsSsAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSs(gI1)),Sqr(Mhh(gI2)))*
      Conj(CpconjUSsSshh(gO2,gI1,gI2))*CpconjUSsSshh(gO1,gI1,gI2)));
   result += SUM(gI2,0,1,Conj(CpconjUSsVWmSc(gO2,gI2))*CpconjUSsVWmSc(gO1,gI2)*
      F0(Sqr(p),Sqr(MSc(gI2)),Sqr(MVWm)));
   result += 1.3333333333333333*SUM(gI2,0,1,Conj(CpconjUSsVGSs(gO2,gI2))*
      CpconjUSsVGSs(gO1,gI2)*F0(Sqr(p),Sqr(MSs(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUSsVPSs(gO2,gI2))*CpconjUSsVPSs(gO1,gI2)*F0
      (Sqr(p),Sqr(MSs(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUSsVZSs(gO2,gI2))*CpconjUSsVZSs(gO1,gI2)*F0
      (Sqr(p),Sqr(MSs(gI2)),Sqr(MVZ)));
   result += SUM(gI2,0,1,(Conj(CpconjUSsFcChaPL(gO2,1,gI2))*CpconjUSsFcChaPL(
      gO1,1,gI2) + Conj(CpconjUSsFcChaPR(gO2,1,gI2))*CpconjUSsFcChaPR(gO1,1,gI2))*
      G0(Sqr(p),Sqr(MFc),Sqr(MCha(gI2))));
   result += -2*MFc*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFc),Sqr(MCha(gI2)))*(Conj(
      CpconjUSsFcChaPR(gO2,1,gI2))*CpconjUSsFcChaPL(gO1,1,gI2) + Conj(
      CpconjUSsFcChaPL(gO2,1,gI2))*CpconjUSsFcChaPR(gO1,1,gI2))*MCha(gI2));
   result += SUM(gI2,0,3,(Conj(CpconjUSsFsChiPL(gO2,1,gI2))*CpconjUSsFsChiPL(
      gO1,1,gI2) + Conj(CpconjUSsFsChiPR(gO2,1,gI2))*CpconjUSsFsChiPR(gO1,1,gI2))*
      G0(Sqr(p),Sqr(MFs),Sqr(MChi(gI2))));
   result += -2*MFs*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFs),Sqr(MChi(gI2)))*(Conj(
      CpconjUSsFsChiPR(gO2,1,gI2))*CpconjUSsFsChiPL(gO1,1,gI2) + Conj(
      CpconjUSsFsChiPL(gO2,1,gI2))*CpconjUSsFsChiPR(gO1,1,gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Ss_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_Ss_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Sc_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpUScconjUScconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUScconjUScconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUScconjUScconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUScconjUScconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUScconjUScVZVZ(gO1,gO2);
   result += 1.3333333333333333*(Conj(CpconjUScGluFcPL(gO2,1,1))*
      CpconjUScGluFcPL(gO1,1,1) + Conj(CpconjUScGluFcPR(gO2,1,1))*CpconjUScGluFcPR
      (gO1,1,1))*G0(Sqr(p),Sqr(MGlu),Sqr(MFc));
   result += -2.6666666666666665*MFc*MGlu*B0(Sqr(p),Sqr(MGlu),Sqr(MFc))*(Conj(
      CpconjUScGluFcPR(gO2,1,1))*CpconjUScGluFcPL(gO1,1,1) + Conj(CpconjUScGluFcPL
      (gO2,1,1))*CpconjUScGluFcPR(gO1,1,1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUScconjUScAhAh(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUScconjUScconjHpmHpm(gO1,gO2,gI1
      ,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUScconjUScconjSbSb(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUScconjUScconjScSc(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUScconjUScconjSdSd(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUScconjUScconjSeSe(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUScconjUScconjSmSm(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUScconjUScconjSsSs(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUScconjUScconjStauStau(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUScconjUScconjStSt(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUScconjUScconjSuSu(gO1,gO2,gI1,
      gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUScconjUSchhhh(gO1,gO2,gI1,
      gI1));
   result += SUM(gI1,0,1,(Conj(CpconjUScbarChaFsPL(gO2,gI1,1))*
      CpconjUScbarChaFsPL(gO1,gI1,1) + Conj(CpconjUScbarChaFsPR(gO2,gI1,1))*
      CpconjUScbarChaFsPR(gO1,gI1,1))*G0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFs)));
   result += -2*MFs*SUM(gI1,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFs))*(Conj(
      CpconjUScbarChaFsPR(gO2,gI1,1))*CpconjUScbarChaFsPL(gO1,gI1,1) + Conj(
      CpconjUScbarChaFsPL(gO2,gI1,1))*CpconjUScbarChaFsPR(gO1,gI1,1))*MCha(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MHpm(gI1)),Sqr(MSs(gI2)))*
      Conj(CpconjUScconjHpmSs(gO2,gI1,gI2))*CpconjUScconjHpmSs(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSc(gI1)),Sqr(MAh(gI2)))*
      Conj(CpconjUScScAh(gO2,gI1,gI2))*CpconjUScScAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSc(gI1)),Sqr(Mhh(gI2)))*
      Conj(CpconjUScSchh(gO2,gI1,gI2))*CpconjUScSchh(gO1,gI1,gI2)));
   result += 1.3333333333333333*SUM(gI2,0,1,Conj(CpconjUScVGSc(gO2,gI2))*
      CpconjUScVGSc(gO1,gI2)*F0(Sqr(p),Sqr(MSc(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUScVPSc(gO2,gI2))*CpconjUScVPSc(gO1,gI2)*F0
      (Sqr(p),Sqr(MSc(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUScVZSc(gO2,gI2))*CpconjUScVZSc(gO1,gI2)*F0
      (Sqr(p),Sqr(MSc(gI2)),Sqr(MVZ)));
   result += SUM(gI2,0,1,Conj(CpconjUScconjVWmSs(gO2,gI2))*CpconjUScconjVWmSs(
      gO1,gI2)*F0(Sqr(p),Sqr(MSs(gI2)),Sqr(MVWm)));
   result += SUM(gI2,0,3,(Conj(CpconjUScFcChiPL(gO2,1,gI2))*CpconjUScFcChiPL(
      gO1,1,gI2) + Conj(CpconjUScFcChiPR(gO2,1,gI2))*CpconjUScFcChiPR(gO1,1,gI2))*
      G0(Sqr(p),Sqr(MFc),Sqr(MChi(gI2))));
   result += -2*MFc*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFc),Sqr(MChi(gI2)))*(Conj(
      CpconjUScFcChiPR(gO2,1,gI2))*CpconjUScFcChiPL(gO1,1,gI2) + Conj(
      CpconjUScFcChiPL(gO2,1,gI2))*CpconjUScFcChiPR(gO1,1,gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Sc_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_Sc_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Sb_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpUSbconjUSbconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUSbconjUSbconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUSbconjUSbconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUSbconjUSbconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUSbconjUSbVZVZ(gO1,gO2);
   result += 1.3333333333333333*(Conj(CpconjUSbGluFbPL(gO2,1,1))*
      CpconjUSbGluFbPL(gO1,1,1) + Conj(CpconjUSbGluFbPR(gO2,1,1))*CpconjUSbGluFbPR
      (gO1,1,1))*G0(Sqr(p),Sqr(MGlu),Sqr(MFb));
   result += -2.6666666666666665*MFb*MGlu*B0(Sqr(p),Sqr(MGlu),Sqr(MFb))*(Conj(
      CpconjUSbGluFbPR(gO2,1,1))*CpconjUSbGluFbPL(gO1,1,1) + Conj(CpconjUSbGluFbPL
      (gO2,1,1))*CpconjUSbGluFbPR(gO1,1,1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUSbconjUSbAhAh(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUSbconjUSbconjHpmHpm(gO1,gO2,gI1
      ,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUSbconjUSbconjSbSb(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUSbconjUSbconjScSc(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUSbconjUSbconjSdSd(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUSbconjUSbconjSeSe(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUSbconjUSbconjSmSm(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUSbconjUSbconjSsSs(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUSbconjUSbconjStauStau(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUSbconjUSbconjStSt(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUSbconjUSbconjSuSu(gO1,gO2,gI1,
      gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUSbconjUSbhhhh(gO1,gO2,gI1,
      gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSb(gI1)),Sqr(MAh(gI2)))*
      Conj(CpconjUSbSbAh(gO2,gI1,gI2))*CpconjUSbSbAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSb(gI1)),Sqr(Mhh(gI2)))*
      Conj(CpconjUSbSbhh(gO2,gI1,gI2))*CpconjUSbSbhh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSt(gI1)),Sqr(MHpm(gI2)))*
      Conj(CpconjUSbStHpm(gO2,gI1,gI2))*CpconjUSbStHpm(gO1,gI1,gI2)));
   result += 1.3333333333333333*SUM(gI2,0,1,Conj(CpconjUSbVGSb(gO2,gI2))*
      CpconjUSbVGSb(gO1,gI2)*F0(Sqr(p),Sqr(MSb(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUSbVPSb(gO2,gI2))*CpconjUSbVPSb(gO1,gI2)*F0
      (Sqr(p),Sqr(MSb(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUSbVZSb(gO2,gI2))*CpconjUSbVZSb(gO1,gI2)*F0
      (Sqr(p),Sqr(MSb(gI2)),Sqr(MVZ)));
   result += SUM(gI2,0,1,Conj(CpconjUSbVWmSt(gO2,gI2))*CpconjUSbVWmSt(gO1,gI2)*
      F0(Sqr(p),Sqr(MSt(gI2)),Sqr(MVWm)));
   result += SUM(gI2,0,1,(Conj(CpconjUSbFtChaPL(gO2,1,gI2))*CpconjUSbFtChaPL(
      gO1,1,gI2) + Conj(CpconjUSbFtChaPR(gO2,1,gI2))*CpconjUSbFtChaPR(gO1,1,gI2))*
      G0(Sqr(p),Sqr(MFt),Sqr(MCha(gI2))));
   result += -2*MFt*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MCha(gI2)))*(Conj(
      CpconjUSbFtChaPR(gO2,1,gI2))*CpconjUSbFtChaPL(gO1,1,gI2) + Conj(
      CpconjUSbFtChaPL(gO2,1,gI2))*CpconjUSbFtChaPR(gO1,1,gI2))*MCha(gI2));
   result += SUM(gI2,0,3,(Conj(CpconjUSbFbChiPL(gO2,1,gI2))*CpconjUSbFbChiPL(
      gO1,1,gI2) + Conj(CpconjUSbFbChiPR(gO2,1,gI2))*CpconjUSbFbChiPR(gO1,1,gI2))*
      G0(Sqr(p),Sqr(MFb),Sqr(MChi(gI2))));
   result += -2*MFb*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFb),Sqr(MChi(gI2)))*(Conj(
      CpconjUSbFbChiPR(gO2,1,gI2))*CpconjUSbFbChiPL(gO1,1,gI2) + Conj(
      CpconjUSbFbChiPL(gO2,1,gI2))*CpconjUSbFbChiPR(gO1,1,gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Sb_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_Sb_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_St_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpUStconjUStconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUStconjUStconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUStconjUStconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUStconjUStconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUStconjUStVZVZ(gO1,gO2);
   result += 1.3333333333333333*(Conj(CpconjUStGluFtPL(gO2,1,1))*
      CpconjUStGluFtPL(gO1,1,1) + Conj(CpconjUStGluFtPR(gO2,1,1))*CpconjUStGluFtPR
      (gO1,1,1))*G0(Sqr(p),Sqr(MGlu),Sqr(MFt));
   result += -2.6666666666666665*MFt*MGlu*B0(Sqr(p),Sqr(MGlu),Sqr(MFt))*(Conj(
      CpconjUStGluFtPR(gO2,1,1))*CpconjUStGluFtPL(gO1,1,1) + Conj(CpconjUStGluFtPL
      (gO2,1,1))*CpconjUStGluFtPR(gO1,1,1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUStconjUStAhAh(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUStconjUStconjHpmHpm(gO1,gO2,gI1
      ,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUStconjUStconjSbSb(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUStconjUStconjScSc(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUStconjUStconjSdSd(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUStconjUStconjSeSe(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUStconjUStconjSmSm(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUStconjUStconjSsSs(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUStconjUStconjStauStau(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUStconjUStconjStSt(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUStconjUStconjSuSu(gO1,gO2,gI1,
      gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUStconjUSthhhh(gO1,gO2,gI1,
      gI1));
   result += SUM(gI1,0,1,(Conj(CpconjUStbarChaFbPL(gO2,gI1,1))*
      CpconjUStbarChaFbPL(gO1,gI1,1) + Conj(CpconjUStbarChaFbPR(gO2,gI1,1))*
      CpconjUStbarChaFbPR(gO1,gI1,1))*G0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFb)));
   result += -2*MFb*SUM(gI1,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFb))*(Conj(
      CpconjUStbarChaFbPR(gO2,gI1,1))*CpconjUStbarChaFbPL(gO1,gI1,1) + Conj(
      CpconjUStbarChaFbPL(gO2,gI1,1))*CpconjUStbarChaFbPR(gO1,gI1,1))*MCha(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MHpm(gI1)),Sqr(MSb(gI2)))*
      Conj(CpconjUStconjHpmSb(gO2,gI1,gI2))*CpconjUStconjHpmSb(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSt(gI1)),Sqr(MAh(gI2)))*
      Conj(CpconjUStStAh(gO2,gI1,gI2))*CpconjUStStAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSt(gI1)),Sqr(Mhh(gI2)))*
      Conj(CpconjUStSthh(gO2,gI1,gI2))*CpconjUStSthh(gO1,gI1,gI2)));
   result += SUM(gI2,0,1,Conj(CpconjUStconjVWmSb(gO2,gI2))*CpconjUStconjVWmSb(
      gO1,gI2)*F0(Sqr(p),Sqr(MSb(gI2)),Sqr(MVWm)));
   result += 1.3333333333333333*SUM(gI2,0,1,Conj(CpconjUStVGSt(gO2,gI2))*
      CpconjUStVGSt(gO1,gI2)*F0(Sqr(p),Sqr(MSt(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUStVPSt(gO2,gI2))*CpconjUStVPSt(gO1,gI2)*F0
      (Sqr(p),Sqr(MSt(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUStVZSt(gO2,gI2))*CpconjUStVZSt(gO1,gI2)*F0
      (Sqr(p),Sqr(MSt(gI2)),Sqr(MVZ)));
   result += SUM(gI2,0,3,(Conj(CpconjUStFtChiPL(gO2,1,gI2))*CpconjUStFtChiPL(
      gO1,1,gI2) + Conj(CpconjUStFtChiPR(gO2,1,gI2))*CpconjUStFtChiPR(gO1,1,gI2))*
      G0(Sqr(p),Sqr(MFt),Sqr(MChi(gI2))));
   result += -2*MFt*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFt),Sqr(MChi(gI2)))*(Conj(
      CpconjUStFtChiPR(gO2,1,gI2))*CpconjUStFtChiPL(gO1,1,gI2) + Conj(
      CpconjUStFtChiPL(gO2,1,gI2))*CpconjUStFtChiPR(gO1,1,gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_St_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_St_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_hh_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(B0(Sqr(p),Sqr(MVWm),Sqr(MVWm))*CpUhhbargWmCgWmC(gO1)*
      CpUhhbargWmCgWmC(gO2));
   result += -(B0(Sqr(p),Sqr(MVWm),Sqr(MVWm))*CpUhhbargWmgWm(gO1)*
      CpUhhbargWmgWm(gO2));
   result += -(B0(Sqr(p),Sqr(MVZ),Sqr(MVZ))*CpUhhbargZgZ(gO1)*CpUhhbargZgZ(gO2)
      );
   result += B0(Sqr(p),Sqr(MSveL),Sqr(MSveL))*Conj(CpUhhconjSveLSveL(gO2))*
      CpUhhconjSveLSveL(gO1);
   result += B0(Sqr(p),Sqr(MSvmL),Sqr(MSvmL))*Conj(CpUhhconjSvmLSvmL(gO2))*
      CpUhhconjSvmLSvmL(gO1);
   result += B0(Sqr(p),Sqr(MSvtL),Sqr(MSvtL))*Conj(CpUhhconjSvtLSvtL(gO2))*
      CpUhhconjSvtLSvtL(gO1);
   result += 4*B0(Sqr(p),Sqr(MVWm),Sqr(MVWm))*Conj(CpUhhconjVWmVWm(gO2))*
      CpUhhconjVWmVWm(gO1);
   result += -(A0(Sqr(MSveL))*CpUhhUhhconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUhhUhhconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUhhUhhconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUhhUhhconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUhhUhhVZVZ(gO1,gO2);
   result += 2*B0(Sqr(p),Sqr(MVZ),Sqr(MVZ))*Conj(CpUhhVZVZ(gO2))*CpUhhVZVZ(gO1)
      ;
   result += 3*(Conj(CpUhhbarFbFbPL(gO2,1,1))*CpUhhbarFbFbPL(gO1,1,1) + Conj(
      CpUhhbarFbFbPR(gO2,1,1))*CpUhhbarFbFbPR(gO1,1,1))*G0(Sqr(p),Sqr(MFb),Sqr(MFb
      ));
   result += 3*(Conj(CpUhhbarFcFcPL(gO2,1,1))*CpUhhbarFcFcPL(gO1,1,1) + Conj(
      CpUhhbarFcFcPR(gO2,1,1))*CpUhhbarFcFcPR(gO1,1,1))*G0(Sqr(p),Sqr(MFc),Sqr(MFc
      ));
   result += 3*(Conj(CpUhhbarFdFdPL(gO2,1,1))*CpUhhbarFdFdPL(gO1,1,1) + Conj(
      CpUhhbarFdFdPR(gO2,1,1))*CpUhhbarFdFdPR(gO1,1,1))*G0(Sqr(p),Sqr(MFd),Sqr(MFd
      ));
   result += (Conj(CpUhhbarFeFePL(gO2))*CpUhhbarFeFePL(gO1) + Conj(
      CpUhhbarFeFePR(gO2))*CpUhhbarFeFePR(gO1))*G0(Sqr(p),Sqr(MFe),Sqr(MFe));
   result += (Conj(CpUhhbarFmFmPL(gO2))*CpUhhbarFmFmPL(gO1) + Conj(
      CpUhhbarFmFmPR(gO2))*CpUhhbarFmFmPR(gO1))*G0(Sqr(p),Sqr(MFm),Sqr(MFm));
   result += 3*(Conj(CpUhhbarFsFsPL(gO2,1,1))*CpUhhbarFsFsPL(gO1,1,1) + Conj(
      CpUhhbarFsFsPR(gO2,1,1))*CpUhhbarFsFsPR(gO1,1,1))*G0(Sqr(p),Sqr(MFs),Sqr(MFs
      ));
   result += 3*(Conj(CpUhhbarFtFtPL(gO2,1,1))*CpUhhbarFtFtPL(gO1,1,1) + Conj(
      CpUhhbarFtFtPR(gO2,1,1))*CpUhhbarFtFtPR(gO1,1,1))*G0(Sqr(p),Sqr(MFt),Sqr(MFt
      ));
   result += (Conj(CpUhhbarFtauFtauPL(gO2))*CpUhhbarFtauFtauPL(gO1) + Conj(
      CpUhhbarFtauFtauPR(gO2))*CpUhhbarFtauFtauPR(gO1))*G0(Sqr(p),Sqr(MFtau),Sqr(
      MFtau));
   result += 3*(Conj(CpUhhbarFuFuPL(gO2,1,1))*CpUhhbarFuFuPL(gO1,1,1) + Conj(
      CpUhhbarFuFuPR(gO2,1,1))*CpUhhbarFuFuPR(gO1,1,1))*G0(Sqr(p),Sqr(MFu),Sqr(MFu
      ));
   result += -6*B0(Sqr(p),Sqr(MFb),Sqr(MFb))*(Conj(CpUhhbarFbFbPR(gO2,1,1))*
      CpUhhbarFbFbPL(gO1,1,1) + Conj(CpUhhbarFbFbPL(gO2,1,1))*CpUhhbarFbFbPR(gO1,1
      ,1))*Sqr(MFb);
   result += -6*B0(Sqr(p),Sqr(MFc),Sqr(MFc))*(Conj(CpUhhbarFcFcPR(gO2,1,1))*
      CpUhhbarFcFcPL(gO1,1,1) + Conj(CpUhhbarFcFcPL(gO2,1,1))*CpUhhbarFcFcPR(gO1,1
      ,1))*Sqr(MFc);
   result += -6*B0(Sqr(p),Sqr(MFd),Sqr(MFd))*(Conj(CpUhhbarFdFdPR(gO2,1,1))*
      CpUhhbarFdFdPL(gO1,1,1) + Conj(CpUhhbarFdFdPL(gO2,1,1))*CpUhhbarFdFdPR(gO1,1
      ,1))*Sqr(MFd);
   result += -2*B0(Sqr(p),Sqr(MFe),Sqr(MFe))*(Conj(CpUhhbarFeFePR(gO2))*
      CpUhhbarFeFePL(gO1) + Conj(CpUhhbarFeFePL(gO2))*CpUhhbarFeFePR(gO1))*Sqr(MFe
      );
   result += -2*B0(Sqr(p),Sqr(MFm),Sqr(MFm))*(Conj(CpUhhbarFmFmPR(gO2))*
      CpUhhbarFmFmPL(gO1) + Conj(CpUhhbarFmFmPL(gO2))*CpUhhbarFmFmPR(gO1))*Sqr(MFm
      );
   result += -6*B0(Sqr(p),Sqr(MFs),Sqr(MFs))*(Conj(CpUhhbarFsFsPR(gO2,1,1))*
      CpUhhbarFsFsPL(gO1,1,1) + Conj(CpUhhbarFsFsPL(gO2,1,1))*CpUhhbarFsFsPR(gO1,1
      ,1))*Sqr(MFs);
   result += -6*B0(Sqr(p),Sqr(MFt),Sqr(MFt))*(Conj(CpUhhbarFtFtPR(gO2,1,1))*
      CpUhhbarFtFtPL(gO1,1,1) + Conj(CpUhhbarFtFtPL(gO2,1,1))*CpUhhbarFtFtPR(gO1,1
      ,1))*Sqr(MFt);
   result += -2*B0(Sqr(p),Sqr(MFtau),Sqr(MFtau))*(Conj(CpUhhbarFtauFtauPR(gO2))
      *CpUhhbarFtauFtauPL(gO1) + Conj(CpUhhbarFtauFtauPL(gO2))*CpUhhbarFtauFtauPR(
      gO1))*Sqr(MFtau);
   result += -6*B0(Sqr(p),Sqr(MFu),Sqr(MFu))*(Conj(CpUhhbarFuFuPR(gO2,1,1))*
      CpUhhbarFuFuPL(gO1,1,1) + Conj(CpUhhbarFuFuPL(gO2,1,1))*CpUhhbarFuFuPR(gO1,1
      ,1))*Sqr(MFu);
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUhhUhhAhAh(gO1,gO2,gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUhhUhhconjHpmHpm(gO1,gO2,gI1,gI1
      ));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUhhUhhconjSbSb(gO1,gO2,gI1,gI1)
      );
   result += -3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUhhUhhconjScSc(gO1,gO2,gI1,gI1)
      );
   result += -3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUhhUhhconjSdSd(gO1,gO2,gI1,gI1)
      );
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUhhUhhconjSeSe(gO1,gO2,gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUhhUhhconjSmSm(gO1,gO2,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUhhUhhconjSsSs(gO1,gO2,gI1,gI1)
      );
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUhhUhhconjStauStau(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUhhUhhconjStSt(gO1,gO2,gI1,gI1)
      );
   result += -3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUhhUhhconjSuSu(gO1,gO2,gI1,gI1)
      );
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUhhUhhhhhh(gO1,gO2,gI1,gI1));
   result += 0.5*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MAh(gI1)),Sqr(MAh(gI2)))
      *Conj(CpUhhAhAh(gO2,gI1,gI2))*CpUhhAhAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MHpm(gI1)),Sqr(MHpm(gI2)))*
      Conj(CpUhhconjHpmHpm(gO2,gI1,gI2))*CpUhhconjHpmHpm(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSb(gI1)),Sqr(MSb(gI2)))*
      Conj(CpUhhconjSbSb(gO2,gI1,gI2))*CpUhhconjSbSb(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSc(gI1)),Sqr(MSc(gI2)))*
      Conj(CpUhhconjScSc(gO2,gI1,gI2))*CpUhhconjScSc(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSd(gI1)),Sqr(MSd(gI2)))*
      Conj(CpUhhconjSdSd(gO2,gI1,gI2))*CpUhhconjSdSd(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSe(gI1)),Sqr(MSe(gI2)))*
      Conj(CpUhhconjSeSe(gO2,gI1,gI2))*CpUhhconjSeSe(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSm(gI1)),Sqr(MSm(gI2)))*
      Conj(CpUhhconjSmSm(gO2,gI1,gI2))*CpUhhconjSmSm(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSs(gI1)),Sqr(MSs(gI2)))*
      Conj(CpUhhconjSsSs(gO2,gI1,gI2))*CpUhhconjSsSs(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MStau(gI1)),Sqr(MStau(gI2)))
      *Conj(CpUhhconjStauStau(gO2,gI1,gI2))*CpUhhconjStauStau(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSt(gI1)),Sqr(MSt(gI2)))*
      Conj(CpUhhconjStSt(gO2,gI1,gI2))*CpUhhconjStSt(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSu(gI1)),Sqr(MSu(gI2)))*
      Conj(CpUhhconjSuSu(gO2,gI1,gI2))*CpUhhconjSuSu(gO1,gI1,gI2)));
   result += 0.5*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(Mhh(gI1)),Sqr(Mhh(gI2)))
      *Conj(CpUhhhhhh(gO2,gI1,gI2))*CpUhhhhhh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,(Conj(CpUhhbarChaChaPL(gO2,gI1,gI2))*
      CpUhhbarChaChaPL(gO1,gI1,gI2) + Conj(CpUhhbarChaChaPR(gO2,gI1,gI2))*
      CpUhhbarChaChaPR(gO1,gI1,gI2))*G0(Sqr(p),Sqr(MCha(gI1)),Sqr(MCha(gI2)))));
   result += -2*SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(
      MCha(gI2)))*(Conj(CpUhhbarChaChaPR(gO2,gI1,gI2))*CpUhhbarChaChaPL(gO1,gI1,
      gI2) + Conj(CpUhhbarChaChaPL(gO2,gI1,gI2))*CpUhhbarChaChaPR(gO1,gI1,gI2))*
      MCha(gI2)));
   result += 0.5*SUM(gI1,0,3,SUM(gI2,0,3,(Conj(CpUhhChiChiPL(gO2,gI1,gI2))*
      CpUhhChiChiPL(gO1,gI1,gI2) + Conj(CpUhhChiChiPR(gO2,gI1,gI2))*CpUhhChiChiPR(
      gO1,gI1,gI2))*G0(Sqr(p),Sqr(MChi(gI1)),Sqr(MChi(gI2)))));
   result += -SUM(gI1,0,3,MChi(gI1)*SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI1)),Sqr(
      MChi(gI2)))*(Conj(CpUhhChiChiPR(gO2,gI1,gI2))*CpUhhChiChiPL(gO1,gI1,gI2) +
      Conj(CpUhhChiChiPL(gO2,gI1,gI2))*CpUhhChiChiPR(gO1,gI1,gI2))*MChi(gI2)));
   result += SUM(gI2,0,1,Conj(CpUhhVZAh(gO2,gI2))*CpUhhVZAh(gO1,gI2)*F0(Sqr(p),
      Sqr(MAh(gI2)),Sqr(MVZ)));
   result += 2*SUM(gI2,0,1,Conj(CpUhhconjVWmHpm(gO2,gI2))*CpUhhconjVWmHpm(gO1,
      gI2)*F0(Sqr(p),Sqr(MHpm(gI2)),Sqr(MVWm)));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_hh_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_hh_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Ah_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -(B0(Sqr(p),Sqr(MVWm),Sqr(MVWm))*CpUAhbargWmCgWmC(gO1)*
      CpUAhbargWmCgWmC(gO2));
   result += -(B0(Sqr(p),Sqr(MVWm),Sqr(MVWm))*CpUAhbargWmgWm(gO1)*
      CpUAhbargWmgWm(gO2));
   result += -(A0(Sqr(MSveL))*CpUAhUAhconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUAhUAhconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUAhUAhconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUAhUAhconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUAhUAhVZVZ(gO1,gO2);
   result += 3*(Conj(CpUAhbarFbFbPL(gO2,1,1))*CpUAhbarFbFbPL(gO1,1,1) + Conj(
      CpUAhbarFbFbPR(gO2,1,1))*CpUAhbarFbFbPR(gO1,1,1))*G0(Sqr(p),Sqr(MFb),Sqr(MFb
      ));
   result += 3*(Conj(CpUAhbarFcFcPL(gO2,1,1))*CpUAhbarFcFcPL(gO1,1,1) + Conj(
      CpUAhbarFcFcPR(gO2,1,1))*CpUAhbarFcFcPR(gO1,1,1))*G0(Sqr(p),Sqr(MFc),Sqr(MFc
      ));
   result += 3*(Conj(CpUAhbarFdFdPL(gO2,1,1))*CpUAhbarFdFdPL(gO1,1,1) + Conj(
      CpUAhbarFdFdPR(gO2,1,1))*CpUAhbarFdFdPR(gO1,1,1))*G0(Sqr(p),Sqr(MFd),Sqr(MFd
      ));
   result += (Conj(CpUAhbarFeFePL(gO2))*CpUAhbarFeFePL(gO1) + Conj(
      CpUAhbarFeFePR(gO2))*CpUAhbarFeFePR(gO1))*G0(Sqr(p),Sqr(MFe),Sqr(MFe));
   result += (Conj(CpUAhbarFmFmPL(gO2))*CpUAhbarFmFmPL(gO1) + Conj(
      CpUAhbarFmFmPR(gO2))*CpUAhbarFmFmPR(gO1))*G0(Sqr(p),Sqr(MFm),Sqr(MFm));
   result += 3*(Conj(CpUAhbarFsFsPL(gO2,1,1))*CpUAhbarFsFsPL(gO1,1,1) + Conj(
      CpUAhbarFsFsPR(gO2,1,1))*CpUAhbarFsFsPR(gO1,1,1))*G0(Sqr(p),Sqr(MFs),Sqr(MFs
      ));
   result += 3*(Conj(CpUAhbarFtFtPL(gO2,1,1))*CpUAhbarFtFtPL(gO1,1,1) + Conj(
      CpUAhbarFtFtPR(gO2,1,1))*CpUAhbarFtFtPR(gO1,1,1))*G0(Sqr(p),Sqr(MFt),Sqr(MFt
      ));
   result += (Conj(CpUAhbarFtauFtauPL(gO2))*CpUAhbarFtauFtauPL(gO1) + Conj(
      CpUAhbarFtauFtauPR(gO2))*CpUAhbarFtauFtauPR(gO1))*G0(Sqr(p),Sqr(MFtau),Sqr(
      MFtau));
   result += 3*(Conj(CpUAhbarFuFuPL(gO2,1,1))*CpUAhbarFuFuPL(gO1,1,1) + Conj(
      CpUAhbarFuFuPR(gO2,1,1))*CpUAhbarFuFuPR(gO1,1,1))*G0(Sqr(p),Sqr(MFu),Sqr(MFu
      ));
   result += -6*B0(Sqr(p),Sqr(MFb),Sqr(MFb))*(Conj(CpUAhbarFbFbPR(gO2,1,1))*
      CpUAhbarFbFbPL(gO1,1,1) + Conj(CpUAhbarFbFbPL(gO2,1,1))*CpUAhbarFbFbPR(gO1,1
      ,1))*Sqr(MFb);
   result += -6*B0(Sqr(p),Sqr(MFc),Sqr(MFc))*(Conj(CpUAhbarFcFcPR(gO2,1,1))*
      CpUAhbarFcFcPL(gO1,1,1) + Conj(CpUAhbarFcFcPL(gO2,1,1))*CpUAhbarFcFcPR(gO1,1
      ,1))*Sqr(MFc);
   result += -6*B0(Sqr(p),Sqr(MFd),Sqr(MFd))*(Conj(CpUAhbarFdFdPR(gO2,1,1))*
      CpUAhbarFdFdPL(gO1,1,1) + Conj(CpUAhbarFdFdPL(gO2,1,1))*CpUAhbarFdFdPR(gO1,1
      ,1))*Sqr(MFd);
   result += -2*B0(Sqr(p),Sqr(MFe),Sqr(MFe))*(Conj(CpUAhbarFeFePR(gO2))*
      CpUAhbarFeFePL(gO1) + Conj(CpUAhbarFeFePL(gO2))*CpUAhbarFeFePR(gO1))*Sqr(MFe
      );
   result += -2*B0(Sqr(p),Sqr(MFm),Sqr(MFm))*(Conj(CpUAhbarFmFmPR(gO2))*
      CpUAhbarFmFmPL(gO1) + Conj(CpUAhbarFmFmPL(gO2))*CpUAhbarFmFmPR(gO1))*Sqr(MFm
      );
   result += -6*B0(Sqr(p),Sqr(MFs),Sqr(MFs))*(Conj(CpUAhbarFsFsPR(gO2,1,1))*
      CpUAhbarFsFsPL(gO1,1,1) + Conj(CpUAhbarFsFsPL(gO2,1,1))*CpUAhbarFsFsPR(gO1,1
      ,1))*Sqr(MFs);
   result += -6*B0(Sqr(p),Sqr(MFt),Sqr(MFt))*(Conj(CpUAhbarFtFtPR(gO2,1,1))*
      CpUAhbarFtFtPL(gO1,1,1) + Conj(CpUAhbarFtFtPL(gO2,1,1))*CpUAhbarFtFtPR(gO1,1
      ,1))*Sqr(MFt);
   result += -2*B0(Sqr(p),Sqr(MFtau),Sqr(MFtau))*(Conj(CpUAhbarFtauFtauPR(gO2))
      *CpUAhbarFtauFtauPL(gO1) + Conj(CpUAhbarFtauFtauPL(gO2))*CpUAhbarFtauFtauPR(
      gO1))*Sqr(MFtau);
   result += -6*B0(Sqr(p),Sqr(MFu),Sqr(MFu))*(Conj(CpUAhbarFuFuPR(gO2,1,1))*
      CpUAhbarFuFuPL(gO1,1,1) + Conj(CpUAhbarFuFuPL(gO2,1,1))*CpUAhbarFuFuPR(gO1,1
      ,1))*Sqr(MFu);
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUAhUAhAhAh(gO1,gO2,gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUAhUAhconjHpmHpm(gO1,gO2,gI1,gI1
      ));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUAhUAhconjSbSb(gO1,gO2,gI1,gI1)
      );
   result += -3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUAhUAhconjScSc(gO1,gO2,gI1,gI1)
      );
   result += -3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUAhUAhconjSdSd(gO1,gO2,gI1,gI1)
      );
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUAhUAhconjSeSe(gO1,gO2,gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUAhUAhconjSmSm(gO1,gO2,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUAhUAhconjSsSs(gO1,gO2,gI1,gI1)
      );
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUAhUAhconjStauStau(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUAhUAhconjStSt(gO1,gO2,gI1,gI1)
      );
   result += -3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUAhUAhconjSuSu(gO1,gO2,gI1,gI1)
      );
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUAhUAhhhhh(gO1,gO2,gI1,gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MHpm(gI1)),Sqr(MHpm(gI2)))*
      Conj(CpUAhconjHpmHpm(gO2,gI1,gI2))*CpUAhconjHpmHpm(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSb(gI1)),Sqr(MSb(gI2)))*
      Conj(CpUAhconjSbSb(gO2,gI1,gI2))*CpUAhconjSbSb(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSc(gI1)),Sqr(MSc(gI2)))*
      Conj(CpUAhconjScSc(gO2,gI1,gI2))*CpUAhconjScSc(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSd(gI1)),Sqr(MSd(gI2)))*
      Conj(CpUAhconjSdSd(gO2,gI1,gI2))*CpUAhconjSdSd(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSe(gI1)),Sqr(MSe(gI2)))*
      Conj(CpUAhconjSeSe(gO2,gI1,gI2))*CpUAhconjSeSe(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSm(gI1)),Sqr(MSm(gI2)))*
      Conj(CpUAhconjSmSm(gO2,gI1,gI2))*CpUAhconjSmSm(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSs(gI1)),Sqr(MSs(gI2)))*
      Conj(CpUAhconjSsSs(gO2,gI1,gI2))*CpUAhconjSsSs(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MStau(gI1)),Sqr(MStau(gI2)))
      *Conj(CpUAhconjStauStau(gO2,gI1,gI2))*CpUAhconjStauStau(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSt(gI1)),Sqr(MSt(gI2)))*
      Conj(CpUAhconjStSt(gO2,gI1,gI2))*CpUAhconjStSt(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSu(gI1)),Sqr(MSu(gI2)))*
      Conj(CpUAhconjSuSu(gO2,gI1,gI2))*CpUAhconjSuSu(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(Mhh(gI1)),Sqr(MAh(gI2)))*
      Conj(CpUAhhhAh(gO2,gI1,gI2))*CpUAhhhAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,(Conj(CpUAhbarChaChaPL(gO2,gI1,gI2))*
      CpUAhbarChaChaPL(gO1,gI1,gI2) + Conj(CpUAhbarChaChaPR(gO2,gI1,gI2))*
      CpUAhbarChaChaPR(gO1,gI1,gI2))*G0(Sqr(p),Sqr(MCha(gI1)),Sqr(MCha(gI2)))));
   result += -2*SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(
      MCha(gI2)))*(Conj(CpUAhbarChaChaPR(gO2,gI1,gI2))*CpUAhbarChaChaPL(gO1,gI1,
      gI2) + Conj(CpUAhbarChaChaPL(gO2,gI1,gI2))*CpUAhbarChaChaPR(gO1,gI1,gI2))*
      MCha(gI2)));
   result += 0.5*SUM(gI1,0,3,SUM(gI2,0,3,(Conj(CpUAhChiChiPL(gO2,gI1,gI2))*
      CpUAhChiChiPL(gO1,gI1,gI2) + Conj(CpUAhChiChiPR(gO2,gI1,gI2))*CpUAhChiChiPR(
      gO1,gI1,gI2))*G0(Sqr(p),Sqr(MChi(gI1)),Sqr(MChi(gI2)))));
   result += -SUM(gI1,0,3,MChi(gI1)*SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI1)),Sqr(
      MChi(gI2)))*(Conj(CpUAhChiChiPR(gO2,gI1,gI2))*CpUAhChiChiPL(gO1,gI1,gI2) +
      Conj(CpUAhChiChiPL(gO2,gI1,gI2))*CpUAhChiChiPR(gO1,gI1,gI2))*MChi(gI2)));
   result += SUM(gI2,0,1,Conj(CpUAhVZhh(gO2,gI2))*CpUAhVZhh(gO1,gI2)*F0(Sqr(p),
      Sqr(Mhh(gI2)),Sqr(MVZ)));
   result += 2*SUM(gI2,0,1,Conj(CpUAhconjVWmHpm(gO2,gI2))*CpUAhconjVWmHpm(gO1,
      gI2)*F0(Sqr(p),Sqr(MHpm(gI2)),Sqr(MVWm)));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Ah_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_Ah_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Hpm_1loop(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += 4*B0(Sqr(p),0,Sqr(MVWm))*Conj(CpconjUHpmVWmVP(gO2))*
      CpconjUHpmVWmVP(gO1);
   result += 4*B0(Sqr(p),Sqr(MVWm),Sqr(MVZ))*Conj(CpconjUHpmVZVWm(gO2))*
      CpconjUHpmVZVWm(gO1);
   result += -(A0(Sqr(MSveL))*CpUHpmconjUHpmconjSveLSveL(gO1,gO2));
   result += -(A0(Sqr(MSvmL))*CpUHpmconjUHpmconjSvmLSvmL(gO1,gO2));
   result += -(A0(Sqr(MSvtL))*CpUHpmconjUHpmconjSvtLSvtL(gO1,gO2));
   result += 4*A0(Sqr(MVWm))*CpUHpmconjUHpmconjVWmVWm(gO1,gO2);
   result += 2*A0(Sqr(MVZ))*CpUHpmconjUHpmVZVZ(gO1,gO2);
   result += -(B0(Sqr(p),Sqr(MVZ),Sqr(MVWm))*CpconjUHpmbargWmCgZ(gO1)*
      CpUHpmgWmCbargZ(gO2));
   result += -(B0(Sqr(p),Sqr(MVWm),Sqr(MVZ))*CpconjUHpmbargZgWm(gO1)*
      CpUHpmgZbargWm(gO2));
   result += 3*(Conj(CpconjUHpmbarFcFsPL(gO2,1,1))*CpconjUHpmbarFcFsPL(gO1,1,1)
      + Conj(CpconjUHpmbarFcFsPR(gO2,1,1))*CpconjUHpmbarFcFsPR(gO1,1,1))*G0(Sqr(p
      ),Sqr(MFc),Sqr(MFs));
   result += 3*(Conj(CpconjUHpmbarFtFbPL(gO2,1,1))*CpconjUHpmbarFtFbPL(gO1,1,1)
      + Conj(CpconjUHpmbarFtFbPR(gO2,1,1))*CpconjUHpmbarFtFbPR(gO1,1,1))*G0(Sqr(p
      ),Sqr(MFt),Sqr(MFb));
   result += 3*(Conj(CpconjUHpmbarFuFdPL(gO2,1,1))*CpconjUHpmbarFuFdPL(gO1,1,1)
      + Conj(CpconjUHpmbarFuFdPR(gO2,1,1))*CpconjUHpmbarFuFdPR(gO1,1,1))*G0(Sqr(p
      ),Sqr(MFu),Sqr(MFd));
   result += (Conj(CpconjUHpmbarFveFePL(gO2))*CpconjUHpmbarFveFePL(gO1) + Conj(
      CpconjUHpmbarFveFePR(gO2))*CpconjUHpmbarFveFePR(gO1))*G0(Sqr(p),Sqr(MFve),
      Sqr(MFe));
   result += (Conj(CpconjUHpmbarFvmFmPL(gO2))*CpconjUHpmbarFvmFmPL(gO1) + Conj(
      CpconjUHpmbarFvmFmPR(gO2))*CpconjUHpmbarFvmFmPR(gO1))*G0(Sqr(p),Sqr(MFvm),
      Sqr(MFm));
   result += (Conj(CpconjUHpmbarFvtFtauPL(gO2))*CpconjUHpmbarFvtFtauPL(gO1) +
      Conj(CpconjUHpmbarFvtFtauPR(gO2))*CpconjUHpmbarFvtFtauPR(gO1))*G0(Sqr(p),Sqr
      (MFvt),Sqr(MFtau));
   result += -6*MFc*MFs*B0(Sqr(p),Sqr(MFc),Sqr(MFs))*(Conj(CpconjUHpmbarFcFsPR(
      gO2,1,1))*CpconjUHpmbarFcFsPL(gO1,1,1) + Conj(CpconjUHpmbarFcFsPL(gO2,1,1))*
      CpconjUHpmbarFcFsPR(gO1,1,1));
   result += -6*MFb*MFt*B0(Sqr(p),Sqr(MFt),Sqr(MFb))*(Conj(CpconjUHpmbarFtFbPR(
      gO2,1,1))*CpconjUHpmbarFtFbPL(gO1,1,1) + Conj(CpconjUHpmbarFtFbPL(gO2,1,1))*
      CpconjUHpmbarFtFbPR(gO1,1,1));
   result += -6*MFd*MFu*B0(Sqr(p),Sqr(MFu),Sqr(MFd))*(Conj(CpconjUHpmbarFuFdPR(
      gO2,1,1))*CpconjUHpmbarFuFdPL(gO1,1,1) + Conj(CpconjUHpmbarFuFdPL(gO2,1,1))*
      CpconjUHpmbarFuFdPR(gO1,1,1));
   result += -2*MFe*MFve*B0(Sqr(p),Sqr(MFve),Sqr(MFe))*(Conj(
      CpconjUHpmbarFveFePR(gO2))*CpconjUHpmbarFveFePL(gO1) + Conj(
      CpconjUHpmbarFveFePL(gO2))*CpconjUHpmbarFveFePR(gO1));
   result += -2*MFm*MFvm*B0(Sqr(p),Sqr(MFvm),Sqr(MFm))*(Conj(
      CpconjUHpmbarFvmFmPR(gO2))*CpconjUHpmbarFvmFmPL(gO1) + Conj(
      CpconjUHpmbarFvmFmPL(gO2))*CpconjUHpmbarFvmFmPR(gO1));
   result += -2*MFtau*MFvt*B0(Sqr(p),Sqr(MFvt),Sqr(MFtau))*(Conj(
      CpconjUHpmbarFvtFtauPR(gO2))*CpconjUHpmbarFvtFtauPL(gO1) + Conj(
      CpconjUHpmbarFvtFtauPL(gO2))*CpconjUHpmbarFvtFtauPR(gO1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUHpmconjUHpmAhAh(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUHpmconjUHpmconjHpmHpm(gO1,gO2,
      gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUHpmconjUHpmconjSbSb(gO1,gO2,
      gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUHpmconjUHpmconjScSc(gO1,gO2,
      gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUHpmconjUHpmconjSdSd(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUHpmconjUHpmconjSeSe(gO1,gO2,gI1,
      gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUHpmconjUHpmconjSmSm(gO1,gO2,gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUHpmconjUHpmconjSsSs(gO1,gO2,
      gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUHpmconjUHpmconjStauStau(gO1,
      gO2,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUHpmconjUHpmconjStSt(gO1,gO2,
      gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUHpmconjUHpmconjSuSu(gO1,gO2,
      gI1,gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUHpmconjUHpmhhhh(gO1,gO2,gI1,
      gI1));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSc(gI1)),Sqr(MSs(gI2)))*
      Conj(CpconjUHpmconjScSs(gO2,gI1,gI2))*CpconjUHpmconjScSs(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSt(gI1)),Sqr(MSb(gI2)))*
      Conj(CpconjUHpmconjStSb(gO2,gI1,gI2))*CpconjUHpmconjStSb(gO1,gI1,gI2)));
   result += 3*SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MSu(gI1)),Sqr(MSd(gI2)))*
      Conj(CpconjUHpmconjSuSd(gO2,gI1,gI2))*CpconjUHpmconjSuSd(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MHpm(gI1)),Sqr(MAh(gI2)))*
      Conj(CpconjUHpmHpmAh(gO2,gI1,gI2))*CpconjUHpmHpmAh(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MHpm(gI1)),Sqr(Mhh(gI2)))*
      Conj(CpconjUHpmHpmhh(gO2,gI1,gI2))*CpconjUHpmHpmhh(gO1,gI1,gI2)));
   result += SUM(gI1,0,3,SUM(gI2,0,1,(Conj(CpconjUHpmChiChaPL(gO2,gI1,gI2))*
      CpconjUHpmChiChaPL(gO1,gI1,gI2) + Conj(CpconjUHpmChiChaPR(gO2,gI1,gI2))*
      CpconjUHpmChiChaPR(gO1,gI1,gI2))*G0(Sqr(p),Sqr(MChi(gI1)),Sqr(MCha(gI2)))));
   result += -2*SUM(gI1,0,3,MChi(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MChi(gI1)),Sqr(
      MCha(gI2)))*(Conj(CpconjUHpmChiChaPR(gO2,gI1,gI2))*CpconjUHpmChiChaPL(gO1,
      gI1,gI2) + Conj(CpconjUHpmChiChaPL(gO2,gI1,gI2))*CpconjUHpmChiChaPR(gO1,gI1,
      gI2))*MCha(gI2)));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MSveL),Sqr(MSe(gI2)))*Conj(
      CpconjUHpmconjSveLSe(gO2,gI2))*CpconjUHpmconjSveLSe(gO1,gI2));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MSvmL),Sqr(MSm(gI2)))*Conj(
      CpconjUHpmconjSvmLSm(gO2,gI2))*CpconjUHpmconjSvmLSm(gO1,gI2));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MSvtL),Sqr(MStau(gI2)))*Conj(
      CpconjUHpmconjSvtLStau(gO2,gI2))*CpconjUHpmconjSvtLStau(gO1,gI2));
   result += SUM(gI2,0,1,Conj(CpconjUHpmVWmAh(gO2,gI2))*CpconjUHpmVWmAh(gO1,gI2
      )*F0(Sqr(p),Sqr(MAh(gI2)),Sqr(MVWm)));
   result += SUM(gI2,0,1,Conj(CpconjUHpmVWmhh(gO2,gI2))*CpconjUHpmVWmhh(gO1,gI2
      )*F0(Sqr(p),Sqr(Mhh(gI2)),Sqr(MVWm)));
   result += SUM(gI2,0,1,Conj(CpconjUHpmVPHpm(gO2,gI2))*CpconjUHpmVPHpm(gO1,gI2
      )*F0(Sqr(p),Sqr(MHpm(gI2)),0));
   result += SUM(gI2,0,1,Conj(CpconjUHpmVZHpm(gO2,gI2))*CpconjUHpmVZHpm(gO1,gI2
      )*F0(Sqr(p),Sqr(MHpm(gI2)),Sqr(MVZ)));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Hpm_1loop(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = i; k < 2; k++)
         self_energy(i, k) = self_energy_Hpm_1loop(p, i, k);

   Hermitianize(self_energy);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_SveL_1loop(double p ) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpSveLconjSveLconjSveLSveL());
   result += -(A0(Sqr(MSvmL))*CpSveLconjSveLconjSvmLSvmL());
   result += -(A0(Sqr(MSvtL))*CpSveLconjSveLconjSvtLSvtL());
   result += 4*A0(Sqr(MVWm))*CpSveLconjSveLconjVWmVWm();
   result += 2*A0(Sqr(MVZ))*CpSveLconjSveLVZVZ();
   result += AbsSqr(CpconjSveLVZSveL())*F0(Sqr(p),Sqr(MSveL),Sqr(MVZ));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpSveLconjSveLAhAh(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpSveLconjSveLconjHpmHpm(gI1,gI1))
      ;
   result += -3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpSveLconjSveLconjSbSb(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpSveLconjSveLconjScSc(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpSveLconjSveLconjSdSd(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpSveLconjSveLconjSeSe(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpSveLconjSveLconjSmSm(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpSveLconjSveLconjSsSs(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpSveLconjSveLconjStauStau(gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpSveLconjSveLconjStSt(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpSveLconjSveLconjSuSu(gI1,gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpSveLconjSveLhhhh(gI1,gI1));
   result += SUM(gI1,0,1,(AbsSqr(CpconjSveLbarChaFePL(gI1)) + AbsSqr(
      CpconjSveLbarChaFePR(gI1)))*G0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFe)));
   result += -2*MFe*SUM(gI1,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFe))*(Conj(
      CpconjSveLbarChaFePR(gI1))*CpconjSveLbarChaFePL(gI1) + Conj(
      CpconjSveLbarChaFePL(gI1))*CpconjSveLbarChaFePR(gI1))*MCha(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjSveLconjHpmSe(gI1,gI2))*B0(
      Sqr(p),Sqr(MHpm(gI1)),Sqr(MSe(gI2)))));
   result += SUM(gI2,0,1,AbsSqr(CpconjSveLSveLhh(gI2))*B0(Sqr(p),Sqr(MSveL),Sqr
      (Mhh(gI2))));
   result += SUM(gI2,0,1,AbsSqr(CpconjSveLconjVWmSe(gI2))*F0(Sqr(p),Sqr(MSe(gI2
      )),Sqr(MVWm)));
   result += SUM(gI2,0,3,(AbsSqr(CpconjSveLFveChiPL(gI2)) + AbsSqr(
      CpconjSveLFveChiPR(gI2)))*G0(Sqr(p),Sqr(MFve),Sqr(MChi(gI2))));
   result += -2*MFve*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFve),Sqr(MChi(gI2)))*(Conj(
      CpconjSveLFveChiPR(gI2))*CpconjSveLFveChiPL(gI2) + Conj(CpconjSveLFveChiPL(
      gI2))*CpconjSveLFveChiPR(gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_SvmL_1loop(double p ) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpSvmLconjSvmLconjSveLSveL());
   result += -(A0(Sqr(MSvmL))*CpSvmLconjSvmLconjSvmLSvmL());
   result += -(A0(Sqr(MSvtL))*CpSvmLconjSvmLconjSvtLSvtL());
   result += 4*A0(Sqr(MVWm))*CpSvmLconjSvmLconjVWmVWm();
   result += 2*A0(Sqr(MVZ))*CpSvmLconjSvmLVZVZ();
   result += AbsSqr(CpconjSvmLVZSvmL())*F0(Sqr(p),Sqr(MSvmL),Sqr(MVZ));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpSvmLconjSvmLAhAh(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpSvmLconjSvmLconjHpmHpm(gI1,gI1))
      ;
   result += -3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpSvmLconjSvmLconjSbSb(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpSvmLconjSvmLconjScSc(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpSvmLconjSvmLconjSdSd(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpSvmLconjSvmLconjSeSe(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpSvmLconjSvmLconjSmSm(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpSvmLconjSvmLconjSsSs(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpSvmLconjSvmLconjStauStau(gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpSvmLconjSvmLconjStSt(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpSvmLconjSvmLconjSuSu(gI1,gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpSvmLconjSvmLhhhh(gI1,gI1));
   result += SUM(gI1,0,1,(AbsSqr(CpconjSvmLbarChaFmPL(gI1)) + AbsSqr(
      CpconjSvmLbarChaFmPR(gI1)))*G0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFm)));
   result += -2*MFm*SUM(gI1,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFm))*(Conj(
      CpconjSvmLbarChaFmPR(gI1))*CpconjSvmLbarChaFmPL(gI1) + Conj(
      CpconjSvmLbarChaFmPL(gI1))*CpconjSvmLbarChaFmPR(gI1))*MCha(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjSvmLconjHpmSm(gI1,gI2))*B0(
      Sqr(p),Sqr(MHpm(gI1)),Sqr(MSm(gI2)))));
   result += SUM(gI2,0,1,AbsSqr(CpconjSvmLSvmLhh(gI2))*B0(Sqr(p),Sqr(MSvmL),Sqr
      (Mhh(gI2))));
   result += SUM(gI2,0,1,AbsSqr(CpconjSvmLconjVWmSm(gI2))*F0(Sqr(p),Sqr(MSm(gI2
      )),Sqr(MVWm)));
   result += SUM(gI2,0,3,(AbsSqr(CpconjSvmLFvmChiPL(gI2)) + AbsSqr(
      CpconjSvmLFvmChiPR(gI2)))*G0(Sqr(p),Sqr(MFvm),Sqr(MChi(gI2))));
   result += -2*MFvm*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFvm),Sqr(MChi(gI2)))*(Conj(
      CpconjSvmLFvmChiPR(gI2))*CpconjSvmLFvmChiPL(gI2) + Conj(CpconjSvmLFvmChiPL(
      gI2))*CpconjSvmLFvmChiPR(gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_SvtL_1loop(double p ) const
{
   std::complex<double> result;

   result += -(A0(Sqr(MSveL))*CpSvtLconjSvtLconjSveLSveL());
   result += -(A0(Sqr(MSvmL))*CpSvtLconjSvtLconjSvmLSvmL());
   result += -(A0(Sqr(MSvtL))*CpSvtLconjSvtLconjSvtLSvtL());
   result += 4*A0(Sqr(MVWm))*CpSvtLconjSvtLconjVWmVWm();
   result += 2*A0(Sqr(MVZ))*CpSvtLconjSvtLVZVZ();
   result += AbsSqr(CpconjSvtLVZSvtL())*F0(Sqr(p),Sqr(MSvtL),Sqr(MVZ));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpSvtLconjSvtLAhAh(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpSvtLconjSvtLconjHpmHpm(gI1,gI1))
      ;
   result += -3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpSvtLconjSvtLconjSbSb(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpSvtLconjSvtLconjScSc(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpSvtLconjSvtLconjSdSd(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpSvtLconjSvtLconjSeSe(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpSvtLconjSvtLconjSmSm(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpSvtLconjSvtLconjSsSs(gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpSvtLconjSvtLconjStauStau(gI1,
      gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpSvtLconjSvtLconjStSt(gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpSvtLconjSvtLconjSuSu(gI1,gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpSvtLconjSvtLhhhh(gI1,gI1));
   result += SUM(gI1,0,1,(AbsSqr(CpconjSvtLbarChaFtauPL(gI1)) + AbsSqr(
      CpconjSvtLbarChaFtauPR(gI1)))*G0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFtau)));
   result += -2*MFtau*SUM(gI1,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MFtau))*(Conj(
      CpconjSvtLbarChaFtauPR(gI1))*CpconjSvtLbarChaFtauPL(gI1) + Conj(
      CpconjSvtLbarChaFtauPL(gI1))*CpconjSvtLbarChaFtauPR(gI1))*MCha(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjSvtLconjHpmStau(gI1,gI2))*B0(
      Sqr(p),Sqr(MHpm(gI1)),Sqr(MStau(gI2)))));
   result += SUM(gI2,0,1,AbsSqr(CpconjSvtLSvtLhh(gI2))*B0(Sqr(p),Sqr(MSvtL),Sqr
      (Mhh(gI2))));
   result += SUM(gI2,0,1,AbsSqr(CpconjSvtLconjVWmStau(gI2))*F0(Sqr(p),Sqr(MStau
      (gI2)),Sqr(MVWm)));
   result += SUM(gI2,0,3,(AbsSqr(CpconjSvtLFvtChiPL(gI2)) + AbsSqr(
      CpconjSvtLFvtChiPR(gI2)))*G0(Sqr(p),Sqr(MFvt),Sqr(MChi(gI2))));
   result += -2*MFvt*SUM(gI2,0,3,B0(Sqr(p),Sqr(MFvt),Sqr(MChi(gI2)))*(Conj(
      CpconjSvtLFvtChiPR(gI2))*CpconjSvtLFvtChiPL(gI2) + Conj(CpconjSvtLFvtChiPL(
      gI2))*CpconjSvtLFvtChiPR(gI2))*MChi(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_VG_1loop(double p ) const
{
   std::complex<double> result;

   result += 3*AbsSqr(CpVGbargGgG())*B00(Sqr(p),Sqr(MVG),Sqr(MVG));
   result += -3*AbsSqr(CpVGVGVG())*(5*B00(Sqr(p),0,0) + 2*B0(Sqr(p),0,0)*Sqr(p)
      );
   result += 0;
   result += 0.5*(AbsSqr(CpVGbarFbFbPL(1,1))*H0(Sqr(p),Sqr(MFb),Sqr(MFb)) +
      AbsSqr(CpVGbarFbFbPR(1,1))*H0(Sqr(p),Sqr(MFb),Sqr(MFb)) + 4*B0(Sqr(p),Sqr(
      MFb),Sqr(MFb))*Re(Conj(CpVGbarFbFbPL(1,1))*CpVGbarFbFbPR(1,1))*Sqr(MFb));
   result += 0.5*(AbsSqr(CpVGbarFcFcPL(1,1))*H0(Sqr(p),Sqr(MFc),Sqr(MFc)) +
      AbsSqr(CpVGbarFcFcPR(1,1))*H0(Sqr(p),Sqr(MFc),Sqr(MFc)) + 4*B0(Sqr(p),Sqr(
      MFc),Sqr(MFc))*Re(Conj(CpVGbarFcFcPL(1,1))*CpVGbarFcFcPR(1,1))*Sqr(MFc));
   result += 0.5*(AbsSqr(CpVGbarFdFdPL(1,1))*H0(Sqr(p),Sqr(MFd),Sqr(MFd)) +
      AbsSqr(CpVGbarFdFdPR(1,1))*H0(Sqr(p),Sqr(MFd),Sqr(MFd)) + 4*B0(Sqr(p),Sqr(
      MFd),Sqr(MFd))*Re(Conj(CpVGbarFdFdPL(1,1))*CpVGbarFdFdPR(1,1))*Sqr(MFd));
   result += 0.5*(AbsSqr(CpVGbarFsFsPL(1,1))*H0(Sqr(p),Sqr(MFs),Sqr(MFs)) +
      AbsSqr(CpVGbarFsFsPR(1,1))*H0(Sqr(p),Sqr(MFs),Sqr(MFs)) + 4*B0(Sqr(p),Sqr(
      MFs),Sqr(MFs))*Re(Conj(CpVGbarFsFsPL(1,1))*CpVGbarFsFsPR(1,1))*Sqr(MFs));
   result += 0.5*(AbsSqr(CpVGbarFtFtPL(1,1))*H0(Sqr(p),Sqr(MFt),Sqr(MFt)) +
      AbsSqr(CpVGbarFtFtPR(1,1))*H0(Sqr(p),Sqr(MFt),Sqr(MFt)) + 4*B0(Sqr(p),Sqr(
      MFt),Sqr(MFt))*Re(Conj(CpVGbarFtFtPL(1,1))*CpVGbarFtFtPR(1,1))*Sqr(MFt));
   result += 0.5*(AbsSqr(CpVGbarFuFuPL(1,1))*H0(Sqr(p),Sqr(MFu),Sqr(MFu)) +
      AbsSqr(CpVGbarFuFuPR(1,1))*H0(Sqr(p),Sqr(MFu),Sqr(MFu)) + 4*B0(Sqr(p),Sqr(
      MFu),Sqr(MFu))*Re(Conj(CpVGbarFuFuPL(1,1))*CpVGbarFuFuPR(1,1))*Sqr(MFu));
   result += 1.5*(AbsSqr(CpVGGluGluPL(1,1))*H0(Sqr(p),Sqr(MGlu),Sqr(MGlu)) +
      AbsSqr(CpVGGluGluPR(1,1))*H0(Sqr(p),Sqr(MGlu),Sqr(MGlu)) + 4*B0(Sqr(p),Sqr(
      MGlu),Sqr(MGlu))*Re(Conj(CpVGGluGluPL(1,1))*CpVGGluGluPR(1,1))*Sqr(MGlu));
   result += 999*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpVGVGconjSbSb(gI1,gI1));
   result += 999*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpVGVGconjScSc(gI1,gI1));
   result += 999*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpVGVGconjSdSd(gI1,gI1));
   result += 999*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpVGVGconjSsSs(gI1,gI1));
   result += 999*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpVGVGconjStSt(gI1,gI1));
   result += 999*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpVGVGconjSuSu(gI1,gI1));
   result += -2*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVGconjSbSb(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSb(gI1)),Sqr(MSb(gI2)))));
   result += -2*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVGconjScSc(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSc(gI1)),Sqr(MSc(gI2)))));
   result += -2*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVGconjSdSd(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSd(gI1)),Sqr(MSd(gI2)))));
   result += -2*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVGconjSsSs(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSs(gI1)),Sqr(MSs(gI2)))));
   result += -2*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVGconjStSt(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSt(gI1)),Sqr(MSt(gI2)))));
   result += -2*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVGconjSuSu(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSu(gI1)),Sqr(MSu(gI2)))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_VP_1loop(double p ) const
{
   std::complex<double> result;

   result += AbsSqr(CpVPbargWmCgWmC())*B00(Sqr(p),Sqr(MVWm),Sqr(MVWm));
   result += AbsSqr(CpVPbargWmgWm())*B00(Sqr(p),Sqr(MVWm),Sqr(MVWm));
   result += -(A0(Sqr(MVWm))*(4*CpVPVPconjVWmVWm1() + CpVPVPconjVWmVWm2() +
      CpVPVPconjVWmVWm3()));
   result += (AbsSqr(CpVPbarFeFePL()) + AbsSqr(CpVPbarFeFePR()))*H0(Sqr(p),Sqr(
      MFe),Sqr(MFe));
   result += (AbsSqr(CpVPbarFmFmPL()) + AbsSqr(CpVPbarFmFmPR()))*H0(Sqr(p),Sqr(
      MFm),Sqr(MFm));
   result += (AbsSqr(CpVPbarFtauFtauPL()) + AbsSqr(CpVPbarFtauFtauPR()))*H0(Sqr
      (p),Sqr(MFtau),Sqr(MFtau));
   result += -2*AbsSqr(CpVPconjVWmVWm())*(A0(Sqr(MVWm)) + 5*B00(Sqr(p),Sqr(MVWm
      ),Sqr(MVWm)) + B0(Sqr(p),Sqr(MVWm),Sqr(MVWm))*(Sqr(MVWm) + 2*Sqr(p)));
   result += 3*(AbsSqr(CpVPbarFbFbPL(1,1))*H0(Sqr(p),Sqr(MFb),Sqr(MFb)) +
      AbsSqr(CpVPbarFbFbPR(1,1))*H0(Sqr(p),Sqr(MFb),Sqr(MFb)) + 4*B0(Sqr(p),Sqr(
      MFb),Sqr(MFb))*Re(Conj(CpVPbarFbFbPL(1,1))*CpVPbarFbFbPR(1,1))*Sqr(MFb));
   result += 3*(AbsSqr(CpVPbarFcFcPL(1,1))*H0(Sqr(p),Sqr(MFc),Sqr(MFc)) +
      AbsSqr(CpVPbarFcFcPR(1,1))*H0(Sqr(p),Sqr(MFc),Sqr(MFc)) + 4*B0(Sqr(p),Sqr(
      MFc),Sqr(MFc))*Re(Conj(CpVPbarFcFcPL(1,1))*CpVPbarFcFcPR(1,1))*Sqr(MFc));
   result += 3*(AbsSqr(CpVPbarFdFdPL(1,1))*H0(Sqr(p),Sqr(MFd),Sqr(MFd)) +
      AbsSqr(CpVPbarFdFdPR(1,1))*H0(Sqr(p),Sqr(MFd),Sqr(MFd)) + 4*B0(Sqr(p),Sqr(
      MFd),Sqr(MFd))*Re(Conj(CpVPbarFdFdPL(1,1))*CpVPbarFdFdPR(1,1))*Sqr(MFd));
   result += 4*B0(Sqr(p),Sqr(MFe),Sqr(MFe))*Re(Conj(CpVPbarFeFePL())*
      CpVPbarFeFePR())*Sqr(MFe);
   result += 4*B0(Sqr(p),Sqr(MFm),Sqr(MFm))*Re(Conj(CpVPbarFmFmPL())*
      CpVPbarFmFmPR())*Sqr(MFm);
   result += 3*(AbsSqr(CpVPbarFsFsPL(1,1))*H0(Sqr(p),Sqr(MFs),Sqr(MFs)) +
      AbsSqr(CpVPbarFsFsPR(1,1))*H0(Sqr(p),Sqr(MFs),Sqr(MFs)) + 4*B0(Sqr(p),Sqr(
      MFs),Sqr(MFs))*Re(Conj(CpVPbarFsFsPL(1,1))*CpVPbarFsFsPR(1,1))*Sqr(MFs));
   result += 4*B0(Sqr(p),Sqr(MFtau),Sqr(MFtau))*Re(Conj(CpVPbarFtauFtauPL())*
      CpVPbarFtauFtauPR())*Sqr(MFtau);
   result += 3*(AbsSqr(CpVPbarFtFtPL(1,1))*H0(Sqr(p),Sqr(MFt),Sqr(MFt)) +
      AbsSqr(CpVPbarFtFtPR(1,1))*H0(Sqr(p),Sqr(MFt),Sqr(MFt)) + 4*B0(Sqr(p),Sqr(
      MFt),Sqr(MFt))*Re(Conj(CpVPbarFtFtPL(1,1))*CpVPbarFtFtPR(1,1))*Sqr(MFt));
   result += 3*(AbsSqr(CpVPbarFuFuPL(1,1))*H0(Sqr(p),Sqr(MFu),Sqr(MFu)) +
      AbsSqr(CpVPbarFuFuPR(1,1))*H0(Sqr(p),Sqr(MFu),Sqr(MFu)) + 4*B0(Sqr(p),Sqr(
      MFu),Sqr(MFu))*Re(Conj(CpVPbarFuFuPL(1,1))*CpVPbarFuFuPR(1,1))*Sqr(MFu));
   result += SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpVPVPconjHpmHpm(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpVPVPconjSbSb(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpVPVPconjScSc(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpVPVPconjSdSd(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpVPVPconjSeSe(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpVPVPconjSmSm(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpVPVPconjSsSs(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpVPVPconjStauStau(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpVPVPconjStSt(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpVPVPconjSuSu(gI1,gI1));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVPconjHpmHpm(gI1,gI2))*B00(Sqr
      (p),Sqr(MHpm(gI1)),Sqr(MHpm(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVPconjSbSb(gI1,gI2))*B00(Sqr(
      p),Sqr(MSb(gI1)),Sqr(MSb(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVPconjScSc(gI1,gI2))*B00(Sqr(
      p),Sqr(MSc(gI1)),Sqr(MSc(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVPconjSdSd(gI1,gI2))*B00(Sqr(
      p),Sqr(MSd(gI1)),Sqr(MSd(gI2)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVPconjSeSe(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSe(gI1)),Sqr(MSe(gI2)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVPconjSmSm(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSm(gI1)),Sqr(MSm(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVPconjSsSs(gI1,gI2))*B00(Sqr(
      p),Sqr(MSs(gI1)),Sqr(MSs(gI2)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVPconjStauStau(gI1,gI2))*B00(
      Sqr(p),Sqr(MStau(gI1)),Sqr(MStau(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVPconjStSt(gI1,gI2))*B00(Sqr(
      p),Sqr(MSt(gI1)),Sqr(MSt(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVPconjSuSu(gI1,gI2))*B00(Sqr(
      p),Sqr(MSu(gI1)),Sqr(MSu(gI2)))));
   result += SUM(gI1,0,1,SUM(gI2,0,1,(AbsSqr(CpVPbarChaChaPL(gI1,gI2)) + AbsSqr
      (CpVPbarChaChaPR(gI1,gI2)))*H0(Sqr(p),Sqr(MCha(gI1)),Sqr(MCha(gI2))) + 4*B0(
      Sqr(p),Sqr(MCha(gI1)),Sqr(MCha(gI2)))*MCha(gI1)*MCha(gI2)*Re(Conj(
      CpVPbarChaChaPL(gI1,gI2))*CpVPbarChaChaPR(gI1,gI2))));
   result += 2*SUM(gI2,0,1,AbsSqr(CpVPconjVWmHpm(gI2))*B0(Sqr(p),Sqr(MVWm),Sqr(
      MHpm(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_VZ_1loop(double p ) const
{
   std::complex<double> result;

   result += AbsSqr(CpVZbargWmCgWmC())*B00(Sqr(p),Sqr(MVWm),Sqr(MVWm));
   result += AbsSqr(CpVZbargWmgWm())*B00(Sqr(p),Sqr(MVWm),Sqr(MVWm));
   result += -4*AbsSqr(CpVZconjSveLSveL())*B00(Sqr(p),Sqr(MSveL),Sqr(MSveL));
   result += -4*AbsSqr(CpVZconjSvmLSvmL())*B00(Sqr(p),Sqr(MSvmL),Sqr(MSvmL));
   result += -4*AbsSqr(CpVZconjSvtLSvtL())*B00(Sqr(p),Sqr(MSvtL),Sqr(MSvtL));
   result += A0(Sqr(MSveL))*CpVZVZconjSveLSveL();
   result += A0(Sqr(MSvmL))*CpVZVZconjSvmLSvmL();
   result += A0(Sqr(MSvtL))*CpVZVZconjSvtLSvtL();
   result += -(A0(Sqr(MVWm))*(4*CpVZVZconjVWmVWm1() + CpVZVZconjVWmVWm2() +
      CpVZVZconjVWmVWm3()));
   result += (AbsSqr(CpVZbarFeFePL()) + AbsSqr(CpVZbarFeFePR()))*H0(Sqr(p),Sqr(
      MFe),Sqr(MFe));
   result += (AbsSqr(CpVZbarFmFmPL()) + AbsSqr(CpVZbarFmFmPR()))*H0(Sqr(p),Sqr(
      MFm),Sqr(MFm));
   result += (AbsSqr(CpVZbarFtauFtauPL()) + AbsSqr(CpVZbarFtauFtauPR()))*H0(Sqr
      (p),Sqr(MFtau),Sqr(MFtau));
   result += (AbsSqr(CpVZbarFveFvePL()) + AbsSqr(CpVZbarFveFvePR()))*H0(Sqr(p),
      Sqr(MFve),Sqr(MFve));
   result += (AbsSqr(CpVZbarFvmFvmPL()) + AbsSqr(CpVZbarFvmFvmPR()))*H0(Sqr(p),
      Sqr(MFvm),Sqr(MFvm));
   result += (AbsSqr(CpVZbarFvtFvtPL()) + AbsSqr(CpVZbarFvtFvtPR()))*H0(Sqr(p),
      Sqr(MFvt),Sqr(MFvt));
   result += -2*AbsSqr(CpVZconjVWmVWm())*(A0(Sqr(MVWm)) + 5*B00(Sqr(p),Sqr(MVWm
      ),Sqr(MVWm)) + B0(Sqr(p),Sqr(MVWm),Sqr(MVWm))*(Sqr(MVWm) + 2*Sqr(p)));
   result += 3*(AbsSqr(CpVZbarFbFbPL(1,1))*H0(Sqr(p),Sqr(MFb),Sqr(MFb)) +
      AbsSqr(CpVZbarFbFbPR(1,1))*H0(Sqr(p),Sqr(MFb),Sqr(MFb)) + 4*B0(Sqr(p),Sqr(
      MFb),Sqr(MFb))*Re(Conj(CpVZbarFbFbPL(1,1))*CpVZbarFbFbPR(1,1))*Sqr(MFb));
   result += 3*(AbsSqr(CpVZbarFcFcPL(1,1))*H0(Sqr(p),Sqr(MFc),Sqr(MFc)) +
      AbsSqr(CpVZbarFcFcPR(1,1))*H0(Sqr(p),Sqr(MFc),Sqr(MFc)) + 4*B0(Sqr(p),Sqr(
      MFc),Sqr(MFc))*Re(Conj(CpVZbarFcFcPL(1,1))*CpVZbarFcFcPR(1,1))*Sqr(MFc));
   result += 3*(AbsSqr(CpVZbarFdFdPL(1,1))*H0(Sqr(p),Sqr(MFd),Sqr(MFd)) +
      AbsSqr(CpVZbarFdFdPR(1,1))*H0(Sqr(p),Sqr(MFd),Sqr(MFd)) + 4*B0(Sqr(p),Sqr(
      MFd),Sqr(MFd))*Re(Conj(CpVZbarFdFdPL(1,1))*CpVZbarFdFdPR(1,1))*Sqr(MFd));
   result += 4*B0(Sqr(p),Sqr(MFe),Sqr(MFe))*Re(Conj(CpVZbarFeFePL())*
      CpVZbarFeFePR())*Sqr(MFe);
   result += 4*B0(Sqr(p),Sqr(MFm),Sqr(MFm))*Re(Conj(CpVZbarFmFmPL())*
      CpVZbarFmFmPR())*Sqr(MFm);
   result += 3*(AbsSqr(CpVZbarFsFsPL(1,1))*H0(Sqr(p),Sqr(MFs),Sqr(MFs)) +
      AbsSqr(CpVZbarFsFsPR(1,1))*H0(Sqr(p),Sqr(MFs),Sqr(MFs)) + 4*B0(Sqr(p),Sqr(
      MFs),Sqr(MFs))*Re(Conj(CpVZbarFsFsPL(1,1))*CpVZbarFsFsPR(1,1))*Sqr(MFs));
   result += 4*B0(Sqr(p),Sqr(MFtau),Sqr(MFtau))*Re(Conj(CpVZbarFtauFtauPL())*
      CpVZbarFtauFtauPR())*Sqr(MFtau);
   result += 3*(AbsSqr(CpVZbarFtFtPL(1,1))*H0(Sqr(p),Sqr(MFt),Sqr(MFt)) +
      AbsSqr(CpVZbarFtFtPR(1,1))*H0(Sqr(p),Sqr(MFt),Sqr(MFt)) + 4*B0(Sqr(p),Sqr(
      MFt),Sqr(MFt))*Re(Conj(CpVZbarFtFtPL(1,1))*CpVZbarFtFtPR(1,1))*Sqr(MFt));
   result += 3*(AbsSqr(CpVZbarFuFuPL(1,1))*H0(Sqr(p),Sqr(MFu),Sqr(MFu)) +
      AbsSqr(CpVZbarFuFuPR(1,1))*H0(Sqr(p),Sqr(MFu),Sqr(MFu)) + 4*B0(Sqr(p),Sqr(
      MFu),Sqr(MFu))*Re(Conj(CpVZbarFuFuPL(1,1))*CpVZbarFuFuPR(1,1))*Sqr(MFu));
   result += 4*B0(Sqr(p),Sqr(MFve),Sqr(MFve))*Re(Conj(CpVZbarFveFvePL())*
      CpVZbarFveFvePR())*Sqr(MFve);
   result += 4*B0(Sqr(p),Sqr(MFvm),Sqr(MFvm))*Re(Conj(CpVZbarFvmFvmPL())*
      CpVZbarFvmFvmPR())*Sqr(MFvm);
   result += 4*B0(Sqr(p),Sqr(MFvt),Sqr(MFvt))*Re(Conj(CpVZbarFvtFvtPL())*
      CpVZbarFvtFvtPR())*Sqr(MFvt);
   result += 0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpVZVZAhAh(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpVZVZconjHpmHpm(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpVZVZconjSbSb(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpVZVZconjScSc(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpVZVZconjSdSd(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpVZVZconjSeSe(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpVZVZconjSmSm(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpVZVZconjSsSs(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpVZVZconjStauStau(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpVZVZconjStSt(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpVZVZconjSuSu(gI1,gI1));
   result += 0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpVZVZhhhh(gI1,gI1));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjHpmHpm(gI1,gI2))*B00(Sqr
      (p),Sqr(MHpm(gI1)),Sqr(MHpm(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSbSb(gI1,gI2))*B00(Sqr(
      p),Sqr(MSb(gI1)),Sqr(MSb(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjScSc(gI1,gI2))*B00(Sqr(
      p),Sqr(MSc(gI1)),Sqr(MSc(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSdSd(gI1,gI2))*B00(Sqr(
      p),Sqr(MSd(gI1)),Sqr(MSd(gI2)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSeSe(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSe(gI1)),Sqr(MSe(gI2)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSmSm(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSm(gI1)),Sqr(MSm(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSsSs(gI1,gI2))*B00(Sqr(
      p),Sqr(MSs(gI1)),Sqr(MSs(gI2)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjStauStau(gI1,gI2))*B00(
      Sqr(p),Sqr(MStau(gI1)),Sqr(MStau(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjStSt(gI1,gI2))*B00(Sqr(
      p),Sqr(MSt(gI1)),Sqr(MSt(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSuSu(gI1,gI2))*B00(Sqr(
      p),Sqr(MSu(gI1)),Sqr(MSu(gI2)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZhhAh(gI1,gI2))*B00(Sqr(p),
      Sqr(MAh(gI2)),Sqr(Mhh(gI1)))));
   result += SUM(gI1,0,1,SUM(gI2,0,1,(AbsSqr(CpVZbarChaChaPL(gI1,gI2)) + AbsSqr
      (CpVZbarChaChaPR(gI1,gI2)))*H0(Sqr(p),Sqr(MCha(gI1)),Sqr(MCha(gI2))) + 4*B0(
      Sqr(p),Sqr(MCha(gI1)),Sqr(MCha(gI2)))*MCha(gI1)*MCha(gI2)*Re(Conj(
      CpVZbarChaChaPL(gI1,gI2))*CpVZbarChaChaPR(gI1,gI2))));
   result += 0.5*SUM(gI1,0,3,SUM(gI2,0,3,(AbsSqr(CpVZChiChiPL(gI1,gI2)) +
      AbsSqr(CpVZChiChiPR(gI1,gI2)))*H0(Sqr(p),Sqr(MChi(gI1)),Sqr(MChi(gI2))) + 4*
      B0(Sqr(p),Sqr(MChi(gI1)),Sqr(MChi(gI2)))*MChi(gI1)*MChi(gI2)*Re(Conj(
      CpVZChiChiPL(gI1,gI2))*CpVZChiChiPR(gI1,gI2))));
   result += 2*SUM(gI2,0,1,AbsSqr(CpVZconjVWmHpm(gI2))*B0(Sqr(p),Sqr(MVWm),Sqr(
      MHpm(gI2))));
   result += SUM(gI2,0,1,AbsSqr(CpVZVZhh(gI2))*B0(Sqr(p),Sqr(MVZ),Sqr(Mhh(gI2))
      ));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_VWm_1loop(double p ) const
{
   std::complex<double> result;

   result += AbsSqr(CpconjVWmbargPgWm())*B00(Sqr(p),Sqr(MVWm),Sqr(MVP));
   result += AbsSqr(CpconjVWmbargWmCgP())*B00(Sqr(p),Sqr(MVP),Sqr(MVWm));
   result += AbsSqr(CpconjVWmbargWmCgZ())*B00(Sqr(p),Sqr(MVZ),Sqr(MVWm));
   result += AbsSqr(CpconjVWmbargZgWm())*B00(Sqr(p),Sqr(MVWm),Sqr(MVZ));
   result += A0(Sqr(MSveL))*CpVWmconjVWmconjSveLSveL();
   result += A0(Sqr(MSvmL))*CpVWmconjVWmconjSvmLSvmL();
   result += A0(Sqr(MSvtL))*CpVWmconjVWmconjSvtLSvtL();
   result += -(A0(Sqr(MVWm))*(4*CpVWmconjVWmconjVWmVWm1() +
      CpVWmconjVWmconjVWmVWm2() + CpVWmconjVWmconjVWmVWm3()));
   result += 0;
   result += -0.5*A0(Sqr(MVZ))*(4*CpVWmconjVWmVZVZ1() + CpVWmconjVWmVZVZ2() +
      CpVWmconjVWmVZVZ3());
   result += (AbsSqr(CpconjVWmbarFveFePL()) + AbsSqr(CpconjVWmbarFveFePR()))*H0
      (Sqr(p),Sqr(MFve),Sqr(MFe));
   result += (AbsSqr(CpconjVWmbarFvmFmPL()) + AbsSqr(CpconjVWmbarFvmFmPR()))*H0
      (Sqr(p),Sqr(MFvm),Sqr(MFm));
   result += (AbsSqr(CpconjVWmbarFvtFtauPL()) + AbsSqr(CpconjVWmbarFvtFtauPR())
      )*H0(Sqr(p),Sqr(MFvt),Sqr(MFtau));
   result += -(AbsSqr(CpconjVWmVWmVP())*(A0(Sqr(MVWm)) + 10*B00(Sqr(p),Sqr(MVWm
      ),0) + B0(Sqr(p),Sqr(MVWm),0)*(Sqr(MVWm) + 4*Sqr(p))));
   result += -(AbsSqr(CpconjVWmVZVWm())*(A0(Sqr(MVWm)) + A0(Sqr(MVZ)) + 10*B00(
      Sqr(p),Sqr(MVZ),Sqr(MVWm)) + B0(Sqr(p),Sqr(MVZ),Sqr(MVWm))*Sqr(MVWm) + B0(
      Sqr(p),Sqr(MVZ),Sqr(MVWm))*Sqr(MVZ) + 4*B0(Sqr(p),Sqr(MVZ),Sqr(MVWm))*Sqr(p)
      ));
   result += 3*(AbsSqr(CpconjVWmbarFcFsPL(1,1))*H0(Sqr(p),Sqr(MFc),Sqr(MFs)) +
      AbsSqr(CpconjVWmbarFcFsPR(1,1))*H0(Sqr(p),Sqr(MFc),Sqr(MFs)) + 4*MFc*MFs*B0(
      Sqr(p),Sqr(MFc),Sqr(MFs))*Re(Conj(CpconjVWmbarFcFsPL(1,1))*
      CpconjVWmbarFcFsPR(1,1)));
   result += 3*(AbsSqr(CpconjVWmbarFtFbPL(1,1))*H0(Sqr(p),Sqr(MFt),Sqr(MFb)) +
      AbsSqr(CpconjVWmbarFtFbPR(1,1))*H0(Sqr(p),Sqr(MFt),Sqr(MFb)) + 4*MFb*MFt*B0(
      Sqr(p),Sqr(MFt),Sqr(MFb))*Re(Conj(CpconjVWmbarFtFbPL(1,1))*
      CpconjVWmbarFtFbPR(1,1)));
   result += 3*(AbsSqr(CpconjVWmbarFuFdPL(1,1))*H0(Sqr(p),Sqr(MFu),Sqr(MFd)) +
      AbsSqr(CpconjVWmbarFuFdPR(1,1))*H0(Sqr(p),Sqr(MFu),Sqr(MFd)) + 4*MFd*MFu*B0(
      Sqr(p),Sqr(MFu),Sqr(MFd))*Re(Conj(CpconjVWmbarFuFdPL(1,1))*
      CpconjVWmbarFuFdPR(1,1)));
   result += 4*MFe*MFve*B0(Sqr(p),Sqr(MFve),Sqr(MFe))*Re(Conj(
      CpconjVWmbarFveFePL())*CpconjVWmbarFveFePR());
   result += 4*MFm*MFvm*B0(Sqr(p),Sqr(MFvm),Sqr(MFm))*Re(Conj(
      CpconjVWmbarFvmFmPL())*CpconjVWmbarFvmFmPR());
   result += 4*MFtau*MFvt*B0(Sqr(p),Sqr(MFvt),Sqr(MFtau))*Re(Conj(
      CpconjVWmbarFvtFtauPL())*CpconjVWmbarFvtFtauPR());
   result += 0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpVWmconjVWmAhAh(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpVWmconjVWmconjHpmHpm(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpVWmconjVWmconjSbSb(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpVWmconjVWmconjScSc(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpVWmconjVWmconjSdSd(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpVWmconjVWmconjSeSe(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpVWmconjVWmconjSmSm(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpVWmconjVWmconjSsSs(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpVWmconjVWmconjStauStau(gI1,gI1))
      ;
   result += 3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpVWmconjVWmconjStSt(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpVWmconjVWmconjSuSu(gI1,gI1));
   result += 0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpVWmconjVWmhhhh(gI1,gI1));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjVWmconjScSs(gI1,gI2))*B00
      (Sqr(p),Sqr(MSs(gI2)),Sqr(MSc(gI1)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjVWmconjStSb(gI1,gI2))*B00
      (Sqr(p),Sqr(MSb(gI2)),Sqr(MSt(gI1)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjVWmconjSuSd(gI1,gI2))*B00
      (Sqr(p),Sqr(MSd(gI2)),Sqr(MSu(gI1)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjVWmHpmAh(gI1,gI2))*B00(Sqr
      (p),Sqr(MAh(gI2)),Sqr(MHpm(gI1)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjVWmHpmhh(gI1,gI2))*B00(Sqr
      (p),Sqr(Mhh(gI2)),Sqr(MHpm(gI1)))));
   result += SUM(gI1,0,3,SUM(gI2,0,1,(AbsSqr(CpconjVWmChiChaPL(gI1,gI2)) +
      AbsSqr(CpconjVWmChiChaPR(gI1,gI2)))*H0(Sqr(p),Sqr(MChi(gI1)),Sqr(MCha(gI2)))
      + 4*B0(Sqr(p),Sqr(MChi(gI1)),Sqr(MCha(gI2)))*MCha(gI2)*MChi(gI1)*Re(Conj(
      CpconjVWmChiChaPL(gI1,gI2))*CpconjVWmChiChaPR(gI1,gI2))));
   result += -4*SUM(gI2,0,1,AbsSqr(CpconjVWmconjSveLSe(gI2))*B00(Sqr(p),Sqr(MSe
      (gI2)),Sqr(MSveL)));
   result += -4*SUM(gI2,0,1,AbsSqr(CpconjVWmconjSvmLSm(gI2))*B00(Sqr(p),Sqr(MSm
      (gI2)),Sqr(MSvmL)));
   result += -4*SUM(gI2,0,1,AbsSqr(CpconjVWmconjSvtLStau(gI2))*B00(Sqr(p),Sqr(
      MStau(gI2)),Sqr(MSvtL)));
   result += SUM(gI2,0,1,AbsSqr(CpconjVWmVPHpm(gI2))*B0(Sqr(p),0,Sqr(MHpm(gI2))
      ));
   result += SUM(gI2,0,1,AbsSqr(CpconjVWmVWmhh(gI2))*B0(Sqr(p),Sqr(MVWm),Sqr(
      Mhh(gI2))));
   result += SUM(gI2,0,1,AbsSqr(CpconjVWmVZHpm(gI2))*B0(Sqr(p),Sqr(MVZ),Sqr(
      MHpm(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Chi_1loop_1(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += MFve*B0(Sqr(p),Sqr(MFve),Sqr(MSveL))*Conj(CpUChibarFveSveLPL(gO2))
      *CpUChibarFveSveLPR(gO1);
   result += MFve*B0(Sqr(p),Sqr(MFve),Sqr(MSveL))*Conj(CpUChiconjSveLFvePL(gO2)
      )*CpUChiconjSveLFvePR(gO1);
   result += MFvm*B0(Sqr(p),Sqr(MFvm),Sqr(MSvmL))*Conj(CpUChibarFvmSvmLPL(gO2))
      *CpUChibarFvmSvmLPR(gO1);
   result += MFvm*B0(Sqr(p),Sqr(MFvm),Sqr(MSvmL))*Conj(CpUChiconjSvmLFvmPL(gO2)
      )*CpUChiconjSvmLFvmPR(gO1);
   result += MFvt*B0(Sqr(p),Sqr(MFvt),Sqr(MSvtL))*Conj(CpUChibarFvtSvtLPL(gO2))
      *CpUChibarFvtSvtLPR(gO1);
   result += MFvt*B0(Sqr(p),Sqr(MFvt),Sqr(MSvtL))*Conj(CpUChiconjSvtLFvtPL(gO2)
      )*CpUChiconjSvtLFvtPR(gO1);
   result += 3*MFb*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFb),Sqr(MSb(gI1)))*Conj(
      CpUChiconjSbFbPL(gO2,gI1,1))*CpUChiconjSbFbPR(gO1,gI1,1));
   result += 3*MFc*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFc),Sqr(MSc(gI1)))*Conj(
      CpUChiconjScFcPL(gO2,gI1,1))*CpUChiconjScFcPR(gO1,gI1,1));
   result += 3*MFd*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFd),Sqr(MSd(gI1)))*Conj(
      CpUChiconjSdFdPL(gO2,gI1,1))*CpUChiconjSdFdPR(gO1,gI1,1));
   result += MFe*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFe),Sqr(MSe(gI1)))*Conj(
      CpUChiconjSeFePL(gO2,gI1))*CpUChiconjSeFePR(gO1,gI1));
   result += MFm*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFm),Sqr(MSm(gI1)))*Conj(
      CpUChiconjSmFmPL(gO2,gI1))*CpUChiconjSmFmPR(gO1,gI1));
   result += 3*MFs*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFs),Sqr(MSs(gI1)))*Conj(
      CpUChiconjSsFsPL(gO2,gI1,1))*CpUChiconjSsFsPR(gO1,gI1,1));
   result += MFtau*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFtau),Sqr(MStau(gI1)))*Conj(
      CpUChiconjStauFtauPL(gO2,gI1))*CpUChiconjStauFtauPR(gO1,gI1));
   result += 3*MFt*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MSt(gI1)))*Conj(
      CpUChiconjStFtPL(gO2,gI1,1))*CpUChiconjStFtPR(gO1,gI1,1));
   result += 3*MFu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFu),Sqr(MSu(gI1)))*Conj(
      CpUChiconjSuFuPL(gO2,gI1,1))*CpUChiconjSuFuPR(gO1,gI1,1));
   result += -4*SUM(gI1,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MVWm))*Conj(
      CpUChibarChaVWmPR(gO2,gI1))*CpUChibarChaVWmPL(gO1,gI1)*MCha(gI1));
   result += SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(
      MHpm(gI2)))*Conj(CpUChibarChaHpmPL(gO2,gI1,gI2))*CpUChibarChaHpmPR(gO1,gI1,
      gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MHpm(gI1)))*
      Conj(CpUChiconjHpmChaPL(gO2,gI1,gI2))*CpUChiconjHpmChaPR(gO1,gI1,gI2)*MCha(
      gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(Mhh(gI1)))*
      Conj(CpUChihhChiPL(gO2,gI1,gI2))*CpUChihhChiPR(gO1,gI1,gI2)*MChi(gI2)));
   result += SUM(gI1,0,3,MChi(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MChi(gI1)),Sqr(MAh
      (gI2)))*Conj(CpUChiChiAhPL(gO2,gI1,gI2))*CpUChiChiAhPR(gO1,gI1,gI2)));
   result += 3*MFb*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFb),Sqr(MSb(gI2)))*Conj(
      CpUChibarFbSbPL(gO2,1,gI2))*CpUChibarFbSbPR(gO1,1,gI2));
   result += 3*MFc*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFc),Sqr(MSc(gI2)))*Conj(
      CpUChibarFcScPL(gO2,1,gI2))*CpUChibarFcScPR(gO1,1,gI2));
   result += 3*MFd*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFd),Sqr(MSd(gI2)))*Conj(
      CpUChibarFdSdPL(gO2,1,gI2))*CpUChibarFdSdPR(gO1,1,gI2));
   result += MFe*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFe),Sqr(MSe(gI2)))*Conj(
      CpUChibarFeSePL(gO2,gI2))*CpUChibarFeSePR(gO1,gI2));
   result += MFm*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFm),Sqr(MSm(gI2)))*Conj(
      CpUChibarFmSmPL(gO2,gI2))*CpUChibarFmSmPR(gO1,gI2));
   result += 3*MFs*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFs),Sqr(MSs(gI2)))*Conj(
      CpUChibarFsSsPL(gO2,1,gI2))*CpUChibarFsSsPR(gO1,1,gI2));
   result += MFtau*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFtau),Sqr(MStau(gI2)))*Conj(
      CpUChibarFtauStauPL(gO2,gI2))*CpUChibarFtauStauPR(gO1,gI2));
   result += 3*MFt*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MSt(gI2)))*Conj(
      CpUChibarFtStPL(gO2,1,gI2))*CpUChibarFtStPR(gO1,1,gI2));
   result += 3*MFu*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFu),Sqr(MSu(gI2)))*Conj(
      CpUChibarFuSuPL(gO2,1,gI2))*CpUChibarFuSuPR(gO1,1,gI2));
   result += -4*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MVWm))*Conj(
      CpUChiconjVWmChaPR(gO2,gI2))*CpUChiconjVWmChaPL(gO1,gI2)*MCha(gI2));
   result += -4*SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MVZ))*Conj(
      CpUChiVZChiPR(gO2,gI2))*CpUChiVZChiPL(gO1,gI2)*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,4,4> CLASSNAME::self_energy_Chi_1loop_1(double p) const
{
   Eigen::Matrix<std::complex<double>,4,4> self_energy;

   for (int i = 0; i < 4; i++)
      for (int k = 0; k < 4; k++)
         self_energy(i, k) = self_energy_Chi_1loop_1(p, i, k);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Chi_1loop_PR(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -0.5*B1(Sqr(p),Sqr(MFve),Sqr(MSveL))*Conj(CpUChibarFveSveLPR(gO2))
      *CpUChibarFveSveLPR(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFvm),Sqr(MSvmL))*Conj(CpUChibarFvmSvmLPR(gO2))
      *CpUChibarFvmSvmLPR(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFvt),Sqr(MSvtL))*Conj(CpUChibarFvtSvtLPR(gO2))
      *CpUChibarFvtSvtLPR(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFve),Sqr(MSveL))*Conj(CpUChiconjSveLFvePR(gO2)
      )*CpUChiconjSveLFvePR(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFvm),Sqr(MSvmL))*Conj(CpUChiconjSvmLFvmPR(gO2)
      )*CpUChiconjSvmLFvmPR(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFvt),Sqr(MSvtL))*Conj(CpUChiconjSvtLFvtPR(gO2)
      )*CpUChiconjSvtLFvtPR(gO1);
   result += -SUM(gI1,0,1,B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MVWm))*Conj(
      CpUChibarChaVWmPL(gO2,gI1))*CpUChibarChaVWmPL(gO1,gI1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFb),Sqr(MSb(gI1)))*Conj(
      CpUChiconjSbFbPR(gO2,gI1,1))*CpUChiconjSbFbPR(gO1,gI1,1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFc),Sqr(MSc(gI1)))*Conj(
      CpUChiconjScFcPR(gO2,gI1,1))*CpUChiconjScFcPR(gO1,gI1,1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFd),Sqr(MSd(gI1)))*Conj(
      CpUChiconjSdFdPR(gO2,gI1,1))*CpUChiconjSdFdPR(gO1,gI1,1));
   result += -0.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFe),Sqr(MSe(gI1)))*Conj(
      CpUChiconjSeFePR(gO2,gI1))*CpUChiconjSeFePR(gO1,gI1));
   result += -0.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFm),Sqr(MSm(gI1)))*Conj(
      CpUChiconjSmFmPR(gO2,gI1))*CpUChiconjSmFmPR(gO1,gI1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFs),Sqr(MSs(gI1)))*Conj(
      CpUChiconjSsFsPR(gO2,gI1,1))*CpUChiconjSsFsPR(gO1,gI1,1));
   result += -0.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFtau),Sqr(MStau(gI1)))*Conj(
      CpUChiconjStauFtauPR(gO2,gI1))*CpUChiconjStauFtauPR(gO1,gI1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFt),Sqr(MSt(gI1)))*Conj(
      CpUChiconjStFtPR(gO2,gI1,1))*CpUChiconjStFtPR(gO1,gI1,1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFu),Sqr(MSu(gI1)))*Conj(
      CpUChiconjSuFuPR(gO2,gI1,1))*CpUChiconjSuFuPR(gO1,gI1,1));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MHpm(gI2
      )))*Conj(CpUChibarChaHpmPR(gO2,gI1,gI2))*CpUChibarChaHpmPR(gO1,gI1,gI2)));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI2)),Sqr(MHpm(gI1
      )))*Conj(CpUChiconjHpmChaPR(gO2,gI1,gI2))*CpUChiconjHpmChaPR(gO1,gI1,gI2)));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,B1(Sqr(p),Sqr(MChi(gI2)),Sqr(Mhh(gI1)
      ))*Conj(CpUChihhChiPR(gO2,gI1,gI2))*CpUChihhChiPR(gO1,gI1,gI2)));
   result += -0.5*SUM(gI1,0,3,SUM(gI2,0,1,B1(Sqr(p),Sqr(MChi(gI1)),Sqr(MAh(gI2)
      ))*Conj(CpUChiChiAhPR(gO2,gI1,gI2))*CpUChiChiAhPR(gO1,gI1,gI2)));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFb),Sqr(MSb(gI2)))*Conj(
      CpUChibarFbSbPR(gO2,1,gI2))*CpUChibarFbSbPR(gO1,1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFc),Sqr(MSc(gI2)))*Conj(
      CpUChibarFcScPR(gO2,1,gI2))*CpUChibarFcScPR(gO1,1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFd),Sqr(MSd(gI2)))*Conj(
      CpUChibarFdSdPR(gO2,1,gI2))*CpUChibarFdSdPR(gO1,1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFe),Sqr(MSe(gI2)))*Conj(
      CpUChibarFeSePR(gO2,gI2))*CpUChibarFeSePR(gO1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFm),Sqr(MSm(gI2)))*Conj(
      CpUChibarFmSmPR(gO2,gI2))*CpUChibarFmSmPR(gO1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFs),Sqr(MSs(gI2)))*Conj(
      CpUChibarFsSsPR(gO2,1,gI2))*CpUChibarFsSsPR(gO1,1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFtau),Sqr(MStau(gI2)))*Conj(
      CpUChibarFtauStauPR(gO2,gI2))*CpUChibarFtauStauPR(gO1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFt),Sqr(MSt(gI2)))*Conj(
      CpUChibarFtStPR(gO2,1,gI2))*CpUChibarFtStPR(gO1,1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFu),Sqr(MSu(gI2)))*Conj(
      CpUChibarFuSuPR(gO2,1,gI2))*CpUChibarFuSuPR(gO1,1,gI2));
   result += -SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI2)),Sqr(MVWm))*Conj(
      CpUChiconjVWmChaPL(gO2,gI2))*CpUChiconjVWmChaPL(gO1,gI2));
   result += -SUM(gI2,0,3,B1(Sqr(p),Sqr(MChi(gI2)),Sqr(MVZ))*Conj(CpUChiVZChiPL
      (gO2,gI2))*CpUChiVZChiPL(gO1,gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,4,4> CLASSNAME::self_energy_Chi_1loop_PR(double p) const
{
   Eigen::Matrix<std::complex<double>,4,4> self_energy;

   for (int i = 0; i < 4; i++)
      for (int k = 0; k < 4; k++)
         self_energy(i, k) = self_energy_Chi_1loop_PR(p, i, k);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Chi_1loop_PL(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -0.5*B1(Sqr(p),Sqr(MFve),Sqr(MSveL))*Conj(CpUChibarFveSveLPL(gO2))
      *CpUChibarFveSveLPL(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFvm),Sqr(MSvmL))*Conj(CpUChibarFvmSvmLPL(gO2))
      *CpUChibarFvmSvmLPL(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFvt),Sqr(MSvtL))*Conj(CpUChibarFvtSvtLPL(gO2))
      *CpUChibarFvtSvtLPL(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFve),Sqr(MSveL))*Conj(CpUChiconjSveLFvePL(gO2)
      )*CpUChiconjSveLFvePL(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFvm),Sqr(MSvmL))*Conj(CpUChiconjSvmLFvmPL(gO2)
      )*CpUChiconjSvmLFvmPL(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFvt),Sqr(MSvtL))*Conj(CpUChiconjSvtLFvtPL(gO2)
      )*CpUChiconjSvtLFvtPL(gO1);
   result += -SUM(gI1,0,1,B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MVWm))*Conj(
      CpUChibarChaVWmPR(gO2,gI1))*CpUChibarChaVWmPR(gO1,gI1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFb),Sqr(MSb(gI1)))*Conj(
      CpUChiconjSbFbPL(gO2,gI1,1))*CpUChiconjSbFbPL(gO1,gI1,1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFc),Sqr(MSc(gI1)))*Conj(
      CpUChiconjScFcPL(gO2,gI1,1))*CpUChiconjScFcPL(gO1,gI1,1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFd),Sqr(MSd(gI1)))*Conj(
      CpUChiconjSdFdPL(gO2,gI1,1))*CpUChiconjSdFdPL(gO1,gI1,1));
   result += -0.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFe),Sqr(MSe(gI1)))*Conj(
      CpUChiconjSeFePL(gO2,gI1))*CpUChiconjSeFePL(gO1,gI1));
   result += -0.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFm),Sqr(MSm(gI1)))*Conj(
      CpUChiconjSmFmPL(gO2,gI1))*CpUChiconjSmFmPL(gO1,gI1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFs),Sqr(MSs(gI1)))*Conj(
      CpUChiconjSsFsPL(gO2,gI1,1))*CpUChiconjSsFsPL(gO1,gI1,1));
   result += -0.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFtau),Sqr(MStau(gI1)))*Conj(
      CpUChiconjStauFtauPL(gO2,gI1))*CpUChiconjStauFtauPL(gO1,gI1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFt),Sqr(MSt(gI1)))*Conj(
      CpUChiconjStFtPL(gO2,gI1,1))*CpUChiconjStFtPL(gO1,gI1,1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFu),Sqr(MSu(gI1)))*Conj(
      CpUChiconjSuFuPL(gO2,gI1,1))*CpUChiconjSuFuPL(gO1,gI1,1));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MHpm(gI2
      )))*Conj(CpUChibarChaHpmPL(gO2,gI1,gI2))*CpUChibarChaHpmPL(gO1,gI1,gI2)));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI2)),Sqr(MHpm(gI1
      )))*Conj(CpUChiconjHpmChaPL(gO2,gI1,gI2))*CpUChiconjHpmChaPL(gO1,gI1,gI2)));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,B1(Sqr(p),Sqr(MChi(gI2)),Sqr(Mhh(gI1)
      ))*Conj(CpUChihhChiPL(gO2,gI1,gI2))*CpUChihhChiPL(gO1,gI1,gI2)));
   result += -0.5*SUM(gI1,0,3,SUM(gI2,0,1,B1(Sqr(p),Sqr(MChi(gI1)),Sqr(MAh(gI2)
      ))*Conj(CpUChiChiAhPL(gO2,gI1,gI2))*CpUChiChiAhPL(gO1,gI1,gI2)));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFb),Sqr(MSb(gI2)))*Conj(
      CpUChibarFbSbPL(gO2,1,gI2))*CpUChibarFbSbPL(gO1,1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFc),Sqr(MSc(gI2)))*Conj(
      CpUChibarFcScPL(gO2,1,gI2))*CpUChibarFcScPL(gO1,1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFd),Sqr(MSd(gI2)))*Conj(
      CpUChibarFdSdPL(gO2,1,gI2))*CpUChibarFdSdPL(gO1,1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFe),Sqr(MSe(gI2)))*Conj(
      CpUChibarFeSePL(gO2,gI2))*CpUChibarFeSePL(gO1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFm),Sqr(MSm(gI2)))*Conj(
      CpUChibarFmSmPL(gO2,gI2))*CpUChibarFmSmPL(gO1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFs),Sqr(MSs(gI2)))*Conj(
      CpUChibarFsSsPL(gO2,1,gI2))*CpUChibarFsSsPL(gO1,1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFtau),Sqr(MStau(gI2)))*Conj(
      CpUChibarFtauStauPL(gO2,gI2))*CpUChibarFtauStauPL(gO1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFt),Sqr(MSt(gI2)))*Conj(
      CpUChibarFtStPL(gO2,1,gI2))*CpUChibarFtStPL(gO1,1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFu),Sqr(MSu(gI2)))*Conj(
      CpUChibarFuSuPL(gO2,1,gI2))*CpUChibarFuSuPL(gO1,1,gI2));
   result += -SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI2)),Sqr(MVWm))*Conj(
      CpUChiconjVWmChaPR(gO2,gI2))*CpUChiconjVWmChaPR(gO1,gI2));
   result += -SUM(gI2,0,3,B1(Sqr(p),Sqr(MChi(gI2)),Sqr(MVZ))*Conj(CpUChiVZChiPR
      (gO2,gI2))*CpUChiVZChiPR(gO1,gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,4,4> CLASSNAME::self_energy_Chi_1loop_PL(double p) const
{
   Eigen::Matrix<std::complex<double>,4,4> self_energy;

   for (int i = 0; i < 4; i++)
      for (int k = 0; k < 4; k++)
         self_energy(i, k) = self_energy_Chi_1loop_PL(p, i, k);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Cha_1loop_1(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += MFe*B0(Sqr(p),Sqr(MFe),Sqr(MSveL))*Conj(CpbarUChaconjSveLFePL(gO2)
      )*CpbarUChaconjSveLFePR(gO1);
   result += MFm*B0(Sqr(p),Sqr(MFm),Sqr(MSvmL))*Conj(CpbarUChaconjSvmLFmPL(gO2)
      )*CpbarUChaconjSvmLFmPR(gO1);
   result += MFtau*B0(Sqr(p),Sqr(MFtau),Sqr(MSvtL))*Conj(
      CpbarUChaconjSvtLFtauPL(gO2))*CpbarUChaconjSvtLFtauPR(gO1);
   result += 3*MFs*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFs),Sqr(MSc(gI1)))*Conj(
      CpbarUChaconjScFsPL(gO2,gI1,1))*CpbarUChaconjScFsPR(gO1,gI1,1));
   result += 3*MFb*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFb),Sqr(MSt(gI1)))*Conj(
      CpbarUChaconjStFbPL(gO2,gI1,1))*CpbarUChaconjStFbPR(gO1,gI1,1));
   result += 3*MFd*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFd),Sqr(MSu(gI1)))*Conj(
      CpbarUChaconjSuFdPL(gO2,gI1,1))*CpbarUChaconjSuFdPR(gO1,gI1,1));
   result += SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MAh
      (gI2)))*Conj(CpbarUChaChaAhPL(gO2,gI1,gI2))*CpbarUChaChaAhPR(gO1,gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(Mhh(gI1)))*
      Conj(CpbarUChahhChaPL(gO2,gI1,gI2))*CpbarUChahhChaPR(gO1,gI1,gI2)*MCha(gI2))
      );
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MHpm(gI1)))*
      Conj(CpbarUChaHpmChiPL(gO2,gI1,gI2))*CpbarUChaHpmChiPR(gO1,gI1,gI2)*MChi(gI2
      )));
   result += 3*MFc*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFc),Sqr(MSs(gI2)))*Conj(
      CpbarUChabarFcSsPL(gO2,1,gI2))*CpbarUChabarFcSsPR(gO1,1,gI2));
   result += 3*MFt*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MSb(gI2)))*Conj(
      CpbarUChabarFtSbPL(gO2,1,gI2))*CpbarUChabarFtSbPR(gO1,1,gI2));
   result += 3*MFu*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFu),Sqr(MSd(gI2)))*Conj(
      CpbarUChabarFuSdPL(gO2,1,gI2))*CpbarUChabarFuSdPR(gO1,1,gI2));
   result += MFve*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFve),Sqr(MSe(gI2)))*Conj(
      CpbarUChabarFveSePL(gO2,gI2))*CpbarUChabarFveSePR(gO1,gI2));
   result += MFvm*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFvm),Sqr(MSm(gI2)))*Conj(
      CpbarUChabarFvmSmPL(gO2,gI2))*CpbarUChabarFvmSmPR(gO1,gI2));
   result += MFvt*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFvt),Sqr(MStau(gI2)))*Conj(
      CpbarUChabarFvtStauPL(gO2,gI2))*CpbarUChabarFvtStauPR(gO1,gI2));
   result += -4*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),0)*Conj(CpbarUChaVPChaPR(
      gO2,gI2))*CpbarUChaVPChaPL(gO1,gI2)*MCha(gI2));
   result += -4*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MVZ))*Conj(
      CpbarUChaVZChaPR(gO2,gI2))*CpbarUChaVZChaPL(gO1,gI2)*MCha(gI2));
   result += -4*SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MVWm))*Conj(
      CpbarUChaVWmChiPR(gO2,gI2))*CpbarUChaVWmChiPL(gO1,gI2)*MChi(gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Cha_1loop_1(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = 0; k < 2; k++)
         self_energy(i, k) = self_energy_Cha_1loop_1(p, i, k);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Cha_1loop_PR(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -0.5*B1(Sqr(p),Sqr(MFe),Sqr(MSveL))*Conj(CpbarUChaconjSveLFePR(gO2
      ))*CpbarUChaconjSveLFePR(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFm),Sqr(MSvmL))*Conj(CpbarUChaconjSvmLFmPR(gO2
      ))*CpbarUChaconjSvmLFmPR(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFtau),Sqr(MSvtL))*Conj(CpbarUChaconjSvtLFtauPR
      (gO2))*CpbarUChaconjSvtLFtauPR(gO1);
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFs),Sqr(MSc(gI1)))*Conj(
      CpbarUChaconjScFsPR(gO2,gI1,1))*CpbarUChaconjScFsPR(gO1,gI1,1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFb),Sqr(MSt(gI1)))*Conj(
      CpbarUChaconjStFbPR(gO2,gI1,1))*CpbarUChaconjStFbPR(gO1,gI1,1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFd),Sqr(MSu(gI1)))*Conj(
      CpbarUChaconjSuFdPR(gO2,gI1,1))*CpbarUChaconjSuFdPR(gO1,gI1,1));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MAh(gI2)
      ))*Conj(CpbarUChaChaAhPR(gO2,gI1,gI2))*CpbarUChaChaAhPR(gO1,gI1,gI2)));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI2)),Sqr(Mhh(gI1)
      ))*Conj(CpbarUChahhChaPR(gO2,gI1,gI2))*CpbarUChahhChaPR(gO1,gI1,gI2)));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,B1(Sqr(p),Sqr(MChi(gI2)),Sqr(MHpm(gI1
      )))*Conj(CpbarUChaHpmChiPR(gO2,gI1,gI2))*CpbarUChaHpmChiPR(gO1,gI1,gI2)));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFc),Sqr(MSs(gI2)))*Conj(
      CpbarUChabarFcSsPR(gO2,1,gI2))*CpbarUChabarFcSsPR(gO1,1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFt),Sqr(MSb(gI2)))*Conj(
      CpbarUChabarFtSbPR(gO2,1,gI2))*CpbarUChabarFtSbPR(gO1,1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFu),Sqr(MSd(gI2)))*Conj(
      CpbarUChabarFuSdPR(gO2,1,gI2))*CpbarUChabarFuSdPR(gO1,1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFve),Sqr(MSe(gI2)))*Conj(
      CpbarUChabarFveSePR(gO2,gI2))*CpbarUChabarFveSePR(gO1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFvm),Sqr(MSm(gI2)))*Conj(
      CpbarUChabarFvmSmPR(gO2,gI2))*CpbarUChabarFvmSmPR(gO1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFvt),Sqr(MStau(gI2)))*Conj(
      CpbarUChabarFvtStauPR(gO2,gI2))*CpbarUChabarFvtStauPR(gO1,gI2));
   result += -SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI2)),0)*Conj(CpbarUChaVPChaPL(gO2
      ,gI2))*CpbarUChaVPChaPL(gO1,gI2));
   result += -SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI2)),Sqr(MVZ))*Conj(
      CpbarUChaVZChaPL(gO2,gI2))*CpbarUChaVZChaPL(gO1,gI2));
   result += -SUM(gI2,0,3,B1(Sqr(p),Sqr(MChi(gI2)),Sqr(MVWm))*Conj(
      CpbarUChaVWmChiPL(gO2,gI2))*CpbarUChaVWmChiPL(gO1,gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Cha_1loop_PR(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = 0; k < 2; k++)
         self_energy(i, k) = self_energy_Cha_1loop_PR(p, i, k);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Cha_1loop_PL(double p , int gO1, int gO2) const
{
   std::complex<double> result;

   result += -0.5*B1(Sqr(p),Sqr(MFe),Sqr(MSveL))*Conj(CpbarUChaconjSveLFePL(gO2
      ))*CpbarUChaconjSveLFePL(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFm),Sqr(MSvmL))*Conj(CpbarUChaconjSvmLFmPL(gO2
      ))*CpbarUChaconjSvmLFmPL(gO1);
   result += -0.5*B1(Sqr(p),Sqr(MFtau),Sqr(MSvtL))*Conj(CpbarUChaconjSvtLFtauPL
      (gO2))*CpbarUChaconjSvtLFtauPL(gO1);
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFs),Sqr(MSc(gI1)))*Conj(
      CpbarUChaconjScFsPL(gO2,gI1,1))*CpbarUChaconjScFsPL(gO1,gI1,1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFb),Sqr(MSt(gI1)))*Conj(
      CpbarUChaconjStFbPL(gO2,gI1,1))*CpbarUChaconjStFbPL(gO1,gI1,1));
   result += -1.5*SUM(gI1,0,1,B1(Sqr(p),Sqr(MFd),Sqr(MSu(gI1)))*Conj(
      CpbarUChaconjSuFdPL(gO2,gI1,1))*CpbarUChaconjSuFdPL(gO1,gI1,1));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MAh(gI2)
      ))*Conj(CpbarUChaChaAhPL(gO2,gI1,gI2))*CpbarUChaChaAhPL(gO1,gI1,gI2)));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI2)),Sqr(Mhh(gI1)
      ))*Conj(CpbarUChahhChaPL(gO2,gI1,gI2))*CpbarUChahhChaPL(gO1,gI1,gI2)));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,B1(Sqr(p),Sqr(MChi(gI2)),Sqr(MHpm(gI1
      )))*Conj(CpbarUChaHpmChiPL(gO2,gI1,gI2))*CpbarUChaHpmChiPL(gO1,gI1,gI2)));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFc),Sqr(MSs(gI2)))*Conj(
      CpbarUChabarFcSsPL(gO2,1,gI2))*CpbarUChabarFcSsPL(gO1,1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFt),Sqr(MSb(gI2)))*Conj(
      CpbarUChabarFtSbPL(gO2,1,gI2))*CpbarUChabarFtSbPL(gO1,1,gI2));
   result += -1.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFu),Sqr(MSd(gI2)))*Conj(
      CpbarUChabarFuSdPL(gO2,1,gI2))*CpbarUChabarFuSdPL(gO1,1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFve),Sqr(MSe(gI2)))*Conj(
      CpbarUChabarFveSePL(gO2,gI2))*CpbarUChabarFveSePL(gO1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFvm),Sqr(MSm(gI2)))*Conj(
      CpbarUChabarFvmSmPL(gO2,gI2))*CpbarUChabarFvmSmPL(gO1,gI2));
   result += -0.5*SUM(gI2,0,1,B1(Sqr(p),Sqr(MFvt),Sqr(MStau(gI2)))*Conj(
      CpbarUChabarFvtStauPL(gO2,gI2))*CpbarUChabarFvtStauPL(gO1,gI2));
   result += -SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI2)),0)*Conj(CpbarUChaVPChaPR(gO2
      ,gI2))*CpbarUChaVPChaPR(gO1,gI2));
   result += -SUM(gI2,0,1,B1(Sqr(p),Sqr(MCha(gI2)),Sqr(MVZ))*Conj(
      CpbarUChaVZChaPR(gO2,gI2))*CpbarUChaVZChaPR(gO1,gI2));
   result += -SUM(gI2,0,3,B1(Sqr(p),Sqr(MChi(gI2)),Sqr(MVWm))*Conj(
      CpbarUChaVWmChiPR(gO2,gI2))*CpbarUChaVWmChiPR(gO1,gI2));

   return result * oneOver16PiSqr;

}

Eigen::Matrix<std::complex<double>,2,2> CLASSNAME::self_energy_Cha_1loop_PL(double p) const
{
   Eigen::Matrix<std::complex<double>,2,2> self_energy;

   for (int i = 0; i < 2; i++)
      for (int k = 0; k < 2; k++)
         self_energy(i, k) = self_energy_Cha_1loop_PL(p, i, k);

   return self_energy;
}

std::complex<double> CLASSNAME::self_energy_Glu_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -12*MGlu*B0(Sqr(p),Sqr(MGlu),0)*Conj(CpGluVGGluPR())*CpGluVGGluPL(
      );
   result += 0.5*MFb*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFb),Sqr(MSb(gI1)))*Conj(
      CpGluconjSbFbPL(gI1,1))*CpGluconjSbFbPR(gI1,1));
   result += 0.5*MFc*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFc),Sqr(MSc(gI1)))*Conj(
      CpGluconjScFcPL(gI1,1))*CpGluconjScFcPR(gI1,1));
   result += 0.5*MFd*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFd),Sqr(MSd(gI1)))*Conj(
      CpGluconjSdFdPL(gI1,1))*CpGluconjSdFdPR(gI1,1));
   result += 0.5*MFs*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFs),Sqr(MSs(gI1)))*Conj(
      CpGluconjSsFsPL(gI1,1))*CpGluconjSsFsPR(gI1,1));
   result += 0.5*MFt*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MSt(gI1)))*Conj(
      CpGluconjStFtPL(gI1,1))*CpGluconjStFtPR(gI1,1));
   result += 0.5*MFu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFu),Sqr(MSu(gI1)))*Conj(
      CpGluconjSuFuPL(gI1,1))*CpGluconjSuFuPR(gI1,1));
   result += 0.5*MFb*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFb),Sqr(MSb(gI2)))*Conj(
      CpGlubarFbSbPL(1,gI2))*CpGlubarFbSbPR(1,gI2));
   result += 0.5*MFc*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFc),Sqr(MSc(gI2)))*Conj(
      CpGlubarFcScPL(1,gI2))*CpGlubarFcScPR(1,gI2));
   result += 0.5*MFd*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFd),Sqr(MSd(gI2)))*Conj(
      CpGlubarFdSdPL(1,gI2))*CpGlubarFdSdPR(1,gI2));
   result += 0.5*MFs*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFs),Sqr(MSs(gI2)))*Conj(
      CpGlubarFsSsPL(1,gI2))*CpGlubarFsSsPR(1,gI2));
   result += 0.5*MFt*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MSt(gI2)))*Conj(
      CpGlubarFtStPL(1,gI2))*CpGlubarFtStPR(1,gI2));
   result += 0.5*MFu*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFu),Sqr(MSu(gI2)))*Conj(
      CpGlubarFuSuPL(1,gI2))*CpGlubarFuSuPR(1,gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Glu_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -3*AbsSqr(CpGluVGGluPL())*B1(Sqr(p),Sqr(MGlu),0);
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjSbFbPR(gI1,1))*B1(Sqr(p),Sqr(MFb
      ),Sqr(MSb(gI1))));
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjScFcPR(gI1,1))*B1(Sqr(p),Sqr(MFc
      ),Sqr(MSc(gI1))));
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjSdFdPR(gI1,1))*B1(Sqr(p),Sqr(MFd
      ),Sqr(MSd(gI1))));
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjSsFsPR(gI1,1))*B1(Sqr(p),Sqr(MFs
      ),Sqr(MSs(gI1))));
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjStFtPR(gI1,1))*B1(Sqr(p),Sqr(MFt
      ),Sqr(MSt(gI1))));
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjSuFuPR(gI1,1))*B1(Sqr(p),Sqr(MFu
      ),Sqr(MSu(gI1))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFbSbPR(1,gI2))*B1(Sqr(p),Sqr(MFb)
      ,Sqr(MSb(gI2))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFcScPR(1,gI2))*B1(Sqr(p),Sqr(MFc)
      ,Sqr(MSc(gI2))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFdSdPR(1,gI2))*B1(Sqr(p),Sqr(MFd)
      ,Sqr(MSd(gI2))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFsSsPR(1,gI2))*B1(Sqr(p),Sqr(MFs)
      ,Sqr(MSs(gI2))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFtStPR(1,gI2))*B1(Sqr(p),Sqr(MFt)
      ,Sqr(MSt(gI2))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFuSuPR(1,gI2))*B1(Sqr(p),Sqr(MFu)
      ,Sqr(MSu(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Glu_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -3*AbsSqr(CpGluVGGluPR())*B1(Sqr(p),Sqr(MGlu),0);
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjSbFbPL(gI1,1))*B1(Sqr(p),Sqr(MFb
      ),Sqr(MSb(gI1))));
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjScFcPL(gI1,1))*B1(Sqr(p),Sqr(MFc
      ),Sqr(MSc(gI1))));
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjSdFdPL(gI1,1))*B1(Sqr(p),Sqr(MFd
      ),Sqr(MSd(gI1))));
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjSsFsPL(gI1,1))*B1(Sqr(p),Sqr(MFs
      ),Sqr(MSs(gI1))));
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjStFtPL(gI1,1))*B1(Sqr(p),Sqr(MFt
      ),Sqr(MSt(gI1))));
   result += -0.25*SUM(gI1,0,1,AbsSqr(CpGluconjSuFuPL(gI1,1))*B1(Sqr(p),Sqr(MFu
      ),Sqr(MSu(gI1))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFbSbPL(1,gI2))*B1(Sqr(p),Sqr(MFb)
      ,Sqr(MSb(gI2))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFcScPL(1,gI2))*B1(Sqr(p),Sqr(MFc)
      ,Sqr(MSc(gI2))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFdSdPL(1,gI2))*B1(Sqr(p),Sqr(MFd)
      ,Sqr(MSd(gI2))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFsSsPL(1,gI2))*B1(Sqr(p),Sqr(MFs)
      ,Sqr(MSs(gI2))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFtStPL(1,gI2))*B1(Sqr(p),Sqr(MFt)
      ,Sqr(MSt(gI2))));
   result += -0.25*SUM(gI2,0,1,AbsSqr(CpGlubarFuSuPL(1,gI2))*B1(Sqr(p),Sqr(MFu)
      ,Sqr(MSu(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fd_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -5.333333333333333*MFd*B0(Sqr(p),Sqr(MFd),0)*Conj(CpbarFdVGFdPR())
      *CpbarFdVGFdPL();
   result += -4*MFd*B0(Sqr(p),Sqr(MFd),0)*Conj(CpbarFdVPFdPR())*CpbarFdVPFdPL()
      ;
   result += -4*MFd*B0(Sqr(p),Sqr(MFd),Sqr(MVZ))*Conj(CpbarFdVZFdPR())*
      CpbarFdVZFdPL();
   result += -4*MFu*B0(Sqr(p),Sqr(MFu),Sqr(MVWm))*Conj(CpbarFdVWmFuPR(1))*
      CpbarFdVWmFuPL(1);
   result += MFd*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFd),Sqr(Mhh(gI1)))*Conj(
      CpbarFdhhFdPL(gI1))*CpbarFdhhFdPR(gI1));
   result += MFu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFu),Sqr(MHpm(gI1)))*Conj(
      CpbarFdHpmFuPL(gI1,1))*CpbarFdHpmFuPR(gI1,1));
   result += 1.3333333333333333*MGlu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MGlu),Sqr(MSd(
      gI1)))*Conj(CpbarFdSdGluPL(gI1,1))*CpbarFdSdGluPR(gI1,1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MSu(gI1)))*
      Conj(CpbarFdSuChaPL(gI1,gI2))*CpbarFdSuChaPR(gI1,gI2)*MCha(gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSd(gI1)))*
      Conj(CpbarFdSdChiPL(gI1,gI2))*CpbarFdSdChiPR(gI1,gI2)*MChi(gI2)));
   result += MFd*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFd),Sqr(MAh(gI2)))*Conj(
      CpbarFdFdAhPL(gI2))*CpbarFdFdAhPR(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fd_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -1.3333333333333333*AbsSqr(CpbarFdVGFdPL())*B1(Sqr(p),Sqr(MFd),0);
   result += -(AbsSqr(CpbarFdVPFdPL())*B1(Sqr(p),Sqr(MFd),0));
   result += -(AbsSqr(CpbarFdVWmFuPL(1))*B1(Sqr(p),Sqr(MFu),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFdVZFdPL())*B1(Sqr(p),Sqr(MFd),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFdhhFdPR(gI1))*B1(Sqr(p),Sqr(MFd),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFdHpmFuPR(gI1,1))*B1(Sqr(p),Sqr(MFu),
      Sqr(MHpm(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFdSdGluPR(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSd(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFdSuChaPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MCha(gI2)),Sqr(MSu(gI1)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFdSdChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSd(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFdFdAhPR(gI2))*B1(Sqr(p),Sqr(MFd),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fd_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -1.3333333333333333*AbsSqr(CpbarFdVGFdPR())*B1(Sqr(p),Sqr(MFd),0);
   result += -(AbsSqr(CpbarFdVPFdPR())*B1(Sqr(p),Sqr(MFd),0));
   result += -(AbsSqr(CpbarFdVWmFuPR(1))*B1(Sqr(p),Sqr(MFu),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFdVZFdPR())*B1(Sqr(p),Sqr(MFd),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFdhhFdPL(gI1))*B1(Sqr(p),Sqr(MFd),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFdHpmFuPL(gI1,1))*B1(Sqr(p),Sqr(MFu),
      Sqr(MHpm(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFdSdGluPL(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSd(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFdSuChaPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MCha(gI2)),Sqr(MSu(gI1)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFdSdChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSd(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFdFdAhPL(gI2))*B1(Sqr(p),Sqr(MFd),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fs_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -4*MFc*B0(Sqr(p),Sqr(MFc),Sqr(MVWm))*Conj(CpbarFsVWmFcPR(1))*
      CpbarFsVWmFcPL(1);
   result += -5.333333333333333*MFs*B0(Sqr(p),Sqr(MFs),0)*Conj(CpbarFsVGFsPR())
      *CpbarFsVGFsPL();
   result += -4*MFs*B0(Sqr(p),Sqr(MFs),0)*Conj(CpbarFsVPFsPR())*CpbarFsVPFsPL()
      ;
   result += -4*MFs*B0(Sqr(p),Sqr(MFs),Sqr(MVZ))*Conj(CpbarFsVZFsPR())*
      CpbarFsVZFsPL();
   result += MFs*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFs),Sqr(Mhh(gI1)))*Conj(
      CpbarFshhFsPL(gI1))*CpbarFshhFsPR(gI1));
   result += MFc*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFc),Sqr(MHpm(gI1)))*Conj(
      CpbarFsHpmFcPL(gI1,1))*CpbarFsHpmFcPR(gI1,1));
   result += 1.3333333333333333*MGlu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MGlu),Sqr(MSs(
      gI1)))*Conj(CpbarFsSsGluPL(gI1,1))*CpbarFsSsGluPR(gI1,1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MSc(gI1)))*
      Conj(CpbarFsScChaPL(gI1,gI2))*CpbarFsScChaPR(gI1,gI2)*MCha(gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSs(gI1)))*
      Conj(CpbarFsSsChiPL(gI1,gI2))*CpbarFsSsChiPR(gI1,gI2)*MChi(gI2)));
   result += MFs*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFs),Sqr(MAh(gI2)))*Conj(
      CpbarFsFsAhPL(gI2))*CpbarFsFsAhPR(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fs_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -1.3333333333333333*AbsSqr(CpbarFsVGFsPL())*B1(Sqr(p),Sqr(MFs),0);
   result += -(AbsSqr(CpbarFsVPFsPL())*B1(Sqr(p),Sqr(MFs),0));
   result += -(AbsSqr(CpbarFsVWmFcPL(1))*B1(Sqr(p),Sqr(MFc),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFsVZFsPL())*B1(Sqr(p),Sqr(MFs),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFshhFsPR(gI1))*B1(Sqr(p),Sqr(MFs),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFsHpmFcPR(gI1,1))*B1(Sqr(p),Sqr(MFc),
      Sqr(MHpm(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFsSsGluPR(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSs(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFsScChaPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MCha(gI2)),Sqr(MSc(gI1)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFsSsChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSs(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFsFsAhPR(gI2))*B1(Sqr(p),Sqr(MFs),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fs_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -1.3333333333333333*AbsSqr(CpbarFsVGFsPR())*B1(Sqr(p),Sqr(MFs),0);
   result += -(AbsSqr(CpbarFsVPFsPR())*B1(Sqr(p),Sqr(MFs),0));
   result += -(AbsSqr(CpbarFsVWmFcPR(1))*B1(Sqr(p),Sqr(MFc),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFsVZFsPR())*B1(Sqr(p),Sqr(MFs),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFshhFsPL(gI1))*B1(Sqr(p),Sqr(MFs),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFsHpmFcPL(gI1,1))*B1(Sqr(p),Sqr(MFc),
      Sqr(MHpm(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFsSsGluPL(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSs(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFsScChaPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MCha(gI2)),Sqr(MSc(gI1)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFsSsChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSs(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFsFsAhPL(gI2))*B1(Sqr(p),Sqr(MFs),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fb_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -5.333333333333333*MFb*B0(Sqr(p),Sqr(MFb),0)*Conj(CpbarFbVGFbPR())
      *CpbarFbVGFbPL();
   result += -4*MFb*B0(Sqr(p),Sqr(MFb),0)*Conj(CpbarFbVPFbPR())*CpbarFbVPFbPL()
      ;
   result += -4*MFb*B0(Sqr(p),Sqr(MFb),Sqr(MVZ))*Conj(CpbarFbVZFbPR())*
      CpbarFbVZFbPL();
   result += -4*MFt*B0(Sqr(p),Sqr(MFt),Sqr(MVWm))*Conj(CpbarFbVWmFtPR(1))*
      CpbarFbVWmFtPL(1);
   result += MFb*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFb),Sqr(Mhh(gI1)))*Conj(
      CpbarFbhhFbPL(gI1))*CpbarFbhhFbPR(gI1));
   result += MFt*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MHpm(gI1)))*Conj(
      CpbarFbHpmFtPL(gI1,1))*CpbarFbHpmFtPR(gI1,1));
   result += 1.3333333333333333*MGlu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MGlu),Sqr(MSb(
      gI1)))*Conj(CpbarFbSbGluPL(gI1,1))*CpbarFbSbGluPR(gI1,1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MSt(gI1)))*
      Conj(CpbarFbStChaPL(gI1,gI2))*CpbarFbStChaPR(gI1,gI2)*MCha(gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSb(gI1)))*
      Conj(CpbarFbSbChiPL(gI1,gI2))*CpbarFbSbChiPR(gI1,gI2)*MChi(gI2)));
   result += MFb*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFb),Sqr(MAh(gI2)))*Conj(
      CpbarFbFbAhPL(gI2))*CpbarFbFbAhPR(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fb_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -1.3333333333333333*AbsSqr(CpbarFbVGFbPL())*B1(Sqr(p),Sqr(MFb),0);
   result += -(AbsSqr(CpbarFbVPFbPL())*B1(Sqr(p),Sqr(MFb),0));
   result += -(AbsSqr(CpbarFbVWmFtPL(1))*B1(Sqr(p),Sqr(MFt),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFbVZFbPL())*B1(Sqr(p),Sqr(MFb),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFbhhFbPR(gI1))*B1(Sqr(p),Sqr(MFb),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFbHpmFtPR(gI1,1))*B1(Sqr(p),Sqr(MFt),
      Sqr(MHpm(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFbSbGluPR(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSb(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFbStChaPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MCha(gI2)),Sqr(MSt(gI1)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFbSbChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSb(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFbFbAhPR(gI2))*B1(Sqr(p),Sqr(MFb),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fb_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -1.3333333333333333*AbsSqr(CpbarFbVGFbPR())*B1(Sqr(p),Sqr(MFb),0);
   result += -(AbsSqr(CpbarFbVPFbPR())*B1(Sqr(p),Sqr(MFb),0));
   result += -(AbsSqr(CpbarFbVWmFtPR(1))*B1(Sqr(p),Sqr(MFt),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFbVZFbPR())*B1(Sqr(p),Sqr(MFb),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFbhhFbPL(gI1))*B1(Sqr(p),Sqr(MFb),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFbHpmFtPL(gI1,1))*B1(Sqr(p),Sqr(MFt),
      Sqr(MHpm(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFbSbGluPL(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSb(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFbStChaPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MCha(gI2)),Sqr(MSt(gI1)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFbSbChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSb(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFbFbAhPL(gI2))*B1(Sqr(p),Sqr(MFb),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fu_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -4*MFd*B0(Sqr(p),Sqr(MFd),Sqr(MVWm))*Conj(CpbarFuconjVWmFdPR(1))*
      CpbarFuconjVWmFdPL(1);
   result += -5.333333333333333*MFu*B0(Sqr(p),Sqr(MFu),0)*Conj(CpbarFuVGFuPR())
      *CpbarFuVGFuPL();
   result += -4*MFu*B0(Sqr(p),Sqr(MFu),0)*Conj(CpbarFuVPFuPR())*CpbarFuVPFuPL()
      ;
   result += -4*MFu*B0(Sqr(p),Sqr(MFu),Sqr(MVZ))*Conj(CpbarFuVZFuPR())*
      CpbarFuVZFuPL();
   result += MFd*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFd),Sqr(MHpm(gI1)))*Conj(
      CpbarFuconjHpmFdPL(gI1,1))*CpbarFuconjHpmFdPR(gI1,1));
   result += MFu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFu),Sqr(Mhh(gI1)))*Conj(
      CpbarFuhhFuPL(gI1))*CpbarFuhhFuPR(gI1));
   result += 1.3333333333333333*MGlu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MGlu),Sqr(MSu(
      gI1)))*Conj(CpbarFuSuGluPL(gI1,1))*CpbarFuSuGluPR(gI1,1));
   result += SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MSd
      (gI2)))*Conj(CpbarFubarChaSdPL(gI1,gI2))*CpbarFubarChaSdPR(gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSu(gI1)))*
      Conj(CpbarFuSuChiPL(gI1,gI2))*CpbarFuSuChiPR(gI1,gI2)*MChi(gI2)));
   result += MFu*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFu),Sqr(MAh(gI2)))*Conj(
      CpbarFuFuAhPL(gI2))*CpbarFuFuAhPR(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fu_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFuconjVWmFdPL(1))*B1(Sqr(p),Sqr(MFd),Sqr(MVWm)));
   result += -1.3333333333333333*AbsSqr(CpbarFuVGFuPL())*B1(Sqr(p),Sqr(MFu),0);
   result += -(AbsSqr(CpbarFuVPFuPL())*B1(Sqr(p),Sqr(MFu),0));
   result += -(AbsSqr(CpbarFuVZFuPL())*B1(Sqr(p),Sqr(MFu),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFuconjHpmFdPR(gI1,1))*B1(Sqr(p),Sqr(
      MFd),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFuhhFuPR(gI1))*B1(Sqr(p),Sqr(MFu),Sqr
      (Mhh(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFuSuGluPR(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSu(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFubarChaSdPR(gI1,gI2))*B1
      (Sqr(p),Sqr(MCha(gI1)),Sqr(MSd(gI2)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFuSuChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSu(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFuFuAhPR(gI2))*B1(Sqr(p),Sqr(MFu),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fu_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFuconjVWmFdPR(1))*B1(Sqr(p),Sqr(MFd),Sqr(MVWm)));
   result += -1.3333333333333333*AbsSqr(CpbarFuVGFuPR())*B1(Sqr(p),Sqr(MFu),0);
   result += -(AbsSqr(CpbarFuVPFuPR())*B1(Sqr(p),Sqr(MFu),0));
   result += -(AbsSqr(CpbarFuVZFuPR())*B1(Sqr(p),Sqr(MFu),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFuconjHpmFdPL(gI1,1))*B1(Sqr(p),Sqr(
      MFd),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFuhhFuPL(gI1))*B1(Sqr(p),Sqr(MFu),Sqr
      (Mhh(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFuSuGluPL(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSu(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFubarChaSdPL(gI1,gI2))*B1
      (Sqr(p),Sqr(MCha(gI1)),Sqr(MSd(gI2)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFuSuChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSu(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFuFuAhPL(gI2))*B1(Sqr(p),Sqr(MFu),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fc_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -5.333333333333333*MFc*B0(Sqr(p),Sqr(MFc),0)*Conj(CpbarFcVGFcPR())
      *CpbarFcVGFcPL();
   result += -4*MFc*B0(Sqr(p),Sqr(MFc),0)*Conj(CpbarFcVPFcPR())*CpbarFcVPFcPL()
      ;
   result += -4*MFc*B0(Sqr(p),Sqr(MFc),Sqr(MVZ))*Conj(CpbarFcVZFcPR())*
      CpbarFcVZFcPL();
   result += -4*MFs*B0(Sqr(p),Sqr(MFs),Sqr(MVWm))*Conj(CpbarFcconjVWmFsPR(1))*
      CpbarFcconjVWmFsPL(1);
   result += MFs*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFs),Sqr(MHpm(gI1)))*Conj(
      CpbarFcconjHpmFsPL(gI1,1))*CpbarFcconjHpmFsPR(gI1,1));
   result += MFc*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFc),Sqr(Mhh(gI1)))*Conj(
      CpbarFchhFcPL(gI1))*CpbarFchhFcPR(gI1));
   result += 1.3333333333333333*MGlu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MGlu),Sqr(MSc(
      gI1)))*Conj(CpbarFcScGluPL(gI1,1))*CpbarFcScGluPR(gI1,1));
   result += SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MSs
      (gI2)))*Conj(CpbarFcbarChaSsPL(gI1,gI2))*CpbarFcbarChaSsPR(gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSc(gI1)))*
      Conj(CpbarFcScChiPL(gI1,gI2))*CpbarFcScChiPR(gI1,gI2)*MChi(gI2)));
   result += MFc*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFc),Sqr(MAh(gI2)))*Conj(
      CpbarFcFcAhPL(gI2))*CpbarFcFcAhPR(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fc_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFcconjVWmFsPL(1))*B1(Sqr(p),Sqr(MFs),Sqr(MVWm)));
   result += -1.3333333333333333*AbsSqr(CpbarFcVGFcPL())*B1(Sqr(p),Sqr(MFc),0);
   result += -(AbsSqr(CpbarFcVPFcPL())*B1(Sqr(p),Sqr(MFc),0));
   result += -(AbsSqr(CpbarFcVZFcPL())*B1(Sqr(p),Sqr(MFc),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFcconjHpmFsPR(gI1,1))*B1(Sqr(p),Sqr(
      MFs),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFchhFcPR(gI1))*B1(Sqr(p),Sqr(MFc),Sqr
      (Mhh(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFcScGluPR(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSc(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFcbarChaSsPR(gI1,gI2))*B1
      (Sqr(p),Sqr(MCha(gI1)),Sqr(MSs(gI2)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFcScChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSc(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFcFcAhPR(gI2))*B1(Sqr(p),Sqr(MFc),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fc_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFcconjVWmFsPR(1))*B1(Sqr(p),Sqr(MFs),Sqr(MVWm)));
   result += -1.3333333333333333*AbsSqr(CpbarFcVGFcPR())*B1(Sqr(p),Sqr(MFc),0);
   result += -(AbsSqr(CpbarFcVPFcPR())*B1(Sqr(p),Sqr(MFc),0));
   result += -(AbsSqr(CpbarFcVZFcPR())*B1(Sqr(p),Sqr(MFc),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFcconjHpmFsPL(gI1,1))*B1(Sqr(p),Sqr(
      MFs),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFchhFcPL(gI1))*B1(Sqr(p),Sqr(MFc),Sqr
      (Mhh(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFcScGluPL(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSc(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFcbarChaSsPL(gI1,gI2))*B1
      (Sqr(p),Sqr(MCha(gI1)),Sqr(MSs(gI2)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFcScChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSc(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFcFcAhPL(gI2))*B1(Sqr(p),Sqr(MFc),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ft_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -4*MFb*B0(Sqr(p),Sqr(MFb),Sqr(MVWm))*Conj(CpbarFtconjVWmFbPR(1))*
      CpbarFtconjVWmFbPL(1);
   result += -5.333333333333333*MFt*B0(Sqr(p),Sqr(MFt),0)*Conj(CpbarFtVGFtPR())
      *CpbarFtVGFtPL();
   result += -4*MFt*B0(Sqr(p),Sqr(MFt),0)*Conj(CpbarFtVPFtPR())*CpbarFtVPFtPL()
      ;
   result += -4*MFt*B0(Sqr(p),Sqr(MFt),Sqr(MVZ))*Conj(CpbarFtVZFtPR())*
      CpbarFtVZFtPL();
   result += MFb*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFb),Sqr(MHpm(gI1)))*Conj(
      CpbarFtconjHpmFbPL(gI1,1))*CpbarFtconjHpmFbPR(gI1,1));
   result += MFt*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFt),Sqr(Mhh(gI1)))*Conj(
      CpbarFthhFtPL(gI1))*CpbarFthhFtPR(gI1));
   result += 1.3333333333333333*MGlu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MGlu),Sqr(MSt(
      gI1)))*Conj(CpbarFtStGluPL(gI1,1))*CpbarFtStGluPR(gI1,1));
   result += SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MSb
      (gI2)))*Conj(CpbarFtbarChaSbPL(gI1,gI2))*CpbarFtbarChaSbPR(gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSt(gI1)))*
      Conj(CpbarFtStChiPL(gI1,gI2))*CpbarFtStChiPR(gI1,gI2)*MChi(gI2)));
   result += MFt*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MAh(gI2)))*Conj(
      CpbarFtFtAhPL(gI2))*CpbarFtFtAhPR(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ft_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFtconjVWmFbPL(1))*B1(Sqr(p),Sqr(MFb),Sqr(MVWm)));
   result += -1.3333333333333333*AbsSqr(CpbarFtVGFtPL())*B1(Sqr(p),Sqr(MFt),0);
   result += -(AbsSqr(CpbarFtVPFtPL())*B1(Sqr(p),Sqr(MFt),0));
   result += -(AbsSqr(CpbarFtVZFtPL())*B1(Sqr(p),Sqr(MFt),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtconjHpmFbPR(gI1,1))*B1(Sqr(p),Sqr(
      MFb),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFthhFtPR(gI1))*B1(Sqr(p),Sqr(MFt),Sqr
      (Mhh(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFtStGluPR(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSt(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFtbarChaSbPR(gI1,gI2))*B1
      (Sqr(p),Sqr(MCha(gI1)),Sqr(MSb(gI2)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFtStChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSt(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtFtAhPR(gI2))*B1(Sqr(p),Sqr(MFt),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ft_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFtconjVWmFbPR(1))*B1(Sqr(p),Sqr(MFb),Sqr(MVWm)));
   result += -1.3333333333333333*AbsSqr(CpbarFtVGFtPR())*B1(Sqr(p),Sqr(MFt),0);
   result += -(AbsSqr(CpbarFtVPFtPR())*B1(Sqr(p),Sqr(MFt),0));
   result += -(AbsSqr(CpbarFtVZFtPR())*B1(Sqr(p),Sqr(MFt),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtconjHpmFbPL(gI1,1))*B1(Sqr(p),Sqr(
      MFb),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFthhFtPL(gI1))*B1(Sqr(p),Sqr(MFt),Sqr
      (Mhh(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFtStGluPL(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSt(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFtbarChaSbPL(gI1,gI2))*B1
      (Sqr(p),Sqr(MCha(gI1)),Sqr(MSb(gI2)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFtStChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSt(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtFtAhPL(gI2))*B1(Sqr(p),Sqr(MFt),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fve_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -4*MFe*B0(Sqr(p),Sqr(MFe),Sqr(MVWm))*Conj(CpbarFveconjVWmFePR())*
      CpbarFveconjVWmFePL();
   result += -4*MFve*B0(Sqr(p),Sqr(MFve),Sqr(MVZ))*Conj(CpbarFveVZFvePR())*
      CpbarFveVZFvePL();
   result += MFe*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFe),Sqr(MHpm(gI1)))*Conj(
      CpbarFveconjHpmFePL(gI1))*CpbarFveconjHpmFePR(gI1));
   result += SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MSe
      (gI2)))*Conj(CpbarFvebarChaSePL(gI1,gI2))*CpbarFvebarChaSePR(gI1,gI2)));
   result += SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSveL))*Conj(
      CpbarFveSveLChiPL(gI2))*CpbarFveSveLChiPR(gI2)*MChi(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fve_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFveconjVWmFePL())*B1(Sqr(p),Sqr(MFe),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFveVZFvePL())*B1(Sqr(p),Sqr(MFve),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFveconjHpmFePR(gI1))*B1(Sqr(p),Sqr(
      MFe),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFvebarChaSePR(gI1,gI2))*
      B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MSe(gI2)))));
   result += -0.5*SUM(gI2,0,3,AbsSqr(CpbarFveSveLChiPR(gI2))*B1(Sqr(p),Sqr(MChi
      (gI2)),Sqr(MSveL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fve_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFveconjVWmFePR())*B1(Sqr(p),Sqr(MFe),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFveVZFvePR())*B1(Sqr(p),Sqr(MFve),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFveconjHpmFePL(gI1))*B1(Sqr(p),Sqr(
      MFe),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFvebarChaSePL(gI1,gI2))*
      B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MSe(gI2)))));
   result += -0.5*SUM(gI2,0,3,AbsSqr(CpbarFveSveLChiPL(gI2))*B1(Sqr(p),Sqr(MChi
      (gI2)),Sqr(MSveL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fvm_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -4*MFm*B0(Sqr(p),Sqr(MFm),Sqr(MVWm))*Conj(CpbarFvmconjVWmFmPR())*
      CpbarFvmconjVWmFmPL();
   result += -4*MFvm*B0(Sqr(p),Sqr(MFvm),Sqr(MVZ))*Conj(CpbarFvmVZFvmPR())*
      CpbarFvmVZFvmPL();
   result += MFm*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFm),Sqr(MHpm(gI1)))*Conj(
      CpbarFvmconjHpmFmPL(gI1))*CpbarFvmconjHpmFmPR(gI1));
   result += SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MSm
      (gI2)))*Conj(CpbarFvmbarChaSmPL(gI1,gI2))*CpbarFvmbarChaSmPR(gI1,gI2)));
   result += SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSvmL))*Conj(
      CpbarFvmSvmLChiPL(gI2))*CpbarFvmSvmLChiPR(gI2)*MChi(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fvm_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFvmconjVWmFmPL())*B1(Sqr(p),Sqr(MFm),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFvmVZFvmPL())*B1(Sqr(p),Sqr(MFvm),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFvmconjHpmFmPR(gI1))*B1(Sqr(p),Sqr(
      MFm),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFvmbarChaSmPR(gI1,gI2))*
      B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MSm(gI2)))));
   result += -0.5*SUM(gI2,0,3,AbsSqr(CpbarFvmSvmLChiPR(gI2))*B1(Sqr(p),Sqr(MChi
      (gI2)),Sqr(MSvmL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fvm_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFvmconjVWmFmPR())*B1(Sqr(p),Sqr(MFm),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFvmVZFvmPR())*B1(Sqr(p),Sqr(MFvm),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFvmconjHpmFmPL(gI1))*B1(Sqr(p),Sqr(
      MFm),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFvmbarChaSmPL(gI1,gI2))*
      B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MSm(gI2)))));
   result += -0.5*SUM(gI2,0,3,AbsSqr(CpbarFvmSvmLChiPL(gI2))*B1(Sqr(p),Sqr(MChi
      (gI2)),Sqr(MSvmL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fvt_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -4*MFtau*B0(Sqr(p),Sqr(MFtau),Sqr(MVWm))*Conj(
      CpbarFvtconjVWmFtauPR())*CpbarFvtconjVWmFtauPL();
   result += -4*MFvt*B0(Sqr(p),Sqr(MFvt),Sqr(MVZ))*Conj(CpbarFvtVZFvtPR())*
      CpbarFvtVZFvtPL();
   result += MFtau*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFtau),Sqr(MHpm(gI1)))*Conj(
      CpbarFvtconjHpmFtauPL(gI1))*CpbarFvtconjHpmFtauPR(gI1));
   result += SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(
      MStau(gI2)))*Conj(CpbarFvtbarChaStauPL(gI1,gI2))*CpbarFvtbarChaStauPR(gI1,
      gI2)));
   result += SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSvtL))*Conj(
      CpbarFvtSvtLChiPL(gI2))*CpbarFvtSvtLChiPR(gI2)*MChi(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fvt_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFvtconjVWmFtauPL())*B1(Sqr(p),Sqr(MFtau),Sqr(MVWm)))
      ;
   result += -(AbsSqr(CpbarFvtVZFvtPL())*B1(Sqr(p),Sqr(MFvt),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFvtconjHpmFtauPR(gI1))*B1(Sqr(p),Sqr(
      MFtau),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFvtbarChaStauPR(gI1,gI2))
      *B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MStau(gI2)))));
   result += -0.5*SUM(gI2,0,3,AbsSqr(CpbarFvtSvtLChiPR(gI2))*B1(Sqr(p),Sqr(MChi
      (gI2)),Sqr(MSvtL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fvt_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFvtconjVWmFtauPR())*B1(Sqr(p),Sqr(MFtau),Sqr(MVWm)))
      ;
   result += -(AbsSqr(CpbarFvtVZFvtPR())*B1(Sqr(p),Sqr(MFvt),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFvtconjHpmFtauPL(gI1))*B1(Sqr(p),Sqr(
      MFtau),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFvtbarChaStauPL(gI1,gI2))
      *B1(Sqr(p),Sqr(MCha(gI1)),Sqr(MStau(gI2)))));
   result += -0.5*SUM(gI2,0,3,AbsSqr(CpbarFvtSvtLChiPL(gI2))*B1(Sqr(p),Sqr(MChi
      (gI2)),Sqr(MSvtL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fe_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -4*MFe*B0(Sqr(p),Sqr(MFe),0)*Conj(CpbarFeVPFePR())*CpbarFeVPFePL()
      ;
   result += -4*MFe*B0(Sqr(p),Sqr(MFe),Sqr(MVZ))*Conj(CpbarFeVZFePR())*
      CpbarFeVZFePL();
   result += -4*MFve*B0(Sqr(p),Sqr(MFve),Sqr(MVWm))*Conj(CpbarFeVWmFvePR())*
      CpbarFeVWmFvePL();
   result += MFe*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFe),Sqr(Mhh(gI1)))*Conj(
      CpbarFehhFePL(gI1))*CpbarFehhFePR(gI1));
   result += MFve*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFve),Sqr(MHpm(gI1)))*Conj(
      CpbarFeHpmFvePL(gI1))*CpbarFeHpmFvePR(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSe(gI1)))*
      Conj(CpbarFeSeChiPL(gI1,gI2))*CpbarFeSeChiPR(gI1,gI2)*MChi(gI2)));
   result += MFe*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFe),Sqr(MAh(gI2)))*Conj(
      CpbarFeFeAhPL(gI2))*CpbarFeFeAhPR(gI2));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MSveL))*Conj(
      CpbarFeSveLChaPL(gI2))*CpbarFeSveLChaPR(gI2)*MCha(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fe_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFeVPFePL())*B1(Sqr(p),Sqr(MFe),0));
   result += -(AbsSqr(CpbarFeVWmFvePL())*B1(Sqr(p),Sqr(MFve),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFeVZFePL())*B1(Sqr(p),Sqr(MFe),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFehhFePR(gI1))*B1(Sqr(p),Sqr(MFe),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFeHpmFvePR(gI1))*B1(Sqr(p),Sqr(MFve),
      Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFeSeChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSe(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFeFeAhPR(gI2))*B1(Sqr(p),Sqr(MFe),Sqr
      (MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFeSveLChaPR(gI2))*B1(Sqr(p),Sqr(MCha(
      gI2)),Sqr(MSveL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fe_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFeVPFePR())*B1(Sqr(p),Sqr(MFe),0));
   result += -(AbsSqr(CpbarFeVWmFvePR())*B1(Sqr(p),Sqr(MFve),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFeVZFePR())*B1(Sqr(p),Sqr(MFe),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFehhFePL(gI1))*B1(Sqr(p),Sqr(MFe),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFeHpmFvePL(gI1))*B1(Sqr(p),Sqr(MFve),
      Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFeSeChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSe(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFeFeAhPL(gI2))*B1(Sqr(p),Sqr(MFe),Sqr
      (MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFeSveLChaPL(gI2))*B1(Sqr(p),Sqr(MCha(
      gI2)),Sqr(MSveL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fm_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -4*MFm*B0(Sqr(p),Sqr(MFm),0)*Conj(CpbarFmVPFmPR())*CpbarFmVPFmPL()
      ;
   result += -4*MFm*B0(Sqr(p),Sqr(MFm),Sqr(MVZ))*Conj(CpbarFmVZFmPR())*
      CpbarFmVZFmPL();
   result += -4*MFvm*B0(Sqr(p),Sqr(MFvm),Sqr(MVWm))*Conj(CpbarFmVWmFvmPR())*
      CpbarFmVWmFvmPL();
   result += MFm*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFm),Sqr(Mhh(gI1)))*Conj(
      CpbarFmhhFmPL(gI1))*CpbarFmhhFmPR(gI1));
   result += MFvm*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFvm),Sqr(MHpm(gI1)))*Conj(
      CpbarFmHpmFvmPL(gI1))*CpbarFmHpmFvmPR(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSm(gI1)))*
      Conj(CpbarFmSmChiPL(gI1,gI2))*CpbarFmSmChiPR(gI1,gI2)*MChi(gI2)));
   result += MFm*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFm),Sqr(MAh(gI2)))*Conj(
      CpbarFmFmAhPL(gI2))*CpbarFmFmAhPR(gI2));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MSvmL))*Conj(
      CpbarFmSvmLChaPL(gI2))*CpbarFmSvmLChaPR(gI2)*MCha(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fm_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFmVPFmPL())*B1(Sqr(p),Sqr(MFm),0));
   result += -(AbsSqr(CpbarFmVWmFvmPL())*B1(Sqr(p),Sqr(MFvm),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFmVZFmPL())*B1(Sqr(p),Sqr(MFm),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFmhhFmPR(gI1))*B1(Sqr(p),Sqr(MFm),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFmHpmFvmPR(gI1))*B1(Sqr(p),Sqr(MFvm),
      Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFmSmChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSm(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFmFmAhPR(gI2))*B1(Sqr(p),Sqr(MFm),Sqr
      (MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFmSvmLChaPR(gI2))*B1(Sqr(p),Sqr(MCha(
      gI2)),Sqr(MSvmL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fm_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFmVPFmPR())*B1(Sqr(p),Sqr(MFm),0));
   result += -(AbsSqr(CpbarFmVWmFvmPR())*B1(Sqr(p),Sqr(MFvm),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFmVZFmPR())*B1(Sqr(p),Sqr(MFm),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFmhhFmPL(gI1))*B1(Sqr(p),Sqr(MFm),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFmHpmFvmPL(gI1))*B1(Sqr(p),Sqr(MFvm),
      Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFmSmChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSm(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFmFmAhPL(gI2))*B1(Sqr(p),Sqr(MFm),Sqr
      (MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFmSvmLChaPL(gI2))*B1(Sqr(p),Sqr(MCha(
      gI2)),Sqr(MSvmL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ftau_1loop_1(double p ) const
{
   std::complex<double> result;

   result += -4*MFtau*B0(Sqr(p),Sqr(MFtau),0)*Conj(CpbarFtauVPFtauPR())*
      CpbarFtauVPFtauPL();
   result += -4*MFtau*B0(Sqr(p),Sqr(MFtau),Sqr(MVZ))*Conj(CpbarFtauVZFtauPR())*
      CpbarFtauVZFtauPL();
   result += -4*MFvt*B0(Sqr(p),Sqr(MFvt),Sqr(MVWm))*Conj(CpbarFtauVWmFvtPR())*
      CpbarFtauVWmFvtPL();
   result += MFtau*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFtau),Sqr(Mhh(gI1)))*Conj(
      CpbarFtauhhFtauPL(gI1))*CpbarFtauhhFtauPR(gI1));
   result += MFvt*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFvt),Sqr(MHpm(gI1)))*Conj(
      CpbarFtauHpmFvtPL(gI1))*CpbarFtauHpmFvtPR(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MStau(gI1)))*
      Conj(CpbarFtauStauChiPL(gI1,gI2))*CpbarFtauStauChiPR(gI1,gI2)*MChi(gI2)));
   result += MFtau*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFtau),Sqr(MAh(gI2)))*Conj(
      CpbarFtauFtauAhPL(gI2))*CpbarFtauFtauAhPR(gI2));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MSvtL))*Conj(
      CpbarFtauSvtLChaPL(gI2))*CpbarFtauSvtLChaPR(gI2)*MCha(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ftau_1loop_PR(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFtauVPFtauPL())*B1(Sqr(p),Sqr(MFtau),0));
   result += -(AbsSqr(CpbarFtauVWmFvtPL())*B1(Sqr(p),Sqr(MFvt),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFtauVZFtauPL())*B1(Sqr(p),Sqr(MFtau),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtauhhFtauPR(gI1))*B1(Sqr(p),Sqr(
      MFtau),Sqr(Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtauHpmFvtPR(gI1))*B1(Sqr(p),Sqr(MFvt
      ),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFtauStauChiPR(gI1,gI2))*
      B1(Sqr(p),Sqr(MChi(gI2)),Sqr(MStau(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtauFtauAhPR(gI2))*B1(Sqr(p),Sqr(
      MFtau),Sqr(MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtauSvtLChaPR(gI2))*B1(Sqr(p),Sqr(
      MCha(gI2)),Sqr(MSvtL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ftau_1loop_PL(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFtauVPFtauPR())*B1(Sqr(p),Sqr(MFtau),0));
   result += -(AbsSqr(CpbarFtauVWmFvtPR())*B1(Sqr(p),Sqr(MFvt),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFtauVZFtauPR())*B1(Sqr(p),Sqr(MFtau),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtauhhFtauPL(gI1))*B1(Sqr(p),Sqr(
      MFtau),Sqr(Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtauHpmFvtPL(gI1))*B1(Sqr(p),Sqr(MFvt
      ),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFtauStauChiPL(gI1,gI2))*
      B1(Sqr(p),Sqr(MChi(gI2)),Sqr(MStau(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtauFtauAhPL(gI2))*B1(Sqr(p),Sqr(
      MFtau),Sqr(MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtauSvtLChaPL(gI2))*B1(Sqr(p),Sqr(
      MCha(gI2)),Sqr(MSvtL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_VZ_1loop_heavy(double p ) const
{
   std::complex<double> result;

   result += -4*AbsSqr(CpVZconjSveLSveL())*B00(Sqr(p),Sqr(MSveL),Sqr(MSveL));
   result += -4*AbsSqr(CpVZconjSvmLSvmL())*B00(Sqr(p),Sqr(MSvmL),Sqr(MSvmL));
   result += -4*AbsSqr(CpVZconjSvtLSvtL())*B00(Sqr(p),Sqr(MSvtL),Sqr(MSvtL));
   result += A0(Sqr(MSveL))*CpVZVZconjSveLSveL();
   result += A0(Sqr(MSvmL))*CpVZVZconjSvmLSvmL();
   result += A0(Sqr(MSvtL))*CpVZVZconjSvtLSvtL();
   result += -4*SUM(gI1,0,1,AbsSqr(CpVZhhAh(gI1,2))*B00(Sqr(p),Sqr(MAh(1)),Sqr(
      Mhh(gI1))));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpVZVZconjSbSb(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpVZVZconjScSc(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpVZVZconjSdSd(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpVZVZconjSeSe(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpVZVZconjSmSm(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpVZVZconjSsSs(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpVZVZconjStauStau(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpVZVZconjStSt(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpVZVZconjSuSu(gI1,gI1));
   result += 0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpVZVZhhhh(gI1,gI1));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSbSb(gI1,gI2))*B00(Sqr(
      p),Sqr(MSb(gI1)),Sqr(MSb(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjScSc(gI1,gI2))*B00(Sqr(
      p),Sqr(MSc(gI1)),Sqr(MSc(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSdSd(gI1,gI2))*B00(Sqr(
      p),Sqr(MSd(gI1)),Sqr(MSd(gI2)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSeSe(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSe(gI1)),Sqr(MSe(gI2)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSmSm(gI1,gI2))*B00(Sqr(p
      ),Sqr(MSm(gI1)),Sqr(MSm(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSsSs(gI1,gI2))*B00(Sqr(
      p),Sqr(MSs(gI1)),Sqr(MSs(gI2)))));
   result += -4*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjStauStau(gI1,gI2))*B00(
      Sqr(p),Sqr(MStau(gI1)),Sqr(MStau(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjStSt(gI1,gI2))*B00(Sqr(
      p),Sqr(MSt(gI1)),Sqr(MSt(gI2)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpVZconjSuSu(gI1,gI2))*B00(Sqr(
      p),Sqr(MSu(gI1)),Sqr(MSu(gI2)))));
   result += SUM(gI1,0,1,SUM(gI2,0,1,(AbsSqr(CpVZbarChaChaPL(gI1,gI2)) + AbsSqr
      (CpVZbarChaChaPR(gI1,gI2)))*H0(Sqr(p),Sqr(MCha(gI1)),Sqr(MCha(gI2))) + 4*B0(
      Sqr(p),Sqr(MCha(gI1)),Sqr(MCha(gI2)))*MCha(gI1)*MCha(gI2)*Re(Conj(
      CpVZbarChaChaPL(gI1,gI2))*CpVZbarChaChaPR(gI1,gI2))));
   result += 0.5*SUM(gI1,0,3,SUM(gI2,0,3,(AbsSqr(CpVZChiChiPL(gI1,gI2)) +
      AbsSqr(CpVZChiChiPR(gI1,gI2)))*H0(Sqr(p),Sqr(MChi(gI1)),Sqr(MChi(gI2))) + 4*
      B0(Sqr(p),Sqr(MChi(gI1)),Sqr(MChi(gI2)))*MChi(gI1)*MChi(gI2)*Re(Conj(
      CpVZChiChiPL(gI1,gI2))*CpVZChiChiPR(gI1,gI2))));
   result += SUM(gI2,0,1,AbsSqr(CpVZVZhh(gI2))*B0(Sqr(p),Sqr(MVZ),Sqr(Mhh(gI2))
      ));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_VWm_1loop_heavy(double p ) const
{
   std::complex<double> result;

   result += A0(Sqr(MSveL))*CpVWmconjVWmconjSveLSveL();
   result += A0(Sqr(MSvmL))*CpVWmconjVWmconjSvmLSvmL();
   result += A0(Sqr(MSvtL))*CpVWmconjVWmconjSvtLSvtL();
   result += 3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpVWmconjVWmconjSbSb(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpVWmconjVWmconjScSc(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpVWmconjVWmconjSdSd(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpVWmconjVWmconjSeSe(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpVWmconjVWmconjSmSm(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpVWmconjVWmconjSsSs(gI1,gI1));
   result += SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpVWmconjVWmconjStauStau(gI1,gI1))
      ;
   result += 3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpVWmconjVWmconjStSt(gI1,gI1));
   result += 3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpVWmconjVWmconjSuSu(gI1,gI1));
   result += 0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpVWmconjVWmhhhh(gI1,gI1));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjVWmconjScSs(gI1,gI2))*B00
      (Sqr(p),Sqr(MSs(gI2)),Sqr(MSc(gI1)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjVWmconjStSb(gI1,gI2))*B00
      (Sqr(p),Sqr(MSb(gI2)),Sqr(MSt(gI1)))));
   result += -12*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpconjVWmconjSuSd(gI1,gI2))*B00
      (Sqr(p),Sqr(MSd(gI2)),Sqr(MSu(gI1)))));
   result += SUM(gI1,0,3,SUM(gI2,0,1,(AbsSqr(CpconjVWmChiChaPL(gI1,gI2)) +
      AbsSqr(CpconjVWmChiChaPR(gI1,gI2)))*H0(Sqr(p),Sqr(MChi(gI1)),Sqr(MCha(gI2)))
      + 4*B0(Sqr(p),Sqr(MChi(gI1)),Sqr(MCha(gI2)))*MCha(gI2)*MChi(gI1)*Re(Conj(
      CpconjVWmChiChaPL(gI1,gI2))*CpconjVWmChiChaPR(gI1,gI2))));
   result += -4*SUM(gI2,0,1,AbsSqr(CpconjVWmconjSveLSe(gI2))*B00(Sqr(p),Sqr(MSe
      (gI2)),Sqr(MSveL)));
   result += -4*SUM(gI2,0,1,AbsSqr(CpconjVWmconjSvmLSm(gI2))*B00(Sqr(p),Sqr(MSm
      (gI2)),Sqr(MSvmL)));
   result += -4*SUM(gI2,0,1,AbsSqr(CpconjVWmconjSvtLStau(gI2))*B00(Sqr(p),Sqr(
      MStau(gI2)),Sqr(MSvtL)));
   result += -4*SUM(gI2,0,1,AbsSqr(CpconjVWmHpmhh(2,gI2))*B00(Sqr(p),Sqr(Mhh(
      gI2)),Sqr(MHpm(1))));
   result += SUM(gI2,0,1,AbsSqr(CpconjVWmVWmhh(gI2))*B0(Sqr(p),Sqr(MVWm),Sqr(
      Mhh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fb_1loop_1_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -4*MFb*B0(Sqr(p),Sqr(MFb),Sqr(MVZ))*Conj(CpbarFbVZFbPR())*
      CpbarFbVZFbPL();
   result += -4*MFt*B0(Sqr(p),Sqr(MFt),Sqr(MVWm))*Conj(CpbarFbVWmFtPR(1))*
      CpbarFbVWmFtPL(1);
   result += MFb*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFb),Sqr(Mhh(gI1)))*Conj(
      CpbarFbhhFbPL(gI1))*CpbarFbhhFbPR(gI1));
   result += MFt*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MHpm(gI1)))*Conj(
      CpbarFbHpmFtPL(gI1,1))*CpbarFbHpmFtPR(gI1,1));
   result += 1.3333333333333333*MGlu*SUM(gI1,0,1,B0(Sqr(p),Sqr(MGlu),Sqr(MSb(
      gI1)))*Conj(CpbarFbSbGluPL(gI1,1))*CpbarFbSbGluPR(gI1,1));
   result += SUM(gI1,0,1,SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MSt(gI1)))*
      Conj(CpbarFbStChaPL(gI1,gI2))*CpbarFbStChaPR(gI1,gI2)*MCha(gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSb(gI1)))*
      Conj(CpbarFbSbChiPL(gI1,gI2))*CpbarFbSbChiPR(gI1,gI2)*MChi(gI2)));
   result += MFb*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFb),Sqr(MAh(gI2)))*Conj(
      CpbarFbFbAhPL(gI2))*CpbarFbFbAhPR(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fb_1loop_PR_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFbVWmFtPL(1))*B1(Sqr(p),Sqr(MFt),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFbVZFbPL())*B1(Sqr(p),Sqr(MFb),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFbhhFbPR(gI1))*B1(Sqr(p),Sqr(MFb),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFbHpmFtPR(gI1,1))*B1(Sqr(p),Sqr(MFt),
      Sqr(MHpm(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFbSbGluPR(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSb(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFbStChaPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MCha(gI2)),Sqr(MSt(gI1)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFbSbChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSb(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFbFbAhPR(gI2))*B1(Sqr(p),Sqr(MFb),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fb_1loop_PL_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFbVWmFtPR(1))*B1(Sqr(p),Sqr(MFt),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFbVZFbPR())*B1(Sqr(p),Sqr(MFb),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFbhhFbPL(gI1))*B1(Sqr(p),Sqr(MFb),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFbHpmFtPL(gI1,1))*B1(Sqr(p),Sqr(MFt),
      Sqr(MHpm(gI1))));
   result += -0.6666666666666666*SUM(gI1,0,1,AbsSqr(CpbarFbSbGluPL(gI1,1))*B1(
      Sqr(p),Sqr(MGlu),Sqr(MSb(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFbStChaPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MCha(gI2)),Sqr(MSt(gI1)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFbSbChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSb(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFbFbAhPL(gI2))*B1(Sqr(p),Sqr(MFb),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fe_1loop_1_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -4*MFe*B0(Sqr(p),Sqr(MFe),Sqr(MVZ))*Conj(CpbarFeVZFePR())*
      CpbarFeVZFePL();
   result += -4*MFve*B0(Sqr(p),Sqr(MFve),Sqr(MVWm))*Conj(CpbarFeVWmFvePR())*
      CpbarFeVWmFvePL();
   result += MFe*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFe),Sqr(Mhh(gI1)))*Conj(
      CpbarFehhFePL(gI1))*CpbarFehhFePR(gI1));
   result += MFve*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFve),Sqr(MHpm(gI1)))*Conj(
      CpbarFeHpmFvePL(gI1))*CpbarFeHpmFvePR(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSe(gI1)))*
      Conj(CpbarFeSeChiPL(gI1,gI2))*CpbarFeSeChiPR(gI1,gI2)*MChi(gI2)));
   result += MFe*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFe),Sqr(MAh(gI2)))*Conj(
      CpbarFeFeAhPL(gI2))*CpbarFeFeAhPR(gI2));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MSveL))*Conj(
      CpbarFeSveLChaPL(gI2))*CpbarFeSveLChaPR(gI2)*MCha(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fe_1loop_PR_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFeVWmFvePL())*B1(Sqr(p),Sqr(MFve),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFeVZFePL())*B1(Sqr(p),Sqr(MFe),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFehhFePR(gI1))*B1(Sqr(p),Sqr(MFe),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFeHpmFvePR(gI1))*B1(Sqr(p),Sqr(MFve),
      Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFeSeChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSe(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFeFeAhPR(gI2))*B1(Sqr(p),Sqr(MFe),Sqr
      (MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFeSveLChaPR(gI2))*B1(Sqr(p),Sqr(MCha(
      gI2)),Sqr(MSveL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fe_1loop_PL_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFeVWmFvePR())*B1(Sqr(p),Sqr(MFve),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFeVZFePR())*B1(Sqr(p),Sqr(MFe),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFehhFePL(gI1))*B1(Sqr(p),Sqr(MFe),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFeHpmFvePL(gI1))*B1(Sqr(p),Sqr(MFve),
      Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFeSeChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSe(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFeFeAhPL(gI2))*B1(Sqr(p),Sqr(MFe),Sqr
      (MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFeSveLChaPL(gI2))*B1(Sqr(p),Sqr(MCha(
      gI2)),Sqr(MSveL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fm_1loop_1_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -4*MFm*B0(Sqr(p),Sqr(MFm),Sqr(MVZ))*Conj(CpbarFmVZFmPR())*
      CpbarFmVZFmPL();
   result += -4*MFvm*B0(Sqr(p),Sqr(MFvm),Sqr(MVWm))*Conj(CpbarFmVWmFvmPR())*
      CpbarFmVWmFvmPL();
   result += MFm*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFm),Sqr(Mhh(gI1)))*Conj(
      CpbarFmhhFmPL(gI1))*CpbarFmhhFmPR(gI1));
   result += MFvm*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFvm),Sqr(MHpm(gI1)))*Conj(
      CpbarFmHpmFvmPL(gI1))*CpbarFmHpmFvmPR(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSm(gI1)))*
      Conj(CpbarFmSmChiPL(gI1,gI2))*CpbarFmSmChiPR(gI1,gI2)*MChi(gI2)));
   result += MFm*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFm),Sqr(MAh(gI2)))*Conj(
      CpbarFmFmAhPL(gI2))*CpbarFmFmAhPR(gI2));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MSvmL))*Conj(
      CpbarFmSvmLChaPL(gI2))*CpbarFmSvmLChaPR(gI2)*MCha(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fm_1loop_PR_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFmVWmFvmPL())*B1(Sqr(p),Sqr(MFvm),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFmVZFmPL())*B1(Sqr(p),Sqr(MFm),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFmhhFmPR(gI1))*B1(Sqr(p),Sqr(MFm),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFmHpmFvmPR(gI1))*B1(Sqr(p),Sqr(MFvm),
      Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFmSmChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSm(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFmFmAhPR(gI2))*B1(Sqr(p),Sqr(MFm),Sqr
      (MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFmSvmLChaPR(gI2))*B1(Sqr(p),Sqr(MCha(
      gI2)),Sqr(MSvmL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Fm_1loop_PL_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFmVWmFvmPR())*B1(Sqr(p),Sqr(MFvm),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFmVZFmPR())*B1(Sqr(p),Sqr(MFm),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFmhhFmPL(gI1))*B1(Sqr(p),Sqr(MFm),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFmHpmFvmPL(gI1))*B1(Sqr(p),Sqr(MFvm),
      Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFmSmChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSm(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFmFmAhPL(gI2))*B1(Sqr(p),Sqr(MFm),Sqr
      (MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFmSvmLChaPL(gI2))*B1(Sqr(p),Sqr(MCha(
      gI2)),Sqr(MSvmL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ftau_1loop_1_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -4*MFtau*B0(Sqr(p),Sqr(MFtau),Sqr(MVZ))*Conj(CpbarFtauVZFtauPR())*
      CpbarFtauVZFtauPL();
   result += -4*MFvt*B0(Sqr(p),Sqr(MFvt),Sqr(MVWm))*Conj(CpbarFtauVWmFvtPR())*
      CpbarFtauVWmFvtPL();
   result += MFtau*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFtau),Sqr(Mhh(gI1)))*Conj(
      CpbarFtauhhFtauPL(gI1))*CpbarFtauhhFtauPR(gI1));
   result += MFvt*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFvt),Sqr(MHpm(gI1)))*Conj(
      CpbarFtauHpmFvtPL(gI1))*CpbarFtauHpmFvtPR(gI1));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MStau(gI1)))*
      Conj(CpbarFtauStauChiPL(gI1,gI2))*CpbarFtauStauChiPR(gI1,gI2)*MChi(gI2)));
   result += MFtau*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFtau),Sqr(MAh(gI2)))*Conj(
      CpbarFtauFtauAhPL(gI2))*CpbarFtauFtauAhPR(gI2));
   result += SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI2)),Sqr(MSvtL))*Conj(
      CpbarFtauSvtLChaPL(gI2))*CpbarFtauSvtLChaPR(gI2)*MCha(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ftau_1loop_PR_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFtauVWmFvtPL())*B1(Sqr(p),Sqr(MFvt),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFtauVZFtauPL())*B1(Sqr(p),Sqr(MFtau),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtauhhFtauPR(gI1))*B1(Sqr(p),Sqr(
      MFtau),Sqr(Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtauHpmFvtPR(gI1))*B1(Sqr(p),Sqr(MFvt
      ),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFtauStauChiPR(gI1,gI2))*
      B1(Sqr(p),Sqr(MChi(gI2)),Sqr(MStau(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtauFtauAhPR(gI2))*B1(Sqr(p),Sqr(
      MFtau),Sqr(MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtauSvtLChaPR(gI2))*B1(Sqr(p),Sqr(
      MCha(gI2)),Sqr(MSvtL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ftau_1loop_PL_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFtauVWmFvtPR())*B1(Sqr(p),Sqr(MFvt),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFtauVZFtauPR())*B1(Sqr(p),Sqr(MFtau),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtauhhFtauPL(gI1))*B1(Sqr(p),Sqr(
      MFtau),Sqr(Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtauHpmFvtPL(gI1))*B1(Sqr(p),Sqr(MFvt
      ),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFtauStauChiPL(gI1,gI2))*
      B1(Sqr(p),Sqr(MChi(gI2)),Sqr(MStau(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtauFtauAhPL(gI2))*B1(Sqr(p),Sqr(
      MFtau),Sqr(MAh(gI2))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtauSvtLChaPL(gI2))*B1(Sqr(p),Sqr(
      MCha(gI2)),Sqr(MSvtL)));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ft_1loop_1_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -4*MFb*B0(Sqr(p),Sqr(MFb),Sqr(MVWm))*Conj(CpbarFtconjVWmFbPR(1))*
      CpbarFtconjVWmFbPL(1);
   result += -4*MFt*B0(Sqr(p),Sqr(MFt),0)*Conj(CpbarFtVPFtPR())*CpbarFtVPFtPL()
      ;
   result += -4*MFt*B0(Sqr(p),Sqr(MFt),Sqr(MVZ))*Conj(CpbarFtVZFtPR())*
      CpbarFtVZFtPL();
   result += MFb*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFb),Sqr(MHpm(gI1)))*Conj(
      CpbarFtconjHpmFbPL(gI1,1))*CpbarFtconjHpmFbPR(gI1,1));
   result += MFt*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFt),Sqr(Mhh(gI1)))*Conj(
      CpbarFthhFtPL(gI1))*CpbarFthhFtPR(gI1));
   result += SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MSb
      (gI2)))*Conj(CpbarFtbarChaSbPL(gI1,gI2))*CpbarFtbarChaSbPR(gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSt(gI1)))*
      Conj(CpbarFtStChiPL(gI1,gI2))*CpbarFtStChiPR(gI1,gI2)*MChi(gI2)));
   result += MFt*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MAh(gI2)))*Conj(
      CpbarFtFtAhPL(gI2))*CpbarFtFtAhPR(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ft_1loop_PR_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFtconjVWmFbPL(1))*B1(Sqr(p),Sqr(MFb),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFtVPFtPL())*B1(Sqr(p),Sqr(MFt),0));
   result += -(AbsSqr(CpbarFtVZFtPL())*B1(Sqr(p),Sqr(MFt),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtconjHpmFbPR(gI1,1))*B1(Sqr(p),Sqr(
      MFb),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFthhFtPR(gI1))*B1(Sqr(p),Sqr(MFt),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFtbarChaSbPR(gI1,gI2))*B1
      (Sqr(p),Sqr(MCha(gI1)),Sqr(MSb(gI2)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFtStChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSt(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtFtAhPR(gI2))*B1(Sqr(p),Sqr(MFt),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ft_1loop_PL_heavy_rotated(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFtconjVWmFbPR(1))*B1(Sqr(p),Sqr(MFb),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFtVPFtPR())*B1(Sqr(p),Sqr(MFt),0));
   result += -(AbsSqr(CpbarFtVZFtPR())*B1(Sqr(p),Sqr(MFt),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtconjHpmFbPL(gI1,1))*B1(Sqr(p),Sqr(
      MFb),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFthhFtPL(gI1))*B1(Sqr(p),Sqr(MFt),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFtbarChaSbPL(gI1,gI2))*B1
      (Sqr(p),Sqr(MCha(gI1)),Sqr(MSb(gI2)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFtStChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSt(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtFtAhPL(gI2))*B1(Sqr(p),Sqr(MFt),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ft_1loop_1_heavy(double p ) const
{
   std::complex<double> result;

   result += -4*MFb*B0(Sqr(p),Sqr(MFb),Sqr(MVWm))*Conj(CpbarFtconjVWmFbPR(1))*
      CpbarFtconjVWmFbPL(1);
   result += -4*MFt*B0(Sqr(p),Sqr(MFt),0)*Conj(CpbarFtVPFtPR())*CpbarFtVPFtPL()
      ;
   result += -4*MFt*B0(Sqr(p),Sqr(MFt),Sqr(MVZ))*Conj(CpbarFtVZFtPR())*
      CpbarFtVZFtPL();
   result += MFb*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFb),Sqr(MHpm(gI1)))*Conj(
      CpbarFtconjHpmFbPL(gI1,1))*CpbarFtconjHpmFbPR(gI1,1));
   result += MFt*SUM(gI1,0,1,B0(Sqr(p),Sqr(MFt),Sqr(Mhh(gI1)))*Conj(
      CpbarFthhFtPL(gI1))*CpbarFthhFtPR(gI1));
   result += SUM(gI1,0,1,MCha(gI1)*SUM(gI2,0,1,B0(Sqr(p),Sqr(MCha(gI1)),Sqr(MSb
      (gI2)))*Conj(CpbarFtbarChaSbPL(gI1,gI2))*CpbarFtbarChaSbPR(gI1,gI2)));
   result += SUM(gI1,0,1,SUM(gI2,0,3,B0(Sqr(p),Sqr(MChi(gI2)),Sqr(MSt(gI1)))*
      Conj(CpbarFtStChiPL(gI1,gI2))*CpbarFtStChiPR(gI1,gI2)*MChi(gI2)));
   result += MFt*SUM(gI2,0,1,B0(Sqr(p),Sqr(MFt),Sqr(MAh(gI2)))*Conj(
      CpbarFtFtAhPL(gI2))*CpbarFtFtAhPR(gI2));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ft_1loop_PR_heavy(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFtconjVWmFbPL(1))*B1(Sqr(p),Sqr(MFb),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFtVPFtPL())*B1(Sqr(p),Sqr(MFt),0));
   result += -(AbsSqr(CpbarFtVZFtPL())*B1(Sqr(p),Sqr(MFt),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtconjHpmFbPR(gI1,1))*B1(Sqr(p),Sqr(
      MFb),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFthhFtPR(gI1))*B1(Sqr(p),Sqr(MFt),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFtbarChaSbPR(gI1,gI2))*B1
      (Sqr(p),Sqr(MCha(gI1)),Sqr(MSb(gI2)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFtStChiPR(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSt(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtFtAhPR(gI2))*B1(Sqr(p),Sqr(MFt),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::self_energy_Ft_1loop_PL_heavy(double p ) const
{
   std::complex<double> result;

   result += -(AbsSqr(CpbarFtconjVWmFbPR(1))*B1(Sqr(p),Sqr(MFb),Sqr(MVWm)));
   result += -(AbsSqr(CpbarFtVPFtPR())*B1(Sqr(p),Sqr(MFt),0));
   result += -(AbsSqr(CpbarFtVZFtPR())*B1(Sqr(p),Sqr(MFt),Sqr(MVZ)));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFtconjHpmFbPL(gI1,1))*B1(Sqr(p),Sqr(
      MFb),Sqr(MHpm(gI1))));
   result += -0.5*SUM(gI1,0,1,AbsSqr(CpbarFthhFtPL(gI1))*B1(Sqr(p),Sqr(MFt),Sqr
      (Mhh(gI1))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,1,AbsSqr(CpbarFtbarChaSbPL(gI1,gI2))*B1
      (Sqr(p),Sqr(MCha(gI1)),Sqr(MSb(gI2)))));
   result += -0.5*SUM(gI1,0,1,SUM(gI2,0,3,AbsSqr(CpbarFtStChiPL(gI1,gI2))*B1(
      Sqr(p),Sqr(MChi(gI2)),Sqr(MSt(gI1)))));
   result += -0.5*SUM(gI2,0,1,AbsSqr(CpbarFtFtAhPL(gI2))*B1(Sqr(p),Sqr(MFt),Sqr
      (MAh(gI2))));

   return result * oneOver16PiSqr;

}

std::complex<double> CLASSNAME::tadpole_hh_1loop(int gO1) const
{
   std::complex<double> result;

   result += A0(Sqr(MVWm))*CpUhhbargWmCgWmC(gO1);
   result += A0(Sqr(MVWm))*CpUhhbargWmgWm(gO1);
   result += A0(Sqr(MVZ))*CpUhhbargZgZ(gO1);
   result += -(A0(Sqr(MSveL))*CpUhhconjSveLSveL(gO1));
   result += -(A0(Sqr(MSvmL))*CpUhhconjSvmLSvmL(gO1));
   result += -(A0(Sqr(MSvtL))*CpUhhconjSvtLSvtL(gO1));
   result += 4*A0(Sqr(MVWm))*CpUhhconjVWmVWm(gO1);
   result += 2*A0(Sqr(MVZ))*CpUhhVZVZ(gO1);
   result += 6*MFb*A0(Sqr(MFb))*(CpUhhbarFbFbPL(gO1,1,1) + CpUhhbarFbFbPR(gO1,1
      ,1));
   result += 6*MFc*A0(Sqr(MFc))*(CpUhhbarFcFcPL(gO1,1,1) + CpUhhbarFcFcPR(gO1,1
      ,1));
   result += 6*MFd*A0(Sqr(MFd))*(CpUhhbarFdFdPL(gO1,1,1) + CpUhhbarFdFdPR(gO1,1
      ,1));
   result += 2*MFe*A0(Sqr(MFe))*(CpUhhbarFeFePL(gO1) + CpUhhbarFeFePR(gO1));
   result += 2*MFm*A0(Sqr(MFm))*(CpUhhbarFmFmPL(gO1) + CpUhhbarFmFmPR(gO1));
   result += 6*MFs*A0(Sqr(MFs))*(CpUhhbarFsFsPL(gO1,1,1) + CpUhhbarFsFsPR(gO1,1
      ,1));
   result += 6*MFt*A0(Sqr(MFt))*(CpUhhbarFtFtPL(gO1,1,1) + CpUhhbarFtFtPR(gO1,1
      ,1));
   result += 2*MFtau*A0(Sqr(MFtau))*(CpUhhbarFtauFtauPL(gO1) +
      CpUhhbarFtauFtauPR(gO1));
   result += 6*MFu*A0(Sqr(MFu))*(CpUhhbarFuFuPL(gO1,1,1) + CpUhhbarFuFuPR(gO1,1
      ,1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(MAh(gI1)))*CpUhhAhAh(gO1,gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MHpm(gI1)))*CpUhhconjHpmHpm(gO1,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSb(gI1)))*CpUhhconjSbSb(gO1,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSc(gI1)))*CpUhhconjScSc(gO1,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSd(gI1)))*CpUhhconjSdSd(gO1,gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSe(gI1)))*CpUhhconjSeSe(gO1,gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MSm(gI1)))*CpUhhconjSmSm(gO1,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSs(gI1)))*CpUhhconjSsSs(gO1,gI1,gI1));
   result += -SUM(gI1,0,1,A0(Sqr(MStau(gI1)))*CpUhhconjStauStau(gO1,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSt(gI1)))*CpUhhconjStSt(gO1,gI1,gI1));
   result += -3*SUM(gI1,0,1,A0(Sqr(MSu(gI1)))*CpUhhconjSuSu(gO1,gI1,gI1));
   result += -0.5*SUM(gI1,0,1,A0(Sqr(Mhh(gI1)))*CpUhhhhhh(gO1,gI1,gI1));
   result += 2*SUM(gI1,0,1,A0(Sqr(MCha(gI1)))*(CpUhhbarChaChaPL(gO1,gI1,gI1) +
      CpUhhbarChaChaPR(gO1,gI1,gI1))*MCha(gI1));
   result += SUM(gI1,0,3,A0(Sqr(MChi(gI1)))*(CpUhhChiChiPL(gO1,gI1,gI1) +
      CpUhhChiChiPR(gO1,gI1,gI1))*MChi(gI1));

   return result * oneOver16PiSqr;

}


void CLASSNAME::calculate_MTopSquark_3rd_generation(double& msf1, double& msf2, double& theta) const {
   sfermions::Mass_data sf_data;
   sf_data.ml2 = Re(mq2(2,2));
   sf_data.mr2 = Re(mu2(2,2));
   sf_data.yf  = Re(Yu(2,2));
   sf_data.vd  = Re(vd);
   sf_data.vu  = Re(vu);
   sf_data.gY  = 0.7745966692414834*g1;
   sf_data.g2  = g2;
   sf_data.Tyf = Re(TYu(2,2));
   sf_data.mu  = Re(Mu);
   sf_data.T3  = sfermions::Isospin[sfermions::up];
   sf_data.Yl  = sfermions::Hypercharge_left[sfermions::up];
   sf_data.Yr  = sfermions::Hypercharge_right[sfermions::up];

   Eigen::Array<double,2,1> msf;

   theta = sfermions::diagonalize_sfermions_2x2(sf_data, msf);
   msf1  = msf(0);
   msf2  = msf(1);
}

void CLASSNAME::calculate_MBottomSquark_3rd_generation(double& msf1, double& msf2, double& theta) const {
   sfermions::Mass_data sf_data;
   sf_data.ml2 = Re(mq2(2,2));
   sf_data.mr2 = Re(md2(2,2));
   sf_data.yf  = Re(Yd(2,2));
   sf_data.vd  = Re(vd);
   sf_data.vu  = Re(vu);
   sf_data.gY  = 0.7745966692414834*g1;
   sf_data.g2  = g2;
   sf_data.Tyf = Re(TYd(2,2));
   sf_data.mu  = Re(Mu);
   sf_data.T3  = sfermions::Isospin[sfermions::down];
   sf_data.Yl  = sfermions::Hypercharge_left[sfermions::down];
   sf_data.Yr  = sfermions::Hypercharge_right[sfermions::down];

   Eigen::Array<double,2,1> msf;

   theta = sfermions::diagonalize_sfermions_2x2(sf_data, msf);
   msf1  = msf(0);
   msf2  = msf(1);
}

void CLASSNAME::calculate_MSneutrino_3rd_generation(double& msf1, double& msf2, double& theta) const {
   sfermions::Mass_data sf_data;
   sf_data.ml2 = Re(ml2(2,2));
   sf_data.mr2 = 0.;
   sf_data.yf  = 0.;
   sf_data.vd  = Re(vd);
   sf_data.vu  = Re(vu);
   sf_data.gY  = 0.7745966692414834*g1;
   sf_data.g2  = g2;
   sf_data.Tyf = 0.;
   sf_data.mu  = Re(Mu);
   sf_data.T3  = sfermions::Isospin[sfermions::neutrino];
   sf_data.Yl  = sfermions::Hypercharge_left[sfermions::neutrino];
   sf_data.Yr  = sfermions::Hypercharge_right[sfermions::neutrino];

   Eigen::Array<double,2,1> msf;

   theta = sfermions::diagonalize_sfermions_2x2(sf_data, msf);
   msf1  = msf(0);
   msf2  = msf(1);
}

void CLASSNAME::calculate_MSelectron_3rd_generation(double& msf1, double& msf2, double& theta) const {
   sfermions::Mass_data sf_data;
   sf_data.ml2 = Re(ml2(2,2));
   sf_data.mr2 = Re(me2(2,2));
   sf_data.yf  = Re(Ye(2,2));
   sf_data.vd  = Re(vd);
   sf_data.vu  = Re(vu);
   sf_data.gY  = 0.7745966692414834*g1;
   sf_data.g2  = g2;
   sf_data.Tyf = Re(TYe(2,2));
   sf_data.mu  = Re(Mu);
   sf_data.T3  = sfermions::Isospin[sfermions::electron];
   sf_data.Yl  = sfermions::Hypercharge_left[sfermions::electron];
   sf_data.Yr  = sfermions::Hypercharge_right[sfermions::electron];

   Eigen::Array<double,2,1> msf;

   theta = sfermions::diagonalize_sfermions_2x2(sf_data, msf);
   msf1  = msf(0);
   msf2  = msf(1);
}


Eigen::Matrix<double,2,2> CLASSNAME::self_energy_hh_2loop() const
{
   using namespace flexiblesusy::mssm_twoloophiggs;

   // calculate 3rd generation sfermion masses and mixing angles
   double mst_1, mst_2, theta_t;
   double msb_1, msb_2, theta_b;
   double mstau_1, mstau_2, theta_tau;
   double msnu_1, msnu_2, theta_nu;

   calculate_MTopSquark_3rd_generation(mst_1, mst_2, theta_t);
   calculate_MBottomSquark_3rd_generation(msb_1, msb_2, theta_b);
   calculate_MSelectron_3rd_generation(mstau_1, mstau_2, theta_tau);
   calculate_MSneutrino_3rd_generation(msnu_1, msnu_2, theta_nu);

   const double mst1sq = Sqr(mst_1), mst2sq = Sqr(mst_2);
   const double msb1sq = Sqr(msb_1), msb2sq = Sqr(msb_2);
   const double mstau1sq = Sqr(mstau_1), mstau2sq = Sqr(mstau_2);
   const double msnusq = Sqr(msnu_2);
   const double sxt = Sin(theta_t), cxt = Cos(theta_t);
   const double sxb = Sin(theta_b), cxb = Cos(theta_b);
   const double sintau = Sin(theta_tau), costau = Cos(theta_tau);
   const double gs = g3;
   const double rmtsq = Sqr(MFt);
   const double scalesq = Sqr(get_scale());
   const double vev2 = Sqr(vd) + Sqr(vu);
   const double tanb = vu/vd;
   const double amu = Re(-Mu);
   const double mg = MGlu;
   const double mAsq = Sqr(get_MPseudoscalarHiggs()(0));
   const double cotbeta = 1.0 / tanb;
   const double rmbsq = Sqr(MFb);
   const double rmtausq = Sqr(MFtau);

   Eigen::Matrix<double,2,2> self_energy_2l(Eigen::Matrix<double,2,2>::Zero());

   if (HIGGS_2LOOP_CORRECTION_AT_AS) {
      self_energy_2l += self_energy_higgs_2loop_at_as_mssm(
         rmtsq, mg, mst1sq, mst2sq, sxt, cxt, scalesq, amu,
         tanb, vev2, gs);
   }

   if (HIGGS_2LOOP_CORRECTION_AB_AS) {
      self_energy_2l += self_energy_higgs_2loop_ab_as_mssm(
         rmbsq, mg, msb1sq, msb2sq, sxb, cxb, scalesq, amu,
         cotbeta, vev2, gs);
   }

   if (HIGGS_2LOOP_CORRECTION_AT_AT) {
      self_energy_2l += self_energy_higgs_2loop_at_at_mssm(
         rmtsq, rmbsq, mAsq, mst1sq, mst2sq, msb1sq, msb2sq,
         sxt, cxt, sxb, cxb, scalesq, amu, tanb, vev2);
   }

   if (HIGGS_2LOOP_CORRECTION_ATAU_ATAU) {
      self_energy_2l += self_energy_higgs_2loop_atau_atau_mssm(
         rmtausq, mAsq, msnusq, mstau1sq, mstau2sq, sintau,
         costau, scalesq, amu, tanb, vev2);
   }

   return self_energy_2l;
}

Eigen::Matrix<double,2,2> CLASSNAME::self_energy_Ah_2loop() const
{
   using namespace flexiblesusy::mssm_twoloophiggs;

   // calculate 3rd generation sfermion masses and mixing angles
   double mst_1, mst_2, theta_t;
   double msb_1, msb_2, theta_b;
   double mstau_1, mstau_2, theta_tau;
   double msnu_1, msnu_2, theta_nu;

   calculate_MTopSquark_3rd_generation(mst_1, mst_2, theta_t);
   calculate_MBottomSquark_3rd_generation(msb_1, msb_2, theta_b);
   calculate_MSelectron_3rd_generation(mstau_1, mstau_2, theta_tau);
   calculate_MSneutrino_3rd_generation(msnu_1, msnu_2, theta_nu);

   const double mst1sq = Sqr(mst_1), mst2sq = Sqr(mst_2);
   const double msb1sq = Sqr(msb_1), msb2sq = Sqr(msb_2);
   const double mstau1sq = Sqr(mstau_1), mstau2sq = Sqr(mstau_2);
   const double msnusq = Sqr(msnu_2);
   const double sxt = Sin(theta_t), cxt = Cos(theta_t);
   const double sxb = Sin(theta_b), cxb = Cos(theta_b);
   const double sintau = Sin(theta_tau), costau = Cos(theta_tau);
   const double gs = g3;
   const double rmtsq = Sqr(MFt);
   const double scalesq = Sqr(get_scale());
   const double vev2 = Sqr(vd) + Sqr(vu);
   const double tanb = vu/vd;
   const double amu = Re(-Mu);
   const double mg = MGlu;
   const double mAsq = Sqr(get_MPseudoscalarHiggs()(0));
   const double cotbeta = 1.0 / tanb;
   const double rmbsq = Sqr(MFb);
   const double rmtausq = Sqr(MFtau);

   Eigen::Matrix<double,2,2> self_energy_2l(Eigen::Matrix<double,2,2>::Zero());

   if (HIGGS_2LOOP_CORRECTION_AT_AS) {
      self_energy_2l += self_energy_pseudoscalar_2loop_at_as_mssm(
         rmtsq, mg, mst1sq, mst2sq, sxt, cxt, scalesq, amu,
         tanb, vev2, gs);
   }

   if (HIGGS_2LOOP_CORRECTION_AB_AS) {
      self_energy_2l += self_energy_pseudoscalar_2loop_ab_as_mssm(
         rmbsq, mg, msb1sq, msb2sq, sxb, cxb, scalesq, amu,
         cotbeta, vev2, gs);
   }

   if (HIGGS_2LOOP_CORRECTION_AT_AT) {
      self_energy_2l += self_energy_pseudoscalar_2loop_at_at_mssm(
         rmtsq, rmbsq, mAsq, mst1sq, mst2sq, msb1sq, msb2sq,
         sxt, cxt, sxb, cxb, scalesq, amu, tanb, vev2);
   }

   if (HIGGS_2LOOP_CORRECTION_ATAU_ATAU) {
      self_energy_2l += self_energy_pseudoscalar_2loop_atau_atau_mssm(
         rmtausq, mAsq, msnusq, mstau1sq, mstau2sq, sintau,
         costau, scalesq, amu, tanb, vev2);
   }

   return self_energy_2l;
}



Eigen::Matrix<double,2,1> CLASSNAME::tadpole_hh_2loop() const
{
   using namespace flexiblesusy::mssm_twoloophiggs;

   // calculate 3rd generation sfermion masses and mixing angles
   double mst_1, mst_2, theta_t;
   double msb_1, msb_2, theta_b;
   double mstau_1, mstau_2, theta_tau;
   double msnu_1, msnu_2, theta_nu;

   calculate_MTopSquark_3rd_generation(mst_1, mst_2, theta_t);
   calculate_MBottomSquark_3rd_generation(msb_1, msb_2, theta_b);
   calculate_MSelectron_3rd_generation(mstau_1, mstau_2, theta_tau);
   calculate_MSneutrino_3rd_generation(msnu_1, msnu_2, theta_nu);

   const double mst1sq = Sqr(mst_1), mst2sq = Sqr(mst_2);
   const double msb1sq = Sqr(msb_1), msb2sq = Sqr(msb_2);
   const double mstau1sq = Sqr(mstau_1), mstau2sq = Sqr(mstau_2);
   const double msnusq = Sqr(msnu_2);
   const double sxt = Sin(theta_t), cxt = Cos(theta_t);
   const double sxb = Sin(theta_b), cxb = Cos(theta_b);
   const double sintau = Sin(theta_tau), costau = Cos(theta_tau);
   const double gs = g3;
   const double rmtsq = Sqr(MFt);
   const double scalesq = Sqr(get_scale());
   const double vev2 = Sqr(vd) + Sqr(vu);
   const double tanb = vu/vd;
   const double amu = Re(-Mu);
   const double mg = MGlu;
   const double mAsq = Sqr(get_MPseudoscalarHiggs()(0));
   const double cotbeta = 1.0 / tanb;
   const double rmbsq = Sqr(MFb);
   const double rmtausq = Sqr(MFtau);

   Eigen::Matrix<double,2,1> tadpole_2l(Eigen::Matrix<double,2,1>::Zero());

   if (HIGGS_2LOOP_CORRECTION_AT_AS) {
      tadpole_2l += tadpole_higgs_2loop_at_as_mssm(
         rmtsq, mg, mst1sq, mst2sq, sxt, cxt, scalesq,
         amu, tanb, vev2, gs);
   }

   if (HIGGS_2LOOP_CORRECTION_AT_AT) {
      tadpole_2l += tadpole_higgs_2loop_at_at_mssm(
         rmtsq, rmbsq, mAsq, mst1sq, mst2sq, msb1sq, msb2sq,
         sxt, cxt, sxb, cxb, scalesq, amu, tanb, vev2);
   }

   if (HIGGS_2LOOP_CORRECTION_AB_AS) {
      tadpole_2l += tadpole_higgs_2loop_ab_as_mssm(
         rmbsq, mg, msb1sq, msb2sq, sxb, cxb, scalesq,
         amu, cotbeta, vev2, gs);
   }

   if (HIGGS_2LOOP_CORRECTION_ATAU_ATAU) {
      tadpole_2l += tadpole_higgs_2loop_atau_atau_mssm(
         rmtausq, mAsq, msnusq, mstau1sq, mstau2sq, sintau,
         costau, scalesq, amu, tanb, vev2);
   }

   tadpole_2l(0) *= vd;
   tadpole_2l(1) *= vu;

   return tadpole_2l;
}


Eigen::Matrix<double,2,2> CLASSNAME::self_energy_hh_3loop() const
{
   Eigen::Matrix<double,2,2> self_energy_3l(Eigen::Matrix<double,2,2>::Zero());

   #ifdef ENABLE_HIMALAYA
   // calculate 3rd generation sfermion masses and mixing angles
   double mst_1, mst_2, theta_t;
   double msb_1, msb_2, theta_b;

   calculate_MTopSquark_3rd_generation(mst_1, mst_2, theta_t);
   calculate_MBottomSquark_3rd_generation(msb_1, msb_2, theta_b);

   himalaya::Parameters pars;
   pars.scale = get_scale();
   pars.mu = Re(Mu);
   pars.g3 = g3;
   pars.vd = vd;
   pars.vu = vu;
   pars.mq2 = Re(mq2);
   pars.md2 = Re(md2);
   pars.mu2 = Re(mu2);
   pars.At = Re(TYu(2,2)/Yu(2,2));
   pars.Ab = Re(TYd(2,2)/Yd(2,2));
   pars.MG = MGlu;
   pars.MW = MVWm;
   pars.MZ = MVZ;
   pars.Mt = MFt;
   pars.Mb = MFb;
   pars.MA = get_MPseudoscalarHiggs()(0);
   pars.MSt << mst_1, mst_2;
   pars.MSb << msb_1, msb_2;
   pars.s2t = Sin(2*theta_t);
   pars.s2b = Sin(2*theta_b);

   if (pars.MSt(0) > pars.MSt(1)) {
      std::swap(pars.MSt(0), pars.MSt(1));
      pars.s2t *= -1;
   }

   if (pars.MSb(0) > pars.MSb(1)) {
      std::swap(pars.MSb(0), pars.MSb(1));
      pars.s2b *= -1;
   }

   try {
      const auto mdrScheme = HIGGS_3LOOP_MDR_SCHEME;
      const bool verbose = false;
      himalaya::HierarchyCalculator hc(pars, verbose);

      if (HIGGS_3LOOP_CORRECTION_AT_AS_AS) {
         const auto hier = hc.calculateDMh3L(false, mdrScheme);

         VERBOSE_MSG("Himalaya top (hierarchy, uncertainties) = ("
                     << hier.getSuitableHierarchy() << ", {"
                     << hier.getExpUncertainty(1) << ", "
                     << hier.getExpUncertainty(2) << ", "
                     << hier.getExpUncertainty(3) << "})");

         // calculate the 3-loop corrections
         self_energy_3l += - hier.getDMh(3);

         if (mdrScheme) {
            // calculate the 1- and 2-loop shift DR -> MDR
            self_energy_3l += - hier.getDRToMDRShift();
         }
      }

      if (HIGGS_3LOOP_CORRECTION_AB_AS_AS) {
         const auto hier = hc.calculateDMh3L(true, mdrScheme);

         VERBOSE_MSG("Himalaya bottom (hierarchy, uncertainties) = ("
                     << hier.getSuitableHierarchy() << ", {"
                     << hier.getExpUncertainty(1) << ", "
                     << hier.getExpUncertainty(2) << ", "
                     << hier.getExpUncertainty(3) << "})");

         // calculate the 3-loop corrections
         self_energy_3l += - hier.getDMh(3);

         if (mdrScheme) {
            // calculate the 1- and 2-loop shift DR -> MDR
            self_energy_3l += - hier.getDRToMDRShift();
         }
      }
   } catch (const std::exception& e) {
      VERBOSE_MSG(e.what());
      VERBOSE_MSG(pars);
      throw HimalayaError(e.what());
   }
   #endif // ENABLE_HIMALAYA

   return self_energy_3l;
}



void CLASSNAME::calculate_MVG_pole()
{
   // diagonalization with medium precision
   PHYSICAL(MVG) = 0.;
}

void CLASSNAME::calculate_MGlu_pole()
{
   // diagonalization with medium precision
   const double M_tree(MGlu);
   const double p = MGlu;
   const double self_energy_1  = Re(self_energy_Glu_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Glu_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Glu_1loop_PR(p));
   const auto M_loop = M_tree - self_energy_1 - M_tree * (self_energy_PL
      + self_energy_PR);
   PHYSICAL(MGlu) = calculate_singlet_mass(M_loop);
}

void CLASSNAME::calculate_MVP_pole()
{
   // diagonalization with medium precision
   PHYSICAL(MVP) = 0.;
}

void CLASSNAME::calculate_MVZ_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::VZ))
      return;

   // diagonalization with medium precision
   const double M_tree(Sqr(MVZ));
   const double p = MVZ;
   const double self_energy = Re(self_energy_VZ_1loop(p));
   const double mass_sqr = M_tree - self_energy;

   if (mass_sqr < 0.)
      problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::VZ);

   PHYSICAL(MVZ) = AbsSqrt(mass_sqr);
}

void CLASSNAME::calculate_MFd_pole()
{
   // diagonalization with medium precision
   const double M_tree(MFd);
   const double p = MFd;
   const double self_energy_1  = Re(self_energy_Fd_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fd_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fd_1loop_PR(p));
   const auto M_loop = M_tree - self_energy_1 - M_tree * (self_energy_PL
      + self_energy_PR);
   PHYSICAL(MFd) = calculate_singlet_mass(M_loop);
}

void CLASSNAME::calculate_MFs_pole()
{
   // diagonalization with medium precision
   const double M_tree(MFs);
   const double p = MFs;
   const double self_energy_1  = Re(self_energy_Fs_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fs_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fs_1loop_PR(p));
   const auto M_loop = M_tree - self_energy_1 - M_tree * (self_energy_PL
      + self_energy_PR);
   PHYSICAL(MFs) = calculate_singlet_mass(M_loop);
}

void CLASSNAME::calculate_MFb_pole()
{
   // diagonalization with medium precision
   const double M_tree(MFb);
   const double p = MFb;
   const double self_energy_1  = Re(self_energy_Fb_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fb_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fb_1loop_PR(p));
   const auto M_loop = M_tree - self_energy_1 - M_tree * (self_energy_PL
      + self_energy_PR);
   PHYSICAL(MFb) = calculate_singlet_mass(M_loop);
}

void CLASSNAME::calculate_MFu_pole()
{
   // diagonalization with medium precision
   const double M_tree(MFu);
   const double p = MFu;
   const double self_energy_1  = Re(self_energy_Fu_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fu_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fu_1loop_PR(p));
   const auto M_loop = M_tree - self_energy_1 - M_tree * (self_energy_PL
      + self_energy_PR);
   PHYSICAL(MFu) = calculate_singlet_mass(M_loop);
}

void CLASSNAME::calculate_MFc_pole()
{
   // diagonalization with medium precision
   const double M_tree(MFc);
   const double p = MFc;
   const double self_energy_1  = Re(self_energy_Fc_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fc_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fc_1loop_PR(p));
   const auto M_loop = M_tree - self_energy_1 - M_tree * (self_energy_PL
      + self_energy_PR);
   PHYSICAL(MFc) = calculate_singlet_mass(M_loop);
}

void CLASSNAME::calculate_MFt_pole()
{
   // diagonalization with medium precision
   const double M_tree(MFt);
   double qcd_1l = 0.;

   {
      double mst_1, mst_2, theta_t;
      calculate_MTopSquark_3rd_generation(mst_1, mst_2, theta_t);

      mssm_twoloop_mt::Parameters pars;
      pars.g3 = g3;
      pars.mt = MFt;
      pars.mg = MGlu;
      pars.mst1 = mst_1;
      pars.mst2 = mst_2;
      pars.msusy = Power(Abs(md2(2,2)),0.25)*Power(Abs(mq2(2,2)),0.25)
         ;
      pars.xt = Sin(2*theta_t) * (Sqr(mst_1) - Sqr(mst_2)) / (2. *
         pars.mt);
      pars.Q = get_scale();

      qcd_1l = - mssm_twoloop_mt::dMt_over_mt_1loop(pars);
   }

   double qcd_2l = 0.;

   if (pole_mass_loop_order > 1 && TOP_POLE_QCD_CORRECTION > 0) {
      double mst_1, mst_2, theta_t;
      calculate_MTopSquark_3rd_generation(mst_1, mst_2, theta_t);

      mssm_twoloop_mt::Parameters pars;
      pars.g3 = g3;
      pars.mt = MFt;
      pars.mg = MGlu;
      pars.mst1 = mst_1;
      pars.mst2 = mst_2;
      pars.msusy = Power(Abs(md2(2,2)),0.25)*Power(Abs(mq2(2,2)),0.25)
         ;
      pars.xt = Sin(2*theta_t) * (Sqr(mst_1) - Sqr(mst_2)) / (2. *
         pars.mt);
      pars.Q = get_scale();

      qcd_2l = - mssm_twoloop_mt::dMt_over_mt_2loop(pars);
   }

   double qcd_3l = 0.;

   if (pole_mass_loop_order > 2 && TOP_POLE_QCD_CORRECTION > 1) {
      const double currentScale = get_scale();
      qcd_3l = 0;
   }

   const double p = MFt;
   const double self_energy_1  = Re(self_energy_Ft_1loop_1_heavy(p));
   const double self_energy_PL = Re(self_energy_Ft_1loop_PL_heavy(p));
   const double self_energy_PR = Re(self_energy_Ft_1loop_PR_heavy(p));
   const auto M_loop = M_tree - self_energy_1 - M_tree * (self_energy_PL
      + self_energy_PR) - M_tree * (qcd_1l + qcd_2l + qcd_3l);

   PHYSICAL(MFt) = calculate_singlet_mass(M_loop);
}

void CLASSNAME::calculate_MFve_pole()
{
   // diagonalization with medium precision
   PHYSICAL(MFve) = 0.;
}

void CLASSNAME::calculate_MFvm_pole()
{
   // diagonalization with medium precision
   PHYSICAL(MFvm) = 0.;
}

void CLASSNAME::calculate_MFvt_pole()
{
   // diagonalization with medium precision
   PHYSICAL(MFvt) = 0.;
}

void CLASSNAME::calculate_MFe_pole()
{
   // diagonalization with medium precision
   const double M_tree(MFe);
   const double p = MFe;
   const double self_energy_1  = Re(self_energy_Fe_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fe_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fe_1loop_PR(p));
   const auto M_loop = M_tree - self_energy_1 - M_tree * (self_energy_PL
      + self_energy_PR);
   PHYSICAL(MFe) = calculate_singlet_mass(M_loop);
}

void CLASSNAME::calculate_MFm_pole()
{
   // diagonalization with medium precision
   const double M_tree(MFm);
   const double p = MFm;
   const double self_energy_1  = Re(self_energy_Fm_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fm_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fm_1loop_PR(p));
   const auto M_loop = M_tree - self_energy_1 - M_tree * (self_energy_PL
      + self_energy_PR);
   PHYSICAL(MFm) = calculate_singlet_mass(M_loop);
}

void CLASSNAME::calculate_MFtau_pole()
{
   // diagonalization with medium precision
   const double M_tree(MFtau);
   const double p = MFtau;
   const double self_energy_1  = Re(self_energy_Ftau_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Ftau_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Ftau_1loop_PR(p));
   const auto M_loop = M_tree - self_energy_1 - M_tree * (self_energy_PL
      + self_energy_PR);
   PHYSICAL(MFtau) = calculate_singlet_mass(M_loop);
}

void CLASSNAME::calculate_MSveL_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::SveL))
      return;

   // diagonalization with medium precision
   const double M_tree(get_mass_matrix_SveL());
   const double p = MSveL;
   double self_energy = Re(self_energy_SveL_1loop(p));
   const double mass_sqr = M_tree - self_energy;

   PHYSICAL(MSveL) = SignedAbsSqrt(mass_sqr);
}

void CLASSNAME::calculate_MSvmL_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::SvmL))
      return;

   // diagonalization with medium precision
   const double M_tree(get_mass_matrix_SvmL());
   const double p = MSvmL;
   double self_energy = Re(self_energy_SvmL_1loop(p));
   const double mass_sqr = M_tree - self_energy;

   PHYSICAL(MSvmL) = SignedAbsSqrt(mass_sqr);
}

void CLASSNAME::calculate_MSvtL_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::SvtL))
      return;

   // diagonalization with medium precision
   const double M_tree(get_mass_matrix_SvtL());
   const double p = MSvtL;
   double self_energy = Re(self_energy_SvtL_1loop(p));
   const double mass_sqr = M_tree - self_energy;

   PHYSICAL(MSvtL) = SignedAbsSqrt(mass_sqr);
}

void CLASSNAME::calculate_MSd_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::Sd))
      return;

   // diagonalization with medium precision
   const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Sd());

   for (int es = 0; es < 2; ++es) {
      const double p = Abs(MSd(es));
      Eigen::Matrix<double,2,2> self_energy = Re(self_energy_Sd_1loop(
         p));
      const Eigen::Matrix<double,2,2> M_loop(M_tree - self_energy);
      Eigen::Array<double,2,1> eigen_values;
      Eigen::Matrix<double,2,2> mix_ZD;
      #ifdef CHECK_EIGENVALUE_ERROR
         double eigenvalue_error;
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZD,
            eigenvalue_error);
         problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Sd,
            eigenvalue_error > precision * Abs(eigen_values(0)));
      #else
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZD);
      #endif

      PHYSICAL(MSd(es)) = SignedAbsSqrt(eigen_values(es));
      if (es == 0)
         PHYSICAL(ZD) = mix_ZD;
   }
}

void CLASSNAME::calculate_MSu_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::Su))
      return;

   // diagonalization with medium precision
   const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Su());

   for (int es = 0; es < 2; ++es) {
      const double p = Abs(MSu(es));
      Eigen::Matrix<double,2,2> self_energy = Re(self_energy_Su_1loop(
         p));
      const Eigen::Matrix<double,2,2> M_loop(M_tree - self_energy);
      Eigen::Array<double,2,1> eigen_values;
      Eigen::Matrix<double,2,2> mix_ZU;
      #ifdef CHECK_EIGENVALUE_ERROR
         double eigenvalue_error;
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZU,
            eigenvalue_error);
         problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Su,
            eigenvalue_error > precision * Abs(eigen_values(0)));
      #else
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZU);
      #endif

      PHYSICAL(MSu(es)) = SignedAbsSqrt(eigen_values(es));
      if (es == 0)
         PHYSICAL(ZU) = mix_ZU;
   }
}

void CLASSNAME::calculate_MSe_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::Se))
      return;

   // diagonalization with medium precision
   const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Se());

   for (int es = 0; es < 2; ++es) {
      const double p = Abs(MSe(es));
      Eigen::Matrix<double,2,2> self_energy = Re(self_energy_Se_1loop(
         p));
      const Eigen::Matrix<double,2,2> M_loop(M_tree - self_energy);
      Eigen::Array<double,2,1> eigen_values;
      Eigen::Matrix<double,2,2> mix_ZE;
      #ifdef CHECK_EIGENVALUE_ERROR
         double eigenvalue_error;
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZE,
            eigenvalue_error);
         problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Se,
            eigenvalue_error > precision * Abs(eigen_values(0)));
      #else
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZE);
      #endif

      PHYSICAL(MSe(es)) = SignedAbsSqrt(eigen_values(es));
      if (es == 0)
         PHYSICAL(ZE) = mix_ZE;
   }
}

void CLASSNAME::calculate_MSm_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::Sm))
      return;

   // diagonalization with medium precision
   const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Sm());

   for (int es = 0; es < 2; ++es) {
      const double p = Abs(MSm(es));
      Eigen::Matrix<double,2,2> self_energy = Re(self_energy_Sm_1loop(
         p));
      const Eigen::Matrix<double,2,2> M_loop(M_tree - self_energy);
      Eigen::Array<double,2,1> eigen_values;
      Eigen::Matrix<double,2,2> mix_ZM;
      #ifdef CHECK_EIGENVALUE_ERROR
         double eigenvalue_error;
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZM,
            eigenvalue_error);
         problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Sm,
            eigenvalue_error > precision * Abs(eigen_values(0)));
      #else
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZM);
      #endif

      PHYSICAL(MSm(es)) = SignedAbsSqrt(eigen_values(es));
      if (es == 0)
         PHYSICAL(ZM) = mix_ZM;
   }
}

void CLASSNAME::calculate_MStau_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::Stau))
      return;

   // diagonalization with medium precision
   const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Stau());

   for (int es = 0; es < 2; ++es) {
      const double p = Abs(MStau(es));
      Eigen::Matrix<double,2,2> self_energy = Re(
         self_energy_Stau_1loop(p));
      const Eigen::Matrix<double,2,2> M_loop(M_tree - self_energy);
      Eigen::Array<double,2,1> eigen_values;
      Eigen::Matrix<double,2,2> mix_ZTau;
      #ifdef CHECK_EIGENVALUE_ERROR
         double eigenvalue_error;
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZTau,
            eigenvalue_error);
         problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Stau,
            eigenvalue_error > precision * Abs(eigen_values(0)));
      #else
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZTau);
      #endif

      PHYSICAL(MStau(es)) = SignedAbsSqrt(eigen_values(es));
      if (es == 0)
         PHYSICAL(ZTau) = mix_ZTau;
   }
}

void CLASSNAME::calculate_MSs_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::Ss))
      return;

   // diagonalization with medium precision
   const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Ss());

   for (int es = 0; es < 2; ++es) {
      const double p = Abs(MSs(es));
      Eigen::Matrix<double,2,2> self_energy = Re(self_energy_Ss_1loop(
         p));
      const Eigen::Matrix<double,2,2> M_loop(M_tree - self_energy);
      Eigen::Array<double,2,1> eigen_values;
      Eigen::Matrix<double,2,2> mix_ZS;
      #ifdef CHECK_EIGENVALUE_ERROR
         double eigenvalue_error;
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZS,
            eigenvalue_error);
         problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Ss,
            eigenvalue_error > precision * Abs(eigen_values(0)));
      #else
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZS);
      #endif

      PHYSICAL(MSs(es)) = SignedAbsSqrt(eigen_values(es));
      if (es == 0)
         PHYSICAL(ZS) = mix_ZS;
   }
}

void CLASSNAME::calculate_MSc_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::Sc))
      return;

   // diagonalization with medium precision
   const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Sc());

   for (int es = 0; es < 2; ++es) {
      const double p = Abs(MSc(es));
      Eigen::Matrix<double,2,2> self_energy = Re(self_energy_Sc_1loop(
         p));
      const Eigen::Matrix<double,2,2> M_loop(M_tree - self_energy);
      Eigen::Array<double,2,1> eigen_values;
      Eigen::Matrix<double,2,2> mix_ZC;
      #ifdef CHECK_EIGENVALUE_ERROR
         double eigenvalue_error;
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZC,
            eigenvalue_error);
         problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Sc,
            eigenvalue_error > precision * Abs(eigen_values(0)));
      #else
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZC);
      #endif

      PHYSICAL(MSc(es)) = SignedAbsSqrt(eigen_values(es));
      if (es == 0)
         PHYSICAL(ZC) = mix_ZC;
   }
}

void CLASSNAME::calculate_MSb_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::Sb))
      return;

   // diagonalization with medium precision
   const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Sb());

   for (int es = 0; es < 2; ++es) {
      const double p = Abs(MSb(es));
      Eigen::Matrix<double,2,2> self_energy = Re(self_energy_Sb_1loop(
         p));
      const Eigen::Matrix<double,2,2> M_loop(M_tree - self_energy);
      Eigen::Array<double,2,1> eigen_values;
      Eigen::Matrix<double,2,2> mix_ZB;
      #ifdef CHECK_EIGENVALUE_ERROR
         double eigenvalue_error;
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZB,
            eigenvalue_error);
         problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Sb,
            eigenvalue_error > precision * Abs(eigen_values(0)));
      #else
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZB);
      #endif

      PHYSICAL(MSb(es)) = SignedAbsSqrt(eigen_values(es));
      if (es == 0)
         PHYSICAL(ZB) = mix_ZB;
   }
}

void CLASSNAME::calculate_MSt_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::St))
      return;

   // diagonalization with medium precision
   const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_St());

   for (int es = 0; es < 2; ++es) {
      const double p = Abs(MSt(es));
      Eigen::Matrix<double,2,2> self_energy = Re(self_energy_St_1loop(
         p));
      const Eigen::Matrix<double,2,2> M_loop(M_tree - self_energy);
      Eigen::Array<double,2,1> eigen_values;
      Eigen::Matrix<double,2,2> mix_ZT;
      #ifdef CHECK_EIGENVALUE_ERROR
         double eigenvalue_error;
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZT,
            eigenvalue_error);
         problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::St,
            eigenvalue_error > precision * Abs(eigen_values(0)));
      #else
         fs_diagonalize_hermitian(M_loop, eigen_values, mix_ZT);
      #endif

      PHYSICAL(MSt(es)) = SignedAbsSqrt(eigen_values(es));
      if (es == 0)
         PHYSICAL(ZT) = mix_ZT;
   }
}

void CLASSNAME::calculate_Mhh_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::hh))
      return;

   // diagonalization with high precision
   const auto number_of_mass_iterations = get_number_of_mass_iterations()
      ;
   int iteration = 0;
   double diff = 0.0;
   decltype(Mhh) old_Mhh(Mhh), new_Mhh(Mhh);

   // two-loop Higgs self-energy contributions
   Eigen::Matrix<double,2,2> self_energy_2l(Eigen::Matrix<double,2,2>
      ::Zero());

   if (pole_mass_loop_order > 1) {
      self_energy_2l = self_energy_hh_2loop();
      for (int i = 0; i < 2; i++) {
         for (int k = 0; k < 2; k++) {
            if (!std::isfinite(self_energy_2l(i,k))) {
               self_energy_2l(i,k) = 0.;
               problems.flag_bad_mass(
                  NUHMSSMNoFVHimalaya_info::hh);
            }
         }
      }
   }

   // three-loop Higgs self-energy contributions
   Eigen::Matrix<double,2,2> self_energy_3l(Eigen::Matrix<double,2,2>
      ::Zero());

   try {
      if (pole_mass_loop_order > 2)
         self_energy_3l = self_energy_hh_3loop();
   } catch (const flexiblesusy::Error& e) {
      WARNING("3-loop Higgs mass calculation failed: " << e.what());
      problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::hh);
   }

   do {
      const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_hh());

      for (int es = 0; es < 2; ++es) {
         const double p = Abs(old_Mhh(es));
         Eigen::Matrix<double,2,2> self_energy = Re(
            self_energy_hh_1loop(p));
         self_energy += self_energy_2l;
         self_energy += self_energy_3l;
         const Eigen::Matrix<double,2,2> M_loop(M_tree -
            self_energy);
         Eigen::Array<double,2,1> eigen_values;
         Eigen::Matrix<double,2,2> mix_ZH;
         #ifdef CHECK_EIGENVALUE_ERROR
            double eigenvalue_error;
            fs_diagonalize_hermitian(M_loop, eigen_values,
               mix_ZH, eigenvalue_error);
            problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::hh,
               eigenvalue_error > precision * Abs(eigen_values(0)));
         #else
            fs_diagonalize_hermitian(M_loop, eigen_values,
               mix_ZH);
         #endif

         PHYSICAL(Mhh(es)) = SignedAbsSqrt(eigen_values(es));
         if (es == 0)
            PHYSICAL(ZH) = mix_ZH;
      }

      new_Mhh = PHYSICAL(Mhh);
      diff = MaxRelDiff(new_Mhh, old_Mhh);
      old_Mhh = new_Mhh;
      iteration++;
   } while (diff > precision
            && iteration < number_of_mass_iterations);

   if (diff > precision)
      problems.flag_no_pole_mass_convergence(
         NUHMSSMNoFVHimalaya_info::hh);
   else
      problems.unflag_no_pole_mass_convergence(
         NUHMSSMNoFVHimalaya_info::hh);
}

void CLASSNAME::calculate_MAh_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::Ah))
      return;

   // diagonalization with high precision
   const auto number_of_mass_iterations = get_number_of_mass_iterations()
      ;
   int iteration = 0;
   double diff = 0.0;
   decltype(MAh) old_MAh(MAh), new_MAh(MAh);

   // two-loop Higgs self-energy contributions
   Eigen::Matrix<double,2,2> self_energy_2l(Eigen::Matrix<double,2,2>
      ::Zero());

   if (pole_mass_loop_order > 1) {
      self_energy_2l = self_energy_Ah_2loop();
      for (int i = 0; i < 2; i++) {
         for (int k = 0; k < 2; k++) {
            if (!std::isfinite(self_energy_2l(i,k))) {
               self_energy_2l(i,k) = 0.;
               problems.flag_bad_mass(
                  NUHMSSMNoFVHimalaya_info::Ah);
            }
         }
      }
   }

   do {
      const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Ah());

      for (int es = 0; es < 2; ++es) {
         const double p = Abs(old_MAh(es));
         Eigen::Matrix<double,2,2> self_energy = Re(
            self_energy_Ah_1loop(p));
         self_energy += self_energy_2l;
         const Eigen::Matrix<double,2,2> M_loop(M_tree -
            self_energy);
         Eigen::Array<double,2,1> eigen_values;
         Eigen::Matrix<double,2,2> mix_ZA;
         #ifdef CHECK_EIGENVALUE_ERROR
            double eigenvalue_error;
            fs_diagonalize_hermitian(M_loop, eigen_values,
               mix_ZA, eigenvalue_error);
            problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Ah,
               eigenvalue_error > precision * Abs(eigen_values(0)));
         #else
            fs_diagonalize_hermitian(M_loop, eigen_values,
               mix_ZA);
         #endif

         PHYSICAL(MAh(es)) = SignedAbsSqrt(eigen_values(es));
         if (es == 1)
            PHYSICAL(ZA) = mix_ZA;
      }

      new_MAh = PHYSICAL(MAh);
      diff = MaxRelDiff(new_MAh, old_MAh);
      old_MAh = new_MAh;
      iteration++;
   } while (diff > precision
            && iteration < number_of_mass_iterations);

   if (diff > precision)
      problems.flag_no_pole_mass_convergence(
         NUHMSSMNoFVHimalaya_info::Ah);
   else
      problems.unflag_no_pole_mass_convergence(
         NUHMSSMNoFVHimalaya_info::Ah);
}

void CLASSNAME::calculate_MHpm_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::Hpm))
      return;

   // diagonalization with high precision
   const auto number_of_mass_iterations = get_number_of_mass_iterations()
      ;
   int iteration = 0;
   double diff = 0.0;
   decltype(MHpm) old_MHpm(MHpm), new_MHpm(MHpm);

   do {
      const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Hpm());

      for (int es = 0; es < 2; ++es) {
         const double p = Abs(old_MHpm(es));
         Eigen::Matrix<double,2,2> self_energy = Re(
            self_energy_Hpm_1loop(p));
         const Eigen::Matrix<double,2,2> M_loop(M_tree -
            self_energy);
         Eigen::Array<double,2,1> eigen_values;
         Eigen::Matrix<double,2,2> mix_ZP;
         #ifdef CHECK_EIGENVALUE_ERROR
            double eigenvalue_error;
            fs_diagonalize_hermitian(M_loop, eigen_values,
               mix_ZP, eigenvalue_error);
            problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Hpm
               , eigenvalue_error > precision * Abs(eigen_values(0)));
         #else
            fs_diagonalize_hermitian(M_loop, eigen_values,
               mix_ZP);
         #endif

         PHYSICAL(MHpm(es)) = SignedAbsSqrt(eigen_values(es));
         if (es == 1)
            PHYSICAL(ZP) = mix_ZP;
      }

      new_MHpm = PHYSICAL(MHpm);
      diff = MaxRelDiff(new_MHpm, old_MHpm);
      old_MHpm = new_MHpm;
      iteration++;
   } while (diff > precision
            && iteration < number_of_mass_iterations);

   if (diff > precision)
      problems.flag_no_pole_mass_convergence(
         NUHMSSMNoFVHimalaya_info::Hpm);
   else
      problems.unflag_no_pole_mass_convergence(
         NUHMSSMNoFVHimalaya_info::Hpm);
}

void CLASSNAME::calculate_MChi_pole()
{
   // diagonalization with medium precision
   const Eigen::Matrix<double,4,4> M_tree(get_mass_matrix_Chi());
   for (int es = 0; es < 4; ++es) {
      const double p = Abs(MChi(es));
      const Eigen::Matrix<double,4,4> self_energy_1  = Re(
         self_energy_Chi_1loop_1(p));
      const Eigen::Matrix<double,4,4> self_energy_PL = Re(
         self_energy_Chi_1loop_PL(p));
      const Eigen::Matrix<double,4,4> self_energy_PR = Re(
         self_energy_Chi_1loop_PR(p));
      const Eigen::Matrix<double,4,4> delta_M(- self_energy_PR *
         M_tree - M_tree * self_energy_PL - self_energy_1);
      const Eigen::Matrix<double,4,4> M_loop(M_tree + 0.5 * (delta_M +
         delta_M.transpose()));
      Eigen::Array<double,4,1> eigen_values;
      decltype(ZN) mix_ZN;
      #ifdef CHECK_EIGENVALUE_ERROR
         double eigenvalue_error;
         fs_diagonalize_symmetric(M_loop, eigen_values, mix_ZN,
            eigenvalue_error);
         problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Chi,
            eigenvalue_error > precision * Abs(eigen_values(0)));
      #else
         fs_diagonalize_symmetric(M_loop, eigen_values, mix_ZN);
      #endif
      if (es == 0)
         PHYSICAL(ZN) = mix_ZN;
      PHYSICAL(MChi(es)) = Abs(eigen_values(es));
   }
}

void CLASSNAME::calculate_MCha_pole()
{
   // diagonalization with medium precision
   const Eigen::Matrix<double,2,2> M_tree(get_mass_matrix_Cha());
   for (int es = 0; es < 2; ++es) {
      const double p = Abs(MCha(es));
      const Eigen::Matrix<double,2,2> self_energy_1  = Re(
         self_energy_Cha_1loop_1(p));
      const Eigen::Matrix<double,2,2> self_energy_PL = Re(
         self_energy_Cha_1loop_PL(p));
      const Eigen::Matrix<double,2,2> self_energy_PR = Re(
         self_energy_Cha_1loop_PR(p));
      const Eigen::Matrix<double,2,2> delta_M(- self_energy_PR *
         M_tree - M_tree * self_energy_PL - self_energy_1);
      const Eigen::Matrix<double,2,2> M_loop(M_tree + delta_M);
      Eigen::Array<double,2,1> eigen_values;
      decltype(UM) mix_UM;
      decltype(UP) mix_UP;
   #ifdef CHECK_EIGENVALUE_ERROR
      double eigenvalue_error;
      fs_svd(M_loop, eigen_values, mix_UM, mix_UP, eigenvalue_error);
      problems.flag_bad_mass(NUHMSSMNoFVHimalaya_info::Cha,
         eigenvalue_error > precision * Abs(eigen_values(0)));
   #else
      fs_svd(M_loop, eigen_values, mix_UM, mix_UP);
   #endif
      if (es == 0) {
         PHYSICAL(UM) = mix_UM;
         PHYSICAL(UP) = mix_UP;
      }
      PHYSICAL(MCha(es)) = Abs(eigen_values(es));
   }
}

void CLASSNAME::calculate_MVWm_pole()
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::VWm))
      return;

   // diagonalization with medium precision
   const double M_tree(Sqr(MVWm));
   const double p = MVWm;
   const double self_energy = Re(self_energy_VWm_1loop(p));
   const double mass_sqr = M_tree - self_energy;

   if (mass_sqr < 0.)
      problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::VWm);

   PHYSICAL(MVWm) = AbsSqrt(mass_sqr);
}

double CLASSNAME::calculate_MVWm_pole(double p)
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::VWm))
      return 0.;

   const double self_energy = Re(self_energy_VWm_1loop(p));
   const double mass_sqr = Sqr(MVWm) - self_energy;

   if (mass_sqr < 0.)
      problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::VWm);

   return AbsSqrt(mass_sqr);
}

double CLASSNAME::calculate_MVZ_pole(double p)
{
   if (!force_output && problems.is_running_tachyon(
      NUHMSSMNoFVHimalaya_info::VZ))
      return 0.;

   const double self_energy = Re(self_energy_VZ_1loop(p));
   const double mass_sqr = Sqr(MVZ) - self_energy;

   if (mass_sqr < 0.)
      problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::VZ);

   return AbsSqrt(mass_sqr);
}


double CLASSNAME::calculate_MFve_DRbar(double m_pole) const
{
   const double p = m_pole;
   const double self_energy_1  = Re(self_energy_Fve_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fve_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fve_1loop_PR(p));

   const double m_drbar = m_pole + self_energy_1 + m_pole * (
      self_energy_PL + self_energy_PR);

   return m_drbar;
}

double CLASSNAME::calculate_MFvm_DRbar(double m_pole) const
{
   const double p = m_pole;
   const double self_energy_1  = Re(self_energy_Fvm_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fvm_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fvm_1loop_PR(p));

   const double m_drbar = m_pole + self_energy_1 + m_pole * (
      self_energy_PL + self_energy_PR);

   return m_drbar;
}

double CLASSNAME::calculate_MFvt_DRbar(double m_pole) const
{
   const double p = m_pole;
   const double self_energy_1  = Re(self_energy_Fvt_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fvt_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fvt_1loop_PR(p));

   const double m_drbar = m_pole + self_energy_1 + m_pole * (
      self_energy_PL + self_energy_PR);

   return m_drbar;
}

double CLASSNAME::calculate_MFe_DRbar(double m_sm_msbar) const
{
   const double p = m_sm_msbar;
   const double self_energy_1  = Re(self_energy_Fe_1loop_1_heavy_rotated(
      p));
   const double self_energy_PL = Re(self_energy_Fe_1loop_PL_heavy_rotated
      (p));
   const double self_energy_PR = Re(self_energy_Fe_1loop_PR_heavy_rotated
      (p));
   const double drbar_conversion = 1 - 0.0023747152416172916*(0.6*Sqr(g1)
      - Sqr(g2));
   const double m_sm_drbar = m_sm_msbar * drbar_conversion;
   const double delta_mf_1loop = - self_energy_1/m_sm_drbar -
      self_energy_PL - self_energy_PR;

   const double m_susy_drbar = m_sm_drbar / (1.0 + delta_mf_1loop);

   return m_susy_drbar;
}

double CLASSNAME::calculate_MFm_DRbar(double m_sm_msbar) const
{
   const double p = m_sm_msbar;
   const double self_energy_1  = Re(self_energy_Fm_1loop_1_heavy_rotated(
      p));
   const double self_energy_PL = Re(self_energy_Fm_1loop_PL_heavy_rotated
      (p));
   const double self_energy_PR = Re(self_energy_Fm_1loop_PR_heavy_rotated
      (p));
   const double drbar_conversion = 1 - 0.0023747152416172916*(0.6*Sqr(g1)
      - Sqr(g2));
   const double m_sm_drbar = m_sm_msbar * drbar_conversion;
   const double delta_mf_1loop = - self_energy_1/m_sm_drbar -
      self_energy_PL - self_energy_PR;

   const double m_susy_drbar = m_sm_drbar / (1.0 + delta_mf_1loop);

   return m_susy_drbar;
}

double CLASSNAME::calculate_MFtau_DRbar(double m_sm_msbar) const
{
   const double p = m_sm_msbar;
   const double self_energy_1  = Re(
      self_energy_Ftau_1loop_1_heavy_rotated(p));
   const double self_energy_PL = Re(
      self_energy_Ftau_1loop_PL_heavy_rotated(p));
   const double self_energy_PR = Re(
      self_energy_Ftau_1loop_PR_heavy_rotated(p));
   const double drbar_conversion = 1 - 0.0023747152416172916*(0.6*Sqr(g1)
      - Sqr(g2));
   const double m_sm_drbar = m_sm_msbar * drbar_conversion;
   const double delta_mf_1loop = - self_energy_1/m_sm_drbar -
      self_energy_PL - self_energy_PR;

   const double m_susy_drbar = m_sm_drbar / (1.0 + delta_mf_1loop);

   return m_susy_drbar;
}

double CLASSNAME::calculate_MFu_DRbar(double m_pole) const
{
   const double p = m_pole;
   const double self_energy_1  = Re(self_energy_Fu_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fu_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fu_1loop_PR(p));

   const double m_drbar = m_pole + self_energy_1 + m_pole * (
      self_energy_PL + self_energy_PR);

   return m_drbar;
}

double CLASSNAME::calculate_MFc_DRbar(double m_pole) const
{
   const double p = m_pole;
   const double self_energy_1  = Re(self_energy_Fc_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fc_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fc_1loop_PR(p));

   const double m_drbar = m_pole + self_energy_1 + m_pole * (
      self_energy_PL + self_energy_PR);

   return m_drbar;
}

double CLASSNAME::calculate_MFt_DRbar(double m_pole) const
{
   const double p = m_pole;
   const double self_energy_1  = Re(self_energy_Ft_1loop_1_heavy_rotated(
      p));
   const double self_energy_PL = Re(self_energy_Ft_1loop_PL_heavy_rotated
      (p));
   const double self_energy_PR = Re(self_energy_Ft_1loop_PR_heavy_rotated
      (p));

   const double currentScale = get_scale();
   double qcd_1l = 0., qcd_2l = 0., qcd_3l = 0.;

   {
      double mst_1, mst_2, theta_t;
      calculate_MTopSquark_3rd_generation(mst_1, mst_2, theta_t);

      mssm_twoloop_mt::Parameters pars;
      pars.g3 = g3;
      pars.mt = MFt;
      pars.mg = MGlu;
      pars.mst1 = mst_1;
      pars.mst2 = mst_2;
      pars.msusy = Power(Abs(md2(2,2)),0.25)*Power(Abs(mq2(2,2)),0.25)
         ;
      pars.xt = Sin(2*theta_t) * (Sqr(mst_1) - Sqr(mst_2)) / (2. *
         pars.mt);
      pars.Q = get_scale();

      qcd_1l = - mssm_twoloop_mt::dMt_over_mt_1loop(pars);
   }

   if (get_thresholds() > 1 && threshold_corrections.mt > 1) {
      double mst_1, mst_2, theta_t;
      calculate_MTopSquark_3rd_generation(mst_1, mst_2, theta_t);

      mssm_twoloop_mt::Parameters pars;
      pars.g3 = g3;
      pars.mt = MFt;
      pars.mg = MGlu;
      pars.mst1 = mst_1;
      pars.mst2 = mst_2;
      pars.msusy = Power(Abs(md2(2,2)),0.25)*Power(Abs(mq2(2,2)),0.25)
         ;
      pars.xt = Sin(2*theta_t) * (Sqr(mst_1) - Sqr(mst_2)) / (2. *
         pars.mt);
      pars.Q = get_scale();

      const double q_2l = mssm_twoloop_mt::dMt_over_mt_2loop(pars);

      qcd_2l = -q_2l + qcd_1l * qcd_1l;
   }

   const double m_susy_drbar = m_pole + self_energy_1 + m_pole * (
      self_energy_PL + self_energy_PR + qcd_1l + qcd_2l + qcd_3l);

   return m_susy_drbar;
}

double CLASSNAME::calculate_MFd_DRbar(double m_pole) const
{
   const double p = m_pole;
   const double self_energy_1  = Re(self_energy_Fd_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fd_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fd_1loop_PR(p));

   const double m_drbar = m_pole + self_energy_1 + m_pole * (
      self_energy_PL + self_energy_PR);

   return m_drbar;
}

double CLASSNAME::calculate_MFs_DRbar(double m_pole) const
{
   const double p = m_pole;
   const double self_energy_1  = Re(self_energy_Fs_1loop_1(p));
   const double self_energy_PL = Re(self_energy_Fs_1loop_PL(p));
   const double self_energy_PR = Re(self_energy_Fs_1loop_PR(p));

   const double m_drbar = m_pole + self_energy_1 + m_pole * (
      self_energy_PL + self_energy_PR);

   return m_drbar;
}

double CLASSNAME::calculate_MFb_DRbar(double m_sm_msbar) const
{
   const double p = m_sm_msbar;
   const double self_energy_1  = Re(self_energy_Fb_1loop_1_heavy_rotated(
      p));
   const double self_energy_PL = Re(self_energy_Fb_1loop_PL_heavy_rotated
      (p));
   const double self_energy_PR = Re(self_energy_Fb_1loop_PR_heavy_rotated
      (p));
   const double m_tree = MFb;
   const double drbar_conversion = 1 - 0.00020496318737651018*Quad(g3) +
      0.0006860288475783287*Sqr(g1) + 0.0023747152416172916*Sqr(g2) -
      0.008443431970194815*Sqr(g3);
   const double m_sm_drbar = m_sm_msbar * drbar_conversion;
   const double delta_mb_1loop = - self_energy_1/m_tree - self_energy_PL
      - self_energy_PR;
   double qcd_2l = 0.;

   if (get_thresholds() > 1 && threshold_corrections.mb > 1) {
      double mst_1, mst_2, theta_t;
      calculate_MTopSquark_3rd_generation(mst_1, mst_2, theta_t);
      double msb_1, msb_2, theta_b;
      calculate_MBottomSquark_3rd_generation(msb_1, msb_2, theta_b);

      mssm_twoloop_mb::Parameters pars;
      pars.g3 = g3;
      pars.mt = MFt;
      pars.mb = MFb;
      pars.mg = MGlu;
      pars.mst1 = mst_1;
      pars.mst2 = mst_2;
      pars.msb1 = msb_1;
      pars.msb2 = msb_2;
      pars.msusy = Power(AbsSqrt(md2(0,0))*AbsSqrt(md2(1,1))*AbsSqrt(
         mq2(0,0))*AbsSqrt(mq2(1,1))*AbsSqrt(mu2(0,0))*AbsSqrt(mu2(1,1)),
         0.16666666666666666);
      pars.xt = Sin(2*theta_t) * (Sqr(mst_1) - Sqr(mst_2)) / (2. *
         pars.mt);
      pars.xb = Sin(2*theta_b) * (Sqr(msb_1) - Sqr(msb_2)) / (2. *
         pars.mb);
      pars.Q = get_scale();

      qcd_2l = mssm_twoloop_mb::delta_mb_2loop(pars);

      const double alpha_s = Sqr(pars.g3) / (4.*Pi);

      // subtract 2-loop epsilon scalar contributions
      qcd_2l -= 31./72. * Sqr(alpha_s / Pi)
         + alpha_s/(3.*Pi) * delta_mb_1loop;
   }

   const double m_susy_drbar = m_sm_drbar / (1.0 + delta_mb_1loop +
      qcd_2l);

   return m_susy_drbar;
}

double CLASSNAME::calculate_MVP_DRbar(double)
{
   return 0.0;
}

double CLASSNAME::calculate_MVZ_DRbar(double m_pole)
{
   const double p = m_pole;
   const double self_energy = Re(self_energy_VZ_1loop(p));
   const double mass_sqr = Sqr(m_pole) + self_energy;

   if (mass_sqr < 0.) {
      problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::VZ);
      return m_pole;
   }

   return AbsSqrt(mass_sqr);
}

double CLASSNAME::calculate_MVWm_DRbar(double m_pole)
{
   const double p = m_pole;
   const double self_energy = Re(self_energy_VWm_1loop(p));
   const double mass_sqr = Sqr(m_pole) + self_energy;

   if (mass_sqr < 0.) {
      problems.flag_pole_tachyon(NUHMSSMNoFVHimalaya_info::VWm);
      return m_pole;
   }

   return AbsSqrt(mass_sqr);
}


double CLASSNAME::v() const
{
   return Sqrt(Sqr(vd) + Sqr(vu));
}

double CLASSNAME::Betax() const
{
   return ArcSin(Abs(ZP(0,1)));
}

double CLASSNAME::Alpha() const
{
   return ArcCos(ZH(0,1));
}

double CLASSNAME::ThetaW() const
{
   return ArcCos(Abs(ZZ(0,0)));
}


std::ostream& operator<<(std::ostream& ostr, const NUHMSSMNoFVHimalaya_mass_eigenstates& model)
{
   model.print(ostr);
   return ostr;
}

} // namespace flexiblesusy
